@@ -34,10 +34,12 @@ import javax.ws.rs.core.Context;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.Response;
 import javax.ws.rs.core.UriInfo;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.broker.api.AccessControl;
 import org.apache.pinot.broker.api.HttpRequesterIdentity;
 import org.apache.pinot.core.auth.FineGrainedAuthUtils;
 import org.apache.pinot.core.auth.ManualAuthorization;
+import org.apache.pinot.spi.auth.AuthorizationResult;
 import org.glassfish.grizzly.http.server.Request;
 
 /**
@@ -82,9 +84,15 @@ public class AuthenticationFilter implements ContainerRequestFilter {
 
     HttpRequesterIdentity httpRequestIdentity = HttpRequesterIdentity.fromRequest(request);
 
-    // default authorization handling
-    if (!accessControl.hasAccess(httpRequestIdentity)) {
-      throw new WebApplicationException("Failed access check for " + httpRequestIdentity.getEndpointUrl(),
+    AuthorizationResult authorizationResult = accessControl.authorize(httpRequestIdentity);
+
+    if (!authorizationResult.hasAccess()) {
+      String failureMessage = authorizationResult.getFailureMessage();
+      if (StringUtils.isNotBlank(failureMessage)) {
+        failureMessage = "Reason: " + failureMessage;
+      }
+      throw new WebApplicationException(
+          "Failed access check for " + httpRequestIdentity.getEndpointUrl() + "." + failureMessage,
           Response.Status.FORBIDDEN);
     }
 
