@@ -22,6 +22,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.helix.model.ExternalView;
 import org.apache.helix.model.IdealState;
 import org.apache.pinot.broker.routing.segmentpreselector.SegmentPreSelector;
@@ -34,8 +35,8 @@ import org.apache.pinot.common.request.BrokerRequest;
 public interface InstanceSelector {
   long NEW_SEGMENT_EXPIRATION_MILLIS = TimeUnit.MINUTES.toMillis(5);
 
-  static boolean isNewSegment(long pushMillis, long nowMillis) {
-    return nowMillis - pushMillis <= NEW_SEGMENT_EXPIRATION_MILLIS;
+  static boolean isNewSegment(long creationTimeMs, long currentTimeMs) {
+    return creationTimeMs > 0 && currentTimeMs - creationTimeMs <= NEW_SEGMENT_EXPIRATION_MILLIS;
   }
 
   /**
@@ -75,16 +76,12 @@ public interface InstanceSelector {
   Set<String> getServingInstances();
 
   class SelectionResult {
-    private final Map<String, String> _segmentToInstanceMap;
+    private final Pair<Map<String, String>, Map<String, String>/*optional segments*/> _segmentToInstanceMap;
     private final List<String> _unavailableSegments;
     private int _numPrunedSegments;
 
-    public SelectionResult(Map<String, String> segmentToInstanceMap, List<String> unavailableSegments) {
-      this(segmentToInstanceMap, unavailableSegments, 0);
-    }
-
-    public SelectionResult(Map<String, String> segmentToInstanceMap, List<String> unavailableSegments,
-        int numPrunedSegments) {
+    public SelectionResult(Pair<Map<String, String>, Map<String, String>> segmentToInstanceMap,
+        List<String> unavailableSegments, int numPrunedSegments) {
       _segmentToInstanceMap = segmentToInstanceMap;
       _unavailableSegments = unavailableSegments;
       _numPrunedSegments = numPrunedSegments;
@@ -94,7 +91,15 @@ public interface InstanceSelector {
      * Returns the map from segment to selected server instance hosting the segment.
      */
     public Map<String, String> getSegmentToInstanceMap() {
-      return _segmentToInstanceMap;
+      return _segmentToInstanceMap.getLeft();
+    }
+
+    /**
+     * Returns the map from optional segment to selected server instance hosting the optional segment.
+     * Optional segments can be skipped by broker or server upon any issue w/o failing the query.
+     */
+    public Map<String, String> getOptionalSegmentToInstanceMap() {
+      return _segmentToInstanceMap.getRight();
     }
 
     /**
