@@ -46,6 +46,7 @@ public class ExecutionStats {
   private static final String NUM_GROUPS_LIMIT_REACHED = "numGroupsLimitReached";
   private static final String BROKER_REDUCE_TIME_MS = "brokerReduceTimeMs";
   private static final String TIME_USED_MS = "timeUsedMs";
+  private static final String PARTIAL_RESULT = "partialResult";
 
   private final JsonNode _brokerResponse;
 
@@ -110,6 +111,10 @@ public class ExecutionStats {
     return _brokerResponse.has(NUM_GROUPS_LIMIT_REACHED) && _brokerResponse.get(NUM_GROUPS_LIMIT_REACHED).asBoolean();
   }
 
+  public boolean isPartialResult() {
+    return _brokerResponse.has(PARTIAL_RESULT) && _brokerResponse.get(PARTIAL_RESULT).asBoolean();
+  }
+
   public long getTimeUsedMs() {
     return _brokerResponse.has(TIME_USED_MS) ? _brokerResponse.get(TIME_USED_MS).asLong() : -1L;
   }
@@ -135,6 +140,7 @@ public class ExecutionStats {
     map.put(NUM_GROUPS_LIMIT_REACHED, isNumGroupsLimitReached());
     map.put(BROKER_REDUCE_TIME_MS, getBrokerReduceTimeMs() + "ms");
     map.put(TIME_USED_MS, getTimeUsedMs() + "ms");
+    map.put(PARTIAL_RESULT, isPartialResult());
     return map.toString();
   }
 }
