@@ -38,7 +38,7 @@ import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pinot.client.controller.PinotControllerTransport;
 import org.apache.pinot.client.controller.PinotControllerTransportFactory;
 import org.apache.pinot.client.utils.DriverUtils;
-import org.apache.pinot.common.utils.TlsUtils;
+import org.apache.pinot.common.utils.tls.TlsUtils;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.slf4j.LoggerFactory;
 
@@ -80,8 +80,14 @@ public class PinotDriver implements Driver {
   @Override
   public Connection connect(String url, Properties info)
       throws SQLException {
+
+      PinotClientTransport pinotClientTransport = null;
+      PinotControllerTransport pinotControllerTransport = null;
     try {
-      LOGGER.info("Initiating connection to database for url: " + url);
+      if (!this.acceptsURL(url)) {
+        return null;
+      }
+      LOGGER.info("Initiating connection to database for url: {}", url);
 
       Map<String, String> urlParams = DriverUtils.getURLParams(url);
       info.putAll(urlParams);
@@ -112,18 +118,43 @@ public class PinotDriver implements Driver {
         pinotControllerTransportFactory.setHeaders(headers);
       }
 
-      PinotClientTransport pinotClientTransport = factory.withConnectionProperties(info).buildTransport();
-      PinotControllerTransport pinotControllerTransport = pinotControllerTransportFactory
+      pinotClientTransport = factory.withConnectionProperties(info).buildTransport();
+      pinotControllerTransport = pinotControllerTransportFactory
               .withConnectionProperties(info)
               .buildTransport();
       String controllerUrl = DriverUtils.getControllerFromURL(url);
       String tenant = info.getProperty(INFO_TENANT, DEFAULT_TENANT);
       return new PinotConnection(info, controllerUrl, pinotClientTransport, tenant, pinotControllerTransport);
     } catch (Exception e) {
+      closeResourcesSafely(pinotClientTransport, pinotControllerTransport);
       throw new SQLException(String.format("Failed to connect to url : %s", url), e);
     }
   }
 
+  /**
+   * If something goes wrong generating the connection, the transports need to be safely closed to prevent leaks.
+   * @param pinotClientTransport connection client transport
+   * @param pinotControllerTransport connection controller transport
+   */
+  private static void closeResourcesSafely(PinotClientTransport pinotClientTransport,
+      PinotControllerTransport pinotControllerTransport) {
+    try {
+      if (pinotClientTransport != null) {
+        pinotClientTransport.close();
+      }
+    } catch (PinotClientException ignored) {
+      // ignored
+    }
+
+    try {
+      if (pinotControllerTransport != null) {
+        pinotControllerTransport.close();
+      }
+    } catch (PinotClientException ignored) {
+      // ignored
+    }
+  }
+
   private Map<String, String> getHeadersFromProperties(Properties info) {
     return info.entrySet().stream().filter(entry -> entry.getKey().toString().startsWith(INFO_HEADERS + ".")).map(
             entry -> Pair.of(entry.getKey().toString().substring(INFO_HEADERS.length() + 1),
