@@ -32,6 +32,7 @@ import java.util.List;
 import java.util.concurrent.CompletionService;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executors;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
 import org.apache.http.util.EntityUtils;
 import org.slf4j.Logger;
@@ -43,6 +44,30 @@ import org.testng.annotations.Test;
 
 
 public class MultiHttpRequestTest {
+  class TestResult {
+    private final int _success;
+    private final int _errors;
+    private final int _timeouts;
+
+    public TestResult(int success, int errors, int timeouts) {
+      _success = success;
+      _errors = errors;
+      _timeouts = timeouts;
+    }
+
+    public int getSuccess() {
+      return _success;
+    }
+
+    public int getErrors() {
+      return _errors;
+    }
+
+    public int getTimeouts() {
+      return _timeouts;
+    }
+  }
+
   private static final Logger LOGGER = LoggerFactory.getLogger(MultiHttpRequest.class);
   private static final String SUCCESS_MSG = "success";
   private static final String ERROR_MSG = "error";
@@ -61,6 +86,7 @@ public class MultiHttpRequestTest {
     startServer(_portStart, createHandler(SUCCESS_CODE, SUCCESS_MSG, 0));
     startServer(_portStart + 1, createHandler(ERROR_CODE, ERROR_MSG, 0));
     startServer(_portStart + 2, createHandler(SUCCESS_CODE, TIMEOUT_MSG, TIMEOUT_MS));
+    startServer(_portStart + 3, createPostHandler(SUCCESS_CODE, SUCCESS_MSG, 0));
   }
 
   @AfterTest
@@ -90,6 +116,33 @@ public class MultiHttpRequestTest {
     };
   }
 
+  private HttpHandler createPostHandler(final int status, final String msg, final int sleepTimeMs) {
+    return new HttpHandler() {
+      @Override
+      public void handle(HttpExchange httpExchange)
+          throws IOException {
+        if (sleepTimeMs > 0) {
+          try {
+            Thread.sleep(sleepTimeMs);
+          } catch (InterruptedException e) {
+            LOGGER.info("Handler interrupted during sleep");
+          }
+        }
+        if (httpExchange.getRequestMethod().equals("POST")) {
+          httpExchange.sendResponseHeaders(status, msg.length());
+          OutputStream responseBody = httpExchange.getResponseBody();
+          responseBody.write(msg.getBytes());
+          responseBody.close();
+        } else {
+          httpExchange.sendResponseHeaders(ERROR_CODE, ERROR_MSG.length());
+          OutputStream responseBody = httpExchange.getResponseBody();
+          responseBody.write(ERROR_MSG.getBytes());
+          responseBody.close();
+        }
+      }
+    };
+  }
+
   private void startServer(int port, HttpHandler handler)
       throws IOException {
     final HttpServer server = HttpServer.create(new InetSocketAddress(port), 0);
@@ -104,22 +157,58 @@ public class MultiHttpRequestTest {
   }
 
   @Test
-  public void testMultiGet() throws Exception {
-    MultiHttpRequest mget =
-        new MultiHttpRequest(Executors.newCachedThreadPool(), new PoolingHttpClientConnectionManager());
+  public void testMultiGet() {
     List<String> urls = Arrays.asList("http://localhost:" + String.valueOf(_portStart) + URI_PATH,
         "http://localhost:" + String.valueOf(_portStart + 1) + URI_PATH,
         "http://localhost:" + String.valueOf(_portStart + 2) + URI_PATH,
         // 2nd request to the same server
-        "http://localhost:" + String.valueOf(_portStart) + URI_PATH);
+        "http://localhost:" + String.valueOf(_portStart) + URI_PATH,
+        "http://localhost:" + String.valueOf(_portStart + 3) + URI_PATH);
+
+    MultiHttpRequest mget =
+        new MultiHttpRequest(Executors.newCachedThreadPool(), new PoolingHttpClientConnectionManager());
+
     // timeout value needs to be less than 5000ms set above for
     // third server
     final int requestTimeoutMs = 1000;
-    CompletionService<MultiHttpRequestResponse> completionService = mget.execute(urls, null, requestTimeoutMs);
+    CompletionService<MultiHttpRequestResponse> completionService = mget.executeGet(urls, null, requestTimeoutMs);
+
+    TestResult result = collectResult(completionService, urls.size());
+    Assert.assertEquals(result.getSuccess(), 2);
+    Assert.assertEquals(result.getErrors(), 2);
+    Assert.assertEquals(result.getTimeouts(), 1);
+  }
+
+  @Test
+  public void testMultiPost() {
+    List<Pair<String, String>> urlsAndRequestBodies =
+        List.of(Pair.of("http://localhost:" + String.valueOf(_portStart) + URI_PATH, "b0"),
+            Pair.of("http://localhost:" + String.valueOf(_portStart + 1) + URI_PATH, "b1"),
+            Pair.of("http://localhost:" + String.valueOf(_portStart + 2) + URI_PATH, "b2"),
+            // 2nd request to the same server
+            Pair.of("http://localhost:" + String.valueOf(_portStart) + URI_PATH, "b3"),
+            Pair.of("http://localhost:" + String.valueOf(_portStart + 3) + URI_PATH, "b4"));
+
+    MultiHttpRequest mpost =
+        new MultiHttpRequest(Executors.newCachedThreadPool(), new PoolingHttpClientConnectionManager());
+
+    // timeout value needs to be less than 5000ms set above for
+    // third server
+    final int requestTimeoutMs = 1000;
+    CompletionService<MultiHttpRequestResponse> completionService =
+        mpost.executePost(urlsAndRequestBodies, null, requestTimeoutMs);
+
+    TestResult result = collectResult(completionService, urlsAndRequestBodies.size());
+    Assert.assertEquals(result.getSuccess(), 3);
+    Assert.assertEquals(result.getErrors(), 1);
+    Assert.assertEquals(result.getTimeouts(), 1);
+  }
+
+  private TestResult collectResult(CompletionService<MultiHttpRequestResponse> completionService, int size) {
     int success = 0;
     int errors = 0;
     int timeouts = 0;
-    for (int i = 0; i < urls.size(); i++) {
+    for (int i = 0; i < size; i++) {
       try (MultiHttpRequestResponse httpRequestResponse = completionService.take().get()) {
         if (httpRequestResponse.getResponse().getStatusLine().getStatusCode() >= 300) {
           errors++;
@@ -143,9 +232,6 @@ public class MultiHttpRequestTest {
         errors++;
       }
     }
-
-    Assert.assertEquals(success, 2);
-    Assert.assertEquals(errors, 1);
-    Assert.assertEquals(timeouts, 1);
+    return new TestResult(success, errors, timeouts);
   }
 }
