@@ -22,8 +22,6 @@ import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
-import java.util.HashMap;
-import java.util.Map;
 import java.util.stream.Stream;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.spi.config.table.TableConfig;
@@ -38,10 +36,7 @@ import static org.testng.AssertJUnit.assertTrue;
 
 
 public class TableConfigTest {
-
-  private static final String TEST_OFFLINE_TABLE_NAME = "testllc_OFFLINE";
-  private static final String TEST_REALTIME_HLC_TABLE_NAME = "testhlc_REALTIME";
-  private static final String TEST_REALTIME_LLC_TABLE_NAME = "testllc_REALTIME";
+  private static final String RAW_TABLE_NAME = "testTable";
 
   @DataProvider
   public Object[][] configs()
@@ -67,34 +62,19 @@ public class TableConfigTest {
   @Test
   public void testGetReplication() {
     TableConfig offlineTableConfig =
-        new TableConfigBuilder(TableType.OFFLINE).setTableName(TEST_OFFLINE_TABLE_NAME).setNumReplicas(2).build();
-    assertEquals(2, offlineTableConfig.getReplication());
-
+        new TableConfigBuilder(TableType.OFFLINE).setTableName(RAW_TABLE_NAME).setNumReplicas(2).build();
+    assertEquals(offlineTableConfig.getReplication(), 2);
     offlineTableConfig.getValidationConfig().setReplication("4");
-    assertEquals(4, offlineTableConfig.getReplication());
-
+    assertEquals(offlineTableConfig.getReplication(), 4);
     offlineTableConfig.getValidationConfig().setReplicasPerPartition("3");
-    assertEquals(4, offlineTableConfig.getReplication());
+    assertEquals(offlineTableConfig.getReplication(), 4);
 
-    TableConfig realtimeLLCTableConfig =
-        new TableConfigBuilder(TableType.REALTIME).setTableName(TEST_REALTIME_LLC_TABLE_NAME)
-            .setStreamConfigs(getStreamConfigMap("lowlevel")).setLLC(true).setNumReplicas(2).build();
-
-    assertEquals(2, realtimeLLCTableConfig.getReplication());
-
-    realtimeLLCTableConfig.getValidationConfig().setReplication("4");
-    assertEquals(2, realtimeLLCTableConfig.getReplication());
-
-    realtimeLLCTableConfig.getValidationConfig().setReplicasPerPartition("3");
-    assertEquals(3, realtimeLLCTableConfig.getReplication());
-  }
-
-  private Map<String, String> getStreamConfigMap(String consumerType) {
-    Map<String, String> configMap = new HashMap<>();
-    configMap.put("streamType", "kafka");
-    configMap.put("stream.kafka.consumer.type", consumerType);
-    configMap.put("stream.kafka.topic.name", "test");
-    configMap.put("stream.kafka.decoder.class.name", "test");
-    return configMap;
+    TableConfig realtimeTableConfig =
+        new TableConfigBuilder(TableType.REALTIME).setTableName(RAW_TABLE_NAME).setNumReplicas(2).build();
+    assertEquals(realtimeTableConfig.getReplication(), 2);
+    realtimeTableConfig.getValidationConfig().setReplication("4");
+    assertEquals(realtimeTableConfig.getReplication(), 4);
+    realtimeTableConfig.getValidationConfig().setReplicasPerPartition("3");
+    assertEquals(realtimeTableConfig.getReplication(), 3);
   }
 }
