@@ -37,6 +37,7 @@ import javax.ws.rs.core.Context;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.UriInfo;
+import org.apache.pinot.common.utils.DatabaseUtils;
 import org.apache.pinot.core.auth.FineGrainedAuthUtils;
 import org.apache.pinot.core.auth.ManualAuthorization;
 import org.glassfish.grizzly.http.server.Request;
@@ -97,6 +98,10 @@ public class AuthenticationFilter implements ContainerRequestFilter {
     //     - "schemaName"
     // If table name is not available, it means the endpoint is not a table-level endpoint.
     String tableName = extractTableName(uriInfo.getPathParameters(), uriInfo.getQueryParameters());
+    if (tableName != null) {
+      // If table name is present, translate it to the fully qualified name based on database header.
+      tableName = DatabaseUtils.translateTableName(tableName, _httpHeaders);
+    }
     AccessType accessType = extractAccessType(endpointMethod);
     AccessControlUtils.validatePermission(tableName, accessType, _httpHeaders, endpointUrl, accessControl);
 
