@@ -49,15 +49,16 @@ import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.Response;
-import org.apache.calcite.jdbc.CalciteSchemaBuilder;
 import org.apache.calcite.sql.SqlNode;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.helix.model.InstanceConfig;
+import org.apache.pinot.calcite.jdbc.CalciteSchemaBuilder;
 import org.apache.pinot.common.Utils;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.response.ProcessingException;
 import org.apache.pinot.common.response.broker.BrokerResponseNative;
+import org.apache.pinot.common.utils.DatabaseUtils;
 import org.apache.pinot.common.utils.config.TagNameUtils;
 import org.apache.pinot.common.utils.request.RequestUtils;
 import org.apache.pinot.controller.ControllerConf;
@@ -72,6 +73,7 @@ import org.apache.pinot.query.catalog.PinotCatalog;
 import org.apache.pinot.query.type.TypeFactory;
 import org.apache.pinot.query.type.TypeSystem;
 import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.spi.exception.DatabaseConflictException;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Broker.Request.QueryOptionKey;
 import org.apache.pinot.spi.utils.JsonUtils;
@@ -196,7 +198,7 @@ public class PinotQueryResource {
   }
 
   private String getMultiStageQueryResponse(String query, String queryOptions, HttpHeaders httpHeaders,
-      String endpointUrl, String traceEnabled) {
+      String endpointUrl, String traceEnabled) throws ProcessingException {
 
     // Validate data access
     // we don't have a cross table access control rule so only ADMIN can make request to multi-stage engine.
@@ -205,8 +207,14 @@ public class PinotQueryResource {
       throw new WebApplicationException("Permission denied", Response.Status.FORBIDDEN);
     }
 
+    Map<String, String> queryOptionsMap = RequestUtils.parseQuery(query).getOptions();
+    if (queryOptions != null) {
+      queryOptionsMap.putAll(RequestUtils.getOptionsFromString(queryOptions));
+    }
+    String database = DatabaseUtils.extractDatabaseFromQueryRequest(queryOptionsMap, httpHeaders);
     QueryEnvironment queryEnvironment = new QueryEnvironment(new TypeFactory(new TypeSystem()),
-        CalciteSchemaBuilder.asRootSchema(new PinotCatalog(_pinotHelixResourceManager.getTableCache())), null, null);
+        CalciteSchemaBuilder.asRootSchema(new PinotCatalog(database, _pinotHelixResourceManager.getTableCache()),
+            database), null, null);
     List<String> tableNames;
     try {
       tableNames = queryEnvironment.getTableNamesForQuery(query);
@@ -229,6 +237,14 @@ public class PinotQueryResource {
             String.format("Unable to dispatch multistage query for tables: [%s]", tableNames))).toString();
       }
       instanceIds = findCommonBrokerInstances(brokerTenantsUnion);
+      if (instanceIds.isEmpty()) {
+        // No common broker found for table tenants
+        LOGGER.error("Unable to find a common broker instance for table tenants. Tables: {}, Tenants: {}",
+            tableNames, brokerTenantsUnion);
+        throw QueryException.getException(QueryException.BROKER_RESOURCE_MISSING_ERROR,
+            new Exception("Unable to find a common broker instance for table tenants. Tables: "
+                + tableNames + ", Tenants: " + brokerTenantsUnion));
+      }
     } else {
       // TODO fail these queries going forward. Added this logic to take care of tautologies like BETWEEN 0 and -1.
       instanceIds = _pinotHelixResourceManager.getAllBrokerInstances();
@@ -239,22 +255,32 @@ public class PinotQueryResource {
   }
 
   private String getQueryResponse(String query, @Nullable SqlNode sqlNode, String traceEnabled, String queryOptions,
-      HttpHeaders httpHeaders) {
+      HttpHeaders httpHeaders) throws ProcessingException {
     // Get resource table name.
     String tableName;
+    Map<String, String> queryOptionsMap = RequestUtils.parseQuery(query).getOptions();
+    if (queryOptions != null) {
+      queryOptionsMap.putAll(RequestUtils.getOptionsFromString(queryOptions));
+    }
+    String database;
+    try {
+      database = DatabaseUtils.extractDatabaseFromQueryRequest(queryOptionsMap, httpHeaders);
+    } catch (DatabaseConflictException e) {
+      return QueryException.getException(QueryException.QUERY_VALIDATION_ERROR, e).toString();
+    }
     try {
       String inputTableName =
           sqlNode != null ? RequestUtils.getTableNames(CalciteSqlParser.compileSqlNodeToPinotQuery(sqlNode)).iterator()
               .next() : CalciteSqlCompiler.compileToBrokerRequest(query).getQuerySource().getTableName();
-      tableName = _pinotHelixResourceManager.getActualTableName(inputTableName);
+      tableName = _pinotHelixResourceManager.getActualTableName(inputTableName, database);
     } catch (Exception e) {
       LOGGER.error("Caught exception while compiling query: {}", query, e);
       try {
         // try to compile the query using multi-stage engine and suggest using it if it succeeds.
         LOGGER.info("Trying to compile query {} using multi-stage engine", query);
         QueryEnvironment queryEnvironment = new QueryEnvironment(new TypeFactory(new TypeSystem()),
-            CalciteSchemaBuilder.asRootSchema(new PinotCatalog(_pinotHelixResourceManager.getTableCache())), null,
-            null);
+            CalciteSchemaBuilder.asRootSchema(new PinotCatalog(database, _pinotHelixResourceManager.getTableCache()),
+                database), null, null);
         queryEnvironment.getTableNamesForQuery(query);
         LOGGER.info("Successfully compiled query using multi-stage engine: {}", query);
         return QueryException.getException(QueryException.SQL_PARSING_ERROR, new Exception(
@@ -299,14 +325,15 @@ public class PinotQueryResource {
     return allTableConfigList;
   }
 
-  private String selectRandomInstanceId(List<String> instanceIds) {
+  private String selectRandomInstanceId(List<String> instanceIds) throws ProcessingException {
     if (instanceIds.isEmpty()) {
-      return QueryException.BROKER_RESOURCE_MISSING_ERROR.toString();
+      throw QueryException.getException(QueryException.BROKER_RESOURCE_MISSING_ERROR, "No broker found for query");
     }
 
     instanceIds.retainAll(_pinotHelixResourceManager.getOnlineInstanceList());
     if (instanceIds.isEmpty()) {
-      return QueryException.BROKER_INSTANCE_MISSING_ERROR.toString();
+      throw QueryException.getException(QueryException.BROKER_INSTANCE_MISSING_ERROR,
+          "No online broker found for query");
     }
 
     // Send query to a random broker.
@@ -387,7 +414,7 @@ public class PinotQueryResource {
         LOGGER.debug("The request is - " + requestStr);
       }*/
 
-      LOGGER.info("url string passed is : " + urlStr);
+      LOGGER.info("url string passed is : {}", urlStr);
       final URL url = new URL(urlStr);
       conn = (HttpURLConnection) url.openConnection();
       conn.setDoOutput(true);
@@ -466,7 +493,7 @@ public class PinotQueryResource {
       final String pinotResultString = sendPostRaw(url, requestJson.toString(), headers);
 
       final long queryTime = System.currentTimeMillis() - startTime;
-      LOGGER.info("Query: " + query + " Time: " + queryTime);
+      LOGGER.info("Query: {} Time: {}", query, queryTime);
 
       return pinotResultString;
     } catch (final Exception ex) {
