@@ -70,6 +70,7 @@ import org.apache.http.conn.HttpClientConnectionManager;
 import org.apache.pinot.common.exception.TableNotFoundException;
 import org.apache.pinot.common.minion.BaseTaskGeneratorInfo;
 import org.apache.pinot.common.minion.TaskManagerStatusCache;
+import org.apache.pinot.common.utils.DatabaseUtils;
 import org.apache.pinot.controller.ControllerConf;
 import org.apache.pinot.controller.api.access.AccessType;
 import org.apache.pinot.controller.api.access.Authenticate;
@@ -102,6 +103,8 @@ import org.quartz.impl.matchers.GroupMatcher;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.pinot.spi.utils.CommonConstants.DATABASE;
+import static org.apache.pinot.spi.utils.CommonConstants.DEFAULT_DATABASE;
 import static org.apache.pinot.spi.utils.CommonConstants.SWAGGER_AUTHORIZATION_KEY;
 
 
@@ -124,9 +127,14 @@ import static org.apache.pinot.spi.utils.CommonConstants.SWAGGER_AUTHORIZATION_K
  *   <li>DELETE '/tasks/{taskType}': Delete all tasks (as well as the task queue) for the given task type</li>
  * </ul>
  */
-@Api(tags = Constants.TASK_TAG, authorizations = {@Authorization(value = SWAGGER_AUTHORIZATION_KEY)})
-@SwaggerDefinition(securityDefinition = @SecurityDefinition(apiKeyAuthDefinitions = @ApiKeyAuthDefinition(name =
-    HttpHeaders.AUTHORIZATION, in = ApiKeyAuthDefinition.ApiKeyLocation.HEADER, key = SWAGGER_AUTHORIZATION_KEY)))
+@Api(tags = Constants.TASK_TAG, authorizations = {@Authorization(value = SWAGGER_AUTHORIZATION_KEY),
+    @Authorization(value = DATABASE)})
+@SwaggerDefinition(securityDefinition = @SecurityDefinition(apiKeyAuthDefinitions = {
+    @ApiKeyAuthDefinition(name = HttpHeaders.AUTHORIZATION, in = ApiKeyAuthDefinition.ApiKeyLocation.HEADER,
+        key = SWAGGER_AUTHORIZATION_KEY),
+    @ApiKeyAuthDefinition(name = DATABASE, in = ApiKeyAuthDefinition.ApiKeyLocation.HEADER, key = DATABASE,
+        description = "Database context passed through http header. If no context is provided 'default' database "
+            + "context will be considered.")}))
 @Path("/")
 public class PinotTaskRestletResource {
   public static final Logger LOGGER = LoggerFactory.getLogger(PinotTaskRestletResource.class);
@@ -167,16 +175,6 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskTypes();
   }
 
-  @Deprecated
-  @GET
-  @Path("/tasks/taskqueues")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("List all task queues (deprecated)")
-  public Set<String> getTaskQueues() {
-    return _pinotHelixTaskResourceManager.getTaskQueues();
-  }
-
   @GET
   @Path("/tasks/{taskType}/state")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
@@ -187,17 +185,6 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskQueueState(taskType);
   }
 
-  @Deprecated
-  @GET
-  @Path("/tasks/taskqueuestate/{taskType}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("Get the state (task queue state) for the given task type (deprecated)")
-  public StringResultResponse getTaskQueueStateDeprecated(
-      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType) {
-    return new StringResultResponse(_pinotHelixTaskResourceManager.getTaskQueueState(taskType).toString());
-  }
-
   @GET
   @Path("/tasks/{taskType}/tasks")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
@@ -215,7 +202,8 @@ public class PinotTaskRestletResource {
   public Map<String, TaskState> getTaskStatesByTable(
       @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
       @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
-          String tableNameWithType) {
+      String tableNameWithType, @Context HttpHeaders headers) {
+    tableNameWithType = DatabaseUtils.translateTableName(tableNameWithType, headers);
     return _pinotHelixTaskResourceManager.getTaskStatesByTable(taskType, tableNameWithType);
   }
 
@@ -227,8 +215,9 @@ public class PinotTaskRestletResource {
   public String getTaskMetadataByTable(
       @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
       @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
-          String tableNameWithType) {
+      String tableNameWithType, @Context HttpHeaders headers) {
     try {
+      tableNameWithType = DatabaseUtils.translateTableName(tableNameWithType, headers);
       return _pinotHelixTaskResourceManager.getTaskMetadataByTable(taskType, tableNameWithType);
     } catch (JsonProcessingException e) {
       throw new ControllerApplicationException(LOGGER, String
@@ -245,7 +234,8 @@ public class PinotTaskRestletResource {
   public SuccessResponse deleteTaskMetadataByTable(
       @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
       @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
-          String tableNameWithType) {
+      String tableNameWithType, @Context HttpHeaders headers) {
+    tableNameWithType = DatabaseUtils.translateTableName(tableNameWithType, headers);
     _pinotHelixTaskResourceManager.deleteTaskMetadataByTable(taskType, tableNameWithType);
     return new SuccessResponse(
         String.format("Successfully deleted metadata for task type: %s from table: %s", taskType, tableNameWithType));
@@ -287,7 +277,8 @@ public class PinotTaskRestletResource {
       @ApiParam(value = "verbosity (Prints information for all the tasks for the given task type and table."
           + "By default, only prints subtask details for running and error tasks. "
           + "Value of > 0 prints subtask details for all tasks)")
-      @DefaultValue("0") @QueryParam("verbosity") int verbosity) {
+      @DefaultValue("0") @QueryParam("verbosity") int verbosity, @Context HttpHeaders headers) {
+    tableNameWithType = DatabaseUtils.translateTableName(tableNameWithType, headers);
     return _pinotHelixTaskResourceManager.getTasksDebugInfoByTable(taskType, tableNameWithType, verbosity);
   }
 
@@ -297,13 +288,13 @@ public class PinotTaskRestletResource {
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
   @ApiOperation("Fetch task generation information for the recent runs of the given task for the given table")
   public String getTaskGenerationDebugInto(
-      @Context HttpHeaders httpHeaders,
       @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
       @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
           String tableNameWithType,
       @ApiParam(value = "Whether to only lookup local cache for logs", defaultValue = "false") @QueryParam("localOnly")
-          boolean localOnly)
+          boolean localOnly, @Context HttpHeaders httpHeaders)
       throws JsonProcessingException {
+    tableNameWithType = DatabaseUtils.translateTableName(tableNameWithType, httpHeaders);
     if (localOnly) {
       BaseTaskGeneratorInfo taskGeneratorMostRecentRunInfo =
           _taskManagerStatusCache.fetchTaskGeneratorInfo(tableNameWithType, taskType);
@@ -320,9 +311,10 @@ public class PinotTaskRestletResource {
     // Relying on original schema that was used to query the controller
     URI uri = _uriInfo.getRequestUri();
     String scheme = uri.getScheme();
+    String finalTableNameWithType = tableNameWithType;
     List<String> controllerUrls = controllers.stream().map(controller -> String
         .format("%s://%s:%d/tasks/generator/%s/%s/debug?localOnly=true", scheme, controller.getHostName(),
-            Integer.parseInt(controller.getPort()), tableNameWithType, taskType)).collect(Collectors.toList());
+            Integer.parseInt(controller.getPort()), finalTableNameWithType, taskType)).collect(Collectors.toList());
 
     CompletionServiceHelper completionServiceHelper =
         new CompletionServiceHelper(_executor, _connectionManager, HashBiMap.create(0));
@@ -360,17 +352,6 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskDebugInfo(taskName, verbosity);
   }
 
-  @Deprecated
-  @GET
-  @Path("/tasks/tasks/{taskType}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("List all tasks for the given task type (deprecated)")
-  public Set<String> getTasksDeprecated(
-      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType) {
-    return _pinotHelixTaskResourceManager.getTasks(taskType);
-  }
-
   @GET
   @Path("/tasks/{taskType}/taskstates")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
@@ -381,17 +362,6 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskStates(taskType);
   }
 
-  @Deprecated
-  @GET
-  @Path("/tasks/taskstates/{taskType}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("Get a map from task to task state for the given task type (deprecated)")
-  public Map<String, TaskState> getTaskStatesDeprecated(
-      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType) {
-    return _pinotHelixTaskResourceManager.getTaskStates(taskType);
-  }
-
   @GET
   @Path("/tasks/task/{taskName}/state")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
@@ -402,17 +372,6 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskState(taskName);
   }
 
-  @Deprecated
-  @GET
-  @Path("/tasks/taskstate/{taskName}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("Get the task state for the given task (deprecated)")
-  public StringResultResponse getTaskStateDeprecated(
-      @ApiParam(value = "Task name", required = true) @PathParam("taskName") String taskName) {
-    return new StringResultResponse(String.valueOf(_pinotHelixTaskResourceManager.getTaskState(taskName)));
-  }
-
   @GET
   @Path("/tasks/subtask/{taskName}/state")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
@@ -443,17 +402,6 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskRuntimeConfig(taskName);
   }
 
-  @Deprecated
-  @GET
-  @Path("/tasks/taskconfig/{taskName}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("Get the task config (a list of child task configs) for the given task (deprecated)")
-  public List<PinotTaskConfig> getTaskConfigsDeprecated(
-      @ApiParam(value = "Task name", required = true) @PathParam("taskName") String taskName) {
-    return _pinotHelixTaskResourceManager.getSubtaskConfigs(taskName);
-  }
-
   @GET
   @Path("/tasks/subtask/{taskName}/config")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_TASK)
@@ -605,17 +553,18 @@ public class PinotTaskRestletResource {
 
   @GET
   @Path("/tasks/scheduler/jobDetails")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_SCHEDULER_INFO)
+  @Authorize(targetType = TargetType.TABLE, paramName = "tableName", action = Actions.Table.GET_SCHEDULER_JOB_DETAILS)
   @Produces(MediaType.APPLICATION_JSON)
-  @ApiOperation("Fetch cron scheduler job keys")
+  @ApiOperation("Fetch job details for table tasks")
   public Map<String, Object> getCronSchedulerJobDetails(
-      @ApiParam(value = "Table name (with type suffix)") @QueryParam("tableName") String tableName,
-      @ApiParam(value = "Task type") @QueryParam("taskType") String taskType)
+      @ApiParam(value = "Table name (with type suffix)", required = true) @QueryParam("tableName") String tableName,
+      @ApiParam(value = "Task type") @QueryParam("taskType") String taskType, @Context HttpHeaders headers)
       throws SchedulerException {
     Scheduler scheduler = _pinotTaskManager.getScheduler();
     if (scheduler == null) {
       throw new NotFoundException("Task scheduler is disabled");
     }
+    tableName = DatabaseUtils.translateTableName(tableName, headers);
     JobKey jobKey = JobKey.jobKey(tableName, taskType);
     if (!scheduler.checkExists(jobKey)) {
       throw new NotFoundException(
@@ -668,15 +617,23 @@ public class PinotTaskRestletResource {
   @Authenticate(AccessType.UPDATE)
   @ApiOperation("Schedule tasks and return a map from task type to task name scheduled")
   public Map<String, String> scheduleTasks(@ApiParam(value = "Task type") @QueryParam("taskType") String taskType,
-      @ApiParam(value = "Table name (with type suffix)") @QueryParam("tableName") String tableName) {
+      @ApiParam(value = "Table name (with type suffix)") @QueryParam("tableName") String tableName,
+      @ApiParam(value = "Minion Instance tag to schedule the task explicitly on") @QueryParam("minionInstanceTag")
+      @Nullable String minionInstanceTag, @Context HttpHeaders headers) {
+    String database = headers != null ? headers.getHeaderString(DATABASE) : DEFAULT_DATABASE;
     if (taskType != null) {
       // Schedule task for the given task type
-      String taskName = tableName != null ? _pinotTaskManager.scheduleTask(taskType, tableName)
-          : _pinotTaskManager.scheduleTask(taskType);
-      return Collections.singletonMap(taskType, taskName);
+      List<String> taskNames = tableName != null ? _pinotTaskManager.scheduleTaskForTable(taskType,
+          DatabaseUtils.translateTableName(tableName, headers), minionInstanceTag)
+          : _pinotTaskManager.scheduleTaskForDatabase(taskType, database, minionInstanceTag);
+      return Collections.singletonMap(taskType, taskNames == null ? null : StringUtils.join(taskNames, ','));
     } else {
       // Schedule tasks for all task types
-      return tableName != null ? _pinotTaskManager.scheduleTasks(tableName) : _pinotTaskManager.scheduleTasks();
+      Map<String, List<String>> allTaskNames = tableName != null ? _pinotTaskManager.scheduleAllTasksForTable(
+          DatabaseUtils.translateTableName(tableName, headers), minionInstanceTag)
+          : _pinotTaskManager.scheduleAllTasksForDatabase(database, minionInstanceTag);
+      return allTaskNames.entrySet().stream()
+          .collect(Collectors.toMap(Map.Entry::getKey, entry -> String.join(",", entry.getValue())));
     }
   }
 
@@ -704,24 +661,13 @@ public class PinotTaskRestletResource {
     } catch (NoTaskScheduledException e) {
       throw new ControllerApplicationException(LOGGER,
           "No task is generated for table: " + adhocTaskConfig.getTableName() + ", with task type: "
-              + adhocTaskConfig.getTaskType(), Response.Status.BAD_REQUEST);
+              + adhocTaskConfig.getTaskType(), Response.Status.BAD_REQUEST, e);
     } catch (Exception e) {
       throw new ControllerApplicationException(LOGGER,
           "Failed to create adhoc task: " + ExceptionUtils.getStackTrace(e), Response.Status.INTERNAL_SERVER_ERROR, e);
     }
   }
 
-  @Deprecated
-  @PUT
-  @Path("/tasks/scheduletasks")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.CREATE_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @Authenticate(AccessType.UPDATE)
-  @ApiOperation("Schedule tasks (deprecated)")
-  public Map<String, String> scheduleTasksDeprecated() {
-    return _pinotTaskManager.scheduleTasks();
-  }
-
   @PUT
   @Path("/tasks/{taskType}/cleanup")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.CLEANUP_TASK)
@@ -734,19 +680,6 @@ public class PinotTaskRestletResource {
     return new SuccessResponse("Successfully cleaned up tasks for task type: " + taskType);
   }
 
-  @Deprecated
-  @PUT
-  @Path("/tasks/cleanuptasks/{taskType}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.CLEANUP_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @Authenticate(AccessType.UPDATE)
-  @ApiOperation("Clean up finished tasks (COMPLETED, FAILED) for the given task type (deprecated)")
-  public SuccessResponse cleanUpTasksDeprecated(
-      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType) {
-    _pinotHelixTaskResourceManager.cleanUpTaskQueue(taskType);
-    return new SuccessResponse("Successfully cleaned up tasks for task type: " + taskType);
-  }
-
   @PUT
   @Path("/tasks/{taskType}/stop")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.STOP_TASK)
@@ -771,28 +704,6 @@ public class PinotTaskRestletResource {
     return new SuccessResponse("Successfully resumed tasks for task type: " + taskType);
   }
 
-  @Deprecated
-  @PUT
-  @Path("/tasks/taskqueue/{taskType}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.UPDATE_TASK_QUEUE)
-  @Produces(MediaType.APPLICATION_JSON)
-  @Authenticate(AccessType.UPDATE)
-  @ApiOperation("Stop/resume a task queue (deprecated)")
-  public SuccessResponse toggleTaskQueueState(
-      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
-      @ApiParam(value = "state", required = true) @QueryParam("state") String state) {
-    switch (state.toUpperCase()) {
-      case TASK_QUEUE_STATE_STOP:
-        _pinotHelixTaskResourceManager.stopTaskQueue(taskType);
-        return new SuccessResponse("Successfully stopped task queue for task type: " + taskType);
-      case TASK_QUEUE_STATE_RESUME:
-        _pinotHelixTaskResourceManager.resumeTaskQueue(taskType);
-        return new SuccessResponse("Successfully resumed task queue for task type: " + taskType);
-      default:
-        throw new IllegalArgumentException("Unsupported state: " + state);
-    }
-  }
-
   @DELETE
   @Path("/tasks/{taskType}")
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.DELETE_TASK)
@@ -820,19 +731,4 @@ public class PinotTaskRestletResource {
     _pinotHelixTaskResourceManager.deleteTask(taskName, forceDelete);
     return new SuccessResponse("Successfully deleted task: " + taskName);
   }
-
-  @Deprecated
-  @DELETE
-  @Path("/tasks/taskqueue/{taskType}")
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.DELETE_TASK)
-  @Produces(MediaType.APPLICATION_JSON)
-  @Authenticate(AccessType.DELETE)
-  @ApiOperation("Delete a task queue (deprecated)")
-  public SuccessResponse deleteTaskQueue(
-      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
-      @ApiParam(value = "Whether to force delete the task queue (expert only option, enable with cautious")
-      @DefaultValue("false") @QueryParam("forceDelete") boolean forceDelete) {
-    _pinotHelixTaskResourceManager.deleteTaskQueue(taskType, forceDelete);
-    return new SuccessResponse("Successfully deleted task queue for task type: " + taskType);
-  }
 }
