@@ -23,6 +23,7 @@ import javax.ws.rs.core.Response;
 import org.apache.http.conn.HttpClientConnectionManager;
 import org.apache.pinot.common.metrics.ControllerMetrics;
 import org.apache.pinot.controller.ControllerConf;
+import org.apache.pinot.controller.LeadControllerManager;
 import org.apache.pinot.controller.api.exception.ControllerApplicationException;
 import org.apache.pinot.controller.helix.core.PinotHelixResourceManager;
 import org.apache.pinot.controller.util.TableSizeReader;
@@ -65,12 +66,13 @@ public class SegmentValidationUtils {
 
   public static void checkStorageQuota(String segmentName, long segmentSizeInBytes, TableConfig tableConfig,
       PinotHelixResourceManager resourceManager, ControllerConf controllerConf, ControllerMetrics controllerMetrics,
-      HttpClientConnectionManager connectionManager, Executor executor, boolean isLeaderForTable) {
+      HttpClientConnectionManager connectionManager, Executor executor, LeadControllerManager leadControllerManager) {
     if (!controllerConf.getEnableStorageQuotaCheck()) {
       return;
     }
+    boolean isLeaderForTable = leadControllerManager.isLeaderForTable(tableConfig.getTableName());
     TableSizeReader tableSizeReader =
-        new TableSizeReader(executor, connectionManager, controllerMetrics, resourceManager);
+        new TableSizeReader(executor, connectionManager, controllerMetrics, resourceManager, leadControllerManager);
     StorageQuotaChecker quotaChecker =
         new StorageQuotaChecker(tableConfig, tableSizeReader, controllerMetrics, isLeaderForTable, resourceManager);
     StorageQuotaChecker.QuotaCheckerResponse response;
