@@ -40,7 +40,7 @@ import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.stream.Collectors;
 import javax.annotation.Nullable;
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.helix.AccessOption;
 import org.apache.helix.Criteria;
 import org.apache.helix.HelixAdmin;
@@ -1483,7 +1483,8 @@ public class PinotLLCRealtimeSegmentManager {
           LOGGER.info("Fixing LLC segment {} whose deep store copy is unavailable", segmentName);
           // Find servers which have online replica
           List<URI> peerSegmentURIs =
-              PeerServerSegmentFinder.getPeerServerURIs(segmentName, CommonConstants.HTTP_PROTOCOL, _helixManager);
+              PeerServerSegmentFinder.getPeerServerURIs(_helixManager, realtimeTableName, segmentName,
+                  CommonConstants.HTTP_PROTOCOL);
           if (peerSegmentURIs.isEmpty()) {
             throw new IllegalStateException(
                 String.format("Failed to upload segment %s to deep store because no online replica is found",
