@@ -24,10 +24,9 @@ import java.util.List;
 import java.util.Properties;
 import java.util.concurrent.TimeUnit;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
+import org.apache.pinot.common.metrics.ControllerMeter;
 import org.apache.pinot.common.metrics.ControllerMetrics;
 import org.apache.pinot.common.metrics.ValidationMetrics;
-import org.apache.pinot.common.utils.HLCSegmentName;
-import org.apache.pinot.common.utils.SegmentName;
 import org.apache.pinot.controller.ControllerConf;
 import org.apache.pinot.controller.LeadControllerManager;
 import org.apache.pinot.controller.helix.core.PinotHelixResourceManager;
@@ -35,7 +34,7 @@ import org.apache.pinot.controller.helix.core.periodictask.ControllerPeriodicTas
 import org.apache.pinot.controller.helix.core.realtime.PinotLLCRealtimeSegmentManager;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.stream.OffsetCriteria;
-import org.apache.pinot.spi.stream.PartitionLevelStreamConfig;
+import org.apache.pinot.spi.stream.StreamConfig;
 import org.apache.pinot.spi.utils.IngestionConfigUtils;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.slf4j.Logger;
@@ -51,6 +50,7 @@ public class RealtimeSegmentValidationManager extends ControllerPeriodicTask<Rea
 
   private final PinotLLCRealtimeSegmentManager _llcRealtimeSegmentManager;
   private final ValidationMetrics _validationMetrics;
+  private final ControllerMetrics _controllerMetrics;
 
   private final int _segmentLevelValidationIntervalInSeconds;
   private long _lastSegmentLevelValidationRunTimeMs = 0L;
@@ -66,6 +66,7 @@ public class RealtimeSegmentValidationManager extends ControllerPeriodicTask<Rea
         leadControllerManager, controllerMetrics);
     _llcRealtimeSegmentManager = llcRealtimeSegmentManager;
     _validationMetrics = validationMetrics;
+    _controllerMetrics = controllerMetrics;
 
     _segmentLevelValidationIntervalInSeconds = config.getSegmentLevelValidationIntervalInSeconds();
     Preconditions.checkState(_segmentLevelValidationIntervalInSeconds > 0);
@@ -102,32 +103,35 @@ public class RealtimeSegmentValidationManager extends ControllerPeriodicTask<Rea
       LOGGER.warn("Failed to find table config for table: {}, skipping validation", tableNameWithType);
       return;
     }
-    PartitionLevelStreamConfig streamConfig = new PartitionLevelStreamConfig(tableConfig.getTableName(),
-        IngestionConfigUtils.getStreamConfigMap(tableConfig));
-
+    StreamConfig streamConfig =
+        new StreamConfig(tableConfig.getTableName(), IngestionConfigUtils.getStreamConfigMap(tableConfig));
     if (context._runSegmentLevelValidation) {
       runSegmentLevelValidation(tableConfig, streamConfig);
     }
-
-    if (streamConfig.hasLowLevelConsumerType()) {
-      _llcRealtimeSegmentManager.ensureAllPartitionsConsuming(tableConfig, streamConfig,
-          context._recreateDeletedConsumingSegment, context._offsetCriteria);
-    }
+    _llcRealtimeSegmentManager.ensureAllPartitionsConsuming(tableConfig, streamConfig,
+        context._recreateDeletedConsumingSegment, context._offsetCriteria);
   }
 
-  private void runSegmentLevelValidation(TableConfig tableConfig, PartitionLevelStreamConfig streamConfig) {
+  private void runSegmentLevelValidation(TableConfig tableConfig, StreamConfig streamConfig) {
     String realtimeTableName = tableConfig.getTableName();
+
     List<SegmentZKMetadata> segmentsZKMetadata = _pinotHelixResourceManager.getSegmentsZKMetadata(realtimeTableName);
 
+    // Delete tmp segments
+    try {
+      long numDeleteTmpSegments = _llcRealtimeSegmentManager.deleteTmpSegments(realtimeTableName, segmentsZKMetadata);
+      LOGGER.info("Deleted {} tmp segments for table: {}", numDeleteTmpSegments, realtimeTableName);
+      _controllerMetrics.addMeteredTableValue(realtimeTableName, ControllerMeter.DELETED_TMP_SEGMENT_COUNT,
+          numDeleteTmpSegments);
+    } catch (Exception e) {
+      LOGGER.error("Failed to delete tmp segments for table: {}", realtimeTableName, e);
+    }
+
     // Update the total document count gauge
-    // Count HLC segments if high level consumer is configured
-    boolean countHLCSegments = streamConfig.hasHighLevelConsumerType();
-    _validationMetrics.updateTotalDocumentCountGauge(realtimeTableName,
-        computeTotalDocumentCount(segmentsZKMetadata, countHLCSegments));
+    _validationMetrics.updateTotalDocumentCountGauge(realtimeTableName, computeTotalDocumentCount(segmentsZKMetadata));
 
     // Check missing segments and upload them to the deep store
-    if (streamConfig.hasLowLevelConsumerType()
-        && _llcRealtimeSegmentManager.isDeepStoreLLCSegmentUploadRetryEnabled()) {
+    if (_llcRealtimeSegmentManager.isDeepStoreLLCSegmentUploadRetryEnabled()) {
       _llcRealtimeSegmentManager.uploadToDeepStoreIfMissing(tableConfig, segmentsZKMetadata);
     }
   }
@@ -137,39 +141,16 @@ public class RealtimeSegmentValidationManager extends ControllerPeriodicTask<Rea
     for (String tableNameWithType : tableNamesWithType) {
       if (TableNameBuilder.isRealtimeTableResource(tableNameWithType)) {
         _validationMetrics.cleanupTotalDocumentCountGauge(tableNameWithType);
+        _controllerMetrics.removeTableMeter(tableNameWithType, ControllerMeter.DELETED_TMP_SEGMENT_COUNT);
       }
     }
   }
 
   @VisibleForTesting
-  static long computeTotalDocumentCount(List<SegmentZKMetadata> segmentsZKMetadata, boolean countHLCSegments) {
+  static long computeTotalDocumentCount(List<SegmentZKMetadata> segmentsZKMetadata) {
     long numTotalDocs = 0;
-    if (countHLCSegments) {
-      String groupId = null;
-      for (SegmentZKMetadata segmentZKMetadata : segmentsZKMetadata) {
-        String segmentName = segmentZKMetadata.getSegmentName();
-        if (SegmentName.isHighLevelConsumerSegmentName(segmentName)) {
-          HLCSegmentName hlcSegmentName = new HLCSegmentName(segmentName);
-          String segmentGroupId = hlcSegmentName.getGroupId();
-          if (groupId == null) {
-            groupId = segmentGroupId;
-            numTotalDocs = segmentZKMetadata.getTotalDocs();
-          } else {
-            // Discard all segments with different group id as they are replicas
-            if (groupId.equals(segmentGroupId)) {
-              numTotalDocs += segmentZKMetadata.getTotalDocs();
-            }
-          }
-        }
-      }
-    } else {
-      for (SegmentZKMetadata segmentZKMetadata : segmentsZKMetadata) {
-        String segmentName = segmentZKMetadata.getSegmentName();
-        if (!SegmentName.isHighLevelConsumerSegmentName(segmentName)) {
-          // LLC segments or uploaded segments
-          numTotalDocs += segmentZKMetadata.getTotalDocs();
-        }
-      }
+    for (SegmentZKMetadata segmentZKMetadata : segmentsZKMetadata) {
+      numTotalDocs += segmentZKMetadata.getTotalDocs();
     }
     return numTotalDocs;
   }
