@@ -18,7 +18,6 @@
  */
 package org.apache.pinot.controller.helix;
 
-import com.google.common.base.Preconditions;
 import java.io.File;
 import java.io.IOException;
 import java.net.URISyntaxException;
@@ -31,11 +30,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import javax.annotation.Nullable;
-import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.collections4.CollectionUtils;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.IOUtils;
-import org.apache.commons.lang.StringUtils;
-import org.apache.commons.lang.math.RandomUtils;
 import org.apache.helix.ConfigAccessor;
 import org.apache.helix.HelixAdmin;
 import org.apache.helix.HelixDataAccessor;
@@ -75,8 +72,8 @@ import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.MetricFieldSpec;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.env.PinotConfiguration;
+import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Helix;
-import org.apache.pinot.spi.utils.CommonConstants.Server;
 import org.apache.pinot.spi.utils.NetUtils;
 import org.apache.pinot.spi.utils.builder.ControllerRequestURLBuilder;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
@@ -86,16 +83,11 @@ import org.slf4j.LoggerFactory;
 
 import static org.apache.pinot.spi.utils.CommonConstants.Helix.UNTAGGED_BROKER_INSTANCE;
 import static org.apache.pinot.spi.utils.CommonConstants.Helix.UNTAGGED_SERVER_INSTANCE;
-import static org.apache.pinot.spi.utils.CommonConstants.Server.DEFAULT_ADMIN_API_PORT;
-import static org.testng.Assert.assertEquals;
-import static org.testng.Assert.assertNotNull;
-import static org.testng.Assert.assertTrue;
-import static org.testng.Assert.fail;
+import static org.testng.Assert.*;
 
 
 public class ControllerTest {
   public static final String LOCAL_HOST = "localhost";
-  public static final int DEFAULT_CONTROLLER_PORT = 18998;
   public static final String DEFAULT_DATA_DIR = new File(FileUtils.getTempDirectoryPath(),
       "test-controller-data-dir" + System.currentTimeMillis()).getAbsolutePath();
   public static final String DEFAULT_LOCAL_TEMP_DIR = new File(FileUtils.getTempDirectoryPath(),
@@ -117,29 +109,34 @@ public class ControllerTest {
    */
   public static final ControllerTest DEFAULT_INSTANCE = new ControllerTest();
 
+  protected static HttpClient _httpClient;
+
   protected final String _clusterName = getClass().getSimpleName();
-
-  protected static HttpClient _httpClient = null;
-
-  private int _controllerPort;
-  private String _controllerBaseApiUrl;
-  protected ControllerConf _controllerConfig;
-  protected ControllerRequestURLBuilder _controllerRequestURLBuilder;
-
-  protected ControllerRequestClient _controllerRequestClient = null;
-
   protected final List<HelixManager> _fakeInstanceHelixManagers = new ArrayList<>();
-  protected String _controllerDataDir;
 
-  protected BaseControllerStarter _controllerStarter;
-  protected PinotHelixResourceManager _helixResourceManager;
-  protected HelixManager _helixManager;
-  protected HelixAdmin _helixAdmin;
-  protected HelixDataAccessor _helixDataAccessor;
-  protected ZkHelixPropertyStore<ZNRecord> _propertyStore;
+  protected int _nextControllerPort = 20000;
+  protected int _nextBrokerPort = _nextControllerPort + 1000;
+  protected int _nextServerPort = _nextBrokerPort + 1000;
+  protected int _nextMinionPort = _nextServerPort + 1000;
 
   private ZkStarter.ZookeeperInstance _zookeeperInstance;
 
+  // The following fields need to be reset when stopping the controller.
+  protected BaseControllerStarter _controllerStarter;
+  protected int _controllerPort;
+  protected ControllerRequestClient _controllerRequestClient;
+
+  // The following fields are always set when controller is started. No need to reset them when stopping the controller.
+  protected ControllerConf _controllerConfig;
+  protected String _controllerBaseApiUrl;
+  protected ControllerRequestURLBuilder _controllerRequestURLBuilder;
+  protected String _controllerDataDir;
+  protected PinotHelixResourceManager _helixResourceManager;
+  protected HelixManager _helixManager;
+  protected HelixDataAccessor _helixDataAccessor;
+  protected HelixAdmin _helixAdmin;
+  protected ZkHelixPropertyStore<ZNRecord> _propertyStore;
+
   /**
    * Acquire the {@link ControllerTest} default instance that can be shared across different test cases.
    *
@@ -210,22 +207,35 @@ public class ControllerTest {
 
   public Map<String, Object> getDefaultControllerConfiguration() {
     Map<String, Object> properties = new HashMap<>();
-
-    properties.put(ControllerConf.CONTROLLER_HOST, LOCAL_HOST);
-    properties.put(ControllerConf.CONTROLLER_PORT,
-        NetUtils.findOpenPort(DEFAULT_CONTROLLER_PORT + RandomUtils.nextInt(10000)));
-    properties.put(ControllerConf.DATA_DIR, DEFAULT_DATA_DIR);
-    properties.put(ControllerConf.LOCAL_TEMP_DIR, DEFAULT_LOCAL_TEMP_DIR);
     properties.put(ControllerConf.ZK_STR, getZkUrl());
     properties.put(ControllerConf.HELIX_CLUSTER_NAME, getHelixClusterName());
+    properties.put(ControllerConf.CONTROLLER_HOST, LOCAL_HOST);
+    int controllerPort = NetUtils.findOpenPort(_nextControllerPort);
+    properties.put(ControllerConf.CONTROLLER_PORT, controllerPort);
+    if (_controllerPort == 0) {
+      _controllerPort = controllerPort;
+    }
+    _nextControllerPort = controllerPort + 1;
+    properties.put(ControllerConf.DATA_DIR, DEFAULT_DATA_DIR);
+    properties.put(ControllerConf.LOCAL_TEMP_DIR, DEFAULT_LOCAL_TEMP_DIR);
     // Enable groovy on the controller
     properties.put(ControllerConf.DISABLE_GROOVY, false);
+    properties.put(CommonConstants.CONFIG_OF_TIMEZONE, "UTC");
     overrideControllerConf(properties);
     return properties;
   }
 
+  /**
+   * Can be overridden to add more properties.
+   */
   protected void overrideControllerConf(Map<String, Object> properties) {
-    // do nothing, to be overridden by tests if they need something specific
+  }
+
+  /**
+   * Can be overridden to use a different implementation.
+   */
+  public BaseControllerStarter createControllerStarter() {
+    return new ControllerStarter();
   }
 
   public void startController()
@@ -235,29 +245,16 @@ public class ControllerTest {
 
   public void startController(Map<String, Object> properties)
       throws Exception {
-    Preconditions.checkState(_controllerStarter == null);
+    assertNull(_controllerStarter, "Controller is already started");
+    assertTrue(_controllerPort > 0, "Controller port is not assigned");
 
-    _controllerConfig = new ControllerConf(properties);
-
-    String controllerScheme = "http";
-    if (StringUtils.isNotBlank(_controllerConfig.getControllerVipProtocol())) {
-      controllerScheme = _controllerConfig.getControllerVipProtocol();
-    }
-
-    _controllerPort = DEFAULT_CONTROLLER_PORT;
-    if (StringUtils.isNotBlank(_controllerConfig.getControllerPort())) {
-      _controllerPort = Integer.parseInt(_controllerConfig.getControllerPort());
-    } else if (StringUtils.isNotBlank(_controllerConfig.getControllerVipPort())) {
-      _controllerPort = Integer.parseInt(_controllerConfig.getControllerVipPort());
-    }
-
-    _controllerBaseApiUrl = controllerScheme + "://localhost:" + _controllerPort;
-    _controllerRequestURLBuilder = ControllerRequestURLBuilder.baseUrl(_controllerBaseApiUrl);
-    _controllerDataDir = _controllerConfig.getDataDir();
-
-    _controllerStarter = getControllerStarter();
+    _controllerStarter = createControllerStarter();
     _controllerStarter.init(new PinotConfiguration(properties));
     _controllerStarter.start();
+    _controllerConfig = _controllerStarter.getConfig();
+    _controllerBaseApiUrl = _controllerConfig.generateVipUrl();
+    _controllerRequestURLBuilder = ControllerRequestURLBuilder.baseUrl(_controllerBaseApiUrl);
+    _controllerDataDir = _controllerConfig.getDataDir();
     _helixResourceManager = _controllerStarter.getHelixResourceManager();
     _helixManager = _controllerStarter.getHelixControllerManager();
     _helixDataAccessor = _helixManager.getHelixDataAccessor();
@@ -287,16 +284,26 @@ public class ControllerTest {
     configAccessor.set(scope, Helix.ENABLE_CASE_INSENSITIVE_KEY, Boolean.toString(true));
     // Set hyperloglog log2m value to 12.
     configAccessor.set(scope, Helix.DEFAULT_HYPERLOGLOG_LOG2M_KEY, Integer.toString(12));
+    assertEquals(System.getProperty("user.timezone"), "UTC");
   }
 
   public void stopController() {
-    Preconditions.checkState(_controllerStarter != null);
+    assertNotNull(_controllerStarter, "Controller hasn't been started");
     _controllerStarter.stop();
     _controllerStarter = null;
+    _controllerPort = 0;
     _controllerRequestClient = null;
     FileUtils.deleteQuietly(new File(_controllerDataDir));
   }
 
+  public void restartController()
+      throws Exception {
+    assertNotNull(_controllerStarter, "Controller hasn't been started");
+    _controllerStarter.stop();
+    _controllerStarter = null;
+    startController(_controllerConfig.toMap());
+  }
+
   public int getFakeBrokerInstanceCount() {
     return _helixAdmin.getInstancesInClusterWithTag(getHelixClusterName(), "DefaultTenant_BROKER").size()
         + _helixAdmin.getInstancesInClusterWithTag(getHelixClusterName(), UNTAGGED_BROKER_INSTANCE).size();
@@ -391,30 +398,13 @@ public class ControllerTest {
 
   public void addFakeServerInstancesToAutoJoinHelixCluster(int numInstances, boolean isSingleTenant)
       throws Exception {
-    addFakeServerInstancesToAutoJoinHelixCluster(numInstances, isSingleTenant, Server.DEFAULT_ADMIN_API_PORT);
-  }
-
-  public void addFakeServerInstancesToAutoJoinHelixCluster(int numInstances, boolean isSingleTenant, int baseAdminPort)
-      throws Exception {
     for (int i = 0; i < numInstances; i++) {
-      addFakeServerInstanceToAutoJoinHelixCluster(SERVER_INSTANCE_ID_PREFIX + i, isSingleTenant,
-          NetUtils.findOpenPort(baseAdminPort + i + RandomUtils.nextInt(10000)));
+      addFakeServerInstanceToAutoJoinHelixCluster(SERVER_INSTANCE_ID_PREFIX + i, isSingleTenant);
     }
   }
 
-  public int getFakeServerInstanceCount() {
-    return _helixAdmin.getInstancesInClusterWithTag(getHelixClusterName(), "DefaultTenant_OFFLINE").size()
-        + _helixAdmin.getInstancesInClusterWithTag(getHelixClusterName(), UNTAGGED_SERVER_INSTANCE).size();
-  }
-
   public void addFakeServerInstanceToAutoJoinHelixCluster(String instanceId, boolean isSingleTenant)
       throws Exception {
-    addFakeServerInstanceToAutoJoinHelixCluster(instanceId, isSingleTenant,
-        NetUtils.findOpenPort(Server.DEFAULT_ADMIN_API_PORT + RandomUtils.nextInt(10000)));
-  }
-
-  public void addFakeServerInstanceToAutoJoinHelixCluster(String instanceId, boolean isSingleTenant, int adminPort)
-      throws Exception {
     HelixManager helixManager =
         HelixManagerFactory.getZKHelixManager(getHelixClusterName(), instanceId, InstanceType.PARTICIPANT, getZkUrl());
     helixManager.getStateMachineEngine()
@@ -430,33 +420,29 @@ public class ControllerTest {
     }
     HelixConfigScope configScope = new HelixConfigScopeBuilder(HelixConfigScope.ConfigScopeProperty.PARTICIPANT,
         getHelixClusterName()).forParticipant(instanceId).build();
-    helixAdmin.setConfig(configScope,
-        Collections.singletonMap(Helix.Instance.ADMIN_PORT_KEY, Integer.toString(adminPort)));
+    int adminPort = NetUtils.findOpenPort(_nextServerPort);
+    helixAdmin.setConfig(configScope, Map.of(Helix.Instance.ADMIN_PORT_KEY, Integer.toString(adminPort)));
+    _nextServerPort = adminPort + 1;
     _fakeInstanceHelixManagers.add(helixManager);
   }
 
   /** Add fake server instances until total number of server instances reaches maxCount */
   public void addMoreFakeServerInstancesToAutoJoinHelixCluster(int maxCount, boolean isSingleTenant)
       throws Exception {
-    addMoreFakeServerInstancesToAutoJoinHelixCluster(maxCount, isSingleTenant,
-        NetUtils.findOpenPort(DEFAULT_ADMIN_API_PORT + RandomUtils.nextInt(10000)));
-  }
-
-  /** Add fake server instances until total number of server instances reaches maxCount */
-  public void addMoreFakeServerInstancesToAutoJoinHelixCluster(int maxCount, boolean isSingleTenant, int baseAdminPort)
-      throws Exception {
-
     // get current instance count
     int currentCount = getFakeServerInstanceCount();
 
     // Add more instances if current count is less than max instance count.
-    if (currentCount < maxCount) {
-      for (int i = currentCount; i < maxCount; i++) {
-        addFakeServerInstanceToAutoJoinHelixCluster(SERVER_INSTANCE_ID_PREFIX + i, isSingleTenant, baseAdminPort + i);
-      }
+    for (int i = currentCount; i < maxCount; i++) {
+      addFakeServerInstanceToAutoJoinHelixCluster(SERVER_INSTANCE_ID_PREFIX + i, isSingleTenant);
     }
   }
 
+  public int getFakeServerInstanceCount() {
+    return _helixAdmin.getInstancesInClusterWithTag(getHelixClusterName(), "DefaultTenant_OFFLINE").size()
+        + _helixAdmin.getInstancesInClusterWithTag(getHelixClusterName(), UNTAGGED_SERVER_INSTANCE).size();
+  }
+
   public static class FakeSegmentOnlineOfflineStateModelFactory extends StateModelFactory<StateModel> {
     private static final String STATE_MODEL_DEF = "SegmentOnlineOfflineStateModel";
 
@@ -930,6 +916,10 @@ public class ControllerTest {
     return _helixAdmin;
   }
 
+  public BaseControllerStarter getControllerStarter() {
+    return _controllerStarter;
+  }
+
   public PinotHelixResourceManager getHelixResourceManager() {
     return _helixResourceManager;
   }
@@ -950,10 +940,6 @@ public class ControllerTest {
     return _controllerPort;
   }
 
-  public BaseControllerStarter getControllerStarter() {
-    return _controllerStarter == null ? new ControllerStarter() : _controllerStarter;
-  }
-
   public ControllerConf getControllerConfig() {
     return _controllerConfig;
   }
@@ -1024,7 +1010,7 @@ public class ControllerTest {
   public void setupSharedStateAndValidate()
       throws Exception {
     if (_zookeeperInstance == null || _helixResourceManager == null) {
-      // this is expected to happen only when running a single test case outside of testNG group, i.e when test
+      // this is expected to happen only when running a single test case outside testNG group, i.e. when test
       // cases are run one at a time within IntelliJ or through maven command line. When running under a testNG
       // group, state will have already been setup by @BeforeGroups method in ControllerTestSetup.
       startSharedTestSetup();
