@@ -101,13 +101,16 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
   private static final String OFFLINE_TABLE_NAME = TableNameBuilder.OFFLINE.tableNameWithType(RAW_TABLE_NAME);
   private static final String REALTIME_TABLE_NAME = TableNameBuilder.REALTIME.tableNameWithType(RAW_TABLE_NAME);
 
+  @Override
+  protected void overrideControllerConf(Map<String, Object> properties) {
+    properties.put(ControllerConf.CLUSTER_TENANT_ISOLATION_ENABLE, false);
+  }
+
   @BeforeClass
   public void setUp()
       throws Exception {
     startZk();
-    Map<String, Object> properties = getDefaultControllerConfiguration();
-    properties.put(ControllerConf.CLUSTER_TENANT_ISOLATION_ENABLE, false);
-    startController(properties);
+    startController();
 
     addFakeBrokerInstancesToAutoJoinHelixCluster(NUM_BROKER_INSTANCES, false);
     addFakeServerInstancesToAutoJoinHelixCluster(NUM_SERVER_INSTANCES, false);
@@ -147,6 +150,17 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
     assertEquals(_helixResourceManager.getOnlineUnTaggedServerInstanceList().size(), 0);
   }
 
+  @Test
+  public void testGetInstancesByTag() {
+    List<String> controllersByTag = _helixResourceManager.getAllInstancesWithTag("controller");
+    List<InstanceConfig> controllerConfigs = _helixResourceManager.getAllControllerInstanceConfigs();
+
+    assertEquals(controllersByTag.size(), controllerConfigs.size());
+    for (InstanceConfig c : controllerConfigs) {
+      assertTrue(controllersByTag.contains(c.getInstanceName()));
+    }
+  }
+
   @Test
   public void testGetDataInstanceAdminEndpoints()
       throws Exception {
@@ -212,6 +226,8 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
     String instanceName = "Server_localhost_" + NUM_SERVER_INSTANCES;
     List<String> allInstances = _helixResourceManager.getAllInstances();
     assertFalse(allInstances.contains(instanceName));
+    List<String> allLiveInstances = _helixResourceManager.getAllLiveInstances();
+    assertFalse(allLiveInstances.contains(instanceName));
 
     Instance instance = new Instance("localhost", NUM_SERVER_INSTANCES, InstanceType.SERVER,
         Collections.singletonList(Helix.UNTAGGED_SERVER_INSTANCE), null, 0, 0, 0, 0, false);
@@ -223,6 +239,8 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
     assertTrue(_helixResourceManager.dropInstance(instanceName).isSuccessful());
     allInstances = _helixResourceManager.getAllInstances();
     assertFalse(allInstances.contains(instanceName));
+    allLiveInstances = _helixResourceManager.getAllLiveInstances();
+    assertFalse(allLiveInstances.contains(instanceName));
   }
 
   @Test
@@ -395,6 +413,30 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
     assertThrows(TableNotFoundException.class, () -> _helixResourceManager.getLiveBrokersForTable("fake_OFFLINE"));
     assertThrows(TableNotFoundException.class, () -> _helixResourceManager.getLiveBrokersForTable("fake_REALTIME"));
 
+    // Test retrieving table name to live broker mapping for table without type suffix
+    Map<String, List<InstanceInfo>> rawTableToLiveBrokersMapping =
+        _helixResourceManager.getTableToLiveBrokersMapping(null, List.of(RAW_TABLE_NAME));
+    assertEquals(rawTableToLiveBrokersMapping.size(), 2);
+    assertEquals(rawTableToLiveBrokersMapping.get(OFFLINE_TABLE_NAME).size(), NUM_BROKER_INSTANCES);
+    assertEquals(rawTableToLiveBrokersMapping.get(REALTIME_TABLE_NAME).size(), NUM_BROKER_INSTANCES);
+
+    // Test retrieving table names list to live broker mapping for each table without type suffix
+    Map<String, List<InstanceInfo>> tablesListToLiveBrokersMapping =
+        _helixResourceManager.getTableToLiveBrokersMapping(List.of(OFFLINE_TABLE_NAME, REALTIME_TABLE_NAME));
+    assertEquals(tablesListToLiveBrokersMapping.size(), 2);
+    assertEquals(tablesListToLiveBrokersMapping.get(OFFLINE_TABLE_NAME).size(), NUM_BROKER_INSTANCES);
+    assertEquals(tablesListToLiveBrokersMapping.get(REALTIME_TABLE_NAME).size(), NUM_BROKER_INSTANCES);
+
+    // Test retrieving table name to live broker mapping for table with type suffix
+    Map<String, List<InstanceInfo>> offlineTableToLiveBrokersMapping =
+        _helixResourceManager.getTableToLiveBrokersMapping(List.of(OFFLINE_TABLE_NAME));
+    assertEquals(offlineTableToLiveBrokersMapping.size(), 1);
+    assertEquals(offlineTableToLiveBrokersMapping.get(OFFLINE_TABLE_NAME).size(), NUM_BROKER_INSTANCES);
+
+    // Test that default value behaves the same as empty for optional argument
+    tableToLiveBrokersMapping = _helixResourceManager.getTableToLiveBrokersMapping();
+    assertEquals(tableToLiveBrokersMapping.size(), 2);
+
     // Delete the tables
     _helixResourceManager.deleteRealtimeTable(RAW_TABLE_NAME);
     _helixResourceManager.deleteOfflineTable(RAW_TABLE_NAME);
@@ -799,7 +841,8 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
    * @throws Exception
    */
   @Test
-  public void testSegmentReplacementWithCustomToSegments() throws Exception {
+  public void testSegmentReplacementWithCustomToSegments()
+      throws Exception {
     // Create the table
     addDummySchema(RAW_TABLE_NAME);
     TableConfig tableConfig =
@@ -882,8 +925,8 @@ public class PinotHelixResourceManagerStatelessTest extends ControllerTest {
     assertThrows(RuntimeException.class,
         () -> _helixResourceManager.endReplaceSegments(REALTIME_TABLE_NAME, lineageEntryId1, null));
     // Invalid lineage entry id
-    assertThrows(RuntimeException.class, () -> _helixResourceManager.
-        endReplaceSegments(OFFLINE_TABLE_NAME, "invalid", null));
+    assertThrows(RuntimeException.class,
+        () -> _helixResourceManager.endReplaceSegments(OFFLINE_TABLE_NAME, "invalid", null));
 
     // New segments not available in the table
     assertThrows(RuntimeException.class,
