@@ -152,8 +152,8 @@ public class PinotTaskManagerStatelessTest extends ControllerTest {
             "MergeRollupTask", ImmutableMap.of("schedule", "0 */10 * ? * * *"))));
     updateTableConfig(tableConfig);
     waitForJobGroupNames(_controllerStarter.getTaskManager(),
-        jgn -> jgn.size() == 2 && jgn.contains(MinionConstants.SegmentGenerationAndPushTask.TASK_TYPE) && jgn
-            .contains(MinionConstants.MergeRollupTask.TASK_TYPE),
+        jgn -> jgn.size() == 2 && jgn.contains(MinionConstants.SegmentGenerationAndPushTask.TASK_TYPE) && jgn.contains(
+            MinionConstants.MergeRollupTask.TASK_TYPE),
         "JobGroupNames should have SegmentGenerationAndPushTask and MergeRollupTask");
     validateJob(MinionConstants.SegmentGenerationAndPushTask.TASK_TYPE, "0 */30 * ? * * *");
     validateJob(MinionConstants.MergeRollupTask.TASK_TYPE, "0 */10 * ? * * *");
@@ -203,10 +203,8 @@ public class PinotTaskManagerStatelessTest extends ControllerTest {
         "JobGroupNames should have SegmentGenerationAndPushTask only");
     validateJob(MinionConstants.SegmentGenerationAndPushTask.TASK_TYPE, "0 */10 * ? * * *");
 
-    // Restart controller
-    stopController();
-    startController(properties);
-    // wait for controller to start correctly.
+    // Restart controller.
+    restartController();
     TestUtils.waitForCondition((aVoid) -> {
       try {
         long tableSize = getTableSize(OFFLINE_TABLE_NAME);
@@ -232,8 +230,8 @@ public class PinotTaskManagerStatelessTest extends ControllerTest {
     // The new MergeRollup task wouldn't be scheduled if not eagerly checking table configs
     // after setting up subscriber on ChildChanges zk event when controller gets restarted.
     waitForJobGroupNames(_controllerStarter.getTaskManager(),
-        jgn -> jgn.size() == 2 && jgn.contains(MinionConstants.SegmentGenerationAndPushTask.TASK_TYPE) && jgn
-            .contains(MinionConstants.MergeRollupTask.TASK_TYPE),
+        jgn -> jgn.size() == 2 && jgn.contains(MinionConstants.SegmentGenerationAndPushTask.TASK_TYPE) && jgn.contains(
+            MinionConstants.MergeRollupTask.TASK_TYPE),
         "JobGroupNames should have SegmentGenerationAndPushTask and MergeRollupTask");
 
     dropOfflineTable(RAW_TABLE_NAME);
