@@ -53,49 +53,63 @@ public class FlushThresholdUpdaterTest {
   public void testFlushThresholdUpdateManager() {
     FlushThresholdUpdateManager flushThresholdUpdateManager = new FlushThresholdUpdateManager();
 
-    // Flush threshold rows larger than 0 - DefaultFlushThresholdUpdater should be returned
-    FlushThresholdUpdater defaultFlushThresholdUpdater = flushThresholdUpdateManager
-        .getFlushThresholdUpdater(mockStreamConfig(StreamConfig.DEFAULT_FLUSH_THRESHOLD_ROWS));
-    assertTrue(defaultFlushThresholdUpdater instanceof DefaultFlushThresholdUpdater);
-    assertEquals(((DefaultFlushThresholdUpdater) defaultFlushThresholdUpdater).getTableFlushSize(),
+    // None of the flush threshold set - DefaultFlushThresholdUpdater should be returned
+    FlushThresholdUpdater flushThresholdUpdater =
+        flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(-1, -1, -1));
+    assertTrue(flushThresholdUpdater instanceof DefaultFlushThresholdUpdater);
+    assertEquals(((DefaultFlushThresholdUpdater) flushThresholdUpdater).getTableFlushSize(),
         StreamConfig.DEFAULT_FLUSH_THRESHOLD_ROWS);
 
+    // Flush threshold rows larger than 0 - DefaultFlushThresholdUpdater should be returned
+    flushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(1234, -1, -1));
+    assertTrue(flushThresholdUpdater instanceof DefaultFlushThresholdUpdater);
+    assertEquals(((DefaultFlushThresholdUpdater) flushThresholdUpdater).getTableFlushSize(), 1234);
+
+    // Flush threshold segment rows larger than 0 - FixedFlushThresholdUpdater should be returned
+    flushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(-1, 1234, -1));
+    assertTrue(flushThresholdUpdater instanceof FixedFlushThresholdUpdater);
+
     // Flush threshold rows set to 0 - SegmentSizeBasedFlushThresholdUpdater should be returned
-    StreamConfig autotuneStreamConfig = mockDefaultAutotuneStreamConfig();
-    FlushThresholdUpdater autotuneFlushThresholdUpdater =
-        flushThresholdUpdateManager.getFlushThresholdUpdater(autotuneStreamConfig);
-    assertTrue(autotuneFlushThresholdUpdater instanceof SegmentSizeBasedFlushThresholdUpdater);
+    FlushThresholdUpdater segmentBasedflushThresholdUpdater =
+        flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(0, -1, -1));
+    assertTrue(segmentBasedflushThresholdUpdater instanceof SegmentSizeBasedFlushThresholdUpdater);
 
-    // Call again with flush threshold rows set to 0 - same Object should be returned
-    assertSame(flushThresholdUpdateManager.getFlushThresholdUpdater(mockAutotuneStreamConfig(10000L, 10000L, 10000)),
-        autotuneFlushThresholdUpdater);
+    // Flush threshold segment size larger than 0 - SegmentSizeBasedFlushThresholdUpdater should be returned
+    flushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(0, -1, 1234));
+    assertSame(flushThresholdUpdater, segmentBasedflushThresholdUpdater);
 
-    // Call again with flush threshold rows set larger than 0 - DefaultFlushThresholdUpdater should be returned
-    defaultFlushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(10000));
-    assertTrue(defaultFlushThresholdUpdater instanceof DefaultFlushThresholdUpdater);
-    assertEquals(((DefaultFlushThresholdUpdater) defaultFlushThresholdUpdater).getTableFlushSize(), 10000);
+    // Flush threshold rows set larger than 0 - DefaultFlushThresholdUpdater should be returned
+    flushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(12345, -1, -1));
+    assertTrue(flushThresholdUpdater instanceof DefaultFlushThresholdUpdater);
+    assertEquals(((DefaultFlushThresholdUpdater) flushThresholdUpdater).getTableFlushSize(), 12345);
 
     // Call again with flush threshold rows set to 0 - a different Object should be returned
-    assertNotSame(flushThresholdUpdateManager.getFlushThresholdUpdater(autotuneStreamConfig),
-        autotuneFlushThresholdUpdater);
+    flushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(0, -1, -1));
+    assertTrue(flushThresholdUpdater instanceof SegmentSizeBasedFlushThresholdUpdater);
+    assertNotSame(flushThresholdUpdater, segmentBasedflushThresholdUpdater);
+    segmentBasedflushThresholdUpdater = flushThresholdUpdater;
 
     // Clear the updater
     flushThresholdUpdateManager.clearFlushThresholdUpdater(REALTIME_TABLE_NAME);
 
     // Call again with flush threshold rows set to 0 - a different Object should be returned
-    assertNotSame(flushThresholdUpdateManager.getFlushThresholdUpdater(autotuneStreamConfig),
-        autotuneFlushThresholdUpdater);
+    flushThresholdUpdater = flushThresholdUpdateManager.getFlushThresholdUpdater(mockStreamConfig(0, -1, -1));
+    assertTrue(flushThresholdUpdater instanceof SegmentSizeBasedFlushThresholdUpdater);
+    assertNotSame(flushThresholdUpdater, segmentBasedflushThresholdUpdater);
   }
 
-  private StreamConfig mockStreamConfig(int flushThresholdRows) {
+  private StreamConfig mockStreamConfig(int flushThresholdRows, int flushThresholdSegmentRows,
+      long flushThresholdSegmentSize) {
     StreamConfig streamConfig = mock(StreamConfig.class);
     when(streamConfig.getTableNameWithType()).thenReturn(REALTIME_TABLE_NAME);
     when(streamConfig.getFlushThresholdRows()).thenReturn(flushThresholdRows);
+    when(streamConfig.getFlushThresholdSegmentRows()).thenReturn(flushThresholdSegmentRows);
+    when(streamConfig.getFlushThresholdSegmentSizeBytes()).thenReturn(flushThresholdSegmentSize);
     return streamConfig;
   }
 
-  private StreamConfig mockAutotuneStreamConfig(long flushSegmentDesiredSizeBytes,
-      long flushThresholdTimeMillis, int flushAutotuneInitialRows) {
+  private StreamConfig mockAutotuneStreamConfig(long flushSegmentDesiredSizeBytes, long flushThresholdTimeMillis,
+      int flushAutotuneInitialRows) {
     StreamConfig streamConfig = mock(StreamConfig.class);
     when(streamConfig.getTableNameWithType()).thenReturn(REALTIME_TABLE_NAME);
     when(streamConfig.getFlushThresholdRows()).thenReturn(0);
@@ -123,8 +137,8 @@ public class FlushThresholdUpdaterTest {
     long segmentSizeLowerLimit = (long) (desiredSegmentSizeBytes * 0.99);
     long segmentSizeHigherLimit = (long) (desiredSegmentSizeBytes * 1.01);
 
-    for (long[] segmentSizesMB : Arrays
-        .asList(EXPONENTIAL_GROWTH_SEGMENT_SIZES_MB, LOGARITHMIC_GROWTH_SEGMENT_SIZES_MB, STEPS_SEGMENT_SIZES_MB)) {
+    for (long[] segmentSizesMB : Arrays.asList(EXPONENTIAL_GROWTH_SEGMENT_SIZES_MB, LOGARITHMIC_GROWTH_SEGMENT_SIZES_MB,
+        STEPS_SEGMENT_SIZES_MB)) {
       SegmentSizeBasedFlushThresholdUpdater flushThresholdUpdater = new SegmentSizeBasedFlushThresholdUpdater();
 
       // Start consumption
@@ -160,8 +174,8 @@ public class FlushThresholdUpdaterTest {
     long segmentSizeLowerLimit = (long) (desiredSegmentSizeBytes * 0.99);
     long segmentSizeHigherLimit = (long) (desiredSegmentSizeBytes * 1.01);
 
-    for (long[] segmentSizesMB : Arrays
-        .asList(EXPONENTIAL_GROWTH_SEGMENT_SIZES_MB, LOGARITHMIC_GROWTH_SEGMENT_SIZES_MB, STEPS_SEGMENT_SIZES_MB)) {
+    for (long[] segmentSizesMB : Arrays.asList(EXPONENTIAL_GROWTH_SEGMENT_SIZES_MB, LOGARITHMIC_GROWTH_SEGMENT_SIZES_MB,
+        STEPS_SEGMENT_SIZES_MB)) {
       SegmentSizeBasedFlushThresholdUpdater flushThresholdUpdater = new SegmentSizeBasedFlushThresholdUpdater();
 
       // Start consumption
