@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.core.common;
 
+import com.google.common.base.Preconditions;
 import java.io.Closeable;
 import java.io.IOException;
 import java.math.BigDecimal;
@@ -32,7 +33,7 @@ import org.apache.pinot.segment.spi.evaluator.TransformEvaluator;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
 import org.apache.pinot.segment.spi.index.reader.ForwardIndexReader;
 import org.apache.pinot.segment.spi.index.reader.ForwardIndexReaderContext;
-import org.apache.pinot.spi.data.FieldSpec;
+import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.trace.Tracing;
 import org.apache.pinot.spi.utils.BytesUtils;
 
@@ -66,10 +67,13 @@ public class DataFetcher {
     for (Map.Entry<String, DataSource> entry : dataSourceMap.entrySet()) {
       String column = entry.getKey();
       DataSource dataSource = entry.getValue();
-      ColumnValueReader columnValueReader =
-          new ColumnValueReader(dataSource.getForwardIndex(), dataSource.getDictionary());
-      _columnValueReaderMap.put(column, columnValueReader);
       DataSourceMetadata dataSourceMetadata = dataSource.getDataSourceMetadata();
+      ForwardIndexReader<?> forwardIndexReader = dataSource.getForwardIndex();
+      Preconditions.checkState(forwardIndexReader != null,
+          "Forward index disabled for column: %s, cannot create DataFetcher!", column);
+      ColumnValueReader columnValueReader =
+          new ColumnValueReader(forwardIndexReader, dataSource.getDictionary());
+      _columnValueReaderMap.put(column, columnValueReader);
       if (!dataSourceMetadata.isSingleValue()) {
         maxNumValuesPerMVEntry = Math.max(maxNumValuesPerMVEntry, dataSourceMetadata.getMaxNumValuesPerMVEntry());
       }
@@ -407,6 +411,18 @@ public class DataFetcher {
     _columnValueReaderMap.get(column).readStringValuesMV(evaluator, inDocIds, length, outValues);
   }
 
+  /**
+   * Fetch the bytes values for a multi-valued column.
+   *
+   * @param column Column name
+   * @param inDocIds Input document Ids buffer
+   * @param length Number of input document Ids
+   * @param outValues Buffer for output
+   */
+  public void fetchBytesValues(String column, int[] inDocIds, int length, byte[][][] outValues) {
+    _columnValueReaderMap.get(column).readBytesValuesMV(inDocIds, length, outValues);
+  }
+
   /**
    * Fetch the number of values for a multi-valued column.
    *
@@ -423,11 +439,13 @@ public class DataFetcher {
    * Helper class to read values for a column from forward index and dictionary. For raw (non-dictionary-encoded)
    * forward index, similar to Dictionary, type conversion among INT, LONG, FLOAT, DOUBLE, STRING is supported; type
    * conversion between STRING and BYTES via Hex encoding/decoding is supported.
+   *
+   * TODO: Type conversion for BOOLEAN and TIMESTAMP is not handled
    */
   private class ColumnValueReader implements Closeable {
     final ForwardIndexReader _reader;
     final Dictionary _dictionary;
-    final FieldSpec.DataType _dataType;
+    final DataType _storedType;
     final boolean _singleValue;
 
     boolean _readerContextCreated;
@@ -436,7 +454,7 @@ public class DataFetcher {
     ColumnValueReader(ForwardIndexReader reader, @Nullable Dictionary dictionary) {
       _reader = reader;
       _dictionary = dictionary;
-      _dataType = reader.getStoredType();
+      _storedType = reader.getStoredType();
       _singleValue = reader.isSingleValue();
     }
 
@@ -450,12 +468,12 @@ public class DataFetcher {
     }
 
     void readDictIds(int[] docIds, int length, int[] dictIdBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       _reader.readDictIds(docIds, length, dictIdBuffer, getReaderContext());
     }
 
     void readIntValues(int[] docIds, int length, int[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
@@ -467,13 +485,13 @@ public class DataFetcher {
     }
 
     void readIntValues(TransformEvaluator evaluator, int[] docIds, int length, int[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valueBuffer);
     }
 
     void readLongValues(int[] docIds, int length, long[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
@@ -485,13 +503,13 @@ public class DataFetcher {
     }
 
     void readLongValues(TransformEvaluator evaluator, int[] docIds, int length, long[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valueBuffer);
     }
 
     void readFloatValues(int[] docIds, int length, float[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
@@ -503,13 +521,13 @@ public class DataFetcher {
     }
 
     void readFloatValues(TransformEvaluator evaluator, int[] docIds, int length, float[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valueBuffer);
     }
 
     void readDoubleValues(int[] docIds, int length, double[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
@@ -521,13 +539,13 @@ public class DataFetcher {
     }
 
     void readDoubleValues(TransformEvaluator evaluator, int[] docIds, int length, double[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valueBuffer);
     }
 
     void readBigDecimalValues(int[] docIds, int length, BigDecimal[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
@@ -539,20 +557,20 @@ public class DataFetcher {
     }
 
     void readBigDecimalValues(TransformEvaluator evaluator, int[] docIds, int length, BigDecimal[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valueBuffer);
     }
 
     void readStringValues(int[] docIds, int length, String[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
         _reader.readDictIds(docIds, length, dictIdBuffer, readerContext);
         _dictionary.readStringValues(dictIdBuffer, length, valueBuffer);
       } else {
-        switch (_reader.getStoredType()) {
+        switch (_storedType) {
           case INT:
             for (int i = 0; i < length; i++) {
               valueBuffer[i] = Integer.toString(_reader.getInt(docIds[i], readerContext));
@@ -573,6 +591,11 @@ public class DataFetcher {
               valueBuffer[i] = Double.toString(_reader.getDouble(docIds[i], readerContext));
             }
             break;
+          case BIG_DECIMAL:
+            for (int i = 0; i < length; i++) {
+              valueBuffer[i] = _reader.getBigDecimal(docIds[i], readerContext).toPlainString();
+            }
+            break;
           case STRING:
             for (int i = 0; i < length; i++) {
               valueBuffer[i] = _reader.getString(docIds[i], readerContext);
@@ -590,146 +613,156 @@ public class DataFetcher {
     }
 
     void readStringValues(TransformEvaluator evaluator, int[] docIds, int length, String[] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valueBuffer);
     }
 
     void readBytesValues(int[] docIds, int length, byte[][] valueBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         int[] dictIdBuffer = THREAD_LOCAL_DICT_IDS.get();
         _reader.readDictIds(docIds, length, dictIdBuffer, readerContext);
         _dictionary.readBytesValues(dictIdBuffer, length, valueBuffer);
       } else {
-        switch (_reader.getStoredType()) {
-          case STRING:
-            for (int i = 0; i < length; i++) {
-              valueBuffer[i] = BytesUtils.toBytes(_reader.getString(docIds[i], readerContext));
-            }
-            break;
-          case BYTES:
-            for (int i = 0; i < length; i++) {
-              valueBuffer[i] = _reader.getBytes(docIds[i], readerContext);
-            }
-            break;
-          default:
-            throw new IllegalStateException();
+        for (int i = 0; i < length; i++) {
+          valueBuffer[i] = _reader.getBytes(docIds[i], readerContext);
         }
       }
     }
 
     void readDictIdsMV(int[] docIds, int length, int[][] dictIdsBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
       for (int i = 0; i < length; i++) {
-        int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, getReaderContext());
+        int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
         dictIdsBuffer[i] = Arrays.copyOfRange(_reusableMVDictIds, 0, numValues);
       }
     }
 
     void readIntValuesMV(int[] docIds, int length, int[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         for (int i = 0; i < length; i++) {
-          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, getReaderContext());
+          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
           int[] values = new int[numValues];
           _dictionary.readIntValues(_reusableMVDictIds, numValues, values);
           valuesBuffer[i] = values;
         }
       } else {
-        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, getReaderContext());
+        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, readerContext);
       }
     }
 
     void readIntValuesMV(TransformEvaluator evaluator, int[] docIds, int length, int[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valuesBuffer);
     }
 
     void readLongValuesMV(int[] docIds, int length, long[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         for (int i = 0; i < length; i++) {
-          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, getReaderContext());
+          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
           long[] values = new long[numValues];
           _dictionary.readLongValues(_reusableMVDictIds, numValues, values);
           valuesBuffer[i] = values;
         }
       } else {
-        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, getReaderContext());
+        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, readerContext);
       }
     }
 
     void readLongValuesMV(TransformEvaluator evaluator, int[] docIds, int length, long[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valuesBuffer);
     }
 
     void readFloatValuesMV(int[] docIds, int length, float[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         for (int i = 0; i < length; i++) {
-          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, getReaderContext());
+          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
           float[] values = new float[numValues];
           _dictionary.readFloatValues(_reusableMVDictIds, numValues, values);
           valuesBuffer[i] = values;
         }
       } else {
-        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, getReaderContext());
+        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, readerContext);
       }
     }
 
     void readFloatValuesMV(TransformEvaluator evaluator, int[] docIds, int length, float[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valuesBuffer);
     }
 
     void readDoubleValuesMV(int[] docIds, int length, double[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         for (int i = 0; i < length; i++) {
-          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, getReaderContext());
+          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
           double[] values = new double[numValues];
           _dictionary.readDoubleValues(_reusableMVDictIds, numValues, values);
           valuesBuffer[i] = values;
         }
       } else {
-        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, getReaderContext());
+        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, readerContext);
       }
     }
 
     void readDoubleValuesMV(TransformEvaluator evaluator, int[] docIds, int length, double[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valuesBuffer);
     }
 
     void readStringValuesMV(int[] docIds, int length, String[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
       if (_dictionary != null) {
         for (int i = 0; i < length; i++) {
-          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, getReaderContext());
+          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
           String[] values = new String[numValues];
           _dictionary.readStringValues(_reusableMVDictIds, numValues, values);
           valuesBuffer[i] = values;
         }
       } else {
-        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, getReaderContext());
+        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, readerContext);
       }
     }
 
     void readStringValuesMV(TransformEvaluator evaluator, int[] docIds, int length, String[][] valuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       evaluator.evaluateBlock(docIds, length, _reader, getReaderContext(), _dictionary, getSVDictIdsBuffer(),
           valuesBuffer);
     }
 
+    void readBytesValuesMV(int[] docIds, int length, byte[][][] valuesBuffer) {
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
+      ForwardIndexReaderContext readerContext = getReaderContext();
+      if (_dictionary != null) {
+        for (int i = 0; i < length; i++) {
+          int numValues = _reader.getDictIdMV(docIds[i], _reusableMVDictIds, readerContext);
+          byte[][] values = new byte[numValues][];
+          _dictionary.readBytesValues(_reusableMVDictIds, numValues, values);
+          valuesBuffer[i] = values;
+        }
+      } else {
+        _reader.readValuesMV(docIds, length, _maxNumValuesPerMVEntry, valuesBuffer, readerContext);
+      }
+    }
+
     public void readNumValuesMV(int[] docIds, int length, int[] numValuesBuffer) {
-      Tracing.activeRecording().setInputDataType(_dataType, _singleValue);
+      Tracing.activeRecording().setInputDataType(_storedType, _singleValue);
       for (int i = 0; i < length; i++) {
         numValuesBuffer[i] = _reader.getNumValuesMV(docIds[i], getReaderContext());
       }
