@@ -52,6 +52,7 @@ import it.unimi.dsi.fastutil.longs.LongIterator;
 import it.unimi.dsi.fastutil.longs.LongOpenHashSet;
 import it.unimi.dsi.fastutil.longs.LongSet;
 import it.unimi.dsi.fastutil.objects.ObjectArrayList;
+import it.unimi.dsi.fastutil.objects.ObjectLinkedOpenHashSet;
 import it.unimi.dsi.fastutil.objects.ObjectOpenHashSet;
 import it.unimi.dsi.fastutil.objects.ObjectSet;
 import java.io.IOException;
@@ -62,6 +63,7 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.PriorityQueue;
 import java.util.Set;
 import org.apache.datasketches.common.ArrayOfStringsSerDe;
 import org.apache.datasketches.cpc.CpcSketch;
@@ -74,12 +76,14 @@ import org.apache.datasketches.tuple.aninteger.IntegerSummary;
 import org.apache.datasketches.tuple.aninteger.IntegerSummaryDeserializer;
 import org.apache.pinot.common.CustomObject;
 import org.apache.pinot.common.utils.HashUtil;
+import org.apache.pinot.core.query.aggregation.function.funnel.FunnelStepEvent;
 import org.apache.pinot.core.query.aggregation.utils.exprminmax.ExprMinMaxObject;
 import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.core.query.utils.idset.IdSet;
 import org.apache.pinot.core.query.utils.idset.IdSets;
 import org.apache.pinot.segment.local.customobject.AvgPair;
 import org.apache.pinot.segment.local.customobject.CovarianceTuple;
+import org.apache.pinot.segment.local.customobject.CpcSketchAccumulator;
 import org.apache.pinot.segment.local.customobject.DoubleLongPair;
 import org.apache.pinot.segment.local.customobject.FloatLongPair;
 import org.apache.pinot.segment.local.customobject.IntLongPair;
@@ -89,6 +93,7 @@ import org.apache.pinot.segment.local.customobject.PinotFourthMoment;
 import org.apache.pinot.segment.local.customobject.QuantileDigest;
 import org.apache.pinot.segment.local.customobject.StringLongPair;
 import org.apache.pinot.segment.local.customobject.ThetaSketchAccumulator;
+import org.apache.pinot.segment.local.customobject.TupleIntSketchAccumulator;
 import org.apache.pinot.segment.local.customobject.VarianceTuple;
 import org.apache.pinot.segment.local.utils.GeometrySerializer;
 import org.apache.pinot.spi.utils.BigDecimalUtils;
@@ -156,7 +161,11 @@ public class ObjectSerDeUtils {
     FloatArrayList(44),
     StringArrayList(45),
     UltraLogLog(46),
-    ThetaSketchAccumulator(47);
+    ThetaSketchAccumulator(47),
+    TupleIntSketchAccumulator(48),
+    CpcSketchAccumulator(49),
+    OrderedStringSet(50),
+    FunnelStepEventAccumulator(51);
 
     private final int _value;
 
@@ -234,6 +243,13 @@ public class ObjectSerDeUtils {
         return ObjectType.FloatSet;
       } else if (value instanceof DoubleSet) {
         return ObjectType.DoubleSet;
+      } else if (value instanceof ObjectLinkedOpenHashSet) {
+        ObjectLinkedOpenHashSet objectSet = (ObjectLinkedOpenHashSet) value;
+        if (objectSet.isEmpty() || objectSet.first() instanceof String) {
+          return ObjectType.OrderedStringSet;
+        }
+        throw new IllegalArgumentException(
+            "Unsupported type of value: " + objectSet.first().getClass().getSimpleName());
       } else if (value instanceof ObjectSet) {
         ObjectSet objectSet = (ObjectSet) value;
         if (objectSet.isEmpty() || objectSet.iterator().next() instanceof String) {
@@ -277,6 +293,17 @@ public class ObjectSerDeUtils {
         return ObjectType.UltraLogLog;
       } else if (value instanceof ThetaSketchAccumulator) {
         return ObjectType.ThetaSketchAccumulator;
+      } else if (value instanceof TupleIntSketchAccumulator) {
+        return ObjectType.TupleIntSketchAccumulator;
+      } else if (value instanceof CpcSketchAccumulator) {
+        return ObjectType.CpcSketchAccumulator;
+      } else if (value instanceof PriorityQueue) {
+        PriorityQueue priorityQueue = (PriorityQueue) value;
+        if (priorityQueue.isEmpty() || priorityQueue.peek() instanceof FunnelStepEvent) {
+          return ObjectType.FunnelStepEventAccumulator;
+        }
+        throw new IllegalArgumentException(
+            "Unsupported type of value: " + priorityQueue.peek().getClass().getSimpleName());
       } else {
         throw new IllegalArgumentException("Unsupported type of value: " + value.getClass().getSimpleName());
       }
@@ -1587,7 +1614,7 @@ public class ObjectSerDeUtils {
     }
   };
 
-  public static final ObjectSerDe<ThetaSketchAccumulator> DATA_SKETCH_SKETCH_ACCUMULATOR_SER_DE =
+  public static final ObjectSerDe<ThetaSketchAccumulator> DATA_SKETCH_THETA_ACCUMULATOR_SER_DE =
       new ObjectSerDe<ThetaSketchAccumulator>() {
 
         @Override
@@ -1614,6 +1641,145 @@ public class ObjectSerDeUtils {
         }
       };
 
+  public static final ObjectSerDe<TupleIntSketchAccumulator> DATA_SKETCH_INT_TUPLE_ACCUMULATOR_SER_DE =
+      new ObjectSerDe<TupleIntSketchAccumulator>() {
+
+        @Override
+        public byte[] serialize(TupleIntSketchAccumulator tupleIntSketchBuffer) {
+          org.apache.datasketches.tuple.Sketch<IntegerSummary> sketch = tupleIntSketchBuffer.getResult();
+          return sketch.toByteArray();
+        }
+
+        @Override
+        public TupleIntSketchAccumulator deserialize(byte[] bytes) {
+          return deserialize(ByteBuffer.wrap(bytes));
+        }
+
+        // Note: The accumulator is designed to serialize as a sketch and should
+        // not be deserialized in practice.
+        @Override
+        public TupleIntSketchAccumulator deserialize(ByteBuffer byteBuffer) {
+          TupleIntSketchAccumulator tupleIntSketchAccumulator = new TupleIntSketchAccumulator();
+          byte[] bytes = new byte[byteBuffer.remaining()];
+          byteBuffer.get(bytes);
+          org.apache.datasketches.tuple.Sketch<IntegerSummary> sketch =
+              org.apache.datasketches.tuple.Sketches.heapifySketch(Memory.wrap(bytes),
+                  new IntegerSummaryDeserializer());
+          tupleIntSketchAccumulator.apply(sketch);
+          return tupleIntSketchAccumulator;
+        }
+      };
+
+  public static final ObjectSerDe<CpcSketchAccumulator> DATA_SKETCH_CPC_ACCUMULATOR_SER_DE =
+      new ObjectSerDe<CpcSketchAccumulator>() {
+
+        @Override
+        public byte[] serialize(CpcSketchAccumulator cpcSketchBuffer) {
+          CpcSketch sketch = cpcSketchBuffer.getResult();
+          return sketch.toByteArray();
+        }
+
+        @Override
+        public CpcSketchAccumulator deserialize(byte[] bytes) {
+          return deserialize(ByteBuffer.wrap(bytes));
+        }
+
+        // Note: The accumulator is designed to serialize as a sketch and should
+        // not be deserialized in practice.
+        @Override
+        public CpcSketchAccumulator deserialize(ByteBuffer byteBuffer) {
+          CpcSketchAccumulator cpcSketchAccumulator = new CpcSketchAccumulator();
+          byte[] bytes = new byte[byteBuffer.remaining()];
+          byteBuffer.get(bytes);
+          CpcSketch sketch = CpcSketch.heapify(Memory.wrap(bytes));
+          cpcSketchAccumulator.apply(sketch);
+          return cpcSketchAccumulator;
+        }
+      };
+
+  public static final ObjectSerDe<ObjectLinkedOpenHashSet<String>> ORDERED_STRING_SET_SER_DE =
+      new ObjectSerDe<ObjectLinkedOpenHashSet<String>>() {
+
+        @Override
+        public byte[] serialize(ObjectLinkedOpenHashSet<String> stringSet) {
+          int size = stringSet.size();
+          // Besides the value bytes, we store: size, length for each value
+          long bufferSize = (1 + (long) size) * Integer.BYTES;
+          byte[][] valueBytesArray = new byte[size][];
+          int index = 0;
+          for (String value : stringSet) {
+            byte[] valueBytes = value.getBytes(UTF_8);
+            bufferSize += valueBytes.length;
+            valueBytesArray[index++] = valueBytes;
+          }
+          Preconditions.checkState(bufferSize <= Integer.MAX_VALUE, "Buffer size exceeds 2GB");
+          byte[] bytes = new byte[(int) bufferSize];
+          ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
+          byteBuffer.putInt(size);
+          for (byte[] valueBytes : valueBytesArray) {
+            byteBuffer.putInt(valueBytes.length);
+            byteBuffer.put(valueBytes);
+          }
+          return bytes;
+        }
+
+        @Override
+        public ObjectLinkedOpenHashSet<String> deserialize(byte[] bytes) {
+          return deserialize(ByteBuffer.wrap(bytes));
+        }
+
+        @Override
+        public ObjectLinkedOpenHashSet<String> deserialize(ByteBuffer byteBuffer) {
+          int size = byteBuffer.getInt();
+          ObjectLinkedOpenHashSet<String> stringSet = new ObjectLinkedOpenHashSet<>(size);
+          for (int i = 0; i < size; i++) {
+            int length = byteBuffer.getInt();
+            byte[] bytes = new byte[length];
+            byteBuffer.get(bytes);
+            stringSet.add(new String(bytes, UTF_8));
+          }
+          return stringSet;
+        }
+      };
+
+  public static final ObjectSerDe<PriorityQueue<FunnelStepEvent>> FUNNEL_STEP_EVENT_ACCUMULATOR_SER_DE =
+      new ObjectSerDe<PriorityQueue<FunnelStepEvent>>() {
+
+        @Override
+        public byte[] serialize(PriorityQueue<FunnelStepEvent> funnelStepEvents) {
+          int numEvents = funnelStepEvents.size();
+          long bufferSize = Integer.BYTES + (long) numEvents * FunnelStepEvent.SIZE_IN_BYTES;
+          Preconditions.checkState(bufferSize <= Integer.MAX_VALUE, "Buffer size exceeds 2GB");
+          byte[] bytes = new byte[(int) bufferSize];
+          ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
+          byteBuffer.putInt(numEvents);
+          for (FunnelStepEvent funnelStepEvent : funnelStepEvents) {
+            byteBuffer.put(funnelStepEvent.getBytes());
+          }
+          return bytes;
+        }
+
+        @Override
+        public PriorityQueue<FunnelStepEvent> deserialize(byte[] bytes) {
+          return deserialize(ByteBuffer.wrap(bytes));
+        }
+
+        @Override
+        public PriorityQueue<FunnelStepEvent> deserialize(ByteBuffer byteBuffer) {
+          int size = byteBuffer.getInt();
+          if (size == 0) {
+            return new PriorityQueue<>();
+          }
+          PriorityQueue<FunnelStepEvent> funnelStepEvents = new PriorityQueue<>(size);
+          byte[] bytes = new byte[FunnelStepEvent.SIZE_IN_BYTES];
+          for (int i = 0; i < size; i++) {
+            byteBuffer.get(bytes);
+            funnelStepEvents.add(new FunnelStepEvent(bytes));
+          }
+          return funnelStepEvents;
+        }
+      };
+
   // NOTE: DO NOT change the order, it has to be the same order as the ObjectType
   //@formatter:off
   private static final ObjectSerDe[] SER_DES = {
@@ -1664,7 +1830,11 @@ public class ObjectSerDeUtils {
       FLOAT_ARRAY_LIST_SER_DE,
       STRING_ARRAY_LIST_SER_DE,
       ULTRA_LOG_LOG_OBJECT_SER_DE,
-      DATA_SKETCH_SKETCH_ACCUMULATOR_SER_DE,
+      DATA_SKETCH_THETA_ACCUMULATOR_SER_DE,
+      DATA_SKETCH_INT_TUPLE_ACCUMULATOR_SER_DE,
+      DATA_SKETCH_CPC_ACCUMULATOR_SER_DE,
+      ORDERED_STRING_SET_SER_DE,
+      FUNNEL_STEP_EVENT_ACCUMULATOR_SER_DE,
   };
   //@formatter:on
 
