@@ -38,6 +38,7 @@ import org.apache.pinot.core.query.request.context.QueryContext;
 @SuppressWarnings({"rawtypes", "unchecked"})
 public abstract class IndexedTable extends BaseTable {
   protected final Map<Key, Record> _lookupMap;
+  protected final boolean _hasFinalInput;
   protected final int _resultSize;
   protected final int _numKeyColumns;
   protected final AggregationFunction[] _aggregationFunctions;
@@ -54,16 +55,18 @@ public abstract class IndexedTable extends BaseTable {
    * Constructor for the IndexedTable.
    *
    * @param dataSchema    Data schema of the table
+   * @param hasFinalInput Whether the input is the final aggregate result
    * @param queryContext  Query context
    * @param resultSize    Number of records to keep in the final result after calling {@link #finish(boolean, boolean)}
    * @param trimSize      Number of records to keep when trimming the table
    * @param trimThreshold Trim the table when the number of records exceeds the threshold
    * @param lookupMap     Map from keys to records
    */
-  protected IndexedTable(DataSchema dataSchema, QueryContext queryContext, int resultSize, int trimSize,
-      int trimThreshold, Map<Key, Record> lookupMap) {
+  protected IndexedTable(DataSchema dataSchema, boolean hasFinalInput, QueryContext queryContext, int resultSize,
+      int trimSize, int trimThreshold, Map<Key, Record> lookupMap) {
     super(dataSchema);
     _lookupMap = lookupMap;
+    _hasFinalInput = hasFinalInput;
     _resultSize = resultSize;
 
     List<ExpressionContext> groupByExpressions = queryContext.getGroupByExpressions();
@@ -74,7 +77,7 @@ public abstract class IndexedTable extends BaseTable {
     if (orderByExpressions != null) {
       // GROUP BY with ORDER BY
       _hasOrderBy = true;
-      _tableResizer = new TableResizer(dataSchema, queryContext);
+      _tableResizer = new TableResizer(dataSchema, hasFinalInput, queryContext);
       // NOTE: trimSize is bounded by trimThreshold/2 to protect the server from using too much memory.
       // TODO: Re-evaluate it as it can lead to in-accurate results
       _trimSize = Math.min(trimSize, trimThreshold / 2);
@@ -102,34 +105,32 @@ public abstract class IndexedTable extends BaseTable {
    * Adds a record with new key or updates a record with existing key.
    */
   protected void addOrUpdateRecord(Key key, Record newRecord) {
-    _lookupMap.compute(key, (k, v) -> {
-      if (v == null) {
-        return newRecord;
-      } else {
-        Object[] existingValues = v.getValues();
-        Object[] newValues = newRecord.getValues();
-        int aggNum = 0;
-        for (int i = _numKeyColumns; i < _numColumns; i++) {
-          existingValues[i] = _aggregationFunctions[aggNum++].merge(existingValues[i], newValues[i]);
-        }
-        return v;
-      }
-    });
+    _lookupMap.compute(key, (k, v) -> v == null ? newRecord : updateRecord(v, newRecord));
   }
 
   /**
    * Updates a record with existing key. Record with new key will be ignored.
    */
   protected void updateExistingRecord(Key key, Record newRecord) {
-    _lookupMap.computeIfPresent(key, (k, v) -> {
-      Object[] existingValues = v.getValues();
-      Object[] newValues = newRecord.getValues();
-      int aggNum = 0;
-      for (int i = _numKeyColumns; i < _numColumns; i++) {
-        existingValues[i] = _aggregationFunctions[aggNum++].merge(existingValues[i], newValues[i]);
+    _lookupMap.computeIfPresent(key, (k, v) -> updateRecord(v, newRecord));
+  }
+
+  private Record updateRecord(Record existingRecord, Record newRecord) {
+    Object[] existingValues = existingRecord.getValues();
+    Object[] newValues = newRecord.getValues();
+    int numAggregations = _aggregationFunctions.length;
+    int index = _numKeyColumns;
+    if (!_hasFinalInput) {
+      for (int i = 0; i < numAggregations; i++, index++) {
+        existingValues[index] = _aggregationFunctions[i].merge(existingValues[index], newValues[index]);
       }
-      return v;
-    });
+    } else {
+      for (int i = 0; i < numAggregations; i++, index++) {
+        existingValues[index] = _aggregationFunctions[i].mergeFinalResult((Comparable) existingValues[index],
+            (Comparable) newValues[index]);
+      }
+    }
+    return existingRecord;
   }
 
   /**
@@ -156,7 +157,8 @@ public abstract class IndexedTable extends BaseTable {
       _topRecords = _lookupMap.values();
     }
     // TODO: Directly return final result in _tableResizer.getTopRecords to avoid extracting final result multiple times
-    if (storeFinalResult) {
+    assert !(_hasFinalInput && !storeFinalResult);
+    if (storeFinalResult && !_hasFinalInput) {
       ColumnDataType[] columnDataTypes = _dataSchema.getColumnDataTypes();
       int numAggregationFunctions = _aggregationFunctions.length;
       for (int i = 0; i < numAggregationFunctions; i++) {
