@@ -70,7 +70,8 @@ public class SegmentPurger {
       throws Exception {
     SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
     String segmentName = segmentMetadata.getName();
-    LOGGER.info("Start purging table: {}, segment: {}", _tableConfig.getTableName(), segmentName);
+    String tableNameWithType = _tableConfig.getTableName();
+    LOGGER.info("Start purging table: {}, segment: {}", tableNameWithType, segmentName);
 
     try (PurgeRecordReader purgeRecordReader = new PurgeRecordReader()) {
       // Make a first pass through the data to see if records need to be purged or modified
@@ -107,8 +108,9 @@ public class SegmentPurger {
       driver.build();
     }
 
-    LOGGER.info("Finish purging table: {}, segment: {}, purged {} records, modified {} records",
-        _tableConfig.getTableName(), segmentName, _numRecordsPurged, _numRecordsModified);
+    LOGGER.info("Finish purging table: {}, segment: {}, purged {} records, modified {} records", tableNameWithType,
+        segmentName, _numRecordsPurged, _numRecordsModified);
+
     return new File(_workingDir, segmentName);
   }
 
