@@ -19,9 +19,12 @@
 package org.apache.pinot.core.operator;
 
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import javax.annotation.Nullable;
+import org.apache.pinot.common.request.context.ExpressionContext;
+import org.apache.pinot.common.utils.HashUtil;
 import org.apache.pinot.core.common.DataBlockCache;
 import org.apache.pinot.core.common.DataFetcher;
 import org.apache.pinot.core.common.Operator;
@@ -31,28 +34,33 @@ import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.spi.trace.Tracing;
 
 
-public class ProjectionOperator extends BaseOperator<ProjectionBlock> {
-
+public class ProjectionOperator extends BaseProjectOperator<ProjectionBlock> {
   private static final String EXPLAIN_NAME = "PROJECT";
 
   private final Map<String, DataSource> _dataSourceMap;
   private final BaseOperator<DocIdSetBlock> _docIdSetOperator;
   private final DataBlockCache _dataBlockCache;
+  private final Map<String, ColumnContext> _columnContextMap;
 
   public ProjectionOperator(Map<String, DataSource> dataSourceMap,
       @Nullable BaseOperator<DocIdSetBlock> docIdSetOperator) {
     _dataSourceMap = dataSourceMap;
     _docIdSetOperator = docIdSetOperator;
     _dataBlockCache = new DataBlockCache(new DataFetcher(dataSourceMap));
+    _columnContextMap = new HashMap<>(HashUtil.getHashMapCapacity(dataSourceMap.size()));
+    dataSourceMap.forEach(
+        (column, dataSource) -> _columnContextMap.put(column, ColumnContext.fromDataSource(dataSource)));
   }
 
-  /**
-   * Returns the map from column to data source.
-   *
-   * @return Map from column to data source
-   */
-  public Map<String, DataSource> getDataSourceMap() {
-    return _dataSourceMap;
+  @Override
+  public Map<String, ColumnContext> getSourceColumnContextMap() {
+    return _columnContextMap;
+  }
+
+  @Override
+  public ColumnContext getResultColumnContext(ExpressionContext expression) {
+    assert expression.getType() == ExpressionContext.Type.IDENTIFIER;
+    return _columnContextMap.get(expression.getIdentifier());
   }
 
   @Override
@@ -64,14 +72,13 @@ public class ProjectionOperator extends BaseOperator<ProjectionBlock> {
       return null;
     } else {
       Tracing.activeRecording().setNumChildren(_dataSourceMap.size());
-      _dataBlockCache.initNewBlock(docIdSetBlock.getDocIdSet(), docIdSetBlock.getSearchableLength());
+      _dataBlockCache.initNewBlock(docIdSetBlock.getDocIds(), docIdSetBlock.getLength());
       return new ProjectionBlock(_dataSourceMap, _dataBlockCache);
     }
   }
 
-
   @Override
-  public List<Operator> getChildOperators() {
+  public List<Operator<DocIdSetBlock>> getChildOperators() {
     return Collections.singletonList(_docIdSetOperator);
   }
 
