@@ -19,7 +19,7 @@
 package org.apache.pinot.core.operator.streaming;
 
 import java.util.List;
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.core.operator.InstanceResponseOperator;
 import org.apache.pinot.core.operator.blocks.InstanceResponseBlock;
@@ -30,21 +30,28 @@ import org.apache.pinot.core.operator.combine.BaseCombineOperator;
 import org.apache.pinot.core.query.executor.ResultsBlockStreamer;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.segment.spi.FetchContext;
-import org.apache.pinot.segment.spi.IndexSegment;
+import org.apache.pinot.segment.spi.SegmentContext;
 import org.apache.pinot.spi.exception.EarlyTerminationException;
 import org.apache.pinot.spi.exception.QueryCancelledException;
 import org.apache.pinot.spi.trace.Tracing;
 
 
+/**
+ * Like {@link InstanceResponseOperator}, but instead of sending all the data to the broker at once, it streams the data
+ * to a given {@link ResultsBlockStreamer}.
+ *
+ * This is used in multi-stage to stream data to the receiving mailboxes.
+ */
 public class StreamingInstanceResponseOperator extends InstanceResponseOperator {
   private static final String EXPLAIN_NAME = "STREAMING_INSTANCE_RESPONSE";
 
   private final BaseStreamingCombineOperator<?> _streamingCombineOperator;
   private final ResultsBlockStreamer _streamer;
 
-  public StreamingInstanceResponseOperator(BaseCombineOperator<?> combinedOperator, List<IndexSegment> indexSegments,
-      List<FetchContext> fetchContexts, ResultsBlockStreamer streamer, QueryContext queryContext) {
-    super(combinedOperator, indexSegments, fetchContexts, queryContext);
+  public StreamingInstanceResponseOperator(BaseCombineOperator<?> combinedOperator,
+      List<SegmentContext> segmentContexts, List<FetchContext> fetchContexts, ResultsBlockStreamer streamer,
+      QueryContext queryContext) {
+    super(combinedOperator, segmentContexts, fetchContexts, queryContext);
     _streamingCombineOperator =
         combinedOperator instanceof BaseStreamingCombineOperator ? (BaseStreamingCombineOperator<?>) combinedOperator
             : null;
@@ -57,7 +64,7 @@ public class StreamingInstanceResponseOperator extends InstanceResponseOperator
       prefetchAll();
       if (_streamingCombineOperator != null) {
         _streamingCombineOperator.start();
-        BaseResultsBlock resultsBlock = _streamingCombineOperator.nextBlock();
+        BaseResultsBlock resultsBlock = getBaseBlock();
         while (!(resultsBlock instanceof MetadataResultsBlock)) {
           if (resultsBlock instanceof ExceptionResultsBlock) {
             return new InstanceResponseBlock(resultsBlock);
@@ -65,20 +72,20 @@ public class StreamingInstanceResponseOperator extends InstanceResponseOperator
           if (resultsBlock.getNumRows() > 0) {
             _streamer.send(resultsBlock);
           }
-          resultsBlock = _streamingCombineOperator.nextBlock();
+          resultsBlock = getBaseBlock();
         }
         // Return a metadata-only block in the end
-        return new InstanceResponseBlock(resultsBlock);
+        return buildInstanceResponseBlock(resultsBlock);
       } else {
         // Handle single block combine operator in streaming fashion
-        BaseResultsBlock resultsBlock = _combineOperator.nextBlock();
+        BaseResultsBlock resultsBlock = getBaseBlock();
         if (resultsBlock instanceof ExceptionResultsBlock) {
           return new InstanceResponseBlock(resultsBlock);
         }
         if (resultsBlock.getNumRows() > 0) {
           _streamer.send(resultsBlock);
         }
-        return new InstanceResponseBlock(resultsBlock).toMetadataOnlyResponseBlock();
+        return buildInstanceResponseBlock(resultsBlock).toMetadataOnlyResponseBlock();
       }
     } catch (EarlyTerminationException e) {
       Exception killedErrorMsg = Tracing.getThreadAccountant().getErrorStatus();
@@ -95,6 +102,10 @@ public class StreamingInstanceResponseOperator extends InstanceResponseOperator
     }
   }
 
+  protected BaseResultsBlock getCombinedResults() {
+    return _combineOperator.nextBlock();
+  }
+
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
