@@ -33,7 +33,14 @@ import org.apache.pinot.core.query.aggregation.function.array.ArrayAggFloatFunct
 import org.apache.pinot.core.query.aggregation.function.array.ArrayAggIntFunction;
 import org.apache.pinot.core.query.aggregation.function.array.ArrayAggLongFunction;
 import org.apache.pinot.core.query.aggregation.function.array.ArrayAggStringFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ListAggDistinctFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ListAggFunction;
+import org.apache.pinot.core.query.aggregation.function.array.SumArrayDoubleAggregationFunction;
+import org.apache.pinot.core.query.aggregation.function.array.SumArrayLongAggregationFunction;
 import org.apache.pinot.core.query.aggregation.function.funnel.FunnelCountAggregationFunctionFactory;
+import org.apache.pinot.core.query.aggregation.function.funnel.window.FunnelCompleteCountAggregationFunction;
+import org.apache.pinot.core.query.aggregation.function.funnel.window.FunnelMatchStepAggregationFunction;
+import org.apache.pinot.core.query.aggregation.function.funnel.window.FunnelMaxStepAggregationFunction;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.exception.BadQueryRequestException;
@@ -49,7 +56,7 @@ public class AggregationFunctionFactory {
 
   /**
    * Given the function information, returns a new instance of the corresponding aggregation function.
-   * <p>NOTE: Underscores in the function name are ignored.
+   * <p>NOTE: Underscores in the function name are ignored in V1.
    */
   public static AggregationFunction getAggregationFunction(FunctionContext function, boolean nullHandlingEnabled) {
     try {
@@ -61,16 +68,16 @@ public class AggregationFunctionFactory {
       if (upperCaseFunctionName.startsWith("PERCENTILE")) {
         String remainingFunctionName = upperCaseFunctionName.substring(10);
         if (remainingFunctionName.equals("SMARTTDIGEST")) {
-          return new PercentileSmartTDigestAggregationFunction(arguments);
+          return new PercentileSmartTDigestAggregationFunction(arguments, nullHandlingEnabled);
         }
         if (remainingFunctionName.equals("KLL")) {
-          return new PercentileKLLAggregationFunction(arguments);
+          return new PercentileKLLAggregationFunction(arguments, nullHandlingEnabled);
         }
         if (remainingFunctionName.equals("KLLMV")) {
           return new PercentileKLLMVAggregationFunction(arguments);
         }
         if (remainingFunctionName.equals("RAWKLL")) {
-          return new PercentileRawKLLAggregationFunction(arguments);
+          return new PercentileRawKLLAggregationFunction(arguments, nullHandlingEnabled);
         }
         if (remainingFunctionName.equals("RAWKLLMV")) {
           return new PercentileRawKLLMVAggregationFunction(arguments);
@@ -80,23 +87,28 @@ public class AggregationFunctionFactory {
           // NOTE: This convention is deprecated. DO NOT add new functions here
           if (remainingFunctionName.matches("\\d+")) {
             // Percentile
-            return new PercentileAggregationFunction(firstArgument, parsePercentileToInt(remainingFunctionName));
+            return new PercentileAggregationFunction(firstArgument, parsePercentileToInt(remainingFunctionName),
+                nullHandlingEnabled);
           } else if (remainingFunctionName.matches("EST\\d+")) {
             // PercentileEst
             String percentileString = remainingFunctionName.substring(3);
-            return new PercentileEstAggregationFunction(firstArgument, parsePercentileToInt(percentileString));
+            return new PercentileEstAggregationFunction(firstArgument, parsePercentileToInt(percentileString),
+                nullHandlingEnabled);
           } else if (remainingFunctionName.matches("RAWEST\\d+")) {
             // PercentileRawEst
             String percentileString = remainingFunctionName.substring(6);
-            return new PercentileRawEstAggregationFunction(firstArgument, parsePercentileToInt(percentileString));
+            return new PercentileRawEstAggregationFunction(firstArgument, parsePercentileToInt(percentileString),
+                nullHandlingEnabled);
           } else if (remainingFunctionName.matches("TDIGEST\\d+")) {
             // PercentileTDigest
             String percentileString = remainingFunctionName.substring(7);
-            return new PercentileTDigestAggregationFunction(firstArgument, parsePercentileToInt(percentileString));
+            return new PercentileTDigestAggregationFunction(firstArgument, parsePercentileToInt(percentileString),
+                nullHandlingEnabled);
           } else if (remainingFunctionName.matches("RAWTDIGEST\\d+")) {
             // PercentileRawTDigest
             String percentileString = remainingFunctionName.substring(10);
-            return new PercentileRawTDigestAggregationFunction(firstArgument, parsePercentileToInt(percentileString));
+            return new PercentileRawTDigestAggregationFunction(firstArgument, parsePercentileToInt(percentileString),
+                nullHandlingEnabled);
           } else if (remainingFunctionName.matches("\\d+MV")) {
             // PercentileMV
             String percentileString = remainingFunctionName.substring(0, remainingFunctionName.length() - 2);
@@ -125,23 +137,23 @@ public class AggregationFunctionFactory {
           Preconditions.checkArgument(percentile >= 0 && percentile <= 100, "Invalid percentile: %s", percentile);
           if (remainingFunctionName.isEmpty()) {
             // Percentile
-            return new PercentileAggregationFunction(firstArgument, percentile);
+            return new PercentileAggregationFunction(firstArgument, percentile, nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("EST")) {
             // PercentileEst
-            return new PercentileEstAggregationFunction(firstArgument, percentile);
+            return new PercentileEstAggregationFunction(firstArgument, percentile, nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("RAWEST")) {
             // PercentileRawEst
-            return new PercentileRawEstAggregationFunction(firstArgument, percentile);
+            return new PercentileRawEstAggregationFunction(firstArgument, percentile, nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("TDIGEST")) {
             // PercentileTDigest
-            return new PercentileTDigestAggregationFunction(firstArgument, percentile);
+            return new PercentileTDigestAggregationFunction(firstArgument, percentile, nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("RAWTDIGEST")) {
             // PercentileRawTDigest
-            return new PercentileRawTDigestAggregationFunction(firstArgument, percentile);
+            return new PercentileRawTDigestAggregationFunction(firstArgument, percentile, nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("MV")) {
             // PercentileMV
@@ -175,11 +187,13 @@ public class AggregationFunctionFactory {
           Preconditions.checkArgument(compressionFactor >= 0, "Invalid compressionFactor: %d", compressionFactor);
           if (remainingFunctionName.equals("TDIGEST")) {
             // PercentileTDigest
-            return new PercentileTDigestAggregationFunction(firstArgument, percentile, compressionFactor);
+            return new PercentileTDigestAggregationFunction(firstArgument, percentile, compressionFactor,
+                nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("RAWTDIGEST")) {
             // PercentileRawTDigest
-            return new PercentileRawTDigestAggregationFunction(firstArgument, percentile, compressionFactor);
+            return new PercentileRawTDigestAggregationFunction(firstArgument, percentile, compressionFactor,
+                nullHandlingEnabled);
           }
           if (remainingFunctionName.equals("TDIGESTMV")) {
             // PercentileTDigestMV
@@ -208,7 +222,7 @@ public class AggregationFunctionFactory {
           case AVG:
             return new AvgAggregationFunction(arguments, nullHandlingEnabled);
           case MODE:
-            return new ModeAggregationFunction(arguments);
+            return new ModeAggregationFunction(arguments, nullHandlingEnabled);
           case FIRSTWITHTIME: {
             Preconditions.checkArgument(numArguments == 3,
                 "FIRST_WITH_TIME expects 3 arguments, got: %s. The function can be used as "
@@ -221,21 +235,46 @@ public class AggregationFunctionFactory {
             DataType dataType = DataType.valueOf(dataTypeExp.getLiteral().getStringValue().toUpperCase());
             switch (dataType) {
               case BOOLEAN:
-                return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol, true);
+                return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled,
+                    true);
               case INT:
-                return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol, false);
+                return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled,
+                    false);
               case LONG:
-                return new FirstLongValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new FirstLongValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               case FLOAT:
-                return new FirstFloatValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new FirstFloatValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               case DOUBLE:
-                return new FirstDoubleValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new FirstDoubleValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               case STRING:
-                return new FirstStringValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new FirstStringValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               default:
                 throw new IllegalArgumentException("Unsupported data type for FIRST_WITH_TIME: " + dataType);
             }
           }
+          case LISTAGG: {
+            Preconditions.checkArgument(numArguments == 2 || numArguments == 3,
+                "LISTAGG expects 2 arguments, got: %s. The function can be used as "
+                    + "listAgg([distinct] expression, 'separator')", numArguments);
+            ExpressionContext separatorExpression = arguments.get(1);
+            Preconditions.checkArgument(separatorExpression.getType() == ExpressionContext.Type.LITERAL,
+                "LISTAGG expects the 2nd argument to be literal, got: %s. The function can be used as "
+                    + "listAgg([distinct] expression, 'separator')", separatorExpression.getType());
+            String separator = separatorExpression.getLiteral().getStringValue();
+            boolean isDistinct = false;
+            if (numArguments == 3) {
+              ExpressionContext isDistinctListAggExp = arguments.get(2);
+              isDistinct = isDistinctListAggExp.getLiteral().getBooleanValue();
+            }
+            if (isDistinct) {
+              return new ListAggDistinctFunction(arguments.get(0), separator, nullHandlingEnabled);
+            }
+            return new ListAggFunction(arguments.get(0), separator, nullHandlingEnabled);
+          }
+          case SUMARRAYLONG:
+            return new SumArrayLongAggregationFunction(arguments);
+          case SUMARRAYDOUBLE:
+            return new SumArrayDoubleAggregationFunction(arguments);
           case ARRAYAGG: {
             Preconditions.checkArgument(numArguments >= 2,
                 "ARRAY_AGG expects 2 or 3 arguments, got: %s. The function can be used as "
@@ -300,23 +339,23 @@ public class AggregationFunctionFactory {
             DataType dataType = DataType.valueOf(dataTypeExp.getLiteral().getStringValue().toUpperCase());
             switch (dataType) {
               case BOOLEAN:
-                return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol, true);
+                return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled, true);
               case INT:
-                return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol, false);
+                return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled, false);
               case LONG:
-                return new LastLongValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new LastLongValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               case FLOAT:
-                return new LastFloatValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new LastFloatValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               case DOUBLE:
-                return new LastDoubleValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new LastDoubleValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               case STRING:
-                return new LastStringValueWithTimeAggregationFunction(firstArgument, timeCol);
+                return new LastStringValueWithTimeAggregationFunction(firstArgument, timeCol, nullHandlingEnabled);
               default:
                 throw new IllegalArgumentException("Unsupported data type for LAST_WITH_TIME: " + dataType);
             }
           }
           case MINMAXRANGE:
-            return new MinMaxRangeAggregationFunction(arguments);
+            return new MinMaxRangeAggregationFunction(arguments, nullHandlingEnabled);
           case DISTINCTCOUNT:
             return new DistinctCountAggregationFunction(arguments, nullHandlingEnabled);
           case DISTINCTCOUNTBITMAP:
@@ -408,13 +447,13 @@ public class AggregationFunctionFactory {
             return new SumValuesIntegerTupleSketchAggregationFunction(arguments, IntegerSummary.Mode.Sum);
           case AVGVALUEINTEGERSUMTUPLESKETCH:
             return new AvgValueIntegerTupleSketchAggregationFunction(arguments, IntegerSummary.Mode.Sum);
-          case PARENTEXPRMAX:
+          case PINOTPARENTAGGEXPRMAX:
             return new ParentExprMinMaxAggregationFunction(arguments, true);
-          case PARENTEXPRMIN:
+          case PINOTPARENTAGGEXPRMIN:
             return new ParentExprMinMaxAggregationFunction(arguments, false);
-          case CHILDEXPRMAX:
+          case PINOTCHILDAGGEXPRMAX:
             return new ChildExprMinMaxAggregationFunction(arguments, true);
-          case CHILDEXPRMIN:
+          case PINOTCHILDAGGEXPRMIN:
             return new ChildExprMinMaxAggregationFunction(arguments, false);
           case EXPRMAX:
           case EXPRMIN:
@@ -422,6 +461,12 @@ public class AggregationFunctionFactory {
                 "Aggregation function: " + functionType + " is only supported in selection without alias.");
           case FUNNELCOUNT:
             return new FunnelCountAggregationFunctionFactory(arguments).get();
+          case FUNNELMAXSTEP:
+            return new FunnelMaxStepAggregationFunction(arguments);
+          case FUNNELMATCHSTEP:
+            return new FunnelMatchStepAggregationFunction(arguments);
+          case FUNNELCOMPLETECOUNT:
+            return new FunnelCompleteCountAggregationFunction(arguments);
           case FREQUENTSTRINGSSKETCH:
             return new FrequentStringsSketchAggregationFunction(arguments);
           case FREQUENTLONGSSKETCH:
