@@ -18,6 +18,11 @@
  */
 package org.apache.pinot.core.query.aggregation.function;
 
+import it.unimi.dsi.fastutil.doubles.DoubleArrayList;
+import it.unimi.dsi.fastutil.floats.FloatArrayList;
+import it.unimi.dsi.fastutil.ints.IntArrayList;
+import it.unimi.dsi.fastutil.longs.LongArrayList;
+import it.unimi.dsi.fastutil.objects.ObjectArrayList;
 import java.sql.Timestamp;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -47,7 +52,7 @@ import org.apache.pinot.core.plan.ProjectPlanNode;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.startree.StarTreeUtils;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
-import org.apache.pinot.segment.spi.IndexSegment;
+import org.apache.pinot.segment.spi.SegmentContext;
 import org.apache.pinot.segment.spi.index.startree.AggregationFunctionColumnPair;
 
 
@@ -141,7 +146,7 @@ public class AggregationFunctionUtils {
    * TODO: Move ser/de into AggregationFunction interface
    */
   public static Object getIntermediateResult(DataTable dataTable, ColumnDataType columnDataType, int rowId, int colId) {
-    switch (columnDataType) {
+    switch (columnDataType.getStoredType()) {
       case INT:
         return dataTable.getInt(rowId, colId);
       case LONG:
@@ -156,9 +161,43 @@ public class AggregationFunctionUtils {
     }
   }
 
+  /**
+   * Reads the final result from the {@link DataTable}.
+   */
+  public static Comparable getFinalResult(DataTable dataTable, ColumnDataType columnDataType, int rowId, int colId) {
+    switch (columnDataType.getStoredType()) {
+      case INT:
+        return dataTable.getInt(rowId, colId);
+      case LONG:
+        return dataTable.getLong(rowId, colId);
+      case FLOAT:
+        return dataTable.getFloat(rowId, colId);
+      case DOUBLE:
+        return dataTable.getDouble(rowId, colId);
+      case BIG_DECIMAL:
+        return dataTable.getBigDecimal(rowId, colId);
+      case STRING:
+        return dataTable.getString(rowId, colId);
+      case BYTES:
+        return dataTable.getBytes(rowId, colId);
+      case INT_ARRAY:
+        return IntArrayList.wrap(dataTable.getIntArray(rowId, colId));
+      case LONG_ARRAY:
+        return LongArrayList.wrap(dataTable.getLongArray(rowId, colId));
+      case FLOAT_ARRAY:
+        return FloatArrayList.wrap(dataTable.getFloatArray(rowId, colId));
+      case DOUBLE_ARRAY:
+        return DoubleArrayList.wrap(dataTable.getDoubleArray(rowId, colId));
+      case STRING_ARRAY:
+        return ObjectArrayList.wrap(dataTable.getStringArray(rowId, colId));
+      default:
+        throw new IllegalStateException("Illegal column data type in final result: " + columnDataType);
+    }
+  }
+
   /**
    * Reads the converted final result from the {@link DataTable}. It should be equivalent to running
-   * {@link AggregationFunction#extractFinalResult(Object)} and {@link ColumnDataType#convert(Object)}.
+   * {@link #getFinalResult} and {@link ColumnDataType#convert}.
    */
   public static Object getConvertedFinalResult(DataTable dataTable, ColumnDataType columnDataType, int rowId,
       int colId) {
@@ -243,16 +282,15 @@ public class AggregationFunctionUtils {
   /**
    * Builds {@link AggregationInfo} for aggregations.
    */
-  public static AggregationInfo buildAggregationInfo(IndexSegment indexSegment, QueryContext queryContext,
+  public static AggregationInfo buildAggregationInfo(SegmentContext segmentContext, QueryContext queryContext,
       AggregationFunction[] aggregationFunctions, @Nullable FilterContext filter, BaseFilterOperator filterOperator,
       List<Pair<Predicate, PredicateEvaluator>> predicateEvaluators) {
     BaseProjectOperator<?> projectOperator = null;
 
     // TODO: Create a short-circuit ProjectOperator when filter result is empty
     if (!filterOperator.isResultEmpty()) {
-      projectOperator =
-          StarTreeUtils.createStarTreeBasedProjectOperator(indexSegment, queryContext, aggregationFunctions, filter,
-              predicateEvaluators);
+      projectOperator = StarTreeUtils.createStarTreeBasedProjectOperator(segmentContext.getIndexSegment(), queryContext,
+          aggregationFunctions, filter, predicateEvaluators);
     }
 
     if (projectOperator != null) {
@@ -262,7 +300,7 @@ public class AggregationFunctionUtils {
           AggregationFunctionUtils.collectExpressionsToTransform(aggregationFunctions,
               queryContext.getGroupByExpressions());
       projectOperator =
-          new ProjectPlanNode(indexSegment, queryContext, expressionsToTransform, DocIdSetPlanNode.MAX_DOC_PER_CALL,
+          new ProjectPlanNode(segmentContext, queryContext, expressionsToTransform, DocIdSetPlanNode.MAX_DOC_PER_CALL,
               filterOperator).run();
       return new AggregationInfo(aggregationFunctions, projectOperator, false);
     }
@@ -271,11 +309,11 @@ public class AggregationFunctionUtils {
   /**
    * Builds swim-lanes (list of {@link AggregationInfo}) for filtered aggregations.
    */
-  public static List<AggregationInfo> buildFilteredAggregationInfos(IndexSegment indexSegment,
+  public static List<AggregationInfo> buildFilteredAggregationInfos(SegmentContext segmentContext,
       QueryContext queryContext) {
     assert queryContext.getAggregationFunctions() != null && queryContext.getFilteredAggregationFunctions() != null;
 
-    FilterPlanNode mainFilterPlan = new FilterPlanNode(indexSegment, queryContext);
+    FilterPlanNode mainFilterPlan = new FilterPlanNode(segmentContext, queryContext);
     BaseFilterOperator mainFilterOperator = mainFilterPlan.run();
     List<Pair<Predicate, PredicateEvaluator>> mainPredicateEvaluators = mainFilterPlan.getPredicateEvaluators();
 
@@ -285,7 +323,7 @@ public class AggregationFunctionUtils {
       Set<ExpressionContext> expressions =
           collectExpressionsToTransform(aggregationFunctions, queryContext.getGroupByExpressions());
       BaseProjectOperator<?> projectOperator =
-          new ProjectPlanNode(indexSegment, queryContext, expressions, DocIdSetPlanNode.MAX_DOC_PER_CALL,
+          new ProjectPlanNode(segmentContext, queryContext, expressions, DocIdSetPlanNode.MAX_DOC_PER_CALL,
               mainFilterOperator).run();
       return Collections.singletonList(new AggregationInfo(aggregationFunctions, projectOperator, false));
     }
@@ -307,7 +345,7 @@ public class AggregationFunctionUtils {
             combinedFilter = FilterContext.forAnd(List.of(mainFilter, filter));
           }
 
-          FilterPlanNode subFilterPlan = new FilterPlanNode(indexSegment, queryContext, filter);
+          FilterPlanNode subFilterPlan = new FilterPlanNode(segmentContext, queryContext, filter);
           BaseFilterOperator subFilterOperator = subFilterPlan.run();
           BaseFilterOperator combinedFilterOperator;
           if (mainFilterOperator.isResultMatchingAll() || subFilterOperator.isResultEmpty()) {
@@ -342,7 +380,7 @@ public class AggregationFunctionUtils {
         AggregationFunction[] aggregationFunctions =
             filteredAggregationContext._aggregationFunctions.toArray(new AggregationFunction[0]);
         aggregationInfos.add(
-            buildAggregationInfo(indexSegment, queryContext, aggregationFunctions, filteredAggregationContext._filter,
+            buildAggregationInfo(segmentContext, queryContext, aggregationFunctions, filteredAggregationContext._filter,
                 filteredAggregationContext._filterOperator, filteredAggregationContext._predicateEvaluators));
       }
     }
@@ -350,7 +388,7 @@ public class AggregationFunctionUtils {
     if (!nonFilteredFunctions.isEmpty()) {
       AggregationFunction[] aggregationFunctions = nonFilteredFunctions.toArray(new AggregationFunction[0]);
       aggregationInfos.add(
-          buildAggregationInfo(indexSegment, queryContext, aggregationFunctions, mainFilter, mainFilterOperator,
+          buildAggregationInfo(segmentContext, queryContext, aggregationFunctions, mainFilter, mainFilterOperator,
               mainPredicateEvaluators));
     }
 
