@@ -204,6 +204,11 @@ public class CountAggregationFunction extends BaseSingleInputAggregationFunction
     return intermediateResult;
   }
 
+  @Override
+  public Long mergeFinalResult(Long finalResult1, Long finalResult2) {
+    return finalResult1 + finalResult2;
+  }
+
   @Override
   public String toExplainString() {
     StringBuilder stringBuilder = new StringBuilder(getType().getName()).append('(');
