@@ -329,6 +329,11 @@ public class DistinctCountBitmapAggregationFunction extends BaseSingleInputAggre
     return intermediateResult.getCardinality();
   }
 
+  @Override
+  public Integer mergeFinalResult(Integer finalResult1, Integer finalResult2) {
+    return finalResult1 + finalResult2;
+  }
+
   /**
    * Returns the dictionary id bitmap from the result holder or creates a new one if it does not exist.
    */
