@@ -363,6 +363,11 @@ public class DistinctCountHLLAggregationFunction extends BaseSingleInputAggregat
     return intermediateResult.cardinality();
   }
 
+  @Override
+  public Long mergeFinalResult(Long finalResult1, Long finalResult2) {
+    return finalResult1 + finalResult2;
+  }
+
   /**
    * Returns the dictionary id bitmap from the result holder or creates a new one if it does not exist.
    */
