@@ -71,4 +71,9 @@ public class DistinctSumAggregationFunction extends BaseDistinctAggregateAggrega
 
     return distinctSum;
   }
+
+  @Override
+  public Double mergeFinalResult(Double finalResult1, Double finalResult2) {
+    return finalResult1 + finalResult2;
+  }
 }
