@@ -37,17 +37,22 @@ public class PercentileRawTDigestAggregationFunction
     extends BaseSingleInputAggregationFunction<TDigest, SerializedTDigest> {
   private final PercentileTDigestAggregationFunction _percentileTDigestAggregationFunction;
 
-  public PercentileRawTDigestAggregationFunction(ExpressionContext expressionContext, int percentile) {
-    this(expressionContext, new PercentileTDigestAggregationFunction(expressionContext, percentile));
-  }
-
-  public PercentileRawTDigestAggregationFunction(ExpressionContext expressionContext, double percentile) {
-    this(expressionContext, new PercentileTDigestAggregationFunction(expressionContext, percentile));
+  public PercentileRawTDigestAggregationFunction(ExpressionContext expressionContext, int percentile,
+      boolean nullHandlingEnabled) {
+    this(expressionContext, new PercentileTDigestAggregationFunction(expressionContext, percentile,
+        nullHandlingEnabled));
   }
 
   public PercentileRawTDigestAggregationFunction(ExpressionContext expressionContext, double percentile,
-      int compressionFactor) {
-    this(expressionContext, new PercentileTDigestAggregationFunction(expressionContext, percentile, compressionFactor));
+      boolean nullHandlingEnabled) {
+    this(expressionContext, new PercentileTDigestAggregationFunction(expressionContext, percentile,
+        nullHandlingEnabled));
+  }
+
+  public PercentileRawTDigestAggregationFunction(ExpressionContext expressionContext, double percentile,
+      int compressionFactor, boolean nullHandlingEnabled) {
+    this(expressionContext, new PercentileTDigestAggregationFunction(expressionContext, percentile, compressionFactor,
+        nullHandlingEnabled));
   }
 
   protected PercentileRawTDigestAggregationFunction(ExpressionContext expression,
