@@ -25,10 +25,13 @@ import java.util.List;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
+import org.apache.pinot.core.common.BlockValSet;
 import org.apache.pinot.core.data.table.Record;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.query.distinct.DistinctExecutor;
 import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
+import org.roaringbitmap.RoaringBitmap;
 
 
 /**
@@ -41,6 +44,7 @@ abstract class BaseRawStringSingleColumnDistinctExecutor implements DistinctExec
   final boolean _nullHandlingEnabled;
 
   final ObjectSet<String> _valueSet;
+  private boolean _hasNull;
 
   BaseRawStringSingleColumnDistinctExecutor(ExpressionContext expression, DataType dataType, int limit,
       boolean nullHandlingEnabled) {
@@ -60,6 +64,47 @@ abstract class BaseRawStringSingleColumnDistinctExecutor implements DistinctExec
     for (String value : _valueSet) {
       records.add(new Record(new Object[]{value}));
     }
+    if (_hasNull) {
+      records.add(new Record(new Object[]{null}));
+    }
+    assert records.size() - (_hasNull ? 1 : 0) <= _limit;
     return new DistinctTable(dataSchema, records, _nullHandlingEnabled);
   }
+
+  @Override
+  public boolean process(ValueBlock valueBlock) {
+    BlockValSet blockValueSet = valueBlock.getBlockValueSet(_expression);
+    int numDocs = valueBlock.getNumDocs();
+    if (blockValueSet.isSingleValue()) {
+      String[] values = blockValueSet.getStringValuesSV();
+      if (_nullHandlingEnabled) {
+        RoaringBitmap nullBitmap = blockValueSet.getNullBitmap();
+        for (int i = 0; i < numDocs; i++) {
+          if (nullBitmap != null && nullBitmap.contains(i)) {
+            _hasNull = true;
+          } else if (add(values[i])) {
+            return true;
+          }
+        }
+      } else {
+        for (int i = 0; i < numDocs; i++) {
+          if (add(values[i])) {
+            return true;
+          }
+        }
+      }
+    } else {
+      String[][] values = blockValueSet.getStringValuesMV();
+      for (int i = 0; i < numDocs; i++) {
+        for (String value : values[i]) {
+          if (add(value)) {
+            return true;
+          }
+        }
+      }
+    }
+    return false;
+  }
+
+  protected abstract boolean add(String val);
 }
