@@ -22,7 +22,9 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.ExecutorService;
 import java.util.function.BiConsumer;
+import javax.annotation.Nullable;
 import org.apache.pinot.core.query.config.SegmentPrunerConfig;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.segment.spi.IndexSegment;
@@ -59,6 +61,7 @@ public class SegmentPrunerService {
             _prunerStatsUpdaters.put(pruner, SegmentPrunerStatistics::setLimitPruned);
             break;
           case SegmentPrunerProvider.COLUMN_VALUE_SEGMENT_PRUNER_NAME:
+          case SegmentPrunerProvider.BLOOM_FILTER_SEGMENT_PRUNER_NAME:
             _prunerStatsUpdaters.put(pruner, SegmentPrunerStatistics::setValuePruned);
             break;
           default:
@@ -96,6 +99,11 @@ public class SegmentPrunerService {
    *                 undefined way. Therefore, this list should not be used after calling this method.
    */
   public List<IndexSegment> prune(List<IndexSegment> segments, QueryContext query, SegmentPrunerStatistics stats) {
+    return prune(segments, query, stats, null);
+  }
+
+  public List<IndexSegment> prune(List<IndexSegment> segments, QueryContext query, SegmentPrunerStatistics stats,
+      @Nullable ExecutorService executorService) {
     try (InvocationScope scope = Tracing.getTracer().createScope(SegmentPrunerService.class)) {
       segments = removeInvalidSegments(segments, query, stats);
       int invokedPrunersCount = 0;
@@ -105,7 +113,7 @@ public class SegmentPrunerService {
           try (InvocationScope prunerScope = Tracing.getTracer().createScope(segmentPruner.getClass())) {
             int originalSegmentsSize = segments.size();
             prunerScope.setNumSegments(originalSegmentsSize);
-            segments = segmentPruner.prune(segments, query);
+            segments = segmentPruner.prune(segments, query, executorService);
             _prunerStatsUpdaters.get(segmentPruner).accept(stats, originalSegmentsSize - segments.size());
           }
         }
