@@ -19,32 +19,23 @@
 package org.apache.pinot.core.query.reduce;
 
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import java.util.function.LongConsumer;
-import java.util.stream.Collectors;
-import javax.annotation.Nullable;
 import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.metrics.BrokerMeter;
 import org.apache.pinot.common.metrics.BrokerMetrics;
 import org.apache.pinot.common.metrics.BrokerTimer;
 import org.apache.pinot.common.response.broker.BrokerResponseNative;
-import org.apache.pinot.common.response.broker.BrokerResponseStats;
 import org.apache.pinot.common.response.broker.QueryProcessingException;
 import org.apache.pinot.core.transport.ServerRoutingInstance;
 import org.apache.pinot.spi.config.table.TableType;
-import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 
 
 public class ExecutionStatsAggregator {
   private final List<QueryProcessingException> _processingExceptions = new ArrayList<>();
-  private final Map<String, Map<String, String>> _operatorStats = new HashMap<>();
-  private final Set<String> _tableNames = new HashSet<>();
   private final Map<String, String> _traceInfo = new HashMap<>();
   private final boolean _enableTrace;
 
@@ -74,67 +65,25 @@ public class ExecutionStatsAggregator {
   private long _explainPlanNumEmptyFilterSegments = 0L;
   private long _explainPlanNumMatchAllFilterSegments = 0L;
   private boolean _numGroupsLimitReached = false;
-  private boolean _maxRowsInJoinReached = false;
-  private int _numBlocks = 0;
-  private int _numRows = 0;
-  private long _stageExecutionTimeMs = 0;
-  private long _stageExecStartTimeMs = -1;
-  private long _stageExecEndTimeMs = -1;
-  private int _stageExecutionUnit = 0;
 
   public ExecutionStatsAggregator(boolean enableTrace) {
     _enableTrace = enableTrace;
   }
 
   public void aggregate(ServerRoutingInstance routingInstance, DataTable dataTable) {
-    aggregate(routingInstance, dataTable.getMetadata(), dataTable.getExceptions());
-  }
-
-  public synchronized void aggregate(@Nullable ServerRoutingInstance routingInstance, Map<String, String> metadata,
-      Map<Integer, String> exceptions) {
-
-    String tableNamesStr = metadata.get(DataTable.MetadataKey.TABLE.getName());
-    String tableName = null;
-
-    if (tableNamesStr != null) {
-      List<String> tableNames = Arrays.stream(tableNamesStr.split("::")).collect(Collectors.toList());
-      _tableNames.addAll(tableNames);
-
-      //TODO: Decide a strategy to split stageLevel stats across tables for brokerMetrics
-      // assigning everything to the first table only for now
-      tableName = tableNames.get(0);
-    }
-
-    TableType tableType = null;
-    String instanceName = null;
-    if (routingInstance != null) {
-      tableType = routingInstance.getTableType();
-      instanceName = routingInstance.getShortName();;
-    } else if (tableName != null) {
-      tableType = TableNameBuilder.getTableTypeFromTableName(tableName);
-      instanceName = tableName;
-    } else {
-      tableType = null;
-      instanceName = null;
-    }
+    TableType tableType = routingInstance.getTableType();
+    String instanceName = routingInstance.getShortName();
+    Map<String, String> metadata = dataTable.getMetadata();
 
     // Reduce on trace info.
-    if (_enableTrace && metadata.containsKey(DataTable.MetadataKey.TRACE_INFO.getName()) && instanceName != null) {
+    if (_enableTrace && metadata.containsKey(DataTable.MetadataKey.TRACE_INFO.getName())) {
       _traceInfo.put(instanceName, metadata.get(DataTable.MetadataKey.TRACE_INFO.getName()));
     }
 
-    String operatorId = metadata.get(DataTable.MetadataKey.OPERATOR_ID.getName());
-    if (operatorId != null) {
-      if (_enableTrace) {
-        _operatorStats.put(operatorId, metadata);
-      } else {
-        _operatorStats.put(operatorId, new HashMap<>());
-      }
-    }
-
     // Reduce on exceptions.
-    for (int key : exceptions.keySet()) {
-      _processingExceptions.add(new QueryProcessingException(key, exceptions.get(key)));
+    Map<Integer, String> exceptions = dataTable.getExceptions();
+    for (Map.Entry<Integer, String> entry : exceptions.entrySet()) {
+      _processingExceptions.add(new QueryProcessingException(entry.getKey(), entry.getValue()));
     }
 
     // Reduce on execution statistics.
@@ -248,51 +197,14 @@ public class ExecutionStatsAggregator {
     if (numTotalDocsString != null) {
       _numTotalDocs += Long.parseLong(numTotalDocsString);
     }
+
     _numGroupsLimitReached |=
         Boolean.parseBoolean(metadata.get(DataTable.MetadataKey.NUM_GROUPS_LIMIT_REACHED.getName()));
-
-    _maxRowsInJoinReached |=
-        Boolean.parseBoolean(metadata.get(DataTable.MetadataKey.MAX_ROWS_IN_JOIN_REACHED.getName()));
-
-    String numBlocksString = metadata.get(DataTable.MetadataKey.NUM_BLOCKS.getName());
-    if (numBlocksString != null) {
-      _numBlocks += Long.parseLong(numBlocksString);
-    }
-
-    String numRowsString = metadata.get(DataTable.MetadataKey.NUM_ROWS.getName());
-    if (numBlocksString != null) {
-      _numRows += Long.parseLong(numRowsString);
-    }
-
-    String operatorExecutionTimeString = metadata.get(DataTable.MetadataKey.OPERATOR_EXECUTION_TIME_MS.getName());
-    if (operatorExecutionTimeString != null) {
-      _stageExecutionTimeMs += Long.parseLong(operatorExecutionTimeString);
-      _stageExecutionUnit += 1;
-    }
-
-    String operatorExecStartTimeString = metadata.get(DataTable.MetadataKey.OPERATOR_EXEC_START_TIME_MS.getName());
-    if (operatorExecStartTimeString != null) {
-      long operatorExecStartTime = Long.parseLong(operatorExecStartTimeString);
-      _stageExecStartTimeMs = _stageExecStartTimeMs == -1 ? operatorExecStartTime
-          : Math.min(operatorExecStartTime, _stageExecStartTimeMs);
-    }
-
-    String operatorExecEndTimeString = metadata.get(DataTable.MetadataKey.OPERATOR_EXEC_END_TIME_MS.getName());
-    if (operatorExecEndTimeString != null) {
-      long operatorExecEndTime = Long.parseLong(operatorExecEndTimeString);
-      _stageExecEndTimeMs = _stageExecEndTimeMs == -1 ? operatorExecEndTime
-          : Math.max(operatorExecEndTime, _stageExecEndTimeMs);
-    }
   }
 
-  public void setStats(BrokerResponseNative brokerResponseNative) {
-    setStats(null, brokerResponseNative, null);
-  }
-
-  public void setStats(@Nullable String rawTableName, BrokerResponseNative brokerResponseNative,
-      @Nullable BrokerMetrics brokerMetrics) {
+  public void setStats(String rawTableName, BrokerResponseNative brokerResponseNative, BrokerMetrics brokerMetrics) {
     // set exception
-    List<QueryProcessingException> processingExceptions = brokerResponseNative.getProcessingExceptions();
+    List<QueryProcessingException> processingExceptions = brokerResponseNative.getExceptions();
     processingExceptions.addAll(_processingExceptions);
 
     // add all trace.
@@ -309,15 +221,12 @@ public class ExecutionStatsAggregator {
     brokerResponseNative.setNumSegmentsMatched(_numSegmentsMatched);
     brokerResponseNative.setTotalDocs(_numTotalDocs);
     brokerResponseNative.setNumGroupsLimitReached(_numGroupsLimitReached);
-    brokerResponseNative.setMaxRowsInJoinReached(_maxRowsInJoinReached);
     brokerResponseNative.setOfflineThreadCpuTimeNs(_offlineThreadCpuTimeNs);
     brokerResponseNative.setRealtimeThreadCpuTimeNs(_realtimeThreadCpuTimeNs);
     brokerResponseNative.setOfflineSystemActivitiesCpuTimeNs(_offlineSystemActivitiesCpuTimeNs);
     brokerResponseNative.setRealtimeSystemActivitiesCpuTimeNs(_realtimeSystemActivitiesCpuTimeNs);
     brokerResponseNative.setOfflineResponseSerializationCpuTimeNs(_offlineResponseSerializationCpuTimeNs);
     brokerResponseNative.setRealtimeResponseSerializationCpuTimeNs(_realtimeResponseSerializationCpuTimeNs);
-    brokerResponseNative.setOfflineTotalCpuTimeNs(_offlineTotalCpuTimeNs);
-    brokerResponseNative.setRealtimeTotalCpuTimeNs(_realtimeTotalCpuTimeNs);
     brokerResponseNative.setNumSegmentsPrunedByServer(_numSegmentsPrunedByServer);
     brokerResponseNative.setNumSegmentsPrunedInvalid(_numSegmentsPrunedInvalid);
     brokerResponseNative.setNumSegmentsPrunedByLimit(_numSegmentsPrunedByLimit);
@@ -334,52 +243,30 @@ public class ExecutionStatsAggregator {
     brokerResponseNative.setNumConsumingSegmentsMatched(_numConsumingSegmentsMatched);
 
     // Update broker metrics.
-    if (brokerMetrics != null && rawTableName != null) {
-      brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.DOCUMENTS_SCANNED, _numDocsScanned);
-      brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.ENTRIES_SCANNED_IN_FILTER,
-          _numEntriesScannedInFilter);
-      brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.ENTRIES_SCANNED_POST_FILTER,
-          _numEntriesScannedPostFilter);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_THREAD_CPU_TIME_NS, _offlineThreadCpuTimeNs,
-          TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_THREAD_CPU_TIME_NS, _realtimeThreadCpuTimeNs,
-          TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_SYSTEM_ACTIVITIES_CPU_TIME_NS,
-          _offlineSystemActivitiesCpuTimeNs, TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_SYSTEM_ACTIVITIES_CPU_TIME_NS,
-          _realtimeSystemActivitiesCpuTimeNs, TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_RESPONSE_SER_CPU_TIME_NS,
-          _offlineResponseSerializationCpuTimeNs, TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_RESPONSE_SER_CPU_TIME_NS,
-          _realtimeResponseSerializationCpuTimeNs, TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_TOTAL_CPU_TIME_NS, _offlineTotalCpuTimeNs,
-          TimeUnit.NANOSECONDS);
-      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_TOTAL_CPU_TIME_NS, _realtimeTotalCpuTimeNs,
-          TimeUnit.NANOSECONDS);
+    brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.DOCUMENTS_SCANNED, _numDocsScanned);
+    brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.ENTRIES_SCANNED_IN_FILTER, _numEntriesScannedInFilter);
+    brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.ENTRIES_SCANNED_POST_FILTER,
+        _numEntriesScannedPostFilter);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_THREAD_CPU_TIME_NS, _offlineThreadCpuTimeNs,
+        TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_THREAD_CPU_TIME_NS, _realtimeThreadCpuTimeNs,
+        TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_SYSTEM_ACTIVITIES_CPU_TIME_NS,
+        _offlineSystemActivitiesCpuTimeNs, TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_SYSTEM_ACTIVITIES_CPU_TIME_NS,
+        _realtimeSystemActivitiesCpuTimeNs, TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_RESPONSE_SER_CPU_TIME_NS,
+        _offlineResponseSerializationCpuTimeNs, TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_RESPONSE_SER_CPU_TIME_NS,
+        _realtimeResponseSerializationCpuTimeNs, TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.OFFLINE_TOTAL_CPU_TIME_NS, _offlineTotalCpuTimeNs,
+        TimeUnit.NANOSECONDS);
+    brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.REALTIME_TOTAL_CPU_TIME_NS, _realtimeTotalCpuTimeNs,
+        TimeUnit.NANOSECONDS);
 
-      if (_minConsumingFreshnessTimeMs != Long.MAX_VALUE) {
-        brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.FRESHNESS_LAG_MS,
-            System.currentTimeMillis() - _minConsumingFreshnessTimeMs, TimeUnit.MILLISECONDS);
-      }
-    }
-  }
-
-  public void setStageLevelStats(@Nullable String rawTableName, BrokerResponseStats brokerResponseStats,
-      @Nullable BrokerMetrics brokerMetrics) {
-    if (_enableTrace) {
-      setStats(rawTableName, brokerResponseStats, brokerMetrics);
-      brokerResponseStats.setOperatorStats(_operatorStats);
-    }
-
-    brokerResponseStats.setNumBlocks(_numBlocks);
-    brokerResponseStats.setNumRows(_numRows);
-    brokerResponseStats.setMaxRowsInJoinReached(_maxRowsInJoinReached);
-    brokerResponseStats.setNumGroupsLimitReached(_numGroupsLimitReached);
-    brokerResponseStats.setStageExecutionTimeMs(_stageExecutionTimeMs);
-    brokerResponseStats.setStageExecutionUnit(_stageExecutionUnit);
-    brokerResponseStats.setTableNames(new ArrayList<>(_tableNames));
-    if (_stageExecStartTimeMs >= 0 && _stageExecEndTimeMs >= 0) {
-      brokerResponseStats.setStageExecWallTimeMs(_stageExecEndTimeMs - _stageExecStartTimeMs);
+    if (_minConsumingFreshnessTimeMs != Long.MAX_VALUE) {
+      brokerMetrics.addTimedTableValue(rawTableName, BrokerTimer.FRESHNESS_LAG_MS,
+          System.currentTimeMillis() - _minConsumingFreshnessTimeMs, TimeUnit.MILLISECONDS);
     }
   }
 
