@@ -20,7 +20,6 @@ package org.apache.pinot.core.query.selection;
 
 import java.util.Collection;
 import java.util.LinkedList;
-import java.util.List;
 import java.util.PriorityQueue;
 import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.response.broker.ResultTable;
@@ -58,22 +57,16 @@ import org.roaringbitmap.RoaringBitmap;
  */
 public class SelectionOperatorService {
   private final QueryContext _queryContext;
-  private final List<String> _selectionColumns;
   private final DataSchema _dataSchema;
+  private final int[] _columnIndices;
   private final int _offset;
   private final int _numRowsToKeep;
   private final PriorityQueue<Object[]> _rows;
 
-  /**
-   * Constructor for <code>SelectionOperatorService</code> with {@link DataSchema}. (Broker side)
-   *
-   * @param queryContext Selection order-by query
-   * @param dataSchema data schema.
-   */
-  public SelectionOperatorService(QueryContext queryContext, DataSchema dataSchema) {
+  public SelectionOperatorService(QueryContext queryContext, DataSchema dataSchema, int[] columnIndices) {
     _queryContext = queryContext;
-    _selectionColumns = SelectionOperatorUtils.getSelectionColumns(queryContext, dataSchema);
     _dataSchema = dataSchema;
+    _columnIndices = columnIndices;
     // Select rows from offset to offset + limit.
     _offset = queryContext.getOffset();
     _numRowsToKeep = _offset + queryContext.getLimit();
@@ -83,25 +76,15 @@ public class SelectionOperatorService {
             _queryContext.isNullHandlingEnabled()).reversed());
   }
 
-  /**
-   * Get the selection results.
-   *
-   * @return selection results.
-   */
-  public PriorityQueue<Object[]> getRows() {
-    return _rows;
-  }
-
   /**
    * Reduces a collection of {@link DataTable}s to selection rows for selection queries with <code>ORDER BY</code>.
-   * (Broker side)
    * TODO: Do merge sort after releasing 0.13.0 when server side results are sorted
    *       Can also consider adding a data table metadata to indicate whether the server side results are sorted
    */
-  public void reduceWithOrdering(Collection<DataTable> dataTables, boolean nullHandlingEnabled) {
+  public void reduceWithOrdering(Collection<DataTable> dataTables) {
     for (DataTable dataTable : dataTables) {
       int numRows = dataTable.getNumberOfRows();
-      if (nullHandlingEnabled) {
+      if (_queryContext.isNullHandlingEnabled()) {
         RoaringBitmap[] nullBitmaps = new RoaringBitmap[dataTable.getDataSchema().size()];
         for (int colId = 0; colId < nullBitmaps.length; colId++) {
           nullBitmaps[colId] = dataTable.getNullRowIds(colId);
@@ -127,32 +110,24 @@ public class SelectionOperatorService {
   }
 
   /**
-   * Render the selection rows to a {@link ResultTable} object for selection queries with <code>ORDER BY</code>.
-   * (Broker side)
-   * <p>{@link ResultTable} object will be used to build the broker response.
-   * <p>Should be called after method "reduceWithOrdering()".
+   * Renders the selection rows to a {@link ResultTable} object for selection queries with <code>ORDER BY</code>.
    */
   public ResultTable renderResultTableWithOrdering() {
-    int[] columnIndices = SelectionOperatorUtils.getColumnIndices(_selectionColumns, _dataSchema);
-    int numColumns = columnIndices.length;
-    DataSchema resultDataSchema = SelectionOperatorUtils.getSchemaForProjection(_dataSchema, columnIndices);
-
-    // Extract the result rows
-    LinkedList<Object[]> rowsInSelectionResults = new LinkedList<>();
+    LinkedList<Object[]> resultRows = new LinkedList<>();
+    DataSchema.ColumnDataType[] columnDataTypes = _dataSchema.getColumnDataTypes();
+    int numColumns = columnDataTypes.length;
     while (_rows.size() > _offset) {
       Object[] row = _rows.poll();
       assert row != null;
-      Object[] extractedRow = new Object[numColumns];
+      Object[] resultRow = new Object[numColumns];
       for (int i = 0; i < numColumns; i++) {
-        Object value = row[columnIndices[i]];
+        Object value = row[_columnIndices[i]];
         if (value != null) {
-          extractedRow[i] = resultDataSchema.getColumnDataType(i).convertAndFormat(value);
+          resultRow[i] = columnDataTypes[i].convertAndFormat(value);
         }
       }
-
-      rowsInSelectionResults.addFirst(extractedRow);
+      resultRows.addFirst(resultRow);
     }
-
-    return new ResultTable(resultDataSchema, rowsInSelectionResults);
+    return new ResultTable(_dataSchema, resultRows);
   }
 }
