@@ -25,7 +25,7 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeoutException;
 import javax.annotation.Nullable;
 import javax.annotation.concurrent.ThreadSafe;
-import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.collections4.CollectionUtils;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pinot.common.config.NettyConfig;
 import org.apache.pinot.common.config.TlsConfig;
@@ -35,6 +35,7 @@ import org.apache.pinot.common.metrics.BrokerMeter;
 import org.apache.pinot.common.metrics.BrokerMetrics;
 import org.apache.pinot.common.request.BrokerRequest;
 import org.apache.pinot.common.request.InstanceRequest;
+import org.apache.pinot.common.utils.config.QueryOptionsUtils;
 import org.apache.pinot.core.transport.server.routing.stats.ServerRoutingStatsManager;
 import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.utils.CommonConstants;
@@ -96,6 +97,9 @@ public class QueryRouter {
     // can prefer but not require TLS until all servers guaranteed to be on TLS
     boolean preferTls = _serverChannelsTls != null;
 
+    // skip unavailable servers if the query option is set
+    boolean skipUnavailableServers = isSkipUnavailableServers(offlineBrokerRequest, realtimeBrokerRequest);
+
     // Build map from server to request based on the routing table
     Map<ServerRoutingInstance, InstanceRequest> requestMap = new HashMap<>();
     if (offlineBrokerRequest != null) {
@@ -126,10 +130,6 @@ public class QueryRouter {
       ServerRoutingInstance serverRoutingInstance = entry.getKey();
       ServerChannels serverChannels = serverRoutingInstance.isTlsEnabled() ? _serverChannelsTls : _serverChannels;
       try {
-        // Record stats related to query submission just before sending the request. Otherwise, if the response is
-        // received immediately, there's a possibility of updating query response stats before updating query
-        // submission stats.
-        _serverRoutingStatsManager.recordStatsAfterQuerySubmission(requestId, serverRoutingInstance.getInstanceId());
         serverChannels.sendRequest(rawTableName, asyncQueryResponse, serverRoutingInstance, entry.getValue(),
             timeoutMs);
         asyncQueryResponse.markRequestSubmitted(serverRoutingInstance);
@@ -141,14 +141,28 @@ public class QueryRouter {
         break;
       } catch (Exception e) {
         _brokerMetrics.addMeteredTableValue(rawTableName, BrokerMeter.REQUEST_SEND_EXCEPTIONS, 1);
-        markQueryFailed(requestId, serverRoutingInstance, asyncQueryResponse, e);
-        break;
+        if (skipUnavailableServers) {
+          asyncQueryResponse.skipServerResponse();
+        } else {
+          markQueryFailed(requestId, serverRoutingInstance, asyncQueryResponse, e);
+          break;
+        }
       }
     }
 
     return asyncQueryResponse;
   }
 
+  private boolean isSkipUnavailableServers(@Nullable BrokerRequest offlineBrokerRequest,
+      @Nullable BrokerRequest realtimeBrokerRequest) {
+    if (offlineBrokerRequest != null && QueryOptionsUtils.isSkipUnavailableServers(
+        offlineBrokerRequest.getPinotQuery().getQueryOptions())) {
+      return true;
+    }
+    return realtimeBrokerRequest != null && QueryOptionsUtils.isSkipUnavailableServers(
+        realtimeBrokerRequest.getPinotQuery().getQueryOptions());
+  }
+
   private void markQueryFailed(long requestId, ServerRoutingInstance serverRoutingInstance,
       AsyncQueryResponse asyncQueryResponse, Exception e) {
     LOGGER.error("Caught exception while sending request {} to server: {}, marking query failed", requestId,
