@@ -29,6 +29,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Random;
 import java.util.Set;
+import javax.annotation.Nullable;
 import org.apache.commons.io.FileUtils;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.core.common.BlockDocIdIterator;
@@ -47,6 +48,7 @@ import org.apache.pinot.segment.local.startree.v2.builder.MultipleTreesBuilder;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.segment.spi.Constants;
 import org.apache.pinot.segment.spi.IndexSegment;
+import org.apache.pinot.segment.spi.SegmentContext;
 import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
 import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
@@ -91,32 +93,45 @@ abstract class BaseStarTreeV2Test<R, A> {
 
   private static final int NUM_SEGMENT_RECORDS = 100_000;
   private static final int MAX_LEAF_RECORDS = RANDOM.nextInt(100) + 1;
-  private static final String DIMENSION_D1 = "d1";
-  private static final String DIMENSION_D2 = "d2";
+  // Using column names with '__' to make sure regular table columns with '__' in the name aren't wrongly interpreted
+  // as AggregationFunctionColumnPair
+  private static final String DIMENSION1 = "d1__COLUMN_NAME";
+  private static final String DIMENSION2 = "DISTINCTCOUNTRAWHLL__d2";
   private static final int DIMENSION_CARDINALITY = 100;
   private static final String METRIC = "m";
 
   // Supported filters
-  private static final String QUERY_FILTER_AND = " WHERE d1 = 0 AND d2 < 10";
+  private static final String QUERY_FILTER_AND = String.format(" WHERE %1$s = 0 AND %2$s < 10", DIMENSION1, DIMENSION2);
   // StarTree supports OR predicates only on a single dimension
-  private static final String QUERY_FILTER_OR = " WHERE d1 > 10 OR d1 < 50";
-  private static final String QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS = " WHERE d2 < 95 AND (d1 > 10 OR d1 < 50)";
+  private static final String QUERY_FILTER_OR = String.format(" WHERE %1$s > 10 OR %1$s < 50", DIMENSION1);
+  private static final String QUERY_FILTER_NOT = String.format(" WHERE NOT %s > 10", DIMENSION1);
+  private static final String QUERY_FILTER_AND_NOT =
+      String.format(" WHERE %1$s > 10 AND NOT %2$s < 10", DIMENSION1, DIMENSION2);
+  private static final String QUERY_FILTER_OR_NOT = String.format(" WHERE %1$s > 50 OR NOT %1$s > 10", DIMENSION1);
+  private static final String QUERY_NOT_NOT = String.format(" WHERE NOT NOT %s > 10", DIMENSION1);
+  private static final String QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS =
+      String.format(" WHERE %2$s < 95 AND (NOT %1$s > 10 OR %1$s > 50)", DIMENSION1, DIMENSION2);
   private static final String QUERY_FILTER_COMPLEX_AND_MULTIPLE_DIMENSIONS_THREE_PREDICATES =
-      " WHERE d2 < 95 AND d2 > 25 AND (d1 > 10 OR d1 < 50)";
+      String.format(" WHERE %2$s < 95 AND NOT %2$s < 25 AND (%1$s > 10 OR %1$s < 50)", DIMENSION1, DIMENSION2);
   private static final String QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS_THREE_PREDICATES =
-      " WHERE (d2 > 95 OR d2 < 25) AND (d1 > 10 OR d1 < 50)";
-  private static final String QUERY_FILTER_COMPLEX_OR_SINGLE_DIMENSION = " WHERE d1 = 95 AND (d1 > 90 OR d1 < 100)";
+      String.format(" WHERE (%2$s > 95 OR %2$s < 25) AND (%1$s > 10 OR %1$s < 50)", DIMENSION1, DIMENSION2);
+  private static final String QUERY_FILTER_COMPLEX_OR_SINGLE_DIMENSION =
+      String.format(" WHERE NOT %1$s = 95 AND (%1$s > 90 OR %1$s < 100)", DIMENSION1);
 
   // Unsupported filters
-  private static final String QUERY_FILTER_OR_MULTIPLE_DIMENSIONS = " WHERE d1 > 10 OR d2 < 50";
-  private static final String QUERY_FILTER_OR_ON_AND = " WHERE (d1 > 10 AND d1 < 50) OR d1 < 50";
-  private static final String QUERY_FILTER_OR_ON_NOT = " WHERE (NOT d1 > 10) OR d1 < 50";
+  private static final String QUERY_FILTER_OR_MULTIPLE_DIMENSIONS =
+      String.format(" WHERE %1$s > 10 OR %2$s < 50", DIMENSION1, DIMENSION2);
+  private static final String QUERY_FILTER_OR_ON_AND =
+      String.format(" WHERE (%1$s > 10 AND %1$s < 50) OR %1$s < 50", DIMENSION1);
+  private static final String QUERY_FILTER_NOT_ON_AND =
+      String.format(" WHERE NOT (%1$s > 10 AND %1$s < 50)", DIMENSION1);
+  private static final String QUERY_FILTER_NOT_ON_OR = String.format(" WHERE NOT (%1$s < 10 OR %1$s > 50)", DIMENSION1);
   // Always false filters
-  private static final String QUERY_FILTER_ALWAYS_FALSE = " WHERE d1 > 100";
-  private static final String QUERY_FILTER_OR_ALWAYS_FALSE = " WHERE d1 > 100 OR d1 < 0";
+  private static final String QUERY_FILTER_ALWAYS_FALSE = String.format(" WHERE %s > 100", DIMENSION1);
+  private static final String QUERY_FILTER_OR_ALWAYS_FALSE = String.format(" WHERE %1$s > 100 OR %1$s < 0", DIMENSION1);
 
-  private static final String QUERY_GROUP_BY = " GROUP BY d2";
-  private static final String FILTER_AGG_CLAUSE = " FILTER(WHERE d1 > 10)";
+  private static final String QUERY_GROUP_BY = " GROUP BY " + DIMENSION2;
+  private static final String FILTER_AGG_CLAUSE = String.format(" FILTER(WHERE %s > 10)", DIMENSION1);
 
   private ValueAggregator _valueAggregator;
   private DataType _aggregatedValueType;
@@ -131,8 +146,8 @@ abstract class BaseStarTreeV2Test<R, A> {
     _aggregatedValueType = _valueAggregator.getAggregatedValueType();
     _aggregation = getAggregation(_valueAggregator.getAggregationType());
 
-    Schema.SchemaBuilder schemaBuilder = new Schema.SchemaBuilder().addSingleValueDimension(DIMENSION_D1, DataType.INT)
-        .addSingleValueDimension(DIMENSION_D2, DataType.INT);
+    Schema.SchemaBuilder schemaBuilder = new Schema.SchemaBuilder().addSingleValueDimension(DIMENSION1, DataType.INT)
+        .addSingleValueDimension(DIMENSION2, DataType.INT);
     DataType rawValueType = getRawValueType();
     // Raw value type will be null for COUNT aggregation function
     if (rawValueType != null) {
@@ -144,8 +159,8 @@ abstract class BaseStarTreeV2Test<R, A> {
     List<GenericRow> segmentRecords = new ArrayList<>(NUM_SEGMENT_RECORDS);
     for (int i = 0; i < NUM_SEGMENT_RECORDS; i++) {
       GenericRow segmentRecord = new GenericRow();
-      segmentRecord.putValue(DIMENSION_D1, RANDOM.nextInt(DIMENSION_CARDINALITY));
-      segmentRecord.putValue(DIMENSION_D2, RANDOM.nextInt(DIMENSION_CARDINALITY));
+      segmentRecord.putValue(DIMENSION1, RANDOM.nextInt(DIMENSION_CARDINALITY));
+      segmentRecord.putValue(DIMENSION2, RANDOM.nextInt(DIMENSION_CARDINALITY));
       if (rawValueType != null) {
         segmentRecord.putValue(METRIC, getRandomRawValue(RANDOM));
       }
@@ -159,10 +174,9 @@ abstract class BaseStarTreeV2Test<R, A> {
     driver.init(segmentGeneratorConfig, new GenericRowRecordReader(segmentRecords));
     driver.build();
 
-    StarTreeIndexConfig starTreeIndexConfig =
-        new StarTreeIndexConfig(Arrays.asList(DIMENSION_D1, DIMENSION_D2), null, null, Collections.singletonList(
-            new StarTreeAggregationConfig(METRIC, _valueAggregator.getAggregationType().getName(),
-                getCompressionCodec())), MAX_LEAF_RECORDS);
+    StarTreeIndexConfig starTreeIndexConfig = new StarTreeIndexConfig(Arrays.asList(DIMENSION1, DIMENSION2), null, null,
+        Collections.singletonList(new StarTreeAggregationConfig(METRIC, _valueAggregator.getAggregationType().getName(),
+            getCompressionCodec(), true, getIndexVersion(), null, null)), MAX_LEAF_RECORDS);
     File indexDir = new File(TEMP_DIR, SEGMENT_NAME);
     // Randomly build star-tree using on-heap or off-heap mode
     MultipleTreesBuilder.BuildMode buildMode =
@@ -193,7 +207,8 @@ abstract class BaseStarTreeV2Test<R, A> {
     String query = String.format("SELECT %s FROM %s", _aggregation, TABLE_NAME);
     testUnsupportedFilter(query + QUERY_FILTER_OR_MULTIPLE_DIMENSIONS);
     testUnsupportedFilter(query + QUERY_FILTER_OR_ON_AND);
-    testUnsupportedFilter(query + QUERY_FILTER_OR_ON_NOT);
+    testUnsupportedFilter(query + QUERY_FILTER_NOT_ON_AND);
+    testUnsupportedFilter(query + QUERY_FILTER_NOT_ON_OR);
     testUnsupportedFilter(query + QUERY_FILTER_ALWAYS_FALSE);
     testUnsupportedFilter(query + QUERY_FILTER_OR_ALWAYS_FALSE);
   }
@@ -207,6 +222,10 @@ abstract class BaseStarTreeV2Test<R, A> {
       testQuery(query);
       testQuery(query + QUERY_FILTER_AND);
       testQuery(query + QUERY_FILTER_OR);
+      testQuery(query + QUERY_FILTER_NOT);
+      testQuery(query + QUERY_FILTER_AND_NOT);
+      testQuery(query + QUERY_FILTER_OR_NOT);
+      testQuery(query + QUERY_NOT_NOT);
       testQuery(query + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS);
       testQuery(query + QUERY_FILTER_COMPLEX_AND_MULTIPLE_DIMENSIONS_THREE_PREDICATES);
       testQuery(query + QUERY_FILTER_COMPLEX_OR_MULTIPLE_DIMENSIONS_THREE_PREDICATES);
@@ -225,7 +244,7 @@ abstract class BaseStarTreeV2Test<R, A> {
 
   private void testUnsupportedFilter(String query) {
     QueryContext queryContext = QueryContextConverterUtils.getQueryContext(query);
-    FilterPlanNode filterPlanNode = new FilterPlanNode(_indexSegment, queryContext);
+    FilterPlanNode filterPlanNode = new FilterPlanNode(new SegmentContext(_indexSegment), queryContext);
     filterPlanNode.run();
     Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap =
         StarTreeUtils.extractPredicateEvaluatorsMap(_indexSegment, queryContext.getFilter(),
@@ -257,7 +276,7 @@ abstract class BaseStarTreeV2Test<R, A> {
     List<String> groupByColumns = new ArrayList<>(groupByColumnSet);
 
     // Filter
-    FilterPlanNode filterPlanNode = new FilterPlanNode(_indexSegment, queryContext);
+    FilterPlanNode filterPlanNode = new FilterPlanNode(new SegmentContext(_indexSegment), queryContext);
     filterPlanNode.run();
     Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap =
         StarTreeUtils.extractPredicateEvaluatorsMap(_indexSegment, queryContext.getFilter(),
@@ -280,7 +299,7 @@ abstract class BaseStarTreeV2Test<R, A> {
         computeStarTreeResult(starTreeFilterPlanNode, starTreeAggregationColumnReaders, starTreeGroupByColumnReaders);
 
     // Extract values without star-tree
-    FilterPlanNode nonStarTreeFilterPlanNode = new FilterPlanNode(_indexSegment, queryContext);
+    FilterPlanNode nonStarTreeFilterPlanNode = new FilterPlanNode(new SegmentContext(_indexSegment), queryContext);
     List<ForwardIndexReader> nonStarTreeAggregationColumnReaders = new ArrayList<>(numAggregations);
     List<Dictionary> nonStarTreeAggregationColumnDictionaries = new ArrayList<>(numAggregations);
     for (AggregationFunctionColumnPair aggregationFunctionColumnPair : aggregationFunctionColumnPairs) {
@@ -463,14 +482,21 @@ abstract class BaseStarTreeV2Test<R, A> {
   /**
    * Can be overridden to force the compression codec.
    */
+  @Nullable
   CompressionCodec getCompressionCodec() {
     CompressionCodec[] compressionCodecs = CompressionCodec.values();
-    while (true) {
-      CompressionCodec compressionCodec = compressionCodecs[RANDOM.nextInt(compressionCodecs.length)];
-      if (compressionCodec.isApplicableToRawIndex()) {
-        return compressionCodec;
-      }
-    }
+    CompressionCodec compressionCodec = compressionCodecs[RANDOM.nextInt(compressionCodecs.length)];
+    return compressionCodec.isApplicableToRawIndex() ? compressionCodec : null;
+  }
+
+  /**
+   * Can be overridden to force the index version.
+   */
+  @Nullable
+  Integer getIndexVersion() {
+    // Allow 2, 3, 4 or null
+    int version = 1 + RANDOM.nextInt(4);
+    return version > 1 ? version : null;
   }
 
   abstract ValueAggregator<R, A> getValueAggregator();
