@@ -29,8 +29,8 @@ import java.util.List;
 import org.apache.commons.io.FileUtils;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.core.common.Operator;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
-import org.apache.pinot.core.operator.query.AggregationGroupByOrderByOperator;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
+import org.apache.pinot.core.operator.query.GroupByOperator;
 import org.apache.pinot.core.query.aggregation.groupby.AggregationGroupByResult;
 import org.apache.pinot.core.query.aggregation.groupby.GroupKeyGenerator;
 import org.apache.pinot.segment.local.indexsegment.immutable.ImmutableSegmentLoader;
@@ -167,9 +167,9 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   @Test
   public void testSimpleSelectOnJsonColumn() {
     try {
-      Operator operator = getOperator("select jsonColumn FROM testTable limit 100");
-      IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-      Collection<Object[]> rows = block.getSelectionResult();
+      Operator<SelectionResultsBlock> operator = getOperator("select jsonColumn FROM testTable limit 100");
+      SelectionResultsBlock block = operator.nextBlock();
+      Collection<Object[]> rows = block.getRows();
       Assert.assertEquals(rows.size(), 13);
       Assert.assertEquals(block.getDataSchema().getColumnDataType(0), DataSchema.ColumnDataType.JSON);
     } catch (IllegalStateException ise) {
@@ -180,11 +180,11 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   /** Test filtering on string value associated with  JSON key*/
   @Test
   public void testExtractScalarWithStringFilter() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select intColumn, json_extract_scalar(jsonColumn, '$.name.last', 'STRING') FROM testTable WHERE "
             + "json_extract_scalar(jsonColumn, '$.name.first', 'STRING') = 'daffy'");
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 1);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -197,11 +197,11 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   /** Test filtering on number value associated with  JSON key*/
   @Test
   public void testExtractScalarWithNumericIntFilter() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.last', 'STRING') FROM testTable WHERE json_extract_scalar"
             + "(jsonColumn, '$.id', 'INT') = 171");
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 1);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -214,11 +214,11 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   public void testExtractScalarWithNumericFloatFilter() {
 
     // query to retrieve result as INT
-    Operator operator1 = getOperator(
+    Operator<SelectionResultsBlock> operator1 = getOperator(
         "select json_extract_scalar(jsonColumn, '$.id', 'INT') FROM testTable WHERE json_extract_scalar(jsonColumn, "
             + "'$.id', 'FLOAT') = 161.5");
-    IntermediateResultsBlock block1 = (IntermediateResultsBlock) operator1.nextBlock();
-    Collection<Object[]> rows1 = block1.getSelectionResult();
+    SelectionResultsBlock block1 = operator1.nextBlock();
+    Collection<Object[]> rows1 = block1.getRows();
     Assert.assertEquals(rows1.size(), 1);
 
     Iterator<Object[]> iterator1 = rows1.iterator();
@@ -226,11 +226,11 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
     Assert.assertEquals(iterator1.next()[0], 161);
 
     // query to retrieve result as DOUBLE
-    Operator operator2 = getOperator(
+    Operator<SelectionResultsBlock> operator2 = getOperator(
         "select json_extract_scalar(jsonColumn, '$.id', 'DOUBLE') FROM testTable WHERE json_extract_scalar"
             + "(jsonColumn, '$.id', 'FLOAT') = 161.5");
-    IntermediateResultsBlock block2 = (IntermediateResultsBlock) operator2.nextBlock();
-    Collection<Object[]> rows2 = block2.getSelectionResult();
+    SelectionResultsBlock block2 = operator2.nextBlock();
+    Collection<Object[]> rows2 = block2.getRows();
     Assert.assertEquals(rows2.size(), 1);
 
     Iterator<Object[]> iterator2 = rows2.iterator();
@@ -241,11 +241,11 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   /** Retrieve JSON array after filtering on string value associated with  JSON key*/
   @Test
   public void testExtractScalarArrayWithStringFilter() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.data', 'STRING') FROM testTable WHERE json_extract_scalar"
             + "(jsonColumn, '$.name.first', 'STRING') = 'daffy'");
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 1);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -256,11 +256,11 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   /** Test filtering on string value within a JSON array*/
   @Test
   public void testExtractScalarWithArrayFilter() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING') FROM testTable WHERE json_extract_scalar"
             + "(jsonColumn, '$.data[0]', 'STRING') IN ('i', 'k')");
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 2);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -272,10 +272,9 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
   @Test
   public void testJsonMatchWithoutIndex() {
     try {
-      Operator operator = getOperator(
-          "select json_extract_scalar(stringColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match"
-              + "(stringColumn, '\"$.id\"=101')");
-      Assert.assertTrue(false);
+      getOperator("select json_extract_scalar(stringColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match"
+          + "(stringColumn, '\"$.id\"=101')");
+      Assert.fail();
     } catch (IllegalStateException ise) {
       Assert.assertTrue(true);
     }
@@ -283,12 +282,12 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
 
   @Test
   public void testJsonMatchAtLevel1WithIndex() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match(jsonColumn,"
             + " '\"$.id\"=101')");
 
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 1);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -298,12 +297,12 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
 
   @Test
   public void testJsonMatchAtLevel2WithIndex() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match(jsonColumn,"
             + " '\"$.name.first\" = ''daffy''')");
 
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 1);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -313,12 +312,12 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
 
   @Test
   public void testJsonMatchArrayWithIndex() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match(jsonColumn,"
             + " '\"$.data[0]\" IN (''k'', ''j'')')");
 
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 2);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -330,12 +329,12 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
 
   @Test
   public void testJsonMatchNestedArrayWithIndex() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match(jsonColumn,"
             + " '\"$.data[0].e[1].y[0].i1\" = 1')");
 
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 1);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -345,12 +344,12 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
 
   @Test
   public void testJsonMatchMultidimensionalArrayWithIndex() {
-    Operator operator = getOperator(
+    Operator<SelectionResultsBlock> operator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING') FROM testTable WHERE json_match(jsonColumn,"
             + " '\"$.data[0][1][0]\" = ''3''')");
 
-    IntermediateResultsBlock block = (IntermediateResultsBlock) operator.nextBlock();
-    Collection<Object[]> rows = block.getSelectionResult();
+    SelectionResultsBlock block = operator.nextBlock();
+    Collection<Object[]> rows = block.getRows();
     Assert.assertEquals(rows.size(), 3);
 
     Iterator<Object[]> iterator = rows.iterator();
@@ -360,7 +359,7 @@ public class JsonDataTypeQueriesTest extends BaseQueriesTest {
 
   @Test
   public void testJsonMatchMultidimensionalArrayGroupByWithIndex() {
-    AggregationGroupByOrderByOperator groupByOperator = getOperator(
+    GroupByOperator groupByOperator = getOperator(
         "select json_extract_scalar(jsonColumn, '$.name.first', 'STRING'), count(*) FROM testTable WHERE json_match"
             + "(jsonColumn, '\"$.data[0][1][0]\" = ''3''') GROUP BY json_extract_scalar(jsonColumn, '$.name.first', "
             + "'STRING')");
