@@ -28,6 +28,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import org.apache.commons.io.FileUtils;
 import org.apache.helix.model.IdealState;
 import org.apache.pinot.client.ResultSetGroup;
 import org.apache.pinot.common.exception.HttpErrorStatusException;
@@ -89,7 +90,7 @@ public class PartialUpsertTableRebalanceIntegrationTest extends BaseClusterInteg
     // Start Kafka and push data into Kafka
     startKafka();
 
-    _resourceManager = getControllerStarter().getHelixResourceManager();
+    _resourceManager = _controllerStarter.getHelixResourceManager();
     _tableRebalancer = new TableRebalancer(_resourceManager.getHelixZkManager());
 
     createSchemaAndTable();
@@ -109,7 +110,7 @@ public class PartialUpsertTableRebalanceIntegrationTest extends BaseClusterInteg
     rebalanceConfig.setIncludeConsuming(true);
 
     // Add a new server
-    BaseServerStarter serverStarter1 = startOneServer(1234);
+    BaseServerStarter serverStarter1 = startOneServer(NUM_SERVERS);
 
     // Now we trigger a rebalance operation
     TableConfig tableConfig = _resourceManager.getTableConfig(REALTIME_TABLE_NAME);
@@ -127,7 +128,7 @@ public class PartialUpsertTableRebalanceIntegrationTest extends BaseClusterInteg
     verifySegmentAssignment(rebalanceResult.getSegmentAssignment(), 5, finalReplicas);
 
     // Add a new server
-    BaseServerStarter serverStarter2 = startOneServer(4567);
+    BaseServerStarter serverStarter2 = startOneServer(NUM_SERVERS + 1);
     rebalanceResult = _tableRebalancer.rebalance(tableConfig, rebalanceConfig, null);
 
     // Check the number of replicas after rebalancing
@@ -299,12 +300,14 @@ public class PartialUpsertTableRebalanceIntegrationTest extends BaseClusterInteg
   }
 
   @AfterClass
-  public void tearDown() {
+  public void tearDown()
+      throws IOException {
     stopServer();
     stopBroker();
     stopController();
     stopKafka();
     stopZk();
+    FileUtils.deleteDirectory(_tempDir);
   }
 
   @Override
