@@ -52,8 +52,8 @@ import org.apache.pinot.client.PinotDriver;
 import org.apache.pinot.client.ResultSetGroup;
 import org.apache.pinot.common.helix.ExtraInstanceConfig;
 import org.apache.pinot.common.utils.SimpleHttpResponse;
-import org.apache.pinot.common.utils.TlsUtils;
 import org.apache.pinot.common.utils.helix.HelixHelper;
+import org.apache.pinot.common.utils.tls.TlsUtils;
 import org.apache.pinot.controller.ControllerConf;
 import org.apache.pinot.core.common.MinionConstants;
 import org.apache.pinot.integration.tests.access.CertBasedTlsChannelAccessControlFactory;
@@ -74,26 +74,22 @@ import org.testng.annotations.Test;
 
 import static org.apache.pinot.integration.tests.BasicAuthTestUtils.AUTH_HEADER;
 import static org.apache.pinot.integration.tests.BasicAuthTestUtils.AUTH_TOKEN;
-import static org.apache.pinot.spi.utils.CommonConstants.Helix.DEFAULT_SERVER_NETTY_PORT;
-import static org.apache.pinot.spi.utils.CommonConstants.Server.DEFAULT_ADMIN_API_PORT;
 
 
 public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   private static final String PASSWORD = "changeit";
   private static final char[] PASSWORD_CHAR = PASSWORD.toCharArray();
   private static final Header CLIENT_HEADER = new BasicHeader("Authorization", AUTH_TOKEN);
-
   private static final String PKCS_12 = "PKCS12";
   private static final String JKS = "JKS";
-
-  private final URL _tlsStoreEmptyPKCS12 = TlsIntegrationTest.class.getResource("/empty.p12");
-  private final URL _tlsStoreEmptyJKS = TlsIntegrationTest.class.getResource("/empty.jks");
-  private final URL _tlsStorePKCS12 = TlsIntegrationTest.class.getResource("/tlstest.p12");
-  private final URL _tlsStoreJKS = TlsIntegrationTest.class.getResource("/tlstest.jks");
+  private static final URL TLS_STORE_EMPTY_PKCS_12 = TlsIntegrationTest.class.getResource("/empty.p12");
+  private static final URL TLS_STORE_EMPTY_JKS = TlsIntegrationTest.class.getResource("/empty.jks");
+  private static final URL TLS_STORE_PKCS_12 = TlsIntegrationTest.class.getResource("/tlstest.p12");
+  private static final URL TLS_STORE_JKS = TlsIntegrationTest.class.getResource("/tlstest.jks");
 
   private int _internalControllerPort;
-  private int _internalBrokerPort;
   private int _externalControllerPort;
+  private int _internalBrokerPort;
   private int _externalBrokerPort;
 
   @BeforeClass
@@ -101,19 +97,12 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
       throws Exception {
     TestUtils.ensureDirectoriesExistAndEmpty(_tempDir);
 
-    _internalControllerPort = NetUtils.findOpenPort(DEFAULT_CONTROLLER_PORT + RANDOM.nextInt(10000));
-    _externalControllerPort = NetUtils.findOpenPort(_internalControllerPort + RANDOM.nextInt(10000));
-
-    _internalBrokerPort = NetUtils.findOpenPort(DEFAULT_BROKER_PORT + RANDOM.nextInt(10000));
-    _externalBrokerPort = NetUtils.findOpenPort(_internalBrokerPort + RANDOM.nextInt(10000));
-    // Start Zookeeper
     startZk();
-    // Start Pinot cluster
-    startKafka();
     startController();
-    startBrokerHttps();
-    startServerHttps();
+    startBroker();
+    startServer();
     startMinion();
+    startKafka();
 
     // Unpack the Avro files
     List<File> avroFiles = unpackAvroData(_tempDir);
@@ -142,108 +131,111 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   }
 
   @Override
-  public Map<String, Object> getDefaultControllerConfiguration() {
-    Map<String, Object> prop = super.getDefaultControllerConfiguration();
+  protected void overrideControllerConf(Map<String, Object> prop) {
+    BasicAuthTestUtils.addControllerConfiguration(prop);
 
     // NOTE: defaults must be suitable for cluster-internal communication
-    prop.put("controller.tls.keystore.path", _tlsStorePKCS12);
+    prop.put("controller.tls.keystore.path", TLS_STORE_PKCS_12);
     prop.put("controller.tls.keystore.password", PASSWORD);
     prop.put("controller.tls.keystore.type", PKCS_12);
-    prop.put("controller.tls.truststore.path", _tlsStorePKCS12);
+    prop.put("controller.tls.truststore.path", TLS_STORE_PKCS_12);
     prop.put("controller.tls.truststore.password", PASSWORD);
     prop.put("controller.tls.truststore.type", PKCS_12);
 
     // CAUTION: order matters. first listener becomes registered as internal address in zookeeper
     prop.put("controller.access.protocols", "internal,external");
     prop.put("controller.access.protocols.internal.protocol", "https");
+    _internalControllerPort = _controllerPort;
     prop.put("controller.access.protocols.internal.port", _internalControllerPort);
     prop.put("controller.access.protocols.internal.tls.client.auth.enabled", "true");
     prop.put("controller.access.protocols.external.protocol", "https");
+    _externalControllerPort = NetUtils.findOpenPort(_internalControllerPort + 1);
     prop.put("controller.access.protocols.external.port", _externalControllerPort);
-    prop.put("controller.access.protocols.external.tls.keystore.path", _tlsStoreJKS);
+    _nextControllerPort = _externalControllerPort + 1;
+    prop.put("controller.access.protocols.external.tls.keystore.path", TLS_STORE_JKS);
     prop.put("controller.access.protocols.external.tls.keystore.type", JKS);
-    prop.put("controller.access.protocols.external.tls.truststore.path", _tlsStoreJKS);
+    prop.put("controller.access.protocols.external.tls.truststore.path", TLS_STORE_JKS);
     prop.put("controller.access.protocols.external.tls.truststore.type", JKS);
 
-    prop.put("controller.broker.protocol", "https");
-
     // announce internal only
-    prop.put("controller.vip.protocol", "https");
-    prop.put("controller.vip.port", _internalControllerPort);
-
-    prop.remove("controller.port");
-
-    return BasicAuthTestUtils.addControllerConfiguration(prop);
+    prop.remove(ControllerConf.CONTROLLER_PORT);
+    prop.put(ControllerConf.CONTROLLER_VIP_PROTOCOL, "https");
+    prop.put(ControllerConf.CONTROLLER_VIP_PORT, _internalControllerPort);
+    prop.put(ControllerConf.CONTROLLER_BROKER_PROTOCOL, "https");
   }
 
   @Override
-  protected PinotConfiguration getDefaultBrokerConfiguration() {
-    Map<String, Object> prop = super.getDefaultBrokerConfiguration().toMap();
+  protected void overrideBrokerConf(PinotConfiguration brokerConf) {
+    BasicAuthTestUtils.addBrokerConfiguration(brokerConf);
 
     // NOTE: defaults must be suitable for cluster-internal communication
-    prop.put("pinot.broker.tls.keystore.path", _tlsStorePKCS12);
-    prop.put("pinot.broker.tls.keystore.password", PASSWORD);
-    prop.put("pinot.broker.tls.keystore.type", PKCS_12);
-    prop.put("pinot.broker.tls.truststore.path", _tlsStorePKCS12);
-    prop.put("pinot.broker.tls.truststore.password", PASSWORD);
-    prop.put("pinot.broker.tls.truststore.type", PKCS_12);
+    brokerConf.setProperty("pinot.broker.tls.keystore.path", TLS_STORE_PKCS_12);
+    brokerConf.setProperty("pinot.broker.tls.keystore.password", PASSWORD);
+    brokerConf.setProperty("pinot.broker.tls.keystore.type", PKCS_12);
+    brokerConf.setProperty("pinot.broker.tls.truststore.path", TLS_STORE_PKCS_12);
+    brokerConf.setProperty("pinot.broker.tls.truststore.password", PASSWORD);
+    brokerConf.setProperty("pinot.broker.tls.truststore.type", PKCS_12);
 
     // CAUTION: order matters. first listener becomes registered as internal address in zookeeper
-    prop.put("pinot.broker.client.access.protocols", "internal,external");
-    prop.put("pinot.broker.client.access.protocols.internal.protocol", "https");
-    prop.put("pinot.broker.client.access.protocols.internal.port", _internalBrokerPort);
-    prop.put("pinot.broker.client.access.protocols.internal.tls.client.auth.enabled", "true");
-    prop.put("pinot.broker.client.access.protocols.external.protocol", "https");
-    prop.put("pinot.broker.client.access.protocols.external.port", _externalBrokerPort);
-    prop.put("pinot.broker.client.access.protocols.external.tls.keystore.path", _tlsStoreJKS);
-    prop.put("pinot.broker.client.access.protocols.external.tls.keystore.type", JKS);
-    prop.put("pinot.broker.client.access.protocols.external.tls.truststore.path", _tlsStoreJKS);
-    prop.put("pinot.broker.client.access.protocols.external.tls.truststore.type", JKS);
+    brokerConf.setProperty("pinot.broker.client.access.protocols", "internal,external");
+    brokerConf.setProperty("pinot.broker.client.access.protocols.internal.protocol", "https");
+    _internalBrokerPort = NetUtils.findOpenPort(_nextBrokerPort);
+    brokerConf.setProperty("pinot.broker.client.access.protocols.internal.port", _internalBrokerPort);
+    _brokerPorts.add(_internalBrokerPort);
+    brokerConf.setProperty("pinot.broker.client.access.protocols.internal.tls.client.auth.enabled", "true");
+    brokerConf.setProperty("pinot.broker.client.access.protocols.external.protocol", "https");
+    _externalBrokerPort = NetUtils.findOpenPort(_internalBrokerPort + 1);
+    brokerConf.setProperty("pinot.broker.client.access.protocols.external.port", _externalBrokerPort);
+    _brokerPorts.add(_externalBrokerPort);
+    _brokerBaseApiUrl = "https://localhost:" + _externalBrokerPort;
+    _nextBrokerPort = _externalBrokerPort + 1;
+    brokerConf.setProperty("pinot.broker.client.access.protocols.external.tls.keystore.path", TLS_STORE_JKS);
+    brokerConf.setProperty("pinot.broker.client.access.protocols.external.tls.keystore.type", JKS);
+    brokerConf.setProperty("pinot.broker.client.access.protocols.external.tls.truststore.path", TLS_STORE_JKS);
+    brokerConf.setProperty("pinot.broker.client.access.protocols.external.tls.truststore.type", JKS);
 
-    prop.put("pinot.broker.nettytls.enabled", "true");
-
-    return BasicAuthTestUtils.addBrokerConfiguration(prop);
+    brokerConf.clearProperty(CommonConstants.Helix.KEY_OF_BROKER_QUERY_PORT);
+    brokerConf.setProperty("pinot.broker.nettytls.enabled", "true");
   }
 
   @Override
-  protected PinotConfiguration getDefaultServerConfiguration() {
-    Map<String, Object> prop = super.getDefaultServerConfiguration().toMap();
+  protected void overrideServerConf(PinotConfiguration serverConf) {
+    BasicAuthTestUtils.addServerConfiguration(serverConf);
 
     // NOTE: defaults must be suitable for cluster-internal communication
-    prop.put("pinot.server.tls.keystore.path", _tlsStorePKCS12);
-    prop.put("pinot.server.tls.keystore.password", PASSWORD);
-    prop.put("pinot.server.tls.keystore.type", PKCS_12);
-    prop.put("pinot.server.tls.truststore.path", _tlsStorePKCS12);
-    prop.put("pinot.server.tls.truststore.password", PASSWORD);
-    prop.put("pinot.server.tls.truststore.type", PKCS_12);
-    prop.put("pinot.server.tls.client.auth.enabled", "true");
+    serverConf.setProperty("pinot.server.tls.keystore.path", TLS_STORE_PKCS_12);
+    serverConf.setProperty("pinot.server.tls.keystore.password", PASSWORD);
+    serverConf.setProperty("pinot.server.tls.keystore.type", PKCS_12);
+    serverConf.setProperty("pinot.server.tls.truststore.path", TLS_STORE_PKCS_12);
+    serverConf.setProperty("pinot.server.tls.truststore.password", PASSWORD);
+    serverConf.setProperty("pinot.server.tls.truststore.type", PKCS_12);
+    serverConf.setProperty("pinot.server.tls.client.auth.enabled", "true");
 
-    prop.put("pinot.server.admin.access.control.factory.class",
+    serverConf.setProperty("pinot.server.admin.access.control.factory.class",
         CertBasedTlsChannelAccessControlFactory.class.getName());
-    prop.put("pinot.server.adminapi.access.protocols", "internal");
-    prop.put("pinot.server.adminapi.access.protocols.internal.protocol", "https");
-    prop.put("pinot.server.adminapi.access.protocols.internal.port",
-        NetUtils.findOpenPort(DEFAULT_ADMIN_API_PORT + RANDOM.nextInt(10000)));
-    prop.put("pinot.server.netty.enabled", "false");
-    prop.put("pinot.server.nettytls.enabled", "true");
-    prop.put("pinot.server.nettytls.port", NetUtils.findOpenPort(DEFAULT_SERVER_NETTY_PORT + RANDOM.nextInt(10000)));
-    prop.put("pinot.server.segment.uploader.protocol", "https");
-
-    return BasicAuthTestUtils.addServerConfiguration(prop);
+    serverConf.setProperty("pinot.server.adminapi.access.protocols", "internal");
+    serverConf.setProperty("pinot.server.adminapi.access.protocols.internal.protocol", "https");
+    int internalAdminPort = NetUtils.findOpenPort(_nextServerPort);
+    serverConf.setProperty("pinot.server.adminapi.access.protocols.internal.port", internalAdminPort);
+    serverConf.setProperty("pinot.server.netty.enabled", "false");
+    serverConf.setProperty("pinot.server.nettytls.enabled", "true");
+    int nettyTlsPort = NetUtils.findOpenPort(internalAdminPort + 1);
+    serverConf.setProperty("pinot.server.nettytls.port", nettyTlsPort);
+    _nextServerPort = nettyTlsPort + 1;
+    serverConf.setProperty("pinot.server.segment.uploader.protocol", "https");
   }
 
   @Override
-  protected PinotConfiguration getDefaultMinionConfiguration() {
-    Map<String, Object> prop = super.getDefaultMinionConfiguration().toMap();
-    prop.put("pinot.minion.tls.keystore.path", _tlsStorePKCS12);
-    prop.put("pinot.minion.tls.keystore.password", "changeit");
-    prop.put("pinot.server.tls.keystore.type", "PKCS12");
-    prop.put("pinot.minion.tls.truststore.path", _tlsStorePKCS12);
-    prop.put("pinot.minion.tls.truststore.password", "changeit");
-    prop.put("pinot.minion.tls.truststore.type", "PKCS12");
-    prop.put("pinot.minion.tls.client.auth.enabled", "true");
+  protected void overrideMinionConf(PinotConfiguration minionConf) {
+    BasicAuthTestUtils.addMinionConfiguration(minionConf);
 
-    return BasicAuthTestUtils.addMinionConfiguration(prop);
+    minionConf.setProperty("pinot.minion.tls.keystore.path", TLS_STORE_PKCS_12);
+    minionConf.setProperty("pinot.minion.tls.keystore.password", "changeit");
+    minionConf.setProperty("pinot.server.tls.keystore.type", "PKCS12");
+    minionConf.setProperty("pinot.minion.tls.truststore.path", TLS_STORE_PKCS_12);
+    minionConf.setProperty("pinot.minion.tls.truststore.password", "changeit");
+    minionConf.setProperty("pinot.minion.tls.truststore.type", "PKCS12");
+    minionConf.setProperty("pinot.minion.tls.client.auth.enabled", "true");
   }
 
   @Override
@@ -293,7 +285,6 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
 
   @Test
   public void testUpdatedBrokerTlsPort() {
-
     List<InstanceConfig> instanceConfigs = HelixHelper.getInstanceConfigs(_helixManager);
     List<ExtraInstanceConfig> securedInstances = instanceConfigs.stream().map(ExtraInstanceConfig::new)
         .filter(pinotInstanceConfig -> pinotInstanceConfig.getTlsPort() != null).collect(Collectors.toList());
@@ -303,49 +294,45 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testControllerConfigValidation()
       throws Exception {
-    PinotConfigUtils.validateControllerConfig(new ControllerConf(getDefaultControllerConfiguration()));
+    PinotConfigUtils.validateControllerConfig(getControllerConfig());
   }
 
   @Test
   public void testControllerConfigValidationImplicitProtocol()
       throws Exception {
-    Map<String, Object> prop = new HashMap<>(getDefaultControllerConfiguration());
+    Map<String, Object> prop = getControllerConfig().toMap();
     prop.put("controller.access.protocols", "https,http");
     prop.put("controller.access.protocols.https.port", _internalControllerPort);
     prop.put("controller.access.protocols.http.port", _externalControllerPort);
-
     PinotConfigUtils.validateControllerConfig(new ControllerConf(prop));
   }
 
   @Test(expectedExceptions = ConfigurationException.class)
   public void testControllerConfigValidationNoProtocol()
       throws Exception {
-    Map<String, Object> prop = new HashMap<>(getDefaultControllerConfiguration());
+    Map<String, Object> prop = getControllerConfig().toMap();
     prop.put("controller.access.protocols", "invalid,http");
     prop.put("controller.access.protocols.invalid.port", _internalControllerPort);
     prop.put("controller.access.protocols.http.port", _externalControllerPort);
-
     PinotConfigUtils.validateControllerConfig(new ControllerConf(prop));
   }
 
   @Test
   public void testControllerExternalTrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(JKS, _tlsStoreJKS, _tlsStoreJKS)) {
-
-      try (CloseableHttpResponse response = client.execute(makeGetTables(_externalControllerPort))) {
-        Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
-        JsonNode tables = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("tables");
-        Assert.assertEquals(tables.size(), 1);
-        Assert.assertEquals(tables.get(0).textValue(), "mytable");
-      }
+    try (CloseableHttpClient client = makeClient(JKS, TLS_STORE_JKS, TLS_STORE_JKS);
+        CloseableHttpResponse response = client.execute(makeGetTables(_externalControllerPort))) {
+      Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
+      JsonNode tables = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("tables");
+      Assert.assertEquals(tables.size(), 1);
+      Assert.assertEquals(tables.get(0).textValue(), "mytable");
     }
   }
 
   @Test
   public void testControllerExternalUntrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(JKS, _tlsStoreJKS, _tlsStoreEmptyJKS)) {
+    try (CloseableHttpClient client = makeClient(JKS, TLS_STORE_JKS, TLS_STORE_EMPTY_JKS)) {
       try {
         client.execute(makeGetTables(_externalControllerPort));
         Assert.fail("Must not allow connection to untrusted server");
@@ -358,20 +345,19 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testControllerInternalTrustedClient()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(PKCS_12, _tlsStorePKCS12, _tlsStorePKCS12)) {
-      try (CloseableHttpResponse response = client.execute(makeGetTables(_internalControllerPort))) {
-        Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
-        JsonNode tables = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("tables");
-        Assert.assertEquals(tables.size(), 1);
-        Assert.assertEquals(tables.get(0).textValue(), "mytable");
-      }
+    try (CloseableHttpClient client = makeClient(PKCS_12, TLS_STORE_PKCS_12, TLS_STORE_PKCS_12);
+        CloseableHttpResponse response = client.execute(makeGetTables(_internalControllerPort))) {
+      Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
+      JsonNode tables = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("tables");
+      Assert.assertEquals(tables.size(), 1);
+      Assert.assertEquals(tables.get(0).textValue(), "mytable");
     }
   }
 
   @Test
   public void testControllerInternalUntrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(PKCS_12, _tlsStorePKCS12, _tlsStoreEmptyPKCS12)) {
+    try (CloseableHttpClient client = makeClient(PKCS_12, TLS_STORE_PKCS_12, TLS_STORE_EMPTY_PKCS_12)) {
       try {
         client.execute(makeGetTables(_internalControllerPort));
         Assert.fail("Must not allow connection to untrusted server");
@@ -384,7 +370,7 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testControllerInternalUntrustedClient()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(PKCS_12, _tlsStoreEmptyPKCS12, _tlsStorePKCS12)) {
+    try (CloseableHttpClient client = makeClient(PKCS_12, TLS_STORE_EMPTY_PKCS_12, TLS_STORE_PKCS_12)) {
       try {
         client.execute(makeGetTables(_internalControllerPort));
         Assert.fail("Must not allow connection from untrusted client");
@@ -397,19 +383,18 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testBrokerExternalTrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(JKS, _tlsStoreEmptyJKS, _tlsStoreJKS)) {
-      try (CloseableHttpResponse response = client.execute(makeQueryBroker(_externalBrokerPort))) {
-        Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
-        JsonNode resultTable = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("resultTable");
-        Assert.assertTrue(resultTable.get("rows").get(0).get(0).longValue() > 100000);
-      }
+    try (CloseableHttpClient client = makeClient(JKS, TLS_STORE_EMPTY_JKS, TLS_STORE_JKS);
+        CloseableHttpResponse response = client.execute(makeQueryBroker(_externalBrokerPort))) {
+      Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
+      JsonNode resultTable = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("resultTable");
+      Assert.assertTrue(resultTable.get("rows").get(0).get(0).longValue() > 100000);
     }
   }
 
   @Test
   public void testBrokerExternalUntrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(JKS, _tlsStoreJKS, _tlsStoreEmptyJKS)) {
+    try (CloseableHttpClient client = makeClient(JKS, TLS_STORE_JKS, TLS_STORE_EMPTY_JKS)) {
       try {
         client.execute(makeQueryBroker(_externalBrokerPort));
         Assert.fail("Must not allow connection to untrusted server");
@@ -422,19 +407,18 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testBrokerInternalTrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(PKCS_12, _tlsStorePKCS12, _tlsStorePKCS12)) {
-      try (CloseableHttpResponse response = client.execute(makeQueryBroker(_internalBrokerPort))) {
-        Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
-        JsonNode resultTable = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("resultTable");
-        Assert.assertTrue(resultTable.get("rows").get(0).get(0).longValue() > 100000);
-      }
+    try (CloseableHttpClient client = makeClient(PKCS_12, TLS_STORE_PKCS_12, TLS_STORE_PKCS_12);
+        CloseableHttpResponse response = client.execute(makeQueryBroker(_internalBrokerPort))) {
+      Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
+      JsonNode resultTable = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("resultTable");
+      Assert.assertTrue(resultTable.get("rows").get(0).get(0).longValue() > 100000);
     }
   }
 
   @Test
   public void testBrokerInternalUntrustedServer()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(PKCS_12, _tlsStorePKCS12, _tlsStoreEmptyJKS)) {
+    try (CloseableHttpClient client = makeClient(PKCS_12, TLS_STORE_PKCS_12, TLS_STORE_EMPTY_JKS)) {
       try {
         client.execute(makeQueryBroker(_internalBrokerPort));
         Assert.fail("Must not allow connection to untrusted server");
@@ -447,7 +431,7 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testBrokerInternalUntrustedClient()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(PKCS_12, _tlsStoreEmptyPKCS12, _tlsStorePKCS12)) {
+    try (CloseableHttpClient client = makeClient(PKCS_12, TLS_STORE_EMPTY_PKCS_12, TLS_STORE_PKCS_12)) {
       try {
         client.execute(makeQueryBroker(_internalBrokerPort));
         Assert.fail("Must not allow connection from untrusted client");
@@ -460,16 +444,14 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
   @Test
   public void testControllerBrokerQueryForward()
       throws Exception {
-    try (CloseableHttpClient client = makeClient(JKS, _tlsStoreJKS, _tlsStoreJKS)) {
-      HttpPost request = new HttpPost("https://localhost:" + _externalControllerPort + "/sql");
-      request.addHeader(CLIENT_HEADER);
-      request.setEntity(new StringEntity("{\"sql\":\"SELECT count(*) FROM mytable\"}"));
-
-      try (CloseableHttpResponse response = client.execute(request)) {
-        Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
-        JsonNode resultTable = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("resultTable");
-        Assert.assertTrue(resultTable.get("rows").get(0).get(0).longValue() > 100000);
-      }
+    HttpPost request = new HttpPost("https://localhost:" + _externalControllerPort + "/sql");
+    request.addHeader(CLIENT_HEADER);
+    request.setEntity(new StringEntity("{\"sql\":\"SELECT count(*) FROM mytable\"}"));
+    try (CloseableHttpClient client = makeClient(JKS, TLS_STORE_JKS, TLS_STORE_JKS);
+        CloseableHttpResponse response = client.execute(request)) {
+      Assert.assertEquals(response.getStatusLine().getStatusCode(), 200);
+      JsonNode resultTable = JsonUtils.inputStreamToJsonNode(response.getEntity().getContent()).get("resultTable");
+      Assert.assertTrue(resultTable.get("rows").get(0).get(0).longValue() > 100000);
     }
   }
 
@@ -489,7 +471,7 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
     Assert.assertTrue(resultBeforeOffline.getResultSet(0).getLong(0) > 0);
 
     // schedule offline segment generation
-    Assert.assertNotNull(_controllerStarter.getTaskManager().scheduleTasks());
+    Assert.assertNotNull(_controllerStarter.getTaskManager().scheduleAllTasksForAllTables(null));
 
     // wait for offline segments
     JsonNode offlineSegments = TestUtils.waitForResult(() -> {
@@ -525,10 +507,8 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
     resultSet.first();
     Assert.assertTrue(resultSet.getLong(1) > 0);
 
-    try {
-      java.sql.Connection invalidConnection = getInValidJDBCConnection(_internalControllerPort);
-      statement = invalidConnection.createStatement();
-      resultSet = statement.executeQuery(query);
+    try (java.sql.Connection invalidConnection = getInValidJDBCConnection(_internalControllerPort)) {
+      invalidConnection.createStatement().executeQuery(query);
       Assert.fail("Should not allow queries with invalid TLS configuration");
     } catch (Exception e) {
       // this should fail
@@ -549,8 +529,8 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
       throws Exception {
     SSLContextBuilder sslContextBuilder = SSLContextBuilder.create();
     sslContextBuilder.setKeyStoreType(PKCS_12);
-    sslContextBuilder.loadKeyMaterial(_tlsStorePKCS12, PASSWORD_CHAR, PASSWORD_CHAR);
-    sslContextBuilder.loadTrustMaterial(_tlsStorePKCS12, PASSWORD_CHAR);
+    sslContextBuilder.loadKeyMaterial(TLS_STORE_PKCS_12, PASSWORD_CHAR, PASSWORD_CHAR);
+    sslContextBuilder.loadTrustMaterial(TLS_STORE_PKCS_12, PASSWORD_CHAR);
 
     PinotDriver pinotDriver = new PinotDriver(sslContextBuilder.build());
     Properties jdbcProps = new Properties();
@@ -563,8 +543,8 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
       throws Exception {
     SSLContextBuilder sslContextBuilder = SSLContextBuilder.create();
     sslContextBuilder.setKeyStoreType(PKCS_12);
-    sslContextBuilder.loadKeyMaterial(_tlsStoreEmptyPKCS12, PASSWORD_CHAR, PASSWORD_CHAR);
-    sslContextBuilder.loadTrustMaterial(_tlsStorePKCS12, PASSWORD_CHAR);
+    sslContextBuilder.loadKeyMaterial(TLS_STORE_EMPTY_PKCS_12, PASSWORD_CHAR, PASSWORD_CHAR);
+    sslContextBuilder.loadTrustMaterial(TLS_STORE_PKCS_12, PASSWORD_CHAR);
 
     PinotDriver pinotDriver = new PinotDriver(sslContextBuilder.build());
     Properties jdbcProps = new Properties();
@@ -574,16 +554,13 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
     return pinotDriver.connect("jdbc:pinot://localhost:" + controllerPort, jdbcProps);
   }
 
-  private static CloseableHttpClient makeClient(String keyStoreType, URL keyStoreUrl, URL trustStoreUrl) {
-    try {
-      SSLContextBuilder sslContextBuilder = SSLContextBuilder.create();
-      sslContextBuilder.setKeyStoreType(keyStoreType);
-      sslContextBuilder.loadKeyMaterial(keyStoreUrl, PASSWORD_CHAR, PASSWORD_CHAR);
-      sslContextBuilder.loadTrustMaterial(trustStoreUrl, PASSWORD_CHAR);
-      return HttpClientBuilder.create().setSSLContext(sslContextBuilder.build()).build();
-    } catch (Exception e) {
-      throw new IllegalStateException("Could not create HTTPS client", e);
-    }
+  private static CloseableHttpClient makeClient(String keyStoreType, URL keyStoreUrl, URL trustStoreUrl)
+      throws Exception {
+    SSLContextBuilder sslContextBuilder = SSLContextBuilder.create();
+    sslContextBuilder.setKeyStoreType(keyStoreType);
+    sslContextBuilder.loadKeyMaterial(keyStoreUrl, PASSWORD_CHAR, PASSWORD_CHAR);
+    sslContextBuilder.loadTrustMaterial(trustStoreUrl, PASSWORD_CHAR);
+    return HttpClientBuilder.create().setSSLContext(sslContextBuilder.build()).build();
   }
 
   private static HttpGet makeGetTables(int port) {
