@@ -22,7 +22,6 @@ import com.google.common.base.Joiner;
 import com.google.common.collect.ImmutableList;
 import java.io.File;
 import java.io.IOException;
-import java.io.InputStream;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
@@ -30,7 +29,7 @@ import java.util.List;
 import java.util.Map;
 import javax.annotation.Nullable;
 import org.apache.commons.io.FileUtils;
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.helix.HelixManager;
 import org.apache.helix.model.InstanceConfig;
 import org.apache.helix.task.TaskState;
@@ -82,8 +81,10 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
   private static final String CSV_DELIMITER = ",";
   private static final String TABLE_NAME = "gameScores";
   private static final int NUM_SERVERS = 2;
-  private static final String PRIMARY_KEY_COL = "playerId";
   private static final String DELETE_COL = "deleted";
+  public static final String PRIMARY_KEY_COL = "playerId";
+  public static final String TIME_COL_NAME = "timestampInEpoch";
+  public static final String UPSERT_SCHEMA_FILE_NAME = "upsert_table_test.schema";
 
   protected PinotTaskManager _taskManager;
   protected PinotHelixTaskResourceManager _helixTaskResourceManager;
@@ -133,13 +134,13 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
 
   @Override
   protected String getSchemaFileName() {
-    return "upsert_table_test.schema";
+    return UPSERT_SCHEMA_FILE_NAME;
   }
 
   @Nullable
   @Override
   protected String getTimeColumnName() {
-    return "timestampInEpoch";
+    return TIME_COL_NAME;
   }
 
   @Override
@@ -167,13 +168,6 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
     return 10;
   }
 
-  private Schema createSchema(String schemaFileName)
-      throws IOException {
-    InputStream inputStream = BaseClusterIntegrationTest.class.getClassLoader().getResourceAsStream(schemaFileName);
-    Assert.assertNotNull(inputStream);
-    return Schema.fromInputStream(inputStream);
-  }
-
   private long queryCountStarWithoutUpsert(String tableName) {
     return getPinotConnection().execute("SELECT COUNT(*) FROM " + tableName + " OPTION(skipUpsert=true)")
         .getResultSet(0).getLong(0);
@@ -400,15 +394,14 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
   @Test
   public void testDefaultMetadataManagerClass()
       throws Exception {
-    PinotConfiguration config = getServerConf(12345);
-    config.setProperty(Joiner.on(".").join(CommonConstants.Server.INSTANCE_DATA_MANAGER_CONFIG_PREFIX,
+    PinotConfiguration serverConf = getServerConf(NUM_SERVERS);
+    serverConf.setProperty(Joiner.on(".").join(CommonConstants.Server.INSTANCE_DATA_MANAGER_CONFIG_PREFIX,
             HelixInstanceDataManagerConfig.UPSERT_CONFIG_PREFIX,
             TableUpsertMetadataManagerFactory.UPSERT_DEFAULT_METADATA_MANAGER_CLASS),
         DummyTableUpsertMetadataManager.class.getName());
 
-    BaseServerStarter serverStarter = null;
+    BaseServerStarter serverStarter = startOneServer(serverConf);
     try {
-      serverStarter = startOneServer(config);
       HelixManager helixManager = serverStarter.getServerInstance().getHelixManager();
       InstanceConfig instanceConfig = HelixHelper.getInstanceConfig(helixManager, serverStarter.getInstanceId());
       // updateInstanceTags
@@ -445,9 +438,7 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
       deleteSchema(dummyTableName);
       waitForEVToDisappear(dummyTableName);
     } finally {
-      if (serverStarter != null) {
-        serverStarter.stop();
-      }
+      serverStarter.stop();
       // Re-initialize the executor for SegmentBuildTimeLeaseExtender to avoid the NullPointerException for the other
       // tests.
       SegmentBuildTimeLeaseExtender.initExecutor();
@@ -469,8 +460,8 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
     waitForAllDocsLoaded(tableName, 600_000L, 1000);
     assertEquals(getScore(tableName), 3692);
     waitForNumQueriedSegmentsToConverge(tableName, 10_000L, 3);
-
-    assertNotNull(_taskManager.scheduleTasks(TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName))
+    String realtimeTableName = TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName);
+    assertNotNull(_taskManager.scheduleAllTasksForTable(realtimeTableName, null)
         .get(MinionConstants.UpsertCompactionTask.TASK_TYPE));
     waitForTaskToComplete();
     waitForAllDocsLoaded(tableName, 600_000L, 3);
@@ -499,8 +490,8 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
     waitForAllDocsLoaded(tableName, 600_000L, 2000);
     assertEquals(getScore(tableName), 3692);
     waitForNumQueriedSegmentsToConverge(tableName, 10_000L, 5);
-
-    assertNotNull(_taskManager.scheduleTasks(TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName))
+    String realtimeTableName = TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName);
+    assertNotNull(_taskManager.scheduleAllTasksForTable(realtimeTableName, null)
         .get(MinionConstants.UpsertCompactionTask.TASK_TYPE));
     waitForTaskToComplete();
     waitForAllDocsLoaded(tableName, 600_000L, 3);
@@ -544,7 +535,8 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
 
     // Run segment compaction. This time, we expect that the deleting rows are still there because they are
     // as part of the consuming segment
-    assertNotNull(_taskManager.scheduleTasks(TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName))
+    String realtimeTableName = TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName);
+    assertNotNull(_taskManager.scheduleAllTasksForTable(realtimeTableName, null)
         .get(MinionConstants.UpsertCompactionTask.TASK_TYPE));
     waitForTaskToComplete();
     waitForAllDocsLoaded(tableName, 600_000L, 3);
@@ -561,7 +553,8 @@ public class UpsertTableIntegrationTest extends BaseClusterIntegrationTestSet {
     assertEquals(getNumDeletedRows(tableName), 2);
 
     // Run segment compaction. This time, we expect that the deleting rows are cleaned up
-    assertNotNull(_taskManager.scheduleTasks(TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName))
+    realtimeTableName = TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(tableName);
+    assertNotNull(_taskManager.scheduleAllTasksForTable(realtimeTableName, null)
         .get(MinionConstants.UpsertCompactionTask.TASK_TYPE));
     waitForTaskToComplete();
     waitForAllDocsLoaded(tableName, 600_000L, 3);
