@@ -18,8 +18,6 @@
  */
 package org.apache.pinot.integration.tests;
 
-import com.fasterxml.jackson.databind.JsonNode;
-import groovy.lang.IntRange;
 import java.io.File;
 import java.io.IOException;
 import java.net.URL;
@@ -33,13 +31,11 @@ import org.apache.pinot.client.ConnectionFactory;
 import org.apache.pinot.client.JsonAsyncHttpPinotClientTransportFactory;
 import org.apache.pinot.client.ResultSetGroup;
 import org.apache.pinot.common.auth.UrlAuthProvider;
-import org.apache.pinot.common.utils.SimpleHttpResponse;
+import org.apache.pinot.controller.helix.ControllerRequestClient;
 import org.apache.pinot.core.common.MinionConstants;
-import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableTaskConfig;
-import org.apache.pinot.spi.data.Schema;
+import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.env.PinotConfiguration;
-import org.apache.pinot.spi.utils.JsonUtils;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.apache.pinot.util.TestUtils;
 import org.testng.Assert;
@@ -97,50 +93,41 @@ public class UrlAuthRealtimeIntegrationTest extends BaseClusterIntegrationTest {
   }
 
   @Override
-  public Map<String, Object> getDefaultControllerConfiguration() {
-    Map<String, Object> conf = BasicAuthTestUtils.addControllerConfiguration(super.getDefaultControllerConfiguration());
-    conf.put("controller.segment.fetcher.auth.provider.class", AUTH_PROVIDER_CLASS);
-    conf.put("controller.segment.fetcher.auth.url", AUTH_URL);
-    conf.put("controller.segment.fetcher.auth.prefix", AUTH_PREFIX);
-
-    return conf;
+  protected void overrideControllerConf(Map<String, Object> properties) {
+    BasicAuthTestUtils.addControllerConfiguration(properties);
+    properties.put("controller.segment.fetcher.auth.provider.class", AUTH_PROVIDER_CLASS);
+    properties.put("controller.segment.fetcher.auth.url", AUTH_URL);
+    properties.put("controller.segment.fetcher.auth.prefix", AUTH_PREFIX);
   }
 
   @Override
-  protected PinotConfiguration getDefaultBrokerConfiguration() {
-    PinotConfiguration conf = BasicAuthTestUtils.addBrokerConfiguration(super.getDefaultBrokerConfiguration().toMap());
-    // no customization yet
-
-    return conf;
+  protected void overrideBrokerConf(PinotConfiguration brokerConf) {
+    BasicAuthTestUtils.addBrokerConfiguration(brokerConf);
   }
 
   @Override
-  protected PinotConfiguration getDefaultServerConfiguration() {
-    PinotConfiguration conf = BasicAuthTestUtils.addServerConfiguration(super.getDefaultServerConfiguration().toMap());
-    conf.setProperty("pinot.server.segment.fetcher.auth.provider.class", AUTH_PROVIDER_CLASS);
-    conf.setProperty("pinot.server.segment.fetcher.auth.url", AUTH_URL);
-    conf.setProperty("pinot.server.segment.fetcher.auth.prefix", AUTH_PREFIX);
-    conf.setProperty("pinot.server.segment.uploader.auth.provider.class", AUTH_PROVIDER_CLASS);
-    conf.setProperty("pinot.server.segment.uploader.auth.url", AUTH_URL);
-    conf.setProperty("pinot.server.segment.uploader.auth.prefix", AUTH_PREFIX);
-    conf.setProperty("pinot.server.instance.auth.provider.class", AUTH_PROVIDER_CLASS);
-    conf.setProperty("pinot.server.instance.auth.url", AUTH_URL);
-    conf.setProperty("pinot.server.instance.auth.prefix", AUTH_PREFIX);
-
-    return conf;
+  protected void overrideServerConf(PinotConfiguration serverConf) {
+    BasicAuthTestUtils.addServerConfiguration(serverConf);
+    serverConf.setProperty("pinot.server.segment.fetcher.auth.provider.class", AUTH_PROVIDER_CLASS);
+    serverConf.setProperty("pinot.server.segment.fetcher.auth.url", AUTH_URL);
+    serverConf.setProperty("pinot.server.segment.fetcher.auth.prefix", AUTH_PREFIX);
+    serverConf.setProperty("pinot.server.segment.uploader.auth.provider.class", AUTH_PROVIDER_CLASS);
+    serverConf.setProperty("pinot.server.segment.uploader.auth.url", AUTH_URL);
+    serverConf.setProperty("pinot.server.segment.uploader.auth.prefix", AUTH_PREFIX);
+    serverConf.setProperty("pinot.server.instance.auth.provider.class", AUTH_PROVIDER_CLASS);
+    serverConf.setProperty("pinot.server.instance.auth.url", AUTH_URL);
+    serverConf.setProperty("pinot.server.instance.auth.prefix", AUTH_PREFIX);
   }
 
   @Override
-  protected PinotConfiguration getDefaultMinionConfiguration() {
-    PinotConfiguration conf = BasicAuthTestUtils.addMinionConfiguration(super.getDefaultMinionConfiguration().toMap());
-    conf.setProperty("segment.fetcher.auth.provider.class", AUTH_PROVIDER_CLASS);
-    conf.setProperty("segment.fetcher.auth.url", AUTH_URL_PREFIXED);
-    conf.setProperty("segment.fetcher.auth.prefix", AUTH_PREFIX);
-    conf.setProperty("task.auth.provider.class", AUTH_PROVIDER_CLASS);
-    conf.setProperty("task.auth.url", AUTH_URL_PREFIXED);
-    conf.setProperty("task.auth.prefix", AUTH_PREFIX);
-
-    return conf;
+  protected void overrideMinionConf(PinotConfiguration minionConf) {
+    BasicAuthTestUtils.addMinionConfiguration(minionConf);
+    minionConf.setProperty("segment.fetcher.auth.provider.class", AUTH_PROVIDER_CLASS);
+    minionConf.setProperty("segment.fetcher.auth.url", AUTH_URL_PREFIXED);
+    minionConf.setProperty("segment.fetcher.auth.prefix", AUTH_PREFIX);
+    minionConf.setProperty("task.auth.provider.class", AUTH_PROVIDER_CLASS);
+    minionConf.setProperty("task.auth.url", AUTH_URL_PREFIXED);
+    minionConf.setProperty("task.auth.prefix", AUTH_PREFIX);
   }
 
   @Override
@@ -153,18 +140,12 @@ public class UrlAuthRealtimeIntegrationTest extends BaseClusterIntegrationTest {
   }
 
   @Override
-  public void addSchema(Schema schema)
-      throws IOException {
-    SimpleHttpResponse response =
-        sendMultipartPostRequest(_controllerRequestURLBuilder.forSchemaCreate(), schema.toSingleLineJsonString(),
-            AUTH_HEADER);
-    Assert.assertEquals(response.getStatusCode(), 200);
-  }
-
-  @Override
-  public void addTableConfig(TableConfig tableConfig)
-      throws IOException {
-    sendPostRequest(_controllerRequestURLBuilder.forTableCreate(), tableConfig.toJsonString(), AUTH_HEADER);
+  public ControllerRequestClient getControllerRequestClient() {
+    if (_controllerRequestClient == null) {
+      _controllerRequestClient =
+          new ControllerRequestClient(_controllerRequestURLBuilder, getHttpClient(), AUTH_HEADER);
+    }
+    return _controllerRequestClient;
   }
 
   @Override
@@ -179,14 +160,6 @@ public class UrlAuthRealtimeIntegrationTest extends BaseClusterIntegrationTest {
     return _pinotConnection;
   }
 
-  @Override
-  public void dropRealtimeTable(String tableName)
-      throws IOException {
-    sendDeleteRequest(
-        _controllerRequestURLBuilder.forTableDelete(TableNameBuilder.REALTIME.tableNameWithType(tableName)),
-        AUTH_HEADER);
-  }
-
   @Test(expectedExceptions = IOException.class)
   public void testUnauthenticatedFailure()
       throws IOException {
@@ -203,15 +176,12 @@ public class UrlAuthRealtimeIntegrationTest extends BaseClusterIntegrationTest {
     Assert.assertTrue(resultBeforeOffline.getResultSet(0).getLong(0) > 0);
 
     // schedule offline segment generation
-    Assert.assertNotNull(_controllerStarter.getTaskManager().scheduleTasks());
+    Assert.assertNotNull(_controllerStarter.getTaskManager().scheduleAllTasksForAllTables(null));
 
     // wait for offline segments
-    JsonNode offlineSegments = TestUtils.waitForResult(() -> {
-      JsonNode segmentSets = JsonUtils.stringToJsonNode(
-          sendGetRequest(_controllerRequestURLBuilder.forSegmentListAPI(getTableName()), AUTH_HEADER));
-      JsonNode currentOfflineSegments =
-          new IntRange(0, segmentSets.size()).stream().map(segmentSets::get).filter(s -> s.has("OFFLINE"))
-              .map(s -> s.get("OFFLINE")).findFirst().get();
+    List<String> offlineSegments = TestUtils.waitForResult(() -> {
+      List<String> currentOfflineSegments =
+          getControllerRequestClient().listSegments(getTableName(), TableType.OFFLINE.name(), false);
       Assert.assertFalse(currentOfflineSegments.isEmpty());
       return currentOfflineSegments;
     }, 30000);
@@ -221,8 +191,7 @@ public class UrlAuthRealtimeIntegrationTest extends BaseClusterIntegrationTest {
     Assert.assertEquals(resultBeforeOffline.getResultSet(0).getLong(0), resultAfterOffline.getResultSet(0).getLong(0));
 
     // download and sanity-check size of offline segment(s)
-    for (int i = 0; i < offlineSegments.size(); i++) {
-      String segment = offlineSegments.get(i).asText();
+    for (String segment : offlineSegments) {
       Assert.assertTrue(
           sendGetRequest(_controllerRequestURLBuilder.forSegmentDownload(getTableName(), segment), AUTH_HEADER).length()
               > 200000); // download segment
