@@ -23,15 +23,19 @@ import com.google.common.cache.Cache;
 import com.google.common.cache.CacheBuilder;
 import com.google.common.collect.ImmutableList;
 import java.io.File;
+import java.util.List;
 import org.apache.avro.file.DataFileWriter;
 import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericDatumWriter;
 import org.apache.commons.lang3.RandomStringUtils;
+import org.apache.pinot.core.util.DoubleComparisonUtil;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.Schema;
-import org.testng.Assert;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.assertTrue;
+
 
 @Test(suiteName = "CustomClusterIntegrationTest")
 public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
@@ -45,12 +49,38 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
   private static final String STRING_COLUMN = "stringCol";
   private static final String TIMESTAMP_COLUMN = "timestampCol";
   private static final String GROUP_BY_COLUMN = "groupKey";
+  private static final String LONG_ARRAY_COLUMN = "longArrayCol";
+  private static final String DOUBLE_ARRAY_COLUMN = "doubleArrayCol";
 
   @Override
   protected long getCountStarResult() {
     return 1000;
   }
 
+  @Test(dataProvider = "useBothQueryEngines")
+  public void testArrayAggWithEmptyPredicate(boolean useMultiStageQueryEngine) throws Exception {
+    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
+    String query =
+        String.format("SELECT "
+            + "arrayAgg(boolCol, 'BOOLEAN'), "
+            + "arrayAgg(intCol, 'INT'), "
+            + "arrayAgg(longCol, 'LONG'), "
+            // NOTE: FLOAT array is auto converted to DOUBLE array
+            + (useMultiStageQueryEngine ? "arrayAgg(floatCol, 'DOUBLE'), " : "arrayAgg(floatCol, 'FLOAT'), ")
+            + "arrayAgg(doubleCol, 'DOUBLE'), "
+            + "arrayAgg(stringCol, 'STRING'), "
+            + "arrayAgg(timestampCol, 'TIMESTAMP') "
+            + "FROM %s WHERE intCol < 0 LIMIT %d", getTableName(), getCountStarResult());
+    JsonNode jsonNode = postQuery(query);
+    JsonNode rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    JsonNode row = rows.get(0);
+    assertEquals(row.size(), 7);
+    for (int i = 0; i < 7; i++) {
+      assertEquals(row.get(i).size(), 0);
+    }
+  }
+
   @Test(dataProvider = "useBothQueryEngines")
   public void testArrayAggQueries(boolean useMultiStageQueryEngine)
       throws Exception {
@@ -68,16 +98,16 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
             + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
     JsonNode jsonNode = postQuery(query);
     JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
+    assertEquals(rows.size(), 1);
     JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 7);
-    Assert.assertEquals(row.get(0).size(), getCountStarResult());
-    Assert.assertEquals(row.get(1).size(), getCountStarResult());
-    Assert.assertEquals(row.get(2).size(), getCountStarResult());
-    Assert.assertEquals(row.get(3).size(), getCountStarResult());
-    Assert.assertEquals(row.get(4).size(), getCountStarResult());
-    Assert.assertEquals(row.get(5).size(), getCountStarResult());
-    Assert.assertEquals(row.get(6).size(), getCountStarResult());
+    assertEquals(row.size(), 7);
+    assertEquals(row.get(0).size(), getCountStarResult());
+    assertEquals(row.get(1).size(), getCountStarResult());
+    assertEquals(row.get(2).size(), getCountStarResult());
+    assertEquals(row.get(3).size(), getCountStarResult());
+    assertEquals(row.get(4).size(), getCountStarResult());
+    assertEquals(row.get(5).size(), getCountStarResult());
+    assertEquals(row.get(6).size(), getCountStarResult());
   }
 
   @Test(dataProvider = "useBothQueryEngines")
@@ -100,17 +130,231 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
             + "LIMIT %d", getTableName(), getCountStarResult());
     JsonNode jsonNode = postQuery(query);
     JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 10);
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
       JsonNode row = rows.get(i);
-      Assert.assertEquals(row.size(), 8);
-      Assert.assertEquals(row.get(0).size(), getCountStarResult() / 10);
-      Assert.assertEquals(row.get(1).size(), getCountStarResult() / 10);
-      Assert.assertEquals(row.get(2).size(), getCountStarResult() / 10);
-      Assert.assertEquals(row.get(3).size(), getCountStarResult() / 10);
-      Assert.assertEquals(row.get(4).size(), getCountStarResult() / 10);
-      Assert.assertEquals(row.get(5).size(), getCountStarResult() / 10);
-      Assert.assertEquals(row.get(6).size(), getCountStarResult() / 10);
+      assertEquals(row.size(), 8);
+      assertEquals(row.get(0).size(), getCountStarResult() / 10);
+      assertEquals(row.get(1).size(), getCountStarResult() / 10);
+      assertEquals(row.get(2).size(), getCountStarResult() / 10);
+      assertEquals(row.get(3).size(), getCountStarResult() / 10);
+      assertEquals(row.get(4).size(), getCountStarResult() / 10);
+      assertEquals(row.get(5).size(), getCountStarResult() / 10);
+      assertEquals(row.get(6).size(), getCountStarResult() / 10);
+    }
+  }
+
+  @Test(dataProvider = "useV2QueryEngine")
+  public void testListAggQueries(boolean useMultiStageQueryEngine)
+      throws Exception {
+    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
+    String query =
+        String.format("SELECT "
+            + "listAgg(stringCol, ' | ') "
+            + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
+    JsonNode jsonNode = postQuery(query);
+    JsonNode rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    JsonNode row = rows.get(0);
+    assertEquals(row.size(), 1);
+    assertEquals(row.get(0).asText().split(" \\| ").length, getCountStarResult());
+
+    query =
+        String.format("SELECT "
+            + "listAgg(stringCol, ' | ') WITHIN GROUP (ORDER BY stringCol) "
+            + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    row = rows.get(0);
+    assertEquals(row.size(), 1);
+    String[] splits = row.get(0).asText().split(" \\| ");
+    assertEquals(splits.length, getCountStarResult());
+    for (int i = 1; i < splits.length; i++) {
+      assertTrue(splits[i].compareTo(splits[i - 1]) >= 0);
+    }
+
+    query =
+        String.format("SELECT "
+            + "listAgg(cast(doubleCol AS VARCHAR), ' | ') WITHIN GROUP (ORDER BY doubleCol) "
+            + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    row = rows.get(0);
+    assertEquals(row.size(), 1);
+    splits = row.get(0).asText().split(" \\| ");
+    assertEquals(splits.length, getCountStarResult());
+    for (int i = 1; i < splits.length; i++) {
+      assertTrue(Double.parseDouble(splits[i]) >= Double.parseDouble(splits[i - 1]));
+    }
+  }
+
+  @Test(dataProvider = "useV2QueryEngine")
+  public void testListAggGroupByQueries(boolean useMultiStageQueryEngine)
+      throws Exception {
+    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
+    String query =
+        String.format("SELECT "
+            + "listAgg(stringCol, ' | '), "
+            + "groupKey "
+            + "FROM %s "
+            + "GROUP BY groupKey "
+            + "LIMIT %d", getTableName(), getCountStarResult());
+    JsonNode jsonNode = postQuery(query);
+    JsonNode rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
+    for (int i = 0; i < 10; i++) {
+      JsonNode row = rows.get(i);
+      assertEquals(row.size(), 2);
+      assertEquals(row.get(0).asText().split(" \\| ").length, getCountStarResult() / 10);
+    }
+
+    query =
+        String.format("SELECT "
+            + "listAgg(stringCol, ' | ') WITHIN GROUP (ORDER BY stringCol), "
+            + "groupKey "
+            + "FROM %s "
+            + "GROUP BY groupKey "
+            + "LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
+    for (int i = 0; i < 10; i++) {
+      JsonNode row = rows.get(i);
+      assertEquals(row.size(), 2);
+      String[] splits = row.get(0).asText().split(" \\| ");
+      assertEquals(splits.length, getCountStarResult() / 10);
+      for (int j = 1; j < splits.length; j++) {
+        assertTrue(splits[j].compareTo(splits[j - 1]) >= 0);
+      }
+    }
+
+    query =
+        String.format("SELECT "
+            + "listAgg(cast(doubleCol AS VARCHAR), ' | ') WITHIN GROUP (ORDER BY doubleCol), "
+            + "groupKey "
+            + "FROM %s "
+            + "GROUP BY groupKey "
+            + "LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
+    for (int i = 0; i < 10; i++) {
+      JsonNode row = rows.get(i);
+      assertEquals(row.size(), 2);
+      String[] splits = row.get(0).asText().split(" \\| ");
+      assertEquals(splits.length, getCountStarResult() / 10);
+      for (int j = 1; j < splits.length; j++) {
+        assertTrue(Double.parseDouble(splits[j]) >= Double.parseDouble(splits[j - 1]));
+      }
+    }
+  }
+
+  @Test(dataProvider = "useV2QueryEngine")
+  public void testListDistinctAggQueries(boolean useMultiStageQueryEngine)
+      throws Exception {
+    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
+    String query =
+        String.format("SELECT "
+            + "listAgg(DISTINCT stringCol, ' | ') "
+            + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
+    JsonNode jsonNode = postQuery(query);
+    JsonNode rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    JsonNode row = rows.get(0);
+    assertEquals(row.size(), 1);
+    assertEquals(row.get(0).asText().split(" \\| ").length, getCountStarResult() / 10);
+
+    query =
+        String.format("SELECT "
+            + "listAgg(DISTINCT stringCol, ' | ') WITHIN GROUP (ORDER BY stringCol) "
+            + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    row = rows.get(0);
+    assertEquals(row.size(), 1);
+    String[] splits = row.get(0).asText().split(" \\| ");
+    assertEquals(splits.length, getCountStarResult() / 10);
+    for (int j = 1; j < splits.length; j++) {
+      assertTrue(splits[j].compareTo(splits[j - 1]) > 0);
+    }
+
+    query =
+        String.format("SELECT "
+            + "listAgg(DISTINCT cast(doubleCol AS VARCHAR), ' | ') WITHIN GROUP (ORDER BY doubleCol) "
+            + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    row = rows.get(0);
+    assertEquals(row.size(), 1);
+    splits = row.get(0).asText().split(" \\| ");
+    assertEquals(splits.length, getCountStarResult() / 10);
+    for (int j = 1; j < splits.length; j++) {
+      assertTrue(Double.parseDouble(splits[j]) > Double.parseDouble(splits[j - 1]));
+    }
+  }
+
+  @Test(dataProvider = "useV2QueryEngine")
+  public void testListAggDistinctGroupByQueries(boolean useMultiStageQueryEngine)
+      throws Exception {
+    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
+    String query =
+        String.format("SELECT "
+            + "listAgg(DISTINCT stringCol, ' | '), "
+            + "groupKey "
+            + "FROM %s "
+            + "GROUP BY groupKey "
+            + "LIMIT %d", getTableName(), getCountStarResult());
+    JsonNode jsonNode = postQuery(query);
+    JsonNode rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
+    for (int i = 0; i < 10; i++) {
+      JsonNode row = rows.get(i);
+      assertEquals(row.size(), 2);
+      assertEquals(row.get(0).asText().split(" \\| ").length, getCountStarResult() / 100);
+    }
+
+    query =
+        String.format("SELECT "
+            + "listAgg(DISTINCT stringCol, ' | ') WITHIN GROUP (ORDER BY stringCol), "
+            + "groupKey "
+            + "FROM %s "
+            + "GROUP BY groupKey "
+            + "LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
+    for (int i = 0; i < 10; i++) {
+      JsonNode row = rows.get(i);
+      assertEquals(row.size(), 2);
+      String[] splits = row.get(0).asText().split(" \\| ");
+      assertEquals(splits.length, getCountStarResult() / 100);
+      for (int j = 1; j < splits.length; j++) {
+        assertTrue(splits[j].compareTo(splits[j - 1]) > 0);
+      }
+    }
+
+    query =
+        String.format("SELECT "
+            + "listAgg(DISTINCT cast(doubleCol AS VARCHAR), ' | ') WITHIN GROUP (ORDER BY doubleCol), "
+            + "groupKey "
+            + "FROM %s "
+            + "GROUP BY groupKey "
+            + "LIMIT %d", getTableName(), getCountStarResult());
+    jsonNode = postQuery(query);
+    rows = jsonNode.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
+    for (int i = 0; i < 10; i++) {
+      JsonNode row = rows.get(i);
+      assertEquals(row.size(), 2);
+      String[] splits = row.get(0).asText().split(" \\| ");
+      assertEquals(splits.length, getCountStarResult() / 100);
+      for (int j = 1; j < splits.length; j++) {
+        assertTrue(Double.parseDouble(splits[j]) > Double.parseDouble(splits[j - 1]));
+      }
     }
   }
 
@@ -132,16 +376,16 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
             + "FROM %s LIMIT %d", getTableName(), getCountStarResult());
     JsonNode jsonNode = postQuery(query);
     JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
+    assertEquals(rows.size(), 1);
     JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 7);
-    Assert.assertEquals(row.get(0).size(), 2);
-    Assert.assertEquals(row.get(1).size(), getCountStarResult() / 10);
-    Assert.assertEquals(row.get(2).size(), getCountStarResult() / 10);
-    Assert.assertEquals(row.get(3).size(), getCountStarResult() / 10);
-    Assert.assertEquals(row.get(4).size(), getCountStarResult() / 10);
-    Assert.assertEquals(row.get(5).size(), getCountStarResult() / 10);
-    Assert.assertEquals(row.get(6).size(), getCountStarResult() / 10);
+    assertEquals(row.size(), 7);
+    assertEquals(row.get(0).size(), 2);
+    assertEquals(row.get(1).size(), getCountStarResult() / 10);
+    assertEquals(row.get(2).size(), getCountStarResult() / 10);
+    assertEquals(row.get(3).size(), getCountStarResult() / 10);
+    assertEquals(row.get(4).size(), getCountStarResult() / 10);
+    assertEquals(row.get(5).size(), getCountStarResult() / 10);
+    assertEquals(row.get(6).size(), getCountStarResult() / 10);
   }
 
   @Test(dataProvider = "useBothQueryEngines")
@@ -165,17 +409,17 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
             + "LIMIT %d", getTableName(), getCountStarResult());
     JsonNode jsonNode = postQuery(query);
     JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 10);
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
       JsonNode row = rows.get(i);
-      Assert.assertEquals(row.size(), 8);
-      Assert.assertEquals(row.get(0).size(), 2);
-      Assert.assertEquals(row.get(1).size(), getCountStarResult() / 100);
-      Assert.assertEquals(row.get(2).size(), getCountStarResult() / 100);
-      Assert.assertEquals(row.get(3).size(), getCountStarResult() / 100);
-      Assert.assertEquals(row.get(4).size(), getCountStarResult() / 100);
-      Assert.assertEquals(row.get(5).size(), getCountStarResult() / 100);
-      Assert.assertEquals(row.get(6).size(), getCountStarResult() / 100);
+      assertEquals(row.size(), 8);
+      assertEquals(row.get(0).size(), 2);
+      assertEquals(row.get(1).size(), getCountStarResult() / 100);
+      assertEquals(row.get(2).size(), getCountStarResult() / 100);
+      assertEquals(row.get(3).size(), getCountStarResult() / 100);
+      assertEquals(row.get(4).size(), getCountStarResult() / 100);
+      assertEquals(row.get(5).size(), getCountStarResult() / 100);
+      assertEquals(row.get(6).size(), getCountStarResult() / 100);
     }
   }
 
@@ -189,188 +433,152 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
             + "FROM %s LIMIT 1", getTableName());
     JsonNode jsonNode = postQuery(query);
     JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
+    assertEquals(rows.size(), 1);
     JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asText(), "t1");
-    Assert.assertEquals(row.get(0).get(1).asText(), "t2");
-    Assert.assertEquals(row.get(0).get(2).asText(), "t3");
+    assertEquals(row.size(), 1);
+    assertEquals(row.get(0).size(), 3);
+    assertEquals(row.get(0).get(0).asText(), "t1");
+    assertEquals(row.get(0).get(1).asText(), "t2");
+    assertEquals(row.get(0).get(2).asText(), "t3");
   }
 
   @Test(dataProvider = "useBothQueryEngines")
   public void testIntArrayLiteral(boolean useMultiStageQueryEngine)
       throws Exception {
     setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query =
-        String.format("SELECT "
-            + "ARRAY[1,2,3] "
-            + "FROM %s LIMIT 1", getTableName());
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asInt(), 1);
-    Assert.assertEquals(row.get(0).get(1).asInt(), 2);
-    Assert.assertEquals(row.get(0).get(2).asInt(), 3);
-  }
-
-  @Test(dataProvider = "useBothQueryEngines")
-  public void testIntArrayLiteralWithoutFrom(boolean useMultiStageQueryEngine)
-      throws Exception {
-    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query = "SELECT ARRAY[1,2,3] ";
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asInt(), 1);
-    Assert.assertEquals(row.get(0).get(1).asInt(), 2);
-    Assert.assertEquals(row.get(0).get(2).asInt(), 3);
+    for (String arrayLiteral : List.of("ARRAY[1,2,3]", "ARRAY'{1,2,3}'")) {
+      for (boolean withFrom : new boolean[]{true, false}) {
+        String query = withFrom ? String.format("SELECT %s FROM %s LIMIT 1", arrayLiteral, getTableName())
+            : "SELECT " + arrayLiteral;
+        JsonNode result = postQuery(query).get("resultTable");
+        // TODO: Check data schema
+        JsonNode rows = result.get("rows");
+        assertEquals(rows.size(), 1);
+        JsonNode row = rows.get(0);
+        assertEquals(row.size(), 1);
+        assertEquals(row.get(0).size(), 3);
+        assertEquals(row.get(0).get(0).asInt(), 1);
+        assertEquals(row.get(0).get(1).asInt(), 2);
+        assertEquals(row.get(0).get(2).asInt(), 3);
+      }
+    }
   }
 
   @Test(dataProvider = "useBothQueryEngines")
   public void testLongArrayLiteral(boolean useMultiStageQueryEngine)
       throws Exception {
     setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query =
-        String.format("SELECT "
-            + "ARRAY[2147483648,2147483649,2147483650] "
-            + "FROM %s LIMIT 1", getTableName());
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asLong(), 2147483648L);
-    Assert.assertEquals(row.get(0).get(1).asLong(), 2147483649L);
-    Assert.assertEquals(row.get(0).get(2).asLong(), 2147483650L);
+    for (String arrayLiteral : List.of("ARRAY[2147483648,2147483649,2147483650]",
+        "ARRAY'{2147483648,2147483649,2147483650}'")) {
+      for (boolean withFrom : new boolean[]{true, false}) {
+        String query = withFrom ? String.format("SELECT %s FROM %s LIMIT 1", arrayLiteral, getTableName())
+            : "SELECT " + arrayLiteral;
+        JsonNode result = postQuery(query).get("resultTable");
+        assertEquals(result.get("dataSchema").get("columnDataTypes").get(0).textValue(), "LONG_ARRAY");
+        JsonNode rows = result.get("rows");
+        assertEquals(rows.size(), 1);
+        JsonNode row = rows.get(0);
+        assertEquals(row.size(), 1);
+        assertEquals(row.get(0).size(), 3);
+        assertEquals(row.get(0).get(0).longValue(), 2147483648L);
+        assertEquals(row.get(0).get(1).longValue(), 2147483649L);
+        assertEquals(row.get(0).get(2).longValue(), 2147483650L);
+      }
+    }
   }
 
   @Test(dataProvider = "useBothQueryEngines")
-  public void testLongArrayLiteralWithoutFrom(boolean useMultiStageQueryEngine)
+  public void testArraySum(boolean useMultiStageQueryEngine)
       throws Exception {
     setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query = "SELECT ARRAY[2147483648,2147483649,2147483650]";
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
+    String query = String.format("SELECT sumArrayLong(%s), sumArrayDouble(%s) FROM %s", LONG_ARRAY_COLUMN,
+        DOUBLE_ARRAY_COLUMN, getTableName());
+    JsonNode result = postQuery(query).get("resultTable");
+    JsonNode columnDataTypesNode = result.get("dataSchema").get("columnDataTypes");
+    assertEquals(columnDataTypesNode.get(0).textValue(), "LONG_ARRAY");
+    assertEquals(columnDataTypesNode.get(1).textValue(), "DOUBLE_ARRAY");
+    JsonNode rows = result.get("rows");
+    assertEquals(rows.size(), 1);
     JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asLong(), 2147483648L);
-    Assert.assertEquals(row.get(0).get(1).asLong(), 2147483649L);
-    Assert.assertEquals(row.get(0).get(2).asLong(), 2147483650L);
+    assertEquals(row.size(), 2);
+    JsonNode entry0 = row.get(0);
+    assertEquals(entry0.size(), 4);
+    assertEquals(entry0.get(0).longValue(), 0L);
+    assertEquals(entry0.get(1).longValue(), 1000L);
+    assertEquals(entry0.get(2).longValue(), 2000L);
+    assertEquals(entry0.get(3).longValue(), 3000L);
+    JsonNode entry1 = row.get(1);
+    assertEquals(entry1.size(), 4);
+    assertEquals(entry1.get(0).doubleValue(), 0.0);
+    // Compare double values:
+    assertEquals(DoubleComparisonUtil.doubleCompare(entry1.get(1).doubleValue(), 100.0, 0.00000000001), 0);
+    assertEquals(DoubleComparisonUtil.doubleCompare(entry1.get(2).doubleValue(), 200.0, 0.00000000001), 0);
+    assertEquals(DoubleComparisonUtil.doubleCompare(entry1.get(3).doubleValue(), 300.0, 0.00000000001), 0);
   }
 
   @Test(dataProvider = "useBothQueryEngines")
   public void testFloatArrayLiteral(boolean useMultiStageQueryEngine)
       throws Exception {
     setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query =
-        String.format("SELECT "
-            + "ARRAY[0.1, 0.2, 0.3] "
-            + "FROM %s LIMIT 1", getTableName());
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asDouble(), 0.1);
-    Assert.assertEquals(row.get(0).get(1).asDouble(), 0.2);
-    Assert.assertEquals(row.get(0).get(2).asDouble(), 0.3);
-  }
-
-  @Test(dataProvider = "useBothQueryEngines")
-  public void testFloatArrayLiteralWithoutFrom(boolean useMultiStageQueryEngine)
-      throws Exception {
-    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query = "SELECT ARRAY[0.1, 0.2, 0.3]";
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asDouble(), 0.1);
-    Assert.assertEquals(row.get(0).get(1).asDouble(), 0.2);
-    Assert.assertEquals(row.get(0).get(2).asDouble(), 0.3);
+    for (String arrayLiteral : List.of("ARRAY[0.1,0.2,0.3]", "ARRAY'{0.1,0.2,0.3}'")) {
+      for (boolean withFrom : new boolean[]{true, false}) {
+        String query = withFrom ? String.format("SELECT %s FROM %s LIMIT 1", arrayLiteral, getTableName())
+            : "SELECT " + arrayLiteral;
+        JsonNode result = postQuery(query).get("resultTable");
+        // TODO: Check data schema
+        JsonNode rows = result.get("rows");
+        assertEquals(rows.size(), 1);
+        JsonNode row = rows.get(0);
+        assertEquals(row.size(), 1);
+        assertEquals(row.get(0).size(), 3);
+        assertEquals(row.get(0).get(0).asDouble(), 0.1);
+        assertEquals(row.get(0).get(1).asDouble(), 0.2);
+        assertEquals(row.get(0).get(2).asDouble(), 0.3);
+      }
+    }
   }
 
   @Test(dataProvider = "useBothQueryEngines")
   public void testDoubleArrayLiteral(boolean useMultiStageQueryEngine)
       throws Exception {
     setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query =
-        String.format("SELECT "
-            + "ARRAY[CAST(0.1 AS DOUBLE), CAST(0.2 AS DOUBLE), CAST(0.3 AS DOUBLE)] "
-            + "FROM %s LIMIT 1", getTableName());
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asDouble(), 0.1);
-    Assert.assertEquals(row.get(0).get(1).asDouble(), 0.2);
-    Assert.assertEquals(row.get(0).get(2).asDouble(), 0.3);
-  }
-
-  @Test(dataProvider = "useBothQueryEngines")
-  public void testDoubleArrayLiteralWithoutFrom(boolean useMultiStageQueryEngine)
-      throws Exception {
-    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query = "SELECT ARRAY[CAST(0.1 AS DOUBLE), CAST(0.2 AS DOUBLE), CAST(0.3 AS DOUBLE)]";
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asDouble(), 0.1);
-    Assert.assertEquals(row.get(0).get(1).asDouble(), 0.2);
-    Assert.assertEquals(row.get(0).get(2).asDouble(), 0.3);
+    String arrayLiteral = "ARRAY[CAST(0.1 AS DOUBLE),CAST(0.2 AS DOUBLE),CAST(0.3 AS DOUBLE)]";
+    for (boolean withFrom : new boolean[]{true, false}) {
+      String query = withFrom ? String.format("SELECT %s FROM %s LIMIT 1", arrayLiteral, getTableName())
+          : "SELECT " + arrayLiteral;
+      JsonNode result = postQuery(query).get("resultTable");
+      // TODO: Check data schema
+      JsonNode rows = result.get("rows");
+      assertEquals(rows.size(), 1);
+      JsonNode row = rows.get(0);
+      assertEquals(row.size(), 1);
+      assertEquals(row.get(0).size(), 3);
+      assertEquals(row.get(0).get(0).asDouble(), 0.1);
+      assertEquals(row.get(0).get(1).asDouble(), 0.2);
+      assertEquals(row.get(0).get(2).asDouble(), 0.3);
+    }
   }
 
   @Test(dataProvider = "useBothQueryEngines")
   public void testStringArrayLiteral(boolean useMultiStageQueryEngine)
       throws Exception {
     setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query =
-        String.format("SELECT "
-            + "ARRAY['a', 'bb', 'ccc'] "
-            + "FROM %s LIMIT 1", getTableName());
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asText(), "a");
-    Assert.assertEquals(row.get(0).get(1).asText(), "bb");
-    Assert.assertEquals(row.get(0).get(2).asText(), "ccc");
-  }
-
-  @Test(dataProvider = "useBothQueryEngines")
-  public void testStringArrayLiteralWithoutFrom(boolean useMultiStageQueryEngine)
-      throws Exception {
-    setUseMultiStageQueryEngine(useMultiStageQueryEngine);
-    String query = "SELECT ARRAY['a', 'bb', 'ccc']";
-    JsonNode jsonNode = postQuery(query);
-    JsonNode rows = jsonNode.get("resultTable").get("rows");
-    Assert.assertEquals(rows.size(), 1);
-    JsonNode row = rows.get(0);
-    Assert.assertEquals(row.size(), 1);
-    Assert.assertEquals(row.get(0).size(), 3);
-    Assert.assertEquals(row.get(0).get(0).asText(), "a");
-    Assert.assertEquals(row.get(0).get(1).asText(), "bb");
-    Assert.assertEquals(row.get(0).get(2).asText(), "ccc");
+    for (String arrayLiteral : List.of("ARRAY['a','bb','ccc']", "ARRAY'{\"a\",\"bb\",\"ccc\"}'")) {
+      for (boolean withFrom : new boolean[]{true, false}) {
+        String query = withFrom ? String.format("SELECT %s FROM %s LIMIT 1", arrayLiteral, getTableName())
+            : "SELECT " + arrayLiteral;
+        JsonNode result = postQuery(query).get("resultTable");
+        assertEquals(result.get("dataSchema").get("columnDataTypes").get(0).textValue(), "STRING_ARRAY");
+        JsonNode rows = result.get("rows");
+        assertEquals(rows.size(), 1);
+        JsonNode row = rows.get(0);
+        assertEquals(row.size(), 1);
+        assertEquals(row.get(0).size(), 3);
+        assertEquals(row.get(0).get(0).textValue(), "a");
+        assertEquals(row.get(0).get(1).textValue(), "bb");
+        assertEquals(row.get(0).get(2).textValue(), "ccc");
+      }
+    }
   }
 
   @Override
@@ -389,6 +597,8 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
         .addSingleValueDimension(STRING_COLUMN, FieldSpec.DataType.STRING)
         .addSingleValueDimension(TIMESTAMP_COLUMN, FieldSpec.DataType.TIMESTAMP)
         .addSingleValueDimension(GROUP_BY_COLUMN, FieldSpec.DataType.STRING)
+        .addMultiValueDimension(LONG_ARRAY_COLUMN, FieldSpec.DataType.LONG)
+        .addMultiValueDimension(DOUBLE_ARRAY_COLUMN, FieldSpec.DataType.DOUBLE)
         .build();
   }
 
@@ -418,6 +628,12 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
             null, null),
         new org.apache.avro.Schema.Field(GROUP_BY_COLUMN,
             org.apache.avro.Schema.create(org.apache.avro.Schema.Type.STRING),
+            null, null),
+        new org.apache.avro.Schema.Field(LONG_ARRAY_COLUMN,
+            org.apache.avro.Schema.createArray(org.apache.avro.Schema.create(org.apache.avro.Schema.Type.LONG)),
+            null, null),
+        new org.apache.avro.Schema.Field(DOUBLE_ARRAY_COLUMN,
+            org.apache.avro.Schema.createArray(org.apache.avro.Schema.create(org.apache.avro.Schema.Type.DOUBLE)),
             null, null)
     ));
 
@@ -440,6 +656,8 @@ public class ArrayTest extends CustomDataQueryClusterIntegrationTest {
               record.put(STRING_COLUMN, RandomStringUtils.random(finalI));
               record.put(TIMESTAMP_COLUMN, finalI);
               record.put(GROUP_BY_COLUMN, String.valueOf(finalI % 10));
+              record.put(LONG_ARRAY_COLUMN, ImmutableList.of(0, 1, 2, 3));
+              record.put(DOUBLE_ARRAY_COLUMN, ImmutableList.of(0.0, 0.1, 0.2, 0.3));
               return record;
             }
         ));
