@@ -114,7 +114,7 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
 
   @Test
   public void testInvalidDelimiterInHeader() {
-    //setup
+    // setup
     CSVRecordReaderConfig csvRecordReaderConfig = new CSVRecordReaderConfig();
     csvRecordReaderConfig.setMultiValueDelimiter(CSV_MULTI_VALUE_DELIMITER);
     csvRecordReaderConfig.setHeader("col1;col2;col3;col4;col5;col6;col7;col8;col9;col10");
@@ -230,12 +230,12 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setSkipUnParseableLines(true);
     readerConfig.setCommentMarker('#');
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
   }
 
@@ -248,12 +248,12 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     // test using line iterator
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setSkipUnParseableLines(true);
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(5, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(5, genericRows.size());
   }
 
@@ -266,12 +266,12 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     // test using line iterator
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setSkipUnParseableLines(true);
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(2, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(2, genericRows.size());
   }
 
@@ -285,12 +285,12 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setSkipUnParseableLines(true);
     readerConfig.setHeader("id,name");
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
   }
 
@@ -303,12 +303,12 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     // test using line iterator
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setSkipUnParseableLines(true);
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(2, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(2, genericRows.size());
   }
 
@@ -320,7 +320,7 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
 
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setSkipUnParseableLines(true);
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(1, genericRows.size());
   }
 
@@ -331,7 +331,7 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     File dataFile = new File(uri);
 
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
-    readCSVRecords(dataFile, readerConfig, false);
+    readCSVRecords(dataFile, readerConfig, null, false);
   }
 
   @Test
@@ -345,7 +345,7 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     readerConfig.setCommentMarker('#');
     readerConfig.setIgnoreEmptyLines(true);
 
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(7, genericRows.size());
   }
 
@@ -359,7 +359,7 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     readerConfig.setCommentMarker('#');
     readerConfig.setIgnoreEmptyLines(true);
 
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, new GenericRow(), false);
     Assert.assertEquals(7, genericRows.size());
   }
 
@@ -373,10 +373,10 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     readerConfig.setSkipUnParseableLines(true);
     readerConfig.setCommentMarker('#');
     readerConfig.setIgnoreEmptyLines(true);
-    readCSVRecords(dataFile, readerConfig, true);
+    readCSVRecords(dataFile, readerConfig, null, true);
 
     // Start reading again; results should be same
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, new GenericRow(), false);
     Assert.assertEquals(7, genericRows.size());
   }
 
@@ -389,10 +389,10 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setCommentMarker('#');
     readerConfig.setIgnoreEmptyLines(true);
-    readCSVRecords(dataFile, readerConfig, true);
+    readCSVRecords(dataFile, readerConfig, null, true);
 
     // Start reading again; results should be same
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(7, genericRows.size());
   }
 
@@ -407,12 +407,12 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     readerConfig.setHeader("firstName,lastName,id");
     readerConfig.setSkipHeader(true);
     readerConfig.setSkipUnParseableLines(true);
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
   }
 
@@ -426,15 +426,57 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
     readerConfig.setDelimiter('|');
     readerConfig.setSkipUnParseableLines(true);
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
   }
 
+  @Test
+  public void testReadingDataFileWithSpaceAroundHeaderFields()
+      throws URISyntaxException, IOException {
+    URI uri = ClassLoader.getSystemResource("dataFileWithSpaceAroundHeaders.csv").toURI();
+    File dataFile = new File(uri);
+
+    // test using line iterator
+    CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
+    readerConfig.setSkipUnParseableLines(true);
+    readerConfig.setIgnoreSurroundingSpaces(true);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(3, genericRows.size());
+    validateSpaceAroundHeadersAreTrimmed(dataFile, readerConfig);
+
+    // test using default CSVRecordReader
+    readerConfig.setSkipUnParseableLines(false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(3, genericRows.size());
+    validateSpaceAroundHeadersAreTrimmed(dataFile, readerConfig);
+  }
+
+  @Test
+  public void testReadingDataFileWithSpaceAroundHeaderAreRetained()
+      throws URISyntaxException, IOException {
+    URI uri = ClassLoader.getSystemResource("dataFileWithSpaceAroundHeaders.csv").toURI();
+    File dataFile = new File(uri);
+
+    // test using line iterator
+    CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
+    readerConfig.setSkipUnParseableLines(true);
+    readerConfig.setIgnoreSurroundingSpaces(false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(3, genericRows.size());
+    validateSpaceAroundHeadersAreRetained(dataFile, readerConfig);
+
+    // test using default CSVRecordReader
+    readerConfig.setSkipUnParseableLines(false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(3, genericRows.size());
+    validateSpaceAroundHeadersAreRetained(dataFile, readerConfig);
+  }
+
   @Test
   public void testRewindMethodAndSkipHeader()
       throws URISyntaxException, IOException {
@@ -445,30 +487,107 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     readerConfig.setSkipUnParseableLines(true);
     readerConfig.setHeader("id,name");
     readerConfig.setSkipHeader(true);
-    readCSVRecords(dataFile, readerConfig, true);
+    readCSVRecords(dataFile, readerConfig, new GenericRow(), true);
 
     // Start reading again; results should be same
-    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
 
     // test using default CSVRecordReader
     readerConfig.setSkipUnParseableLines(false);
-    readCSVRecords(dataFile, readerConfig, true);
+    readCSVRecords(dataFile, readerConfig, null, true);
 
     // Start reading again; results should be same
-    genericRows = readCSVRecords(dataFile, readerConfig, false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
     Assert.assertEquals(3, genericRows.size());
   }
 
-  private List<GenericRow> readCSVRecords(File dataFile, CSVRecordReaderConfig readerConfig, boolean rewind)
+  @Test
+  public void testReadingDataFileWithPartialLastRow()
+      throws URISyntaxException, IOException {
+    URI uri = ClassLoader.getSystemResource("dataFileWithPartialLastRow.csv").toURI();
+    File dataFile = new File(uri);
+
+    // test using line iterator
+    CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
+    readerConfig.setSkipUnParseableLines(true);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(2, genericRows.size());
+
+    // Note: The default CSVRecordReader cannot handle unparseable rows
+  }
+
+  @Test
+  public void testReadingDataFileWithNoRecords()
+      throws URISyntaxException, IOException {
+    URI uri = ClassLoader.getSystemResource("dataFileWithNoRecords.csv").toURI();
+    File dataFile = new File(uri);
+
+    // test using line iterator
+    CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
+    readerConfig.setSkipUnParseableLines(true);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(0, genericRows.size());
+
+    // Note: The default CSVRecordReader cannot handle unparseable rows
+  }
+
+  @Test
+  public void testReadingDataFileWithNoHeaderAndDataRecordsWithEmptyValues()
+      throws URISyntaxException, IOException {
+    URI uri = ClassLoader.getSystemResource("dataFileWithNoHeader2.csv").toURI();
+    File dataFile = new File(uri);
+
+    // test using line iterator
+    CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
+    readerConfig.setSkipUnParseableLines(true);
+    readerConfig.setHeader("key,num0,num1");
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(4, genericRows.size());
+
+    // test using default CSVRecordReader
+    readerConfig.setSkipUnParseableLines(false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(4, genericRows.size());
+  }
+
+  @Test
+  public void testReadingDataFileWithValidHeaders()
+      throws URISyntaxException, IOException {
+    URI uri = ClassLoader.getSystemResource("dataFileWithValidHeaders.csv").toURI();
+    File dataFile = new File(uri);
+
+    // test using line iterator
+    CSVRecordReaderConfig readerConfig = new CSVRecordReaderConfig();
+    readerConfig.setSkipUnParseableLines(true);
+    // No explicit header is set and attempt to skip the header should be ignored. 1st line would be treated as the
+    // header line.
+    readerConfig.setSkipHeader(false);
+    List<GenericRow> genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(4, genericRows.size());
+
+    // test using default CSVRecordReader
+    readerConfig.setSkipUnParseableLines(false);
+    genericRows = readCSVRecords(dataFile, readerConfig, null, false);
+    Assert.assertEquals(4, genericRows.size());
+  }
+
+  private List<GenericRow> readCSVRecords(File dataFile,
+      CSVRecordReaderConfig readerConfig, GenericRow genericRow, boolean rewind)
       throws IOException {
     List<GenericRow> genericRows = new ArrayList<>();
 
     try (CSVRecordReader recordReader = new CSVRecordReader()) {
       recordReader.init(dataFile, null, readerConfig);
+      GenericRow reuse = new GenericRow();
       while (recordReader.hasNext()) {
-        GenericRow genericRow = recordReader.next();
-        genericRows.add(genericRow);
+        if (genericRow != null) {
+          recordReader.next(reuse);
+          genericRows.add(reuse);
+        } else {
+          GenericRow nextRow = recordReader.next();
+          genericRows.add(nextRow);
+        }
       }
 
       if (rewind) {
@@ -478,4 +597,32 @@ public class CSVRecordReaderTest extends AbstractRecordReaderTest {
     }
     return genericRows;
   }
+
+  private void validateSpaceAroundHeadersAreTrimmed(File dataFile, CSVRecordReaderConfig readerConfig)
+      throws IOException {
+    try (CSVRecordReader recordReader = new CSVRecordReader()) {
+      recordReader.init(dataFile, null, readerConfig);
+      Map<String, Integer> headerMap = recordReader.getCSVHeaderMap();
+      Assert.assertEquals(3, headerMap.size());
+      List<String> headers = List.of("firstName", "lastName", "id");
+      for (String header : headers) {
+        // surrounding spaces in headers are trimmed
+        Assert.assertTrue(headerMap.containsKey(header));
+      }
+    }
+  }
+
+  private void validateSpaceAroundHeadersAreRetained(File dataFile, CSVRecordReaderConfig readerConfig)
+      throws IOException {
+    try (CSVRecordReader recordReader = new CSVRecordReader()) {
+      recordReader.init(dataFile, null, readerConfig);
+      Map<String, Integer> headerMap = recordReader.getCSVHeaderMap();
+      Assert.assertEquals(3, headerMap.size());
+      List<String> headers = List.of(" firstName ", " lastName ", " id");
+      for (String header : headers) {
+        // surrounding spaces in headers are trimmed
+        Assert.assertTrue(headerMap.containsKey(header));
+      }
+    }
+  }
 }
