@@ -19,7 +19,9 @@
 package org.apache.pinot.plugin.minion.tasks;
 
 import java.net.URI;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import org.apache.helix.HelixAdmin;
 import org.apache.helix.model.ExternalView;
@@ -29,6 +31,8 @@ import org.apache.pinot.common.utils.config.InstanceUtils;
 import org.apache.pinot.controller.helix.core.minion.ClusterInfoAccessor;
 import org.apache.pinot.controller.util.ServerSegmentMetadataReader;
 import org.apache.pinot.minion.MinionContext;
+import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.spi.config.table.TableTaskConfig;
 import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.filesystem.LocalPinotFS;
 import org.apache.pinot.spi.filesystem.PinotFS;
@@ -90,8 +94,8 @@ public class MinionTaskUtils {
       String pushMode = IngestionConfigUtils.getPushMode(taskConfigs);
 
       Map<String, String> singleFileGenerationTaskConfig = new HashMap<>(taskConfigs);
-      if (pushMode == null
-          || pushMode.toUpperCase().contentEquals(BatchConfigProperties.SegmentPushType.TAR.toString())) {
+      if (pushMode == null || pushMode.toUpperCase()
+          .contentEquals(BatchConfigProperties.SegmentPushType.TAR.toString())) {
         singleFileGenerationTaskConfig.put(BatchConfigProperties.PUSH_MODE,
             BatchConfigProperties.SegmentPushType.TAR.toString());
       } else {
@@ -144,18 +148,28 @@ public class MinionTaskUtils {
     HelixAdmin helixAdmin = minionContext.getHelixManager().getClusterManagmentTool();
     String clusterName = minionContext.getHelixManager().getClusterName();
 
-    String server = getServer(segmentName, tableNameWithType, helixAdmin, clusterName);
-    InstanceConfig instanceConfig = helixAdmin.getInstanceConfig(clusterName, server);
-    String endpoint = InstanceUtils.getServerAdminEndpoint(instanceConfig);
+    List<String> servers = getServers(segmentName, tableNameWithType, helixAdmin, clusterName);
+    for (String server : servers) {
+      InstanceConfig instanceConfig = helixAdmin.getInstanceConfig(clusterName, server);
+      String endpoint = InstanceUtils.getServerAdminEndpoint(instanceConfig);
 
-    // We only need aggregated table size and the total number of docs/rows. Skipping column related stats, by
-    // passing an empty list.
-    ServerSegmentMetadataReader serverSegmentMetadataReader = new ServerSegmentMetadataReader();
-    return serverSegmentMetadataReader.getValidDocIdsBitmapFromServer(tableNameWithType, segmentName, endpoint,
-        validDocIdsType, 60_000);
+      // We only need aggregated table size and the total number of docs/rows. Skipping column related stats, by
+      // passing an empty list.
+      ServerSegmentMetadataReader serverSegmentMetadataReader = new ServerSegmentMetadataReader();
+      try {
+        return serverSegmentMetadataReader.getValidDocIdsBitmapFromServer(tableNameWithType, segmentName, endpoint,
+            validDocIdsType, 60_000);
+      } catch (Exception e) {
+        LOGGER.warn(
+            String.format("Unable to retrieve validDocIds bitmap for segment: %s from endpoint: %s", segmentName,
+                endpoint), e);
+      }
+    }
+    throw new IllegalStateException(
+        String.format("Unable to retrieve validDocIds bitmap for segment: %s from servers: %s", segmentName, servers));
   }
 
-  public static String getServer(String segmentName, String tableNameWithType, HelixAdmin helixAdmin,
+  public static List<String> getServers(String segmentName, String tableNameWithType, HelixAdmin helixAdmin,
       String clusterName) {
     ExternalView externalView = helixAdmin.getResourceExternalView(clusterName, tableNameWithType);
     if (externalView == null) {
@@ -165,11 +179,31 @@ public class MinionTaskUtils {
     if (instanceStateMap == null) {
       throw new IllegalStateException("Failed to find segment: " + segmentName);
     }
+    ArrayList<String> servers = new ArrayList<>();
     for (Map.Entry<String, String> entry : instanceStateMap.entrySet()) {
       if (entry.getValue().equals(CommonConstants.Helix.StateModel.SegmentStateModel.ONLINE)) {
-        return entry.getKey();
+        servers.add(entry.getKey());
       }
     }
-    throw new IllegalStateException("Failed to find ONLINE server for segment: " + segmentName);
+    if (servers.isEmpty()) {
+      throw new IllegalStateException("Failed to find any ONLINE servers for segment: " + segmentName);
+    }
+    return servers;
+  }
+
+  /**
+   * Extract allowDownloadFromServer config from table task config
+   */
+  public static boolean extractMinionAllowDownloadFromServer(TableConfig tableConfig, String taskType,
+      boolean defaultValue) {
+    TableTaskConfig tableTaskConfig = tableConfig.getTaskConfig();
+    if (tableTaskConfig != null) {
+      Map<String, String> configs = tableTaskConfig.getConfigsForTaskType(taskType);
+      if (configs != null && !configs.isEmpty()) {
+        return Boolean.parseBoolean(
+            configs.getOrDefault(TableTaskConfig.MINION_ALLOW_DOWNLOAD_FROM_SERVER, String.valueOf(defaultValue)));
+      }
+    }
+    return defaultValue;
   }
 }
