@@ -28,6 +28,7 @@ import java.util.Map;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.helix.task.TaskState;
 import org.apache.pinot.common.exception.InvalidConfigException;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
@@ -56,7 +57,8 @@ public class UpsertCompactionTaskGenerator extends BaseTaskGenerator {
   private static final Logger LOGGER = LoggerFactory.getLogger(UpsertCompactionTaskGenerator.class);
   private static final String DEFAULT_BUFFER_PERIOD = "7d";
   private static final double DEFAULT_INVALID_RECORDS_THRESHOLD_PERCENT = 0.0;
-  private static final long DEFAULT_INVALID_RECORDS_THRESHOLD_COUNT = 0;
+  private static final long DEFAULT_INVALID_RECORDS_THRESHOLD_COUNT = 1;
+  private static final int DEFAULT_NUM_SEGMENTS_BATCH_PER_SERVER_REQUEST = 500;
 
   public static class SegmentSelectionResult {
 
@@ -136,15 +138,18 @@ public class UpsertCompactionTaskGenerator extends BaseTaskGenerator {
           new ServerSegmentMetadataReader(_clusterInfoAccessor.getExecutor(),
               _clusterInfoAccessor.getConnectionManager());
 
-      // TODO: currently, we put segmentNames=null to get metadata for all segments. We can change this to get
-      // valid doc id metadata in batches with the loop.
-
       // By default, we use 'snapshot' for validDocIdsType. This means that we will use the validDocIds bitmap from
       // the snapshot from Pinot segment. This will require 'enableSnapshot' from UpsertConfig to be set to true.
       String validDocIdsTypeStr =
           taskConfigs.getOrDefault(UpsertCompactionTask.VALID_DOC_IDS_TYPE, ValidDocIdsType.SNAPSHOT.toString());
       ValidDocIdsType validDocIdsType = ValidDocIdsType.valueOf(validDocIdsTypeStr.toUpperCase());
 
+      // Number of segments to query per server request. If a table has a lot of segments, then we might send a
+      // huge payload to pinot-server in request. Batching the requests will help in reducing the payload size.
+      int numSegmentsBatchPerServerRequest =
+          Integer.parseInt(taskConfigs.getOrDefault(UpsertCompactionTask.NUM_SEGMENTS_BATCH_PER_SERVER_REQUEST,
+              String.valueOf(DEFAULT_NUM_SEGMENTS_BATCH_PER_SERVER_REQUEST)));
+
       // Validate that the snapshot is enabled if validDocIdsType is validDocIdsSnapshot
       if (validDocIdsType == ValidDocIdsType.SNAPSHOT) {
         UpsertConfig upsertConfig = tableConfig.getUpsertConfig();
@@ -162,7 +167,7 @@ public class UpsertCompactionTaskGenerator extends BaseTaskGenerator {
 
       List<ValidDocIdsMetadataInfo> validDocIdsMetadataList =
           serverSegmentMetadataReader.getValidDocIdsMetadataFromServer(tableNameWithType, serverToSegments,
-              serverToEndpoints, null, 60_000, validDocIdsType.toString());
+              serverToEndpoints, null, 60_000, validDocIdsType.toString(), numSegmentsBatchPerServerRequest);
 
       Map<String, SegmentZKMetadata> completedSegmentsMap =
           completedSegments.stream().collect(Collectors.toMap(SegmentZKMetadata::getSegmentName, Function.identity()));
@@ -188,9 +193,8 @@ public class UpsertCompactionTaskGenerator extends BaseTaskGenerator {
           LOGGER.warn("Skipping segment {} for task {} as download url is empty", segment.getSegmentName(), taskType);
           continue;
         }
-        Map<String, String> configs = new HashMap<>();
-        configs.put(MinionConstants.TABLE_NAME_KEY, tableNameWithType);
-        configs.put(MinionConstants.SEGMENT_NAME_KEY, segment.getSegmentName());
+        Map<String, String> configs = new HashMap<>(getBaseTaskConfigs(tableConfig,
+            List.of(segment.getSegmentName())));
         configs.put(MinionConstants.DOWNLOAD_URL_KEY, segment.getDownloadUrl());
         configs.put(MinionConstants.UPLOAD_URL_KEY, _clusterInfoAccessor.getVipUrl() + "/segments");
         configs.put(MinionConstants.ORIGINAL_SEGMENT_CRC_KEY, String.valueOf(segment.getCrc()));
@@ -212,7 +216,7 @@ public class UpsertCompactionTaskGenerator extends BaseTaskGenerator {
     long invalidRecordsThresholdCount = Long.parseLong(
         taskConfigs.getOrDefault(UpsertCompactionTask.INVALID_RECORDS_THRESHOLD_COUNT,
             String.valueOf(DEFAULT_INVALID_RECORDS_THRESHOLD_COUNT)));
-    List<SegmentZKMetadata> segmentsForCompaction = new ArrayList<>();
+    List<Pair<SegmentZKMetadata, Long>> segmentsForCompaction = new ArrayList<>();
     List<String> segmentsForDeletion = new ArrayList<>();
     for (ValidDocIdsMetadataInfo validDocIdsMetadata : validDocIdsMetadataInfoList) {
       long totalInvalidDocs = validDocIdsMetadata.getTotalInvalidDocs();
@@ -235,12 +239,23 @@ public class UpsertCompactionTaskGenerator extends BaseTaskGenerator {
       double invalidRecordPercent = ((double) totalInvalidDocs / totalDocs) * 100;
       if (totalInvalidDocs == totalDocs) {
         segmentsForDeletion.add(segment.getSegmentName());
-      } else if (invalidRecordPercent > invalidRecordsThresholdPercent
-          && totalInvalidDocs > invalidRecordsThresholdCount) {
-        segmentsForCompaction.add(segment);
+      } else if (invalidRecordPercent >= invalidRecordsThresholdPercent
+          && totalInvalidDocs >= invalidRecordsThresholdCount) {
+        segmentsForCompaction.add(Pair.of(segment, totalInvalidDocs));
       }
     }
-    return new SegmentSelectionResult(segmentsForCompaction, segmentsForDeletion);
+    segmentsForCompaction.sort((o1, o2) -> {
+      if (o1.getValue() > o2.getValue()) {
+        return -1;
+      } else if (o1.getValue().equals(o2.getValue())) {
+        return 0;
+      }
+      return 1;
+    });
+
+    return new SegmentSelectionResult(
+        segmentsForCompaction.stream().map(Map.Entry::getKey).collect(Collectors.toList()),
+        segmentsForDeletion);
   }
 
   @VisibleForTesting
