@@ -44,9 +44,8 @@ import org.apache.calcite.sql.validate.SqlValidatorImpl;
 public class Validator extends SqlValidatorImpl {
 
   public Validator(SqlOperatorTable opTab, SqlValidatorCatalogReader catalogReader, RelDataTypeFactory typeFactory) {
-    // TODO: support BABEL validator. Currently parser conformance is set to use BABEL.
     super(opTab, catalogReader, typeFactory,
-        Config.DEFAULT.withSqlConformance(SqlConformanceEnum.LENIENT).withIdentifierExpansion(true));
+        Config.DEFAULT.withConformance(SqlConformanceEnum.BABEL).withIdentifierExpansion(true));
   }
 
   /**
