@@ -19,18 +19,14 @@
 package org.apache.pinot.query.runtime.operator;
 
 import com.google.common.base.Preconditions;
-import com.google.common.collect.ImmutableList;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import javax.annotation.Nullable;
-import org.apache.calcite.rel.hint.PinotHintOptions;
-import org.apache.calcite.sql.SqlKind;
 import org.apache.pinot.common.datablock.DataBlock;
-import org.apache.pinot.common.datatable.DataTable;
-import org.apache.pinot.common.request.Literal;
+import org.apache.pinot.common.datatable.StatMap;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FunctionContext;
 import org.apache.pinot.common.utils.DataSchema;
@@ -43,14 +39,14 @@ import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
 import org.apache.pinot.core.query.aggregation.function.AggregationFunctionFactory;
 import org.apache.pinot.core.query.aggregation.function.CountAggregationFunction;
 import org.apache.pinot.core.util.DataBlockExtractUtils;
-import org.apache.pinot.query.planner.logical.LiteralHintUtils;
+import org.apache.pinot.query.parser.CalciteRexExpressionParser;
 import org.apache.pinot.query.planner.logical.RexExpression;
-import org.apache.pinot.query.planner.plannode.AbstractPlanNode;
-import org.apache.pinot.query.planner.plannode.AggregateNode.AggType;
+import org.apache.pinot.query.planner.plannode.AggregateNode;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
-import org.apache.pinot.query.runtime.blocks.TransferableBlockUtils;
 import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
 import org.roaringbitmap.RoaringBitmap;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 
 /**
@@ -59,71 +55,75 @@ import org.roaringbitmap.RoaringBitmap;
  * When the list of aggregation calls is empty, this class is used to calculate distinct result based on group by keys.
  */
 public class AggregateOperator extends MultiStageOperator {
+  private static final Logger LOGGER = LoggerFactory.getLogger(AggregateOperator.class);
   private static final String EXPLAIN_NAME = "AGGREGATE_OPERATOR";
   private static final CountAggregationFunction COUNT_STAR_AGG_FUNCTION =
       new CountAggregationFunction(Collections.singletonList(ExpressionContext.forIdentifier("*")), false);
-  private static final ExpressionContext PLACEHOLDER_IDENTIFIER = ExpressionContext.forIdentifier("__PLACEHOLDER__");
 
-  private final MultiStageOperator _inputOperator;
+  private final MultiStageOperator _input;
   private final DataSchema _resultSchema;
-  private final AggType _aggType;
   private final MultistageAggregationExecutor _aggregationExecutor;
   private final MultistageGroupByExecutor _groupByExecutor;
+  @Nullable
+  private TransferableBlock _eosBlock;
+  private final StatMap<StatKey> _statMap = new StatMap<>(StatKey.class);
 
   private boolean _hasConstructedAggregateBlock;
 
-  public AggregateOperator(OpChainExecutionContext context, MultiStageOperator inputOperator, DataSchema resultSchema,
-      List<RexExpression> aggCalls, List<RexExpression> groupSet, AggType aggType, List<Integer> filterArgIndices,
-      @Nullable AbstractPlanNode.NodeHint nodeHint) {
+  public AggregateOperator(OpChainExecutionContext context, MultiStageOperator input, AggregateNode node) {
     super(context);
-    _inputOperator = inputOperator;
-    _resultSchema = resultSchema;
-    _aggType = aggType;
-
-    // Process literal hints
-    Map<Integer, Map<Integer, Literal>> literalArgumentsMap = null;
-    if (nodeHint != null) {
-      Map<String, String> aggOptions = nodeHint._hintOptions.get(PinotHintOptions.INTERNAL_AGG_OPTIONS);
-      if (aggOptions != null) {
-        literalArgumentsMap = LiteralHintUtils.hintStringToLiteralMap(
-            aggOptions.get(PinotHintOptions.InternalAggregateOptions.AGG_CALL_SIGNATURE));
-      }
-    }
-    if (literalArgumentsMap == null) {
-      literalArgumentsMap = Collections.emptyMap();
-    }
+    _input = input;
+    _resultSchema = node.getDataSchema();
 
     // Initialize the aggregation functions
-    AggregationFunction<?, ?>[] aggFunctions = getAggFunctions(aggCalls, literalArgumentsMap);
+    AggregationFunction<?, ?>[] aggFunctions = getAggFunctions(node.getAggCalls());
+    int numFunctions = aggFunctions.length;
 
     // Process the filter argument indices
-    int numFunctions = aggFunctions.length;
+    List<Integer> filterArgs = node.getFilterArgs();
     int[] filterArgIds = new int[numFunctions];
     int maxFilterArgId = -1;
     for (int i = 0; i < numFunctions; i++) {
-      filterArgIds[i] = filterArgIndices.get(i);
+      filterArgIds[i] = filterArgs.get(i);
       maxFilterArgId = Math.max(maxFilterArgId, filterArgIds[i]);
     }
 
     // Initialize the appropriate executor.
-    if (groupSet.isEmpty()) {
+    List<Integer> groupKeys = node.getGroupKeys();
+    AggregateNode.AggType aggType = node.getAggType();
+    if (groupKeys.isEmpty()) {
       _aggregationExecutor =
           new MultistageAggregationExecutor(aggFunctions, filterArgIds, maxFilterArgId, aggType, _resultSchema);
       _groupByExecutor = null;
     } else {
       _groupByExecutor =
-          new MultistageGroupByExecutor(getGroupKeyIds(groupSet), aggFunctions, filterArgIds, maxFilterArgId, aggType,
-              _resultSchema, context.getOpChainMetadata(), nodeHint);
+          new MultistageGroupByExecutor(getGroupKeyIds(groupKeys), aggFunctions, filterArgIds, maxFilterArgId, aggType,
+              _resultSchema, context.getOpChainMetadata(), node.getNodeHint());
       _aggregationExecutor = null;
     }
   }
 
   @Override
+  public void registerExecution(long time, int numRows) {
+    _statMap.merge(StatKey.EXECUTION_TIME_MS, time);
+    _statMap.merge(StatKey.EMITTED_ROWS, numRows);
+  }
+
+  @Override
+  public Type getOperatorType() {
+    return Type.AGGREGATE;
+  }
+
+  @Override
+  protected Logger logger() {
+    return LOGGER;
+  }
+
+  @Override
   public List<MultiStageOperator> getChildOperators() {
-    return ImmutableList.of(_inputOperator);
+    return List.of(_input);
   }
 
-  @Nullable
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
@@ -132,13 +132,16 @@ public class AggregateOperator extends MultiStageOperator {
   @Override
   protected TransferableBlock getNextBlock() {
     if (_hasConstructedAggregateBlock) {
-      return TransferableBlockUtils.getEndOfStreamTransferableBlock();
+      assert _eosBlock != null;
+      return _eosBlock;
     }
     TransferableBlock finalBlock = _aggregationExecutor != null ? consumeAggregation() : consumeGroupBy();
     // returning upstream error block if finalBlock contains error.
     if (finalBlock.isErrorBlock()) {
       return finalBlock;
     }
+    assert finalBlock.isSuccessfulEndOfStreamBlock() : "Final block must be EOS block";
+    _eosBlock = updateEosBlock(finalBlock, _statMap);
     return produceAggregatedBlock();
   }
 
@@ -149,13 +152,12 @@ public class AggregateOperator extends MultiStageOperator {
     } else {
       List<Object[]> rows = _groupByExecutor.getResult();
       if (rows.isEmpty()) {
-        return TransferableBlockUtils.getEndOfStreamTransferableBlock();
+        return _eosBlock;
       } else {
         TransferableBlock dataBlock = new TransferableBlock(rows, _resultSchema, DataBlock.Type.ROW);
         if (_groupByExecutor.isNumGroupsLimitReached()) {
-          OperatorStats operatorStats = _opChainStats.getOperatorStats(_context, _operatorId);
-          operatorStats.recordSingleStat(DataTable.MetadataKey.NUM_GROUPS_LIMIT_REACHED.getName(), "true");
-          _inputOperator.earlyTerminate();
+          _statMap.merge(StatKey.NUM_GROUPS_LIMIT_REACHED, true);
+          _input.earlyTerminate();
         }
         return dataBlock;
       }
@@ -164,51 +166,42 @@ public class AggregateOperator extends MultiStageOperator {
 
   /**
    * Consumes the input blocks as a group by
+   *
    * @return the last block, which must always be either an error or the end of the stream
    */
   private TransferableBlock consumeGroupBy() {
-    TransferableBlock block = _inputOperator.nextBlock();
+    TransferableBlock block = _input.nextBlock();
     while (block.isDataBlock()) {
       _groupByExecutor.processBlock(block);
-      block = _inputOperator.nextBlock();
+      block = _input.nextBlock();
     }
     return block;
   }
 
   /**
    * Consumes the input blocks as an aggregation
+   *
    * @return the last block, which must always be either an error or the end of the stream
    */
   private TransferableBlock consumeAggregation() {
-    TransferableBlock block = _inputOperator.nextBlock();
+    TransferableBlock block = _input.nextBlock();
     while (block.isDataBlock()) {
       _aggregationExecutor.processBlock(block);
-      block = _inputOperator.nextBlock();
+      block = _input.nextBlock();
     }
     return block;
   }
 
-  private AggregationFunction<?, ?>[] getAggFunctions(List<RexExpression> aggCalls,
-      Map<Integer, Map<Integer, Literal>> literalArgumentsMap) {
+  private AggregationFunction<?, ?>[] getAggFunctions(List<RexExpression.FunctionCall> aggCalls) {
     int numFunctions = aggCalls.size();
     AggregationFunction<?, ?>[] aggFunctions = new AggregationFunction[numFunctions];
-    if (!_aggType.isInputIntermediateFormat()) {
-      for (int i = 0; i < numFunctions; i++) {
-        Map<Integer, Literal> literalArguments = literalArgumentsMap.getOrDefault(i, Collections.emptyMap());
-        aggFunctions[i] = getAggFunctionForRawInput((RexExpression.FunctionCall) aggCalls.get(i), literalArguments);
-      }
-    } else {
-      for (int i = 0; i < numFunctions; i++) {
-        Map<Integer, Literal> literalArguments = literalArgumentsMap.getOrDefault(i, Collections.emptyMap());
-        aggFunctions[i] =
-            getAggFunctionForIntermediateInput((RexExpression.FunctionCall) aggCalls.get(i), literalArguments);
-      }
+    for (int i = 0; i < numFunctions; i++) {
+      aggFunctions[i] = getAggFunction(aggCalls.get(i));
     }
     return aggFunctions;
   }
 
-  private AggregationFunction<?, ?> getAggFunctionForRawInput(RexExpression.FunctionCall functionCall,
-      Map<Integer, Literal> literalArguments) {
+  private AggregationFunction<?, ?> getAggFunction(RexExpression.FunctionCall functionCall) {
     String functionName = functionCall.getFunctionName();
     List<RexExpression> operands = functionCall.getFunctionOperands();
     int numArguments = operands.size();
@@ -218,66 +211,20 @@ public class AggregateOperator extends MultiStageOperator {
       return COUNT_STAR_AGG_FUNCTION;
     }
     List<ExpressionContext> arguments = new ArrayList<>(numArguments);
-    for (int i = 0; i < numArguments; i++) {
-      Literal literalArgument = literalArguments.get(i);
-      if (literalArgument != null) {
-        arguments.add(ExpressionContext.forLiteralContext(literalArgument));
+    for (RexExpression operand : operands) {
+      if (operand instanceof RexExpression.InputRef) {
+        RexExpression.InputRef inputRef = (RexExpression.InputRef) operand;
+        arguments.add(ExpressionContext.forIdentifier(fromColIdToIdentifier(inputRef.getIndex())));
       } else {
-        RexExpression operand = operands.get(i);
-        switch (operand.getKind()) {
-          case INPUT_REF:
-            RexExpression.InputRef inputRef = (RexExpression.InputRef) operand;
-            arguments.add(ExpressionContext.forIdentifier(fromColIdToIdentifier(inputRef.getIndex())));
-            break;
-          case LITERAL:
-            RexExpression.Literal literalRexExp = (RexExpression.Literal) operand;
-            arguments.add(ExpressionContext.forLiteralContext(literalRexExp.getDataType().toDataType(),
-                literalRexExp.getValue()));
-            break;
-          default:
-            throw new IllegalStateException("Illegal aggregation function operand type: " + operand.getKind());
-        }
+        assert operand instanceof RexExpression.Literal;
+        arguments.add(
+            ExpressionContext.forLiteral(CalciteRexExpressionParser.toLiteral((RexExpression.Literal) operand)));
       }
     }
-
     return AggregationFunctionFactory.getAggregationFunction(
         new FunctionContext(FunctionContext.Type.AGGREGATION, functionName, arguments), true);
   }
 
-  private static AggregationFunction<?, ?> getAggFunctionForIntermediateInput(RexExpression.FunctionCall functionCall,
-      Map<Integer, Literal> literalArguments) {
-    String functionName = functionCall.getFunctionName();
-    List<RexExpression> operands = functionCall.getFunctionOperands();
-    int numArguments = operands.size();
-    Preconditions.checkState(numArguments == 1, "Intermediate aggregate must have 1 argument, got: %s", numArguments);
-    RexExpression operand = operands.get(0);
-    Preconditions.checkState(operand.getKind() == SqlKind.INPUT_REF,
-        "Intermediate aggregate argument must be an input reference, got: %s", operand.getKind());
-    // We might need to append extra arguments extracted from the hint to match the signature of the aggregation
-    Literal numArgumentsLiteral = literalArguments.get(-1);
-    if (numArgumentsLiteral == null) {
-      return AggregationFunctionFactory.getAggregationFunction(
-          new FunctionContext(FunctionContext.Type.AGGREGATION, functionName, Collections.singletonList(
-              ExpressionContext.forIdentifier(fromColIdToIdentifier(((RexExpression.InputRef) operand).getIndex())))),
-          true);
-    } else {
-      int numExpectedArguments = numArgumentsLiteral.getIntValue();
-      List<ExpressionContext> arguments = new ArrayList<>(numExpectedArguments);
-      arguments.add(
-          ExpressionContext.forIdentifier(fromColIdToIdentifier(((RexExpression.InputRef) operand).getIndex())));
-      for (int i = 1; i < numExpectedArguments; i++) {
-        Literal literalArgument = literalArguments.get(i);
-        if (literalArgument != null) {
-          arguments.add(ExpressionContext.forLiteralContext(literalArgument));
-        } else {
-          arguments.add(PLACEHOLDER_IDENTIFIER);
-        }
-      }
-      return AggregationFunctionFactory.getAggregationFunction(
-          new FunctionContext(FunctionContext.Type.AGGREGATION, functionName, arguments), true);
-    }
-  }
-
   private static String fromColIdToIdentifier(int colId) {
     return "$" + colId;
   }
@@ -288,14 +235,11 @@ public class AggregateOperator extends MultiStageOperator {
     return Integer.parseInt(identifier.substring(1));
   }
 
-  private int[] getGroupKeyIds(List<RexExpression> groupSet) {
-    int numKeys = groupSet.size();
+  private int[] getGroupKeyIds(List<Integer> groupKeys) {
+    int numKeys = groupKeys.size();
     int[] groupKeyIds = new int[numKeys];
     for (int i = 0; i < numKeys; i++) {
-      RexExpression rexExp = groupSet.get(i);
-      Preconditions.checkState(rexExp.getKind() == SqlKind.INPUT_REF, "Group key must be an input reference, got: %s",
-          rexExp.getKind());
-      groupKeyIds[i] = ((RexExpression.InputRef) rexExp).getIndex();
+      groupKeyIds[i] = groupKeys.get(i);
     }
     return groupKeyIds;
   }
@@ -407,4 +351,33 @@ public class AggregateOperator extends MultiStageOperator {
       return DataBlockExtractUtils.extractColumn(block.getDataBlock(), colId);
     }
   }
+
+  public enum StatKey implements StatMap.Key {
+    //@formatter:off
+    EXECUTION_TIME_MS(StatMap.Type.LONG) {
+      @Override
+      public boolean includeDefaultInJson() {
+        return true;
+      }
+    },
+    EMITTED_ROWS(StatMap.Type.LONG) {
+      @Override
+      public boolean includeDefaultInJson() {
+        return true;
+      }
+    },
+    NUM_GROUPS_LIMIT_REACHED(StatMap.Type.BOOLEAN);
+    //@formatter:on
+
+    private final StatMap.Type _type;
+
+    StatKey(StatMap.Type type) {
+      _type = type;
+    }
+
+    @Override
+    public StatMap.Type getType() {
+      return _type;
+    }
+  }
 }
