@@ -19,19 +19,20 @@
 package org.apache.pinot.query.runtime.operator;
 
 import com.google.common.base.Preconditions;
-import com.google.common.collect.ImmutableList;
 import java.util.ArrayList;
 import java.util.List;
-import javax.annotation.Nullable;
 import org.apache.pinot.common.datablock.DataBlock;
+import org.apache.pinot.common.datatable.StatMap;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
-import org.apache.pinot.query.planner.logical.RexExpression;
+import org.apache.pinot.query.planner.plannode.FilterNode;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperand;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperandFactory;
 import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
 import org.apache.pinot.spi.utils.BooleanUtils;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 
 /*
@@ -48,28 +49,45 @@ import org.apache.pinot.spi.utils.BooleanUtils;
     Note: Scalar functions are the ones we have in v1 engine and only do function name and arg # matching.
  */
 public class FilterOperator extends MultiStageOperator {
+
+  private static final Logger LOGGER = LoggerFactory.getLogger(FilterOperator.class);
   private static final String EXPLAIN_NAME = "FILTER";
 
-  private final MultiStageOperator _upstreamOperator;
+  private final MultiStageOperator _input;
   private final TransformOperand _filterOperand;
   private final DataSchema _dataSchema;
+  private final StatMap<StatKey> _statMap = new StatMap<>(StatKey.class);
 
-  public FilterOperator(OpChainExecutionContext context, MultiStageOperator upstreamOperator, DataSchema dataSchema,
-      RexExpression filter) {
+  public FilterOperator(OpChainExecutionContext context, MultiStageOperator input, FilterNode node) {
     super(context);
-    _upstreamOperator = upstreamOperator;
-    _dataSchema = dataSchema;
-    _filterOperand = TransformOperandFactory.getTransformOperand(filter, dataSchema);
+    _input = input;
+    _dataSchema = node.getDataSchema();
+    _filterOperand = TransformOperandFactory.getTransformOperand(node.getCondition(), _dataSchema);
     Preconditions.checkState(_filterOperand.getResultType() == ColumnDataType.BOOLEAN,
         "Filter operand must return BOOLEAN, got: %s", _filterOperand.getResultType());
   }
 
   @Override
+  public void registerExecution(long time, int numRows) {
+    _statMap.merge(StatKey.EXECUTION_TIME_MS, time);
+    _statMap.merge(StatKey.EMITTED_ROWS, numRows);
+  }
+
+  @Override
+  public Type getOperatorType() {
+    return Type.FILTER;
+  }
+
+  @Override
+  protected Logger logger() {
+    return LOGGER;
+  }
+
+  @Override
   public List<MultiStageOperator> getChildOperators() {
-    return ImmutableList.of(_upstreamOperator);
+    return List.of(_input);
   }
 
-  @Nullable
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
@@ -77,9 +95,12 @@ public class FilterOperator extends MultiStageOperator {
 
   @Override
   protected TransferableBlock getNextBlock() {
-    TransferableBlock block = _upstreamOperator.nextBlock();
+    TransferableBlock block = _input.nextBlock();
     if (block.isEndOfStreamBlock()) {
-      return block;
+      if (block.isErrorBlock()) {
+        return block;
+      }
+      return updateEosBlock(block, _statMap);
     }
     List<Object[]> resultRows = new ArrayList<>();
     for (Object[] row : block.getContainer()) {
@@ -90,4 +111,32 @@ public class FilterOperator extends MultiStageOperator {
     }
     return new TransferableBlock(resultRows, _dataSchema, DataBlock.Type.ROW);
   }
+
+  public enum StatKey implements StatMap.Key {
+    //@formatter:off
+    EXECUTION_TIME_MS(StatMap.Type.LONG) {
+      @Override
+      public boolean includeDefaultInJson() {
+        return true;
+      }
+    },
+    EMITTED_ROWS(StatMap.Type.LONG) {
+      @Override
+      public boolean includeDefaultInJson() {
+        return true;
+      }
+    };
+    //@formatter:on
+
+    private final StatMap.Type _type;
+
+    StatKey(StatMap.Type type) {
+      _type = type;
+    }
+
+    @Override
+    public StatMap.Type getType() {
+      return _type;
+    }
+  }
 }
