@@ -19,8 +19,6 @@
 package org.apache.pinot.query.runtime.operator;
 
 import com.google.common.base.Preconditions;
-import com.google.common.collect.ImmutableList;
-import com.google.common.collect.ImmutableSet;
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.HashMap;
@@ -29,9 +27,9 @@ import java.util.Map;
 import java.util.Set;
 import javax.annotation.Nullable;
 import org.apache.calcite.rel.core.JoinRelType;
-import org.apache.calcite.rel.hint.PinotHintOptions;
+import org.apache.pinot.calcite.rel.hint.PinotHintOptions;
 import org.apache.pinot.common.datablock.DataBlock;
-import org.apache.pinot.common.datatable.DataTable;
+import org.apache.pinot.common.datatable.StatMap;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.response.ProcessingException;
 import org.apache.pinot.common.utils.DataSchema;
@@ -39,16 +37,19 @@ import org.apache.pinot.common.utils.config.QueryOptionsUtils;
 import org.apache.pinot.query.planner.logical.RexExpression;
 import org.apache.pinot.query.planner.partitioning.KeySelector;
 import org.apache.pinot.query.planner.partitioning.KeySelectorFactory;
-import org.apache.pinot.query.planner.plannode.AbstractPlanNode;
 import org.apache.pinot.query.planner.plannode.JoinNode;
+import org.apache.pinot.query.planner.plannode.PlanNode;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
 import org.apache.pinot.query.runtime.blocks.TransferableBlockUtils;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperand;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperandFactory;
+import org.apache.pinot.query.runtime.plan.MultiStageQueryStats;
 import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
 import org.apache.pinot.spi.utils.BooleanUtils;
+import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.MultiStageQueryRunner.JoinOverFlowMode;
-
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * This basic {@code BroadcastJoinOperator} implement a basic broadcast join algorithm.
@@ -65,13 +66,14 @@ import org.apache.pinot.spi.utils.CommonConstants.MultiStageQueryRunner.JoinOver
 // TODO: Move inequi out of hashjoin. (https://github.com/apache/pinot/issues/9728)
 // TODO: Support memory size based resource limit.
 public class HashJoinOperator extends MultiStageOperator {
+  private static final Logger LOGGER = LoggerFactory.getLogger(HashJoinOperator.class);
   private static final String EXPLAIN_NAME = "HASH_JOIN";
   private static final int INITIAL_HEURISTIC_SIZE = 16;
   private static final int DEFAULT_MAX_ROWS_IN_JOIN = 1024 * 1024; // 2^20, around 1MM rows
   private static final JoinOverFlowMode DEFAULT_JOIN_OVERFLOW_MODE = JoinOverFlowMode.THROW;
 
   private static final Set<JoinRelType> SUPPORTED_JOIN_TYPES =
-      ImmutableSet.of(JoinRelType.INNER, JoinRelType.LEFT, JoinRelType.RIGHT, JoinRelType.FULL, JoinRelType.SEMI,
+      Set.of(JoinRelType.INNER, JoinRelType.LEFT, JoinRelType.RIGHT, JoinRelType.FULL, JoinRelType.SEMI,
           JoinRelType.ANTI);
 
   private final Map<Object, ArrayList<Object[]>> _broadcastRightTable;
@@ -80,14 +82,15 @@ public class HashJoinOperator extends MultiStageOperator {
   // Only used for right join and full join to output non-matched right rows.
   private final Map<Object, BitSet> _matchedRightRows;
 
-  private final MultiStageOperator _leftTableOperator;
-  private final MultiStageOperator _rightTableOperator;
+  private final MultiStageOperator _leftInput;
+  private final MultiStageOperator _rightInput;
   private final JoinRelType _joinType;
   private final DataSchema _resultSchema;
   private final int _leftColumnSize;
   private final int _resultColumnSize;
-  private final List<TransformOperand> _joinClauseEvaluators;
+  private final List<TransformOperand> _nonEquiEvaluators;
   private boolean _isHashTableBuilt;
+  private final StatMap<StatKey> _statMap = new StatMap<>(StatKey.class);
 
   // Used by non-inner join.
   // Needed to indicate we have finished processing all results after returning last block.
@@ -110,29 +113,30 @@ public class HashJoinOperator extends MultiStageOperator {
    */
   private final JoinOverFlowMode _joinOverflowMode;
 
-  private int _currentRowsInHashTable = 0;
+  private int _currentRowsInHashTable;
+  private MultiStageQueryStats _leftSideStats;
+  private MultiStageQueryStats _rightSideStats;
 
-  public HashJoinOperator(OpChainExecutionContext context, MultiStageOperator leftTableOperator,
-      MultiStageOperator rightTableOperator, DataSchema leftSchema, JoinNode node) {
+  public HashJoinOperator(OpChainExecutionContext context, MultiStageOperator leftInput, DataSchema leftSchema,
+      MultiStageOperator rightInput, JoinNode node) {
     super(context);
-    Preconditions.checkState(SUPPORTED_JOIN_TYPES.contains(node.getJoinRelType()),
-        "Join type: " + node.getJoinRelType() + " is not supported!");
-    _joinType = node.getJoinRelType();
-    JoinNode.JoinKeys joinKeys = node.getJoinKeys();
-    _leftKeySelector = KeySelectorFactory.getKeySelector(joinKeys.getLeftKeys());
-    _rightKeySelector = KeySelectorFactory.getKeySelector(joinKeys.getRightKeys());
+    Preconditions.checkState(SUPPORTED_JOIN_TYPES.contains(node.getJoinType()),
+        "Join type: " + node.getJoinType() + " is not supported!");
+    _joinType = node.getJoinType();
+    _leftKeySelector = KeySelectorFactory.getKeySelector(node.getLeftKeys());
+    _rightKeySelector = KeySelectorFactory.getKeySelector(node.getRightKeys());
     _leftColumnSize = leftSchema.size();
-    Preconditions.checkState(_leftColumnSize > 0, "leftColumnSize has to be greater than zero:" + _leftColumnSize);
     _resultSchema = node.getDataSchema();
     _resultColumnSize = _resultSchema.size();
     Preconditions.checkState(_resultColumnSize >= _leftColumnSize,
-        "Result column size" + _leftColumnSize + " has to be greater than or equal to left column size:"
-            + _leftColumnSize);
-    _leftTableOperator = leftTableOperator;
-    _rightTableOperator = rightTableOperator;
-    _joinClauseEvaluators = new ArrayList<>(node.getJoinClauses().size());
-    for (RexExpression joinClause : node.getJoinClauses()) {
-      _joinClauseEvaluators.add(TransformOperandFactory.getTransformOperand(joinClause, _resultSchema));
+        "Result column size: %s has to be greater than or equal to left column size: %s", _resultColumnSize,
+        _leftColumnSize);
+    _leftInput = leftInput;
+    _rightInput = rightInput;
+    List<RexExpression> nonEquiConditions = node.getNonEquiConditions();
+    _nonEquiEvaluators = new ArrayList<>(nonEquiConditions.size());
+    for (RexExpression nonEquiCondition : nonEquiConditions) {
+      _nonEquiEvaluators.add(TransformOperandFactory.getTransformOperand(nonEquiCondition, _resultSchema));
     }
     _isHashTableBuilt = false;
     _broadcastRightTable = new HashMap<>();
@@ -142,13 +146,30 @@ public class HashJoinOperator extends MultiStageOperator {
       _matchedRightRows = null;
     }
     Map<String, String> metadata = context.getOpChainMetadata();
-    _maxRowsInHashTable = getMaxRowInJoin(metadata, node.getJoinHints());
-    _joinOverflowMode = getJoinOverflowMode(metadata, node.getJoinHints());
+    PlanNode.NodeHint nodeHint = node.getNodeHint();
+    _maxRowsInHashTable = getMaxRowInJoin(metadata, nodeHint);
+    _joinOverflowMode = getJoinOverflowMode(metadata, nodeHint);
   }
 
-  private int getMaxRowInJoin(Map<String, String> opChainMetadata, @Nullable AbstractPlanNode.NodeHint nodeHint) {
+  @Override
+  public void registerExecution(long time, int numRows) {
+    _statMap.merge(StatKey.EXECUTION_TIME_MS, time);
+    _statMap.merge(StatKey.EMITTED_ROWS, numRows);
+  }
+
+  @Override
+  public Type getOperatorType() {
+    return Type.HASH_JOIN;
+  }
+
+  @Override
+  protected Logger logger() {
+    return LOGGER;
+  }
+
+  private int getMaxRowInJoin(Map<String, String> opChainMetadata, @Nullable PlanNode.NodeHint nodeHint) {
     if (nodeHint != null) {
-      Map<String, String> joinOptions = nodeHint._hintOptions.get(PinotHintOptions.JOIN_HINT_OPTIONS);
+      Map<String, String> joinOptions = nodeHint.getHintOptions().get(PinotHintOptions.JOIN_HINT_OPTIONS);
       if (joinOptions != null) {
         String maxRowsInJoinStr = joinOptions.get(PinotHintOptions.JoinHintOptions.MAX_ROWS_IN_JOIN);
         if (maxRowsInJoinStr != null) {
@@ -161,9 +182,9 @@ public class HashJoinOperator extends MultiStageOperator {
   }
 
   private JoinOverFlowMode getJoinOverflowMode(Map<String, String> contextMetadata,
-      @Nullable AbstractPlanNode.NodeHint nodeHint) {
+      @Nullable PlanNode.NodeHint nodeHint) {
     if (nodeHint != null) {
-      Map<String, String> joinOptions = nodeHint._hintOptions.get(PinotHintOptions.JOIN_HINT_OPTIONS);
+      Map<String, String> joinOptions = nodeHint.getHintOptions().get(PinotHintOptions.JOIN_HINT_OPTIONS);
       if (joinOptions != null) {
         String joinOverflowModeStr = joinOptions.get(PinotHintOptions.JoinHintOptions.JOIN_OVERFLOW_MODE);
         if (joinOverflowModeStr != null) {
@@ -175,13 +196,11 @@ public class HashJoinOperator extends MultiStageOperator {
     return joinOverflowMode != null ? joinOverflowMode : DEFAULT_JOIN_OVERFLOW_MODE;
   }
 
-  // TODO: Separate left and right table operator.
   @Override
   public List<MultiStageOperator> getChildOperators() {
-    return ImmutableList.of(_leftTableOperator, _rightTableOperator);
+    return List.of(_leftInput, _rightInput);
   }
 
-  @Nullable
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
@@ -191,7 +210,8 @@ public class HashJoinOperator extends MultiStageOperator {
   protected TransferableBlock getNextBlock()
       throws ProcessingException {
     if (_isTerminated) {
-      return TransferableBlockUtils.getEndOfStreamTransferableBlock();
+      assert _leftSideStats != null;
+      return TransferableBlockUtils.getEndOfStreamTransferableBlock(_leftSideStats);
     }
     if (!_isHashTableBuilt) {
       // Build JOIN hash table
@@ -200,14 +220,15 @@ public class HashJoinOperator extends MultiStageOperator {
     if (_upstreamErrorBlock != null) {
       return _upstreamErrorBlock;
     }
-    TransferableBlock leftBlock = _leftTableOperator.nextBlock();
+    TransferableBlock leftBlock = _leftInput.nextBlock();
     // JOIN each left block with the constructed right hash table.
     return buildJoinedDataBlock(leftBlock);
   }
 
   private void buildBroadcastHashTable()
       throws ProcessingException {
-    TransferableBlock rightBlock = _rightTableOperator.nextBlock();
+    long startTime = System.currentTimeMillis();
+    TransferableBlock rightBlock = _rightInput.nextBlock();
     while (!TransferableBlockUtils.isEndOfStream(rightBlock)) {
       List<Object[]> container = rightBlock.getContainer();
       // Row based overflow check.
@@ -216,17 +237,24 @@ public class HashJoinOperator extends MultiStageOperator {
           ProcessingException resourceLimitExceededException =
               new ProcessingException(QueryException.SERVER_RESOURCE_LIMIT_EXCEEDED_ERROR_CODE);
           resourceLimitExceededException.setMessage(
-              "Cannot build in memory hash table for join operator, reach number of rows limit: "
-                  + _maxRowsInHashTable);
+              "Cannot build in memory hash table for join operator, reach number of rows limit: " + _maxRowsInHashTable
+                  + ". Consider increasing the limit for the maximum number of rows in a join either via the query "
+                  + "option '" + CommonConstants.Broker.Request.QueryOptionKey.MAX_ROWS_IN_JOIN + "' or the '"
+                  + PinotHintOptions.JoinHintOptions.MAX_ROWS_IN_JOIN + "' hint in the '"
+                  + PinotHintOptions.JOIN_HINT_OPTIONS + "'. Alternatively, if partial results are acceptable, the join"
+                  + " overflow mode can be set to '" + JoinOverFlowMode.BREAK.name() + "' either via the query option '"
+                  + CommonConstants.Broker.Request.QueryOptionKey.JOIN_OVERFLOW_MODE + "' or the '"
+                  + PinotHintOptions.JoinHintOptions.JOIN_OVERFLOW_MODE + "' hint in the '"
+                  + PinotHintOptions.JOIN_HINT_OPTIONS + "'. Furthermore, if there is a large disparity in the size of "
+                  + "the two tables being joined, use the smaller table as the right input instead of the left.");
           throw resourceLimitExceededException;
         } else {
           // Just fill up the buffer.
           int remainingRows = _maxRowsInHashTable - _currentRowsInHashTable;
           container = container.subList(0, remainingRows);
-          OperatorStats operatorStats = _opChainStats.getOperatorStats(_context, _operatorId);
-          operatorStats.recordSingleStat(DataTable.MetadataKey.MAX_ROWS_IN_JOIN_REACHED.getName(), "true");
+          _statMap.merge(StatKey.MAX_ROWS_IN_JOIN_REACHED, true);
           // setting only the rightTableOperator to be early terminated and awaits EOS block next.
-          _rightTableOperator.earlyTerminate();
+          _rightInput.earlyTerminate();
         }
       }
       // put all the rows into corresponding hash collections keyed by the key selector function.
@@ -240,13 +268,16 @@ public class HashJoinOperator extends MultiStageOperator {
         hashCollection.add(row);
       }
       _currentRowsInHashTable += container.size();
-      rightBlock = _rightTableOperator.nextBlock();
+      rightBlock = _rightInput.nextBlock();
     }
     if (rightBlock.isErrorBlock()) {
       _upstreamErrorBlock = rightBlock;
     } else {
       _isHashTableBuilt = true;
+      _rightSideStats = rightBlock.getQueryStats();
+      assert _rightSideStats != null;
     }
+    _statMap.merge(StatKey.TIME_BUILDING_HASH_TABLE_MS, System.currentTimeMillis() - startTime);
   }
 
   private TransferableBlock buildJoinedDataBlock(TransferableBlock leftBlock) {
@@ -255,8 +286,13 @@ public class HashJoinOperator extends MultiStageOperator {
       return _upstreamErrorBlock;
     }
     if (leftBlock.isSuccessfulEndOfStreamBlock()) {
+      assert _rightSideStats != null;
+      _leftSideStats = leftBlock.getQueryStats();
+      assert _leftSideStats != null;
+      _leftSideStats.mergeInOrder(_rightSideStats, getOperatorType(), _statMap);
+
       if (!needUnmatchedRightRows()) {
-        return leftBlock;
+        return TransferableBlockUtils.getEndOfStreamTransferableBlock(_leftSideStats);
       }
       // TODO: Moved to a different function.
       // Return remaining non-matched rows for non-inner join.
@@ -334,7 +370,7 @@ public class HashJoinOperator extends MultiStageOperator {
         Object[] rightRow = rightRows.get(i);
         // TODO: Optimize this to avoid unnecessary object copy.
         Object[] resultRow = joinRow(leftRow, rightRow);
-        if (_joinClauseEvaluators.isEmpty() || _joinClauseEvaluators.stream()
+        if (_nonEquiEvaluators.isEmpty() || _nonEquiEvaluators.stream()
             .allMatch(evaluator -> BooleanUtils.isTrueInternalValue(evaluator.apply(resultRow)))) {
           rows.add(resultRow);
           hasMatchForLeftRow = true;
@@ -390,4 +426,37 @@ public class HashJoinOperator extends MultiStageOperator {
   private boolean needUnmatchedLeftRows() {
     return _joinType == JoinRelType.LEFT || _joinType == JoinRelType.FULL;
   }
+
+  public enum StatKey implements StatMap.Key {
+    //@formatter:off
+    EXECUTION_TIME_MS(StatMap.Type.LONG) {
+      @Override
+      public boolean includeDefaultInJson() {
+        return true;
+      }
+    },
+    EMITTED_ROWS(StatMap.Type.LONG) {
+      @Override
+      public boolean includeDefaultInJson() {
+        return true;
+      }
+    },
+    MAX_ROWS_IN_JOIN_REACHED(StatMap.Type.BOOLEAN),
+    /**
+     * How long (CPU time) has been spent on building the hash table.
+     */
+    TIME_BUILDING_HASH_TABLE_MS(StatMap.Type.LONG);
+    //@formatter:on
+
+    private final StatMap.Type _type;
+
+    StatKey(StatMap.Type type) {
+      _type = type;
+    }
+
+    @Override
+    public StatMap.Type getType() {
+      return _type;
+    }
+  }
 }
