@@ -18,10 +18,12 @@
  */
 package org.apache.pinot.query.runtime.operator;
 
-import org.apache.calcite.rel.RelDistribution;
 import org.apache.pinot.query.mailbox.ReceivingMailbox;
+import org.apache.pinot.query.planner.plannode.MailboxReceiveNode;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
 import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 
 /**
@@ -29,10 +31,11 @@ import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
  * {@link MultiStageOperator#getNextBlock()} API.
  */
 public class MailboxReceiveOperator extends BaseMailboxReceiveOperator {
+  private static final Logger LOGGER = LoggerFactory.getLogger(MailboxReceiveOperator.class);
   private static final String EXPLAIN_NAME = "MAILBOX_RECEIVE";
 
-  public MailboxReceiveOperator(OpChainExecutionContext context, RelDistribution.Type exchangeType, int senderStageId) {
-    super(context, exchangeType, senderStageId);
+  public MailboxReceiveOperator(OpChainExecutionContext context, MailboxReceiveNode node) {
+    super(context, node);
   }
 
   @Override
@@ -40,15 +43,23 @@ public class MailboxReceiveOperator extends BaseMailboxReceiveOperator {
     return EXPLAIN_NAME;
   }
 
+  @Override
+  protected Logger logger() {
+    return LOGGER;
+  }
+
   @Override
   protected TransferableBlock getNextBlock() {
-    TransferableBlock block = getMultiConsumer().readBlockBlocking();
+    TransferableBlock block = _multiConsumer.readBlockBlocking();
     // When early termination flag is set, caller is expecting an EOS block to be returned, however since the 2 stages
     // between sending/receiving mailbox are setting early termination flag asynchronously, there's chances that the
     // next block pulled out of the ReceivingMailbox to be an already buffered normal data block. This requires the
     // MailboxReceiveOperator to continue pulling and dropping data block until an EOS block is observed.
     while (_isEarlyTerminated && !block.isEndOfStreamBlock()) {
-      block = getMultiConsumer().readBlockBlocking();
+      block = _multiConsumer.readBlockBlocking();
+    }
+    if (block.isSuccessfulEndOfStreamBlock()) {
+      updateEosBlock(block, _statMap);
     }
     return block;
   }
