@@ -18,28 +18,28 @@
  */
 package org.apache.pinot.query.runtime.operator;
 
-import com.google.common.collect.ImmutableList;
-import com.google.common.collect.ImmutableMap;
-import java.util.Arrays;
-import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import org.apache.calcite.rel.RelDistribution;
+import org.apache.calcite.rel.RelFieldCollation;
 import org.apache.calcite.rel.RelFieldCollation.Direction;
 import org.apache.calcite.rel.RelFieldCollation.NullDirection;
+import org.apache.pinot.common.datatable.StatMap;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.query.mailbox.MailboxService;
 import org.apache.pinot.query.mailbox.ReceivingMailbox;
-import org.apache.pinot.query.planner.logical.RexExpression;
 import org.apache.pinot.query.planner.physical.MailboxIdUtils;
+import org.apache.pinot.query.planner.plannode.MailboxReceiveNode;
 import org.apache.pinot.query.routing.MailboxInfo;
 import org.apache.pinot.query.routing.MailboxInfos;
 import org.apache.pinot.query.routing.SharedMailboxInfos;
 import org.apache.pinot.query.routing.StageMetadata;
 import org.apache.pinot.query.routing.WorkerMetadata;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
+import org.apache.pinot.query.runtime.blocks.TransferableBlockTestUtils;
 import org.apache.pinot.query.runtime.blocks.TransferableBlockUtils;
 import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
 import org.mockito.Mock;
@@ -52,6 +52,7 @@ import org.testng.annotations.Test;
 import static org.apache.pinot.common.utils.DataSchema.ColumnDataType.INT;
 import static org.apache.pinot.common.utils.DataSchema.ColumnDataType.STRING;
 import static org.mockito.Mockito.eq;
+import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertTrue;
@@ -60,9 +61,8 @@ import static org.testng.Assert.assertTrue;
 public class SortedMailboxReceiveOperatorTest {
   private static final DataSchema DATA_SCHEMA =
       new DataSchema(new String[]{"col1", "col2"}, new DataSchema.ColumnDataType[]{INT, INT});
-  private static final List<RexExpression> COLLATION_KEYS = Collections.singletonList(new RexExpression.InputRef(0));
-  private static final List<Direction> COLLATION_DIRECTIONS = Collections.singletonList(Direction.ASCENDING);
-  private static final List<NullDirection> COLLATION_NULL_DIRECTIONS = Collections.singletonList(NullDirection.LAST);
+  private static final List<RelFieldCollation> FIELD_COLLATIONS =
+      List.of(new RelFieldCollation(0, Direction.ASCENDING, NullDirection.LAST));
   private static final String MAILBOX_ID_1 = MailboxIdUtils.toMailboxId(0, 1, 0, 0, 0);
   private static final String MAILBOX_ID_2 = MailboxIdUtils.toMailboxId(0, 1, 1, 0, 0);
 
@@ -79,14 +79,13 @@ public class SortedMailboxReceiveOperatorTest {
 
   @BeforeClass
   public void setUp() {
-    MailboxInfos mailboxInfosBoth = new SharedMailboxInfos(new MailboxInfo("localhost", 1234, ImmutableList.of(0, 1)));
-    _stageMetadataBoth = new StageMetadata(0, Stream.of(0, 1)
-        .map(workerId -> new WorkerMetadata(workerId, ImmutableMap.of(1, mailboxInfosBoth), ImmutableMap.of()))
-        .collect(Collectors.toList()), ImmutableMap.of());
-    MailboxInfos mailboxInfos1 = new SharedMailboxInfos(new MailboxInfo("localhost", 1234, ImmutableList.of(0)));
-    _stageMetadata1 = new StageMetadata(0,
-        ImmutableList.of(new WorkerMetadata(0, ImmutableMap.of(1, mailboxInfos1), ImmutableMap.of())),
-        ImmutableMap.of());
+    MailboxInfos mailboxInfosBoth = new SharedMailboxInfos(new MailboxInfo("localhost", 1234, List.of(0, 1)));
+    _stageMetadataBoth = new StageMetadata(0,
+        Stream.of(0, 1).map(workerId -> new WorkerMetadata(workerId, Map.of(1, mailboxInfosBoth), Map.of()))
+            .collect(Collectors.toList()), Map.of());
+    MailboxInfos mailboxInfos1 = new SharedMailboxInfos(new MailboxInfo("localhost", 1234, List.of(0)));
+    _stageMetadata1 =
+        new StageMetadata(0, List.of(new WorkerMetadata(0, Map.of(1, mailboxInfos1), Map.of())), Map.of());
   }
 
   @BeforeMethod
@@ -94,6 +93,8 @@ public class SortedMailboxReceiveOperatorTest {
     _mocks = MockitoAnnotations.openMocks(this);
     when(_mailboxService.getHostname()).thenReturn("localhost");
     when(_mailboxService.getPort()).thenReturn(1234);
+    when(_mailbox1.getStatMap()).thenReturn(new StatMap<>(ReceivingMailbox.StatKey.class));
+    when(_mailbox2.getStatMap()).thenReturn(new StatMap<>(ReceivingMailbox.StatKey.class));
   }
 
   @AfterMethod
@@ -104,35 +105,21 @@ public class SortedMailboxReceiveOperatorTest {
 
   @Test(expectedExceptions = IllegalStateException.class, expectedExceptionsMessageRegExp = ".*RANGE_DISTRIBUTED.*")
   public void shouldThrowRangeDistributionNotSupported() {
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadata1);
-    //noinspection resource
-    new SortedMailboxReceiveOperator(context, RelDistribution.Type.RANGE_DISTRIBUTED, DATA_SCHEMA, COLLATION_KEYS,
-        COLLATION_DIRECTIONS, COLLATION_NULL_DIRECTIONS, false, 1);
+    getOperator(_stageMetadata1, RelDistribution.Type.RANGE_DISTRIBUTED);
   }
 
-  @Test(expectedExceptions = IllegalStateException.class, expectedExceptionsMessageRegExp = "Collation keys.*")
+  @Test(expectedExceptions = IllegalStateException.class, expectedExceptionsMessageRegExp = "Field collations.*")
   public void shouldThrowOnEmptyCollationKey() {
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_1))).thenReturn(_mailbox1);
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadata1);
-    //noinspection resource
-    new SortedMailboxReceiveOperator(context, RelDistribution.Type.SINGLETON, DATA_SCHEMA, Collections.emptyList(),
-        Collections.emptyList(), Collections.emptyList(), false, 1);
+    getOperator(_stageMetadata1, RelDistribution.Type.SINGLETON, DATA_SCHEMA, List.of(), Long.MAX_VALUE);
   }
 
   @Test
-  public void shouldTimeout()
-      throws InterruptedException {
+  public void shouldTimeout() {
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_1))).thenReturn(_mailbox1);
-
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, System.currentTimeMillis() + 1000L, _stageMetadata1);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.SINGLETON, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS, COLLATION_NULL_DIRECTIONS,
-        false, 1)) {
-      Thread.sleep(100L);
-      TransferableBlock block = receiveOp.nextBlock();
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadata1, RelDistribution.Type.SINGLETON,
+        DATA_SCHEMA, FIELD_COLLATIONS, System.currentTimeMillis() + 100L)) {
+      TransferableBlock block = operator.nextBlock();
       assertTrue(block.isErrorBlock());
       assertTrue(block.getExceptions().containsKey(QueryException.EXECUTION_TIMEOUT_ERROR_CODE));
     }
@@ -141,13 +128,9 @@ public class SortedMailboxReceiveOperatorTest {
   @Test
   public void shouldReceiveEosDirectlyFromSender() {
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_1))).thenReturn(_mailbox1);
-    when(_mailbox1.poll()).thenReturn(TransferableBlockUtils.getEndOfStreamTransferableBlock());
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadata1);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.SINGLETON, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS, COLLATION_NULL_DIRECTIONS,
-        false, 1)) {
-      assertTrue(receiveOp.nextBlock().isEndOfStreamBlock());
+    when(_mailbox1.poll()).thenReturn(TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadata1, RelDistribution.Type.SINGLETON)) {
+      assertTrue(operator.nextBlock().isSuccessfulEndOfStreamBlock());
     }
   }
 
@@ -156,16 +139,12 @@ public class SortedMailboxReceiveOperatorTest {
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_1))).thenReturn(_mailbox1);
     Object[] row = new Object[]{1, 1};
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadata1);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.SINGLETON, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS, COLLATION_NULL_DIRECTIONS,
-        false, 1)) {
-      List<Object[]> actualRows = receiveOp.nextBlock().getContainer();
-      assertEquals(actualRows.size(), 1);
-      assertEquals(actualRows.get(0), row);
-      assertTrue(receiveOp.nextBlock().isEndOfStreamBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadata1, RelDistribution.Type.SINGLETON)) {
+      List<Object[]> resultRows = operator.nextBlock().getContainer();
+      assertEquals(resultRows.size(), 1);
+      assertEquals(resultRows.get(0), row);
+      assertTrue(operator.nextBlock().isSuccessfulEndOfStreamBlock());
     }
   }
 
@@ -175,12 +154,8 @@ public class SortedMailboxReceiveOperatorTest {
     String errorMessage = "TEST ERROR";
     when(_mailbox1.poll()).thenReturn(
         TransferableBlockUtils.getErrorTransferableBlock(new RuntimeException(errorMessage)));
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadata1);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.SINGLETON, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS, COLLATION_NULL_DIRECTIONS,
-        false, 1)) {
-      TransferableBlock block = receiveOp.nextBlock();
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadata1, RelDistribution.Type.SINGLETON)) {
+      TransferableBlock block = operator.nextBlock();
       assertTrue(block.isErrorBlock());
       assertTrue(block.getExceptions().get(QueryException.UNKNOWN_ERROR_CODE).contains(errorMessage));
     }
@@ -189,23 +164,17 @@ public class SortedMailboxReceiveOperatorTest {
   @Test
   public void shouldReceiveMailboxFromTwoServersOneNull() {
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_1))).thenReturn(_mailbox1);
-    when(_mailbox1.poll()).thenReturn(null, TransferableBlockUtils.getEndOfStreamTransferableBlock());
+    when(_mailbox1.poll()).thenReturn(null, TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_2))).thenReturn(_mailbox2);
     Object[] row = new Object[]{1, 1};
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadataBoth);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.HASH_DISTRIBUTED, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS,
-        COLLATION_NULL_DIRECTIONS, false, 1)) {
-      TransferableBlock firstBlock = receiveOp.nextBlock();
-      List<Object[]> actualRows = firstBlock.getContainer();
-      assertEquals(actualRows.size(), 1);
-      assertEquals(actualRows.get(0), row);
-
-      TransferableBlock secondBlock = receiveOp.nextBlock();
-      assertTrue(secondBlock.isEndOfStreamBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadataBoth,
+        RelDistribution.Type.HASH_DISTRIBUTED)) {
+      List<Object[]> resultRows = operator.nextBlock().getContainer();
+      assertEquals(resultRows.size(), 1);
+      assertEquals(resultRows.get(0), row);
+      assertTrue(operator.nextBlock().isSuccessfulEndOfStreamBlock());
     }
   }
 
@@ -218,13 +187,10 @@ public class SortedMailboxReceiveOperatorTest {
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_2))).thenReturn(_mailbox2);
     Object[] row = new Object[]{3, 3};
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadataBoth);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.HASH_DISTRIBUTED, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS,
-        COLLATION_NULL_DIRECTIONS, false, 1)) {
-      TransferableBlock block = receiveOp.nextBlock();
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadataBoth,
+        RelDistribution.Type.HASH_DISTRIBUTED)) {
+      TransferableBlock block = operator.nextBlock();
       assertTrue(block.isErrorBlock());
       assertTrue(block.getExceptions().get(QueryException.UNKNOWN_ERROR_CODE).contains(errorMessage));
     }
@@ -236,21 +202,18 @@ public class SortedMailboxReceiveOperatorTest {
     Object[] row1 = new Object[]{3, 3};
     Object[] row2 = new Object[]{1, 1};
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row1),
-        OperatorTestUtil.block(DATA_SCHEMA, row2), TransferableBlockUtils.getEndOfStreamTransferableBlock());
+        OperatorTestUtil.block(DATA_SCHEMA, row2), TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_2))).thenReturn(_mailbox2);
     Object[] row3 = new Object[]{4, 2};
     Object[] row4 = new Object[]{2, 4};
     Object[] row5 = new Object[]{-1, 95};
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row3),
         OperatorTestUtil.block(DATA_SCHEMA, row4), OperatorTestUtil.block(DATA_SCHEMA, row5),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadataBoth);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.HASH_DISTRIBUTED, DATA_SCHEMA, COLLATION_KEYS, COLLATION_DIRECTIONS,
-        COLLATION_NULL_DIRECTIONS, false, 1)) {
-      assertEquals(receiveOp.nextBlock().getContainer(), Arrays.asList(row5, row2, row4, row1, row3));
-      assertTrue(receiveOp.nextBlock().isEndOfStreamBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadataBoth,
+        RelDistribution.Type.HASH_DISTRIBUTED)) {
+      assertEquals(operator.nextBlock().getContainer(), List.of(row5, row2, row4, row1, row3));
+      assertTrue(operator.nextBlock().isSuccessfulEndOfStreamBlock());
     }
   }
 
@@ -258,30 +221,39 @@ public class SortedMailboxReceiveOperatorTest {
   public void shouldReceiveMailboxFromTwoServersWithCollationKeyTwoColumns() {
     DataSchema dataSchema =
         new DataSchema(new String[]{"col1", "col2", "col3"}, new DataSchema.ColumnDataType[]{INT, INT, STRING});
-    List<RexExpression> collationKeys = Arrays.asList(new RexExpression.InputRef(2), new RexExpression.InputRef(0));
-    List<Direction> collationDirections = Arrays.asList(Direction.DESCENDING, Direction.ASCENDING);
-    List<NullDirection> collationNullDirections = Arrays.asList(NullDirection.FIRST, NullDirection.LAST);
-
+    List<RelFieldCollation> collations = List.of(new RelFieldCollation(2, Direction.DESCENDING, NullDirection.FIRST),
+        new RelFieldCollation(0, Direction.ASCENDING, NullDirection.LAST));
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_1))).thenReturn(_mailbox1);
     Object[] row1 = new Object[]{3, 3, "queen"};
     Object[] row2 = new Object[]{1, 1, "pink floyd"};
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(dataSchema, row1),
-        OperatorTestUtil.block(dataSchema, row2), TransferableBlockUtils.getEndOfStreamTransferableBlock());
+        OperatorTestUtil.block(dataSchema, row2), TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
     when(_mailboxService.getReceivingMailbox(eq(MAILBOX_ID_2))).thenReturn(_mailbox2);
     Object[] row3 = new Object[]{4, 2, "pink floyd"};
     Object[] row4 = new Object[]{2, 4, "aerosmith"};
     Object[] row5 = new Object[]{-1, 95, "foo fighters"};
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(dataSchema, row3),
         OperatorTestUtil.block(dataSchema, row4), OperatorTestUtil.block(dataSchema, row5),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
-
-    OpChainExecutionContext context =
-        OperatorTestUtil.getOpChainContext(_mailboxService, Long.MAX_VALUE, _stageMetadataBoth);
-    try (SortedMailboxReceiveOperator receiveOp = new SortedMailboxReceiveOperator(context,
-        RelDistribution.Type.HASH_DISTRIBUTED, dataSchema, collationKeys, collationDirections, collationNullDirections,
-        false, 1)) {
-      assertEquals(receiveOp.nextBlock().getContainer(), Arrays.asList(row1, row2, row3, row5, row4));
-      assertTrue(receiveOp.nextBlock().isEndOfStreamBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
+    try (SortedMailboxReceiveOperator operator = getOperator(_stageMetadataBoth, RelDistribution.Type.HASH_DISTRIBUTED,
+        dataSchema, collations, Long.MAX_VALUE)) {
+      assertEquals(operator.nextBlock().getContainer(), List.of(row1, row2, row3, row5, row4));
+      assertTrue(operator.nextBlock().isSuccessfulEndOfStreamBlock());
     }
   }
+
+  private SortedMailboxReceiveOperator getOperator(StageMetadata stageMetadata, RelDistribution.Type distributionType,
+      DataSchema resultSchema, List<RelFieldCollation> collations, long deadlineMs) {
+    OpChainExecutionContext context = OperatorTestUtil.getOpChainContext(_mailboxService, deadlineMs, stageMetadata);
+    MailboxReceiveNode node = mock(MailboxReceiveNode.class);
+    when(node.getDistributionType()).thenReturn(distributionType);
+    when(node.getSenderStageId()).thenReturn(1);
+    when(node.getDataSchema()).thenReturn(resultSchema);
+    when(node.getCollations()).thenReturn(collations);
+    return new SortedMailboxReceiveOperator(context, node);
+  }
+
+  private SortedMailboxReceiveOperator getOperator(StageMetadata stageMetadata, RelDistribution.Type distributionType) {
+    return getOperator(stageMetadata, distributionType, DATA_SCHEMA, FIELD_COLLATIONS, Long.MAX_VALUE);
+  }
 }
