@@ -27,21 +27,23 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import org.apache.calcite.rel.RelDistribution;
 import org.apache.calcite.rel.core.JoinRelType;
-import org.apache.calcite.rel.logical.PinotRelExchangeType;
+import org.apache.pinot.calcite.rel.logical.PinotRelExchangeType;
+import org.apache.pinot.common.datatable.StatMap;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.query.mailbox.MailboxService;
 import org.apache.pinot.query.mailbox.ReceivingMailbox;
 import org.apache.pinot.query.planner.physical.MailboxIdUtils;
 import org.apache.pinot.query.planner.plannode.JoinNode;
 import org.apache.pinot.query.planner.plannode.MailboxReceiveNode;
+import org.apache.pinot.query.planner.plannode.PlanNode;
 import org.apache.pinot.query.routing.MailboxInfo;
 import org.apache.pinot.query.routing.MailboxInfos;
 import org.apache.pinot.query.routing.SharedMailboxInfos;
 import org.apache.pinot.query.routing.StageMetadata;
 import org.apache.pinot.query.routing.StagePlan;
-import org.apache.pinot.query.routing.VirtualServerAddress;
 import org.apache.pinot.query.routing.WorkerMetadata;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
+import org.apache.pinot.query.runtime.blocks.TransferableBlockTestUtils;
 import org.apache.pinot.query.runtime.blocks.TransferableBlockUtils;
 import org.apache.pinot.query.runtime.executor.ExecutorServiceUtils;
 import org.apache.pinot.query.runtime.executor.OpChainSchedulerService;
@@ -64,7 +66,6 @@ public class PipelineBreakerExecutorTest {
   private static final String MAILBOX_ID_1 = MailboxIdUtils.toMailboxId(0, 1, 0, 0, 0);
   private static final String MAILBOX_ID_2 = MailboxIdUtils.toMailboxId(0, 2, 0, 0, 0);
 
-  private final VirtualServerAddress _server = new VirtualServerAddress("localhost", 123, 0);
   private final ExecutorService _executor = Executors.newCachedThreadPool();
   private final OpChainSchedulerService _scheduler = new OpChainSchedulerService(_executor);
   private final MailboxInfos _mailboxInfos =
@@ -89,7 +90,9 @@ public class PipelineBreakerExecutorTest {
     when(_mailboxService.getPort()).thenReturn(123);
 
     when(_mailbox1.getId()).thenReturn("mailbox1");
+    when(_mailbox1.getStatMap()).thenReturn(new StatMap<>(ReceivingMailbox.StatKey.class));
     when(_mailbox2.getId()).thenReturn("mailbox2");
+    when(_mailbox2.getStatMap()).thenReturn(new StatMap<>(ReceivingMailbox.StatKey.class));
   }
 
   @AfterMethod
@@ -105,9 +108,7 @@ public class PipelineBreakerExecutorTest {
 
   @Test
   public void shouldReturnBlocksUponNormalOperation() {
-    MailboxReceiveNode mailboxReceiveNode =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 1, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
+    MailboxReceiveNode mailboxReceiveNode = getPBReceiveNode(1);
     StagePlan stagePlan = new StagePlan(mailboxReceiveNode, _stageMetadata);
 
     // when
@@ -116,7 +117,7 @@ public class PipelineBreakerExecutorTest {
     Object[] row2 = new Object[]{2, 3};
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row1),
         OperatorTestUtil.block(DATA_SCHEMA, row2),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock(OperatorTestUtil.getDummyStats(0, 1, _server)));
+        TransferableBlockUtils.getEndOfStreamTransferableBlock(OperatorTestUtil.getDummyStats(1)));
 
     PipelineBreakerResult pipelineBreakerResult =
         PipelineBreakerExecutor.executePipelineBreakers(_scheduler, _mailboxService, _workerMetadata, stagePlan,
@@ -130,22 +131,18 @@ public class PipelineBreakerExecutorTest {
     Assert.assertEquals(pipelineBreakerResult.getResultMap().values().iterator().next().size(), 2);
 
     // should collect stats from previous stage here
-    Assert.assertNotNull(pipelineBreakerResult.getOpChainStats());
-    Assert.assertEquals(pipelineBreakerResult.getOpChainStats().getOperatorStatsMap().size(), 1);
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats());
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats().getUpstreamStageStats(1),
+        "Stats for stage 1 should be sent");
   }
 
   @Test
   public void shouldWorkWithMultiplePBNodeUponNormalOperation() {
-    MailboxReceiveNode mailboxReceiveNode1 =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 1, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
-    MailboxReceiveNode mailboxReceiveNode2 =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 2, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
+    MailboxReceiveNode mailboxReceiveNode1 = getPBReceiveNode(1);
+    MailboxReceiveNode mailboxReceiveNode2 = getPBReceiveNode(2);
     JoinNode joinNode =
-        new JoinNode(0, DATA_SCHEMA, DATA_SCHEMA, DATA_SCHEMA, JoinRelType.INNER, null, null, ImmutableList.of());
-    joinNode.addInput(mailboxReceiveNode1);
-    joinNode.addInput(mailboxReceiveNode2);
+        new JoinNode(0, DATA_SCHEMA, PlanNode.NodeHint.EMPTY, List.of(mailboxReceiveNode1, mailboxReceiveNode2),
+            JoinRelType.INNER, List.of(0), List.of(0), List.of());
     StagePlan stagePlan = new StagePlan(joinNode, _stageMetadata);
 
     // when
@@ -154,9 +151,9 @@ public class PipelineBreakerExecutorTest {
     Object[] row1 = new Object[]{1, 1};
     Object[] row2 = new Object[]{2, 3};
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row1),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock(OperatorTestUtil.getDummyStats(0, 1, _server)));
+        TransferableBlockUtils.getEndOfStreamTransferableBlock(OperatorTestUtil.getDummyStats(1)));
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row2),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock(OperatorTestUtil.getDummyStats(0, 2, _server)));
+        TransferableBlockUtils.getEndOfStreamTransferableBlock(OperatorTestUtil.getDummyStats(2)));
 
     PipelineBreakerResult pipelineBreakerResult =
         PipelineBreakerExecutor.executePipelineBreakers(_scheduler, _mailboxService, _workerMetadata, stagePlan,
@@ -173,15 +170,16 @@ public class PipelineBreakerExecutorTest {
     Assert.assertFalse(it.hasNext());
 
     // should collect stats from previous stage here
-    Assert.assertNotNull(pipelineBreakerResult.getOpChainStats());
-    Assert.assertEquals(pipelineBreakerResult.getOpChainStats().getOperatorStatsMap().size(), 2);
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats());
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats().getUpstreamStageStats(1),
+        "Stats for stage 1 should be sent");
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats().getUpstreamStageStats(2),
+        "Stats for stage 2 should be sent");
   }
 
   @Test
   public void shouldReturnEmptyBlockWhenPBExecuteWithIncorrectMailboxNode() {
-    MailboxReceiveNode incorrectlyConfiguredMailboxNode =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 3, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
+    MailboxReceiveNode incorrectlyConfiguredMailboxNode = getPBReceiveNode(3);
     StagePlan stagePlan = new StagePlan(incorrectlyConfiguredMailboxNode, _stageMetadata);
 
     // when
@@ -197,14 +195,12 @@ public class PipelineBreakerExecutorTest {
     List<TransferableBlock> resultBlocks = pipelineBreakerResult.getResultMap().values().iterator().next();
     Assert.assertEquals(resultBlocks.size(), 0);
 
-    Assert.assertNotNull(pipelineBreakerResult.getOpChainStats());
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats());
   }
 
   @Test
   public void shouldReturnErrorBlocksFailureWhenPBTimeout() {
-    MailboxReceiveNode mailboxReceiveNode =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 1, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
+    MailboxReceiveNode mailboxReceiveNode = getPBReceiveNode(1);
     StagePlan stagePlan = new StagePlan(mailboxReceiveNode, _stageMetadata);
 
     // when
@@ -212,7 +208,7 @@ public class PipelineBreakerExecutorTest {
     CountDownLatch latch = new CountDownLatch(1);
     when(_mailbox1.poll()).thenAnswer(invocation -> {
       latch.await();
-      return TransferableBlockUtils.getEndOfStreamTransferableBlock();
+      return TransferableBlockTestUtils.getEndOfStreamTransferableBlock(1);
     });
 
     PipelineBreakerResult pipelineBreakerResult =
@@ -231,16 +227,11 @@ public class PipelineBreakerExecutorTest {
 
   @Test
   public void shouldReturnWhenAnyPBReturnsEmpty() {
-    MailboxReceiveNode mailboxReceiveNode1 =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 1, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
-    MailboxReceiveNode incorrectlyConfiguredMailboxNode =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 3, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
-    JoinNode joinNode =
-        new JoinNode(0, DATA_SCHEMA, DATA_SCHEMA, DATA_SCHEMA, JoinRelType.INNER, null, null, ImmutableList.of());
-    joinNode.addInput(mailboxReceiveNode1);
-    joinNode.addInput(incorrectlyConfiguredMailboxNode);
+    MailboxReceiveNode mailboxReceiveNode1 = getPBReceiveNode(1);
+    MailboxReceiveNode incorrectlyConfiguredMailboxNode = getPBReceiveNode(3);
+    JoinNode joinNode = new JoinNode(0, DATA_SCHEMA, PlanNode.NodeHint.EMPTY,
+        List.of(mailboxReceiveNode1, incorrectlyConfiguredMailboxNode), JoinRelType.INNER, List.of(0), List.of(0),
+        List.of());
     StagePlan stagePlan = new StagePlan(joinNode, _stageMetadata);
 
     // when
@@ -249,9 +240,9 @@ public class PipelineBreakerExecutorTest {
     Object[] row1 = new Object[]{1, 1};
     Object[] row2 = new Object[]{2, 3};
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row1),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(1));
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row2),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(1));
 
     PipelineBreakerResult pipelineBreakerResult =
         PipelineBreakerExecutor.executePipelineBreakers(_scheduler, _mailboxService, _workerMetadata, stagePlan,
@@ -264,21 +255,16 @@ public class PipelineBreakerExecutorTest {
     Assert.assertEquals(pipelineBreakerResult.getResultMap().get(0).size(), 1);
     Assert.assertEquals(pipelineBreakerResult.getResultMap().get(1).size(), 0);
 
-    Assert.assertNotNull(pipelineBreakerResult.getOpChainStats());
+    Assert.assertNotNull(pipelineBreakerResult.getStageQueryStats());
   }
 
   @Test
   public void shouldReturnErrorBlocksWhenReceivedErrorFromSender() {
-    MailboxReceiveNode mailboxReceiveNode1 =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 1, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
-    MailboxReceiveNode incorrectlyConfiguredMailboxNode =
-        new MailboxReceiveNode(0, DATA_SCHEMA, 2, RelDistribution.Type.SINGLETON, PinotRelExchangeType.PIPELINE_BREAKER,
-            null, null, false, false, null);
-    JoinNode joinNode =
-        new JoinNode(0, DATA_SCHEMA, DATA_SCHEMA, DATA_SCHEMA, JoinRelType.INNER, null, null, ImmutableList.of());
-    joinNode.addInput(mailboxReceiveNode1);
-    joinNode.addInput(incorrectlyConfiguredMailboxNode);
+    MailboxReceiveNode mailboxReceiveNode1 = getPBReceiveNode(1);
+    MailboxReceiveNode incorrectlyConfiguredMailboxNode = getPBReceiveNode(2);
+    JoinNode joinNode = new JoinNode(0, DATA_SCHEMA, PlanNode.NodeHint.EMPTY,
+        List.of(mailboxReceiveNode1, incorrectlyConfiguredMailboxNode), JoinRelType.INNER, List.of(0), List.of(0),
+        List.of());
     StagePlan stagePlan = new StagePlan(joinNode, _stageMetadata);
 
     // when
@@ -289,7 +275,7 @@ public class PipelineBreakerExecutorTest {
     when(_mailbox1.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row1),
         TransferableBlockUtils.getErrorTransferableBlock(new RuntimeException("ERROR ON 1")));
     when(_mailbox2.poll()).thenReturn(OperatorTestUtil.block(DATA_SCHEMA, row2),
-        TransferableBlockUtils.getEndOfStreamTransferableBlock());
+        TransferableBlockTestUtils.getEndOfStreamTransferableBlock(0));
 
     PipelineBreakerResult pipelineBreakerResult =
         PipelineBreakerExecutor.executePipelineBreakers(_scheduler, _mailboxService, _workerMetadata, stagePlan,
@@ -302,4 +288,9 @@ public class PipelineBreakerExecutorTest {
     Assert.assertNotNull(errorBlock);
     Assert.assertTrue(errorBlock.isErrorBlock());
   }
+
+  private static MailboxReceiveNode getPBReceiveNode(int senderStageId) {
+    return new MailboxReceiveNode(0, DATA_SCHEMA, List.of(), senderStageId, PinotRelExchangeType.PIPELINE_BREAKER,
+        RelDistribution.Type.SINGLETON, null, null, false, false, null);
+  }
 }
