@@ -24,6 +24,7 @@ import java.io.File;
 import java.io.IOException;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.stream.Collectors;
 import org.apache.commons.io.FileUtils;
@@ -71,23 +72,22 @@ public class JsonIndexTest {
     FileUtils.deleteDirectory(INDEX_DIR);
   }
 
-
   @Test
   public void testSmallIndex()
       throws Exception {
     // @formatter: off
     // CHECKSTYLE:OFF
     String[] records = new String[]{
-        "{" + "\"name\":\"adam\"," + "\"age\":20," + "\"addresses\":["
+        "{" + "\"name\":\"adam\"," + "\"age\":20," + "\"score\":1.25," + "\"addresses\":["
             + "   {\"street\":\"street-00\",\"country\":\"us\"}," + "   {\"street\":\"street-01\",\"country\":\"us\"},"
             + "   {\"street\":\"street-02\",\"country\":\"ca\"}]," + "\"skills\":[\"english\",\"programming\"]" + "}",
-        "{" + "\"name\":\"bob\"," + "\"age\":25," + "\"addresses\":["
+        "{" + "\"name\":\"bob\"," + "\"age\":25," + "\"score\":1.94," + "\"addresses\":["
             + "   {\"street\":\"street-10\",\"country\":\"ca\"}," + "   {\"street\":\"street-11\",\"country\":\"us\"},"
             + "   {\"street\":\"street-12\",\"country\":\"in\"}]," + "\"skills\":[]" + "}",
-        "{" + "\"name\":\"charles\"," + "\"age\":30," + "\"addresses\":["
+        "{" + "\"name\":\"charles\"," + "\"age\":30," + "\"score\":0.90,"  + "\"addresses\":["
             + "   {\"street\":\"street-20\",\"country\":\"jp\"}," + "   {\"street\":\"street-21\",\"country\":\"kr\"},"
             + "   {\"street\":\"street-22\",\"country\":\"cn\"}]," + "\"skills\":[\"japanese\",\"korean\",\"chinese\"]"
-            + "}", "{" + "\"name\":\"david\"," + "\"age\":35," + "\"addresses\":["
+            + "}", "{" + "\"name\":\"david\"," + "\"age\":35," + "\"score\":0.9999,"  + "\"addresses\":["
         + "   {\"street\":\"street-30\",\"country\":\"ca\",\"types\":[\"home\",\"office\"]},"
         + "   {\"street\":\"street-31\",\"country\":\"ca\"}," + "   {\"street\":\"street-32\",\"country\":\"ca\"}],"
         + "\"skills\":null" + "}"
@@ -120,12 +120,54 @@ public class JsonIndexTest {
         matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].street\" = 'street-21'");
         Assert.assertEquals(matchingDocIds.toArray(), new int[]{2});
 
+        matchingDocIds = getMatchingDocIds(indexReader, "REGEXP_LIKE(\"addresses[*].street\", 'street-2.*')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{2});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"age\" > 25");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{2, 3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"age\" >= 25");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{1, 2, 3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"age\" < 25");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"age\" <= 25");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"name\" > 'adam'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{1, 2, 3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"name\" > 'a'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2, 3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"score\" > 1");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"score\" > 1.0");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"score\" > 0.99");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "REGEXP_LIKE(\"score\", '[0-1]\\.[6-9].*')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{1, 2, 3});
+
         matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].street\" NOT IN ('street-10', 'street-22')");
-        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 3});
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2, 3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].country\" != 'ca'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"skills[*]\" NOT IN ('english', 'japanese')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 2});
 
         matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[0].country\" IN ('ca', 'us')");
         Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 3});
 
+        matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[0].country\" NOT IN ('ca', 'us')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{2});
+
         matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].types[1]\" = 'office'");
         Assert.assertEquals(matchingDocIds.toArray(), new int[]{3});
 
@@ -138,8 +180,11 @@ public class JsonIndexTest {
         matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].types[*]\" IS NULL");
         Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2});
 
-        matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[1].types[*]\" IS NULL");
-        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2, 3});
+        matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].types[*]\" IS NOT NULL");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{3});
+
+        matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[1].types[*]\" IS NOT NULL");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[0]);
 
         matchingDocIds = getMatchingDocIds(indexReader, "abc IS NULL");
         Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2, 3});
@@ -153,6 +198,45 @@ public class JsonIndexTest {
 
         matchingDocIds = getMatchingDocIds(indexReader, "\"addresses[*].country\" = 'us' OR \"skills[*]\" IS NOT NULL");
         Assert.assertEquals(matchingDocIds.toArray(), new int[]{0, 1, 2});
+
+        // Nested exclusive predicates
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"addresses[0].street\" = 'street-00' AND \"addresses[0].country\" != 'ca'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0});
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"age\" = '20' AND \"addresses[*].country\" NOT IN ('us')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{0});
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"age\" = '20' AND \"addresses[*].country\" NOT IN ('us', 'ca')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[0]);
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"addresses[*].street\" = 'street-21' AND \"addresses[*].country\" != 'kr'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[0]);
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"addresses[*].street\" = 'street-21' AND \"addresses[*].country\" != 'us'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{2});
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"addresses[*].street\" = 'street-30' AND \"addresses[*].country\" NOT IN ('us', 'kr')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{3});
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "REGEXP_LIKE(\"addresses[*].street\", 'street-0.*') AND \"addresses[*].country\" NOT IN ('us', 'ca')");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[0]);
+
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "REGEXP_LIKE(\"addresses[*].street\", 'street-3.*') AND \"addresses[*].country\" != 'us'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{3});
+
+        // A single matching flattened doc ID will result in the overall doc being matched
+        matchingDocIds = getMatchingDocIds(indexReader,
+            "\"addresses[*].street\" = 'street-21' AND \"skills[*]\" != 'japanese'");
+        Assert.assertEquals(matchingDocIds.toArray(), new int[]{2});
       }
     }
   }
@@ -274,8 +358,8 @@ public class JsonIndexTest {
   /**
    * Creates a JSON index with the given config and adds the given records
    * @param createOnHeap Whether to create an on-heap index
-   * @param jsonIndexConfig
-   * @param records
+   * @param jsonIndexConfig the JSON index config
+   * @param records the records to be added to the index
    * @throws IOException on error
    */
   private void createIndex(boolean createOnHeap, JsonIndexConfig jsonIndexConfig, String[] records)
@@ -294,19 +378,98 @@ public class JsonIndexTest {
     return indexReader.getMatchingDocIds(filter);
   }
 
+  @Test
+  public void testGetValueToFlattenedDocIdsMap()
+      throws Exception {
+    // @formatter: off
+    // CHECKSTYLE:OFF
+    String[] records = new String[]{
+        "{\"arrField\": " + "[{\"intKey01\": 1, \"stringKey01\": \"abc\"},"
+            + " {\"intKey01\": 1, \"stringKey01\": \"foo\"}, " + " {\"intKey01\": 3, \"stringKey01\": \"bar\"},"
+            + " {\"intKey01\": 5, \"stringKey01\": \"fuzz\"}]}",
+        "{\"arrField\": " + "[{\"intKey01\": 7, \"stringKey01\": \"pqrS\"},"
+            + " {\"intKey01\": 6, \"stringKey01\": \"foo\"}, " + " {\"intKey01\": 8, \"stringKey01\": \"test\"},"
+            + " {\"intKey01\": 9, \"stringKey01\": \"testf2\"}]}",
+        "{\"arrField\": " + "[{\"intKey01\": 1, \"stringKey01\": \"pqr\"},"
+            + " {\"intKey01\": 1, \"stringKey01\": \"foo\"}, " + " {\"intKey01\": 6, \"stringKey01\": \"test\"},"
+            + " {\"intKey01\": 3, \"stringKey01\": \"testf2\"}]}",
+    };
+    // CHECKSTYLE:ON
+    // @formatter: on
+
+    String[][] testKeys = new String[][]{
+        // Without filters
+        {"$.arrField[*].intKey01", null},
+        {"$.arrField[*].stringKey01", null},
+
+        // With regexp filter
+        {"$.arrField[*].intKey01", "REGEXP_LIKE(\"arrField..stringKey01\", '.*f.*')"},
+        // With range filter
+        {"$.arrField[*].stringKey01", "\"arrField..intKey01\" > 2"},
+        // With AND filters
+        {"$.arrField[*].intKey01", "\"arrField..intKey01\" > 2 AND REGEXP_LIKE(\"arrField..stringKey01\", "
+            + "'[a-b][a-b].*')"},
+        // Exclusive filters
+        {"$.arrField[*].intKey01", "\"arrField[*].stringKey01\" != 'bar'"},
+        {"$.arrField[*].stringKey01", "\"arrField[*].intKey01\" != '3'"},
+    };
+
+    String colName = "col";
+    try (JsonIndexCreator offHeapIndexCreator = new OffHeapJsonIndexCreator(INDEX_DIR, colName, new JsonIndexConfig());
+        MutableJsonIndexImpl mutableJsonIndex = new MutableJsonIndexImpl(new JsonIndexConfig())) {
+      for (String record : records) {
+        offHeapIndexCreator.add(record);
+        mutableJsonIndex.add(record);
+      }
+      offHeapIndexCreator.seal();
+
+      File offHeapIndexFile = new File(INDEX_DIR, colName + V1Constants.Indexes.JSON_INDEX_FILE_EXTENSION);
+      Assert.assertTrue(offHeapIndexFile.exists());
+
+      try (PinotDataBuffer offHeapDataBuffer = PinotDataBuffer.mapReadOnlyBigEndianFile(offHeapIndexFile);
+          ImmutableJsonIndexReader offHeapIndexReader = new ImmutableJsonIndexReader(offHeapDataBuffer,
+              records.length)) {
+        int[] docMask = new int[]{0, 2, 1};
+        int docIdValidLength = 2;
+        String[][][] expectedValues = new String[][][]{
+            {{"1", "1", "3", "5"}, {"1", "1", "6", "3"}},
+            {{"abc", "foo", "bar", "fuzz"}, {"pqr", "foo", "test", "testf2"}},
+            {{"1", "5"}, {"1", "3"}},
+            {{"bar", "fuzz"}, {"test", "testf2"}},
+            {{"3"}, {}},
+            {{"1", "1", "5"}, {"1", "1", "6", "3"}},
+            {{"abc", "foo", "fuzz"}, {"pqr", "foo", "test"}}
+        };
+        for (int i = 0; i < testKeys.length; i++) {
+          Map<String, RoaringBitmap> context =
+              offHeapIndexReader.getMatchingFlattenedDocsMap(testKeys[i][0], testKeys[i][1]);
+          String[][] values = offHeapIndexReader.getValuesMV(docMask, docIdValidLength, context);
+
+          for (int j = 0; j < docIdValidLength; j++) {
+            Assert.assertEquals(values[j], expectedValues[i][j]);
+          }
+
+          context = mutableJsonIndex.getMatchingFlattenedDocsMap(testKeys[i][0], testKeys[i][1]);
+          values = mutableJsonIndex.getValuesMV(docMask, docIdValidLength, context);
+          Assert.assertEquals(values, expectedValues[i]);
+        }
+      }
+    }
+  }
+
   @Test
   public void testGetValuesForKeyAndDocs()
       throws Exception {
     // @formatter: off
     // CHECKSTYLE:OFF
-    String[] records = new String[] {
+    String[] records = new String[]{
         "{\"field1\":\"value1\",\"field2\":\"value2\",\"field3\":\"value3\"}",
         "{\"field1\":\"value2\", \"field2\":[\"value1\",\"value2\"]}",
         "{\"field1\":\"value1\",\"field2\":\"value4\"}",
     };
     // CHECKSTYLE:ON
     // @formatter: on
-    String[] testKeys = new String[]{".field1", ".field2", ".field3", ".field4"};
+    String[] testKeys = new String[]{"$.field1", "$.field2", "$.field3", "$.field4"};
 
     String colName = "col";
     try (
@@ -331,12 +494,20 @@ public class JsonIndexTest {
             new String[][]{{"value1", "value2", "value1"}, {"value2", null, "value4"}, {"value3", null, null},
                 {null, null, null}};
         for (int i = 0; i < testKeys.length; i++) {
-          Map<String, RoaringBitmap> context = offHeapIndexReader.getMatchingDocsMap(testKeys[i]);
-          String[] values = offHeapIndexReader.getValuesForKeyAndDocs(docMask, context);
+          Map<String, RoaringBitmap> context = offHeapIndexReader.getMatchingFlattenedDocsMap(testKeys[i], null);
+          String[] values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, true);
           Assert.assertEquals(values, expectedValues[i]);
 
-          context = mutableJsonIndex.getMatchingDocsMap(testKeys[i]);
-          values = mutableJsonIndex.getValuesForKeyAndDocs(docMask, context);
+          offHeapIndexReader.convertFlattenedDocIdsToDocIds(context);
+          values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, false);
+          Assert.assertEquals(values, expectedValues[i]);
+
+          context = mutableJsonIndex.getMatchingFlattenedDocsMap(testKeys[i], null);
+          values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, true);
+          Assert.assertEquals(values, expectedValues[i]);
+
+          mutableJsonIndex.convertFlattenedDocIdsToDocIds(context);
+          values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, false);
           Assert.assertEquals(values, expectedValues[i]);
         }
 
@@ -344,31 +515,55 @@ public class JsonIndexTest {
         docMask = new int[]{1, 2};
         expectedValues = new String[][]{{"value2", "value1"}, {null, "value4"}, {null, null}, {null, null}};
         for (int i = 0; i < testKeys.length; i++) {
-          Map<String, RoaringBitmap> context = offHeapIndexReader.getMatchingDocsMap(testKeys[i]);
-          String[] values = offHeapIndexReader.getValuesForKeyAndDocs(docMask, context);
+          Map<String, RoaringBitmap> context = offHeapIndexReader.getMatchingFlattenedDocsMap(testKeys[i], null);
+          String[] values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, true);
           Assert.assertEquals(values, expectedValues[i]);
 
-          context = mutableJsonIndex.getMatchingDocsMap(testKeys[i]);
-          values = mutableJsonIndex.getValuesForKeyAndDocs(docMask, context);
+          offHeapIndexReader.convertFlattenedDocIdsToDocIds(context);
+          values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, false);
+          Assert.assertEquals(values, expectedValues[i]);
+
+          context = mutableJsonIndex.getMatchingFlattenedDocsMap(testKeys[i], null);
+          values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, true);
+          Assert.assertEquals(values, expectedValues[i]);
+
+          mutableJsonIndex.convertFlattenedDocIdsToDocIds(context);
+          values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, false);
           Assert.assertEquals(values, expectedValues[i]);
         }
 
         // Immutable index, context is reused for the second method call
-        Map<String, RoaringBitmap> context = offHeapIndexReader.getMatchingDocsMap(".field1");
+        Map<String, RoaringBitmap> context = offHeapIndexReader.getMatchingFlattenedDocsMap("$.field1", null);
         docMask = new int[]{0};
-        String[] values = offHeapIndexReader.getValuesForKeyAndDocs(docMask, context);
+        String[] values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, true);
         Assert.assertEquals(values, new String[]{"value1"});
         docMask = new int[]{1, 2};
-        values = offHeapIndexReader.getValuesForKeyAndDocs(docMask, context);
+        values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, true);
+        Assert.assertEquals(values, new String[]{"value2", "value1"});
+
+        offHeapIndexReader.convertFlattenedDocIdsToDocIds(context);
+        docMask = new int[]{0};
+        values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, false);
+        Assert.assertEquals(values, new String[]{"value1"});
+        docMask = new int[]{1, 2};
+        values = offHeapIndexReader.getValuesSV(docMask, docMask.length, context, false);
         Assert.assertEquals(values, new String[]{"value2", "value1"});
 
         // Mutable index, context is reused for the second method call
-        context = mutableJsonIndex.getMatchingDocsMap(".field1");;
+        context = mutableJsonIndex.getMatchingFlattenedDocsMap("$.field1", null);
         docMask = new int[]{0};
-        values = mutableJsonIndex.getValuesForKeyAndDocs(docMask, context);
+        values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, true);
         Assert.assertEquals(values, new String[]{"value1"});
         docMask = new int[]{1, 2};
-        values = mutableJsonIndex.getValuesForKeyAndDocs(docMask, context);
+        values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, true);
+        Assert.assertEquals(values, new String[]{"value2", "value1"});
+
+        mutableJsonIndex.convertFlattenedDocIdsToDocIds(context);
+        docMask = new int[]{0};
+        values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, false);
+        Assert.assertEquals(values, new String[]{"value1"});
+        docMask = new int[]{1, 2};
+        values = mutableJsonIndex.getValuesSV(docMask, docMask.length, context, false);
         Assert.assertEquals(values, new String[]{"value2", "value1"});
       }
     }
@@ -397,9 +592,9 @@ public class JsonIndexTest {
       for (String record : records) {
         mutableJsonIndex.add(record);
       }
-      Map<String, RoaringBitmap> onHeapRes = onHeapIndexReader.getMatchingDocsMap("");
-      Map<String, RoaringBitmap> offHeapRes = offHeapIndexReader.getMatchingDocsMap("");
-      Map<String, RoaringBitmap> mutableRes = mutableJsonIndex.getMatchingDocsMap("");
+      Map<String, RoaringBitmap> onHeapRes = onHeapIndexReader.getMatchingFlattenedDocsMap("$", null);
+      Map<String, RoaringBitmap> offHeapRes = offHeapIndexReader.getMatchingFlattenedDocsMap("$", null);
+      Map<String, RoaringBitmap> mutableRes = mutableJsonIndex.getMatchingFlattenedDocsMap("$", null);
       Map<String, RoaringBitmap> expectedRes = Collections.singletonMap(JsonUtils.SKIPPED_VALUE_REPLACEMENT,
           RoaringBitmap.bitmapOf(0));
       Assert.assertEquals(expectedRes, onHeapRes);
@@ -418,6 +613,66 @@ public class JsonIndexTest {
     createIndex(true, jsonIndexConfig, records);
   }
 
+
+  @Test
+  public void testGetMatchingValDocIdsPairForArrayPath() throws Exception {
+    String[] records = {
+            "{\"foo\":[{\"bar\":[\"x\",\"y\"]},{\"bar\":[\"a\",\"b\"]}],\"foo2\":[\"u\"]}",
+            "{\"foo\":[{\"bar\":[\"y\",\"z\"]}],\"foo2\":[\"u\"]}"
+    };
+    JsonIndexConfig jsonIndexConfig = new JsonIndexConfig();
+
+    createIndex(true, jsonIndexConfig, records);
+    File onHeapIndexFile = new File(INDEX_DIR, ON_HEAP_COLUMN_NAME + V1Constants.Indexes.JSON_INDEX_FILE_EXTENSION);
+    Assert.assertTrue(onHeapIndexFile.exists());
+
+    createIndex(false, jsonIndexConfig, records);
+    File offHeapIndexFile = new File(INDEX_DIR, OFF_HEAP_COLUMN_NAME + V1Constants.Indexes.JSON_INDEX_FILE_EXTENSION);
+    Assert.assertTrue(offHeapIndexFile.exists());
+
+    String[] keys = {"$.foo[0].bar[1]", "$.foo[1].bar[0]", "$.foo2[0]", "$.foo[100].bar[100]", "$.foo[0].bar[*]",
+            "$.foo[*].bar[0]", "$.foo[*].bar[*]"};
+    List<Map<String, RoaringBitmap>> expected = List.of(
+            Map.of("y", RoaringBitmap.bitmapOf(0), "z", RoaringBitmap.bitmapOf(1)),
+            Map.of("a", RoaringBitmap.bitmapOf(0)),
+            Map.of("u", RoaringBitmap.bitmapOf(0, 1)),
+            Collections.emptyMap(),
+            Map.of("x", RoaringBitmap.bitmapOf(0),
+                    "y", RoaringBitmap.bitmapOf(0, 1),
+                    "z", RoaringBitmap.bitmapOf(1)),
+            Map.of("x", RoaringBitmap.bitmapOf(0),
+                    "a", RoaringBitmap.bitmapOf(0),
+                    "y", RoaringBitmap.bitmapOf(1)),
+            Map.of("x", RoaringBitmap.bitmapOf(0),
+                    "y", RoaringBitmap.bitmapOf(0, 1),
+                    "z", RoaringBitmap.bitmapOf(1),
+                    "a", RoaringBitmap.bitmapOf(0),
+                    "b", RoaringBitmap.bitmapOf(0))
+    );
+
+    try (PinotDataBuffer onHeapDataBuffer = PinotDataBuffer.mapReadOnlyBigEndianFile(onHeapIndexFile);
+         PinotDataBuffer offHeapDataBuffer = PinotDataBuffer.mapReadOnlyBigEndianFile(offHeapIndexFile);
+         ImmutableJsonIndexReader onHeapIndexReader = new ImmutableJsonIndexReader(onHeapDataBuffer, records.length);
+         ImmutableJsonIndexReader offHeapIndexReader = new ImmutableJsonIndexReader(offHeapDataBuffer, records.length);
+         MutableJsonIndexImpl mutableJsonIndex = new MutableJsonIndexImpl(jsonIndexConfig)) {
+      for (String record : records) {
+        mutableJsonIndex.add(record);
+      }
+
+      for (int i = 0; i < keys.length; i++) {
+        Map<String, RoaringBitmap> onHeapRes = onHeapIndexReader.getMatchingFlattenedDocsMap(keys[i], null);
+        onHeapIndexReader.convertFlattenedDocIdsToDocIds(onHeapRes);
+        Map<String, RoaringBitmap> offHeapRes = offHeapIndexReader.getMatchingFlattenedDocsMap(keys[i], null);
+        offHeapIndexReader.convertFlattenedDocIdsToDocIds(offHeapRes);
+        Map<String, RoaringBitmap> mutableRes = mutableJsonIndex.getMatchingFlattenedDocsMap(keys[i], null);
+        mutableJsonIndex.convertFlattenedDocIdsToDocIds(mutableRes);
+        Assert.assertEquals(expected.get(i), onHeapRes);
+        Assert.assertEquals(expected.get(i), offHeapRes);
+        Assert.assertEquals(mutableRes, expected.get(i));
+      }
+    }
+  }
+
   public static class ConfTest extends AbstractSerdeIndexContract {
 
     @Test
