@@ -25,8 +25,8 @@ import java.nio.ByteOrder;
 import java.nio.file.Files;
 import java.util.Random;
 import org.apache.commons.io.FileUtils;
-import org.apache.commons.lang.RandomStringUtils;
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.RandomStringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.segment.local.io.writer.impl.VarByteChunkForwardIndexWriter;
 import org.apache.pinot.segment.local.segment.creator.impl.fwd.SingleValueVarByteRawIndexCreator;
 import org.apache.pinot.segment.local.segment.index.readers.forward.ChunkReaderContext;
@@ -75,6 +75,12 @@ public class VarByteChunkSVForwardIndexTest {
     test(ChunkCompressionType.LZ4);
   }
 
+  @Test
+  public void testWithGZIPCompression()
+      throws Exception {
+    test(ChunkCompressionType.GZIP);
+  }
+
   /**
    * This test writes {@link #NUM_ENTRIES} using {@link VarByteChunkForwardIndexWriter}. It then reads
    * the strings & bytes using {@link VarByteChunkSVForwardIndexReader}, and asserts that what was written is the
@@ -177,36 +183,43 @@ public class VarByteChunkSVForwardIndexTest {
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 10, 1000);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 10, 1000);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 10, 1000);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 10, 1000);
 
     testLargeVarcharHelper(ChunkCompressionType.SNAPPY, 100, 1000);
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 100, 1000);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 100, 1000);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 100, 1000);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 100, 1000);
 
     testLargeVarcharHelper(ChunkCompressionType.SNAPPY, 1000, 1000);
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 1000, 1000);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 1000, 1000);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 1000, 1000);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 1000, 1000);
 
     testLargeVarcharHelper(ChunkCompressionType.SNAPPY, 10000, 100);
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 10000, 100);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 10000, 100);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 10000, 100);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 10000, 100);
 
     testLargeVarcharHelper(ChunkCompressionType.SNAPPY, 100000, 10);
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 100000, 10);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 100000, 10);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 100000, 10);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 100000, 10);
 
     testLargeVarcharHelper(ChunkCompressionType.SNAPPY, 1000000, 10);
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 1000000, 10);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 1000000, 10);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 1000000, 10);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 1000000, 10);
 
     testLargeVarcharHelper(ChunkCompressionType.SNAPPY, 2000000, 10);
     testLargeVarcharHelper(ChunkCompressionType.PASS_THROUGH, 2000000, 10);
     testLargeVarcharHelper(ChunkCompressionType.ZSTANDARD, 2000000, 10);
     testLargeVarcharHelper(ChunkCompressionType.LZ4, 2000000, 10);
+    testLargeVarcharHelper(ChunkCompressionType.GZIP, 2000000, 10);
   }
 
   private void testLargeVarcharHelper(ChunkCompressionType compressionType, int numChars, int numDocs)
@@ -224,7 +237,7 @@ public class VarByteChunkSVForwardIndexTest {
       maxStringLengthInBytes = Math.max(maxStringLengthInBytes, value.getBytes(UTF_8).length);
     }
 
-    int numDocsPerChunk = SingleValueVarByteRawIndexCreator.getNumDocsPerChunk(maxStringLengthInBytes);
+    int numDocsPerChunk = SingleValueVarByteRawIndexCreator.getNumDocsPerChunk(maxStringLengthInBytes, 1024 * 1024);
     try (VarByteChunkForwardIndexWriter writer = new VarByteChunkForwardIndexWriter(outFile, compressionType, numDocs,
         numDocsPerChunk, maxStringLengthInBytes, 3)) {
       // NOTE: No need to test BYTES explicitly because STRING is handled as UTF-8 encoded bytes
