@@ -53,7 +53,11 @@ public class StarTreeV2Constants {
     public static final String AGGREGATION_PREFIX = "aggregation.";
     public static final String FUNCTION_TYPE = "function.type";
     public static final String COLUMN_NAME = "column.name";
+    public static final String INDEX_VERSION = "index.version";
     public static final String COMPRESSION_CODEC = "compression.codec";
+    public static final String DERIVE_NUM_DOCS_PER_CHUNK = "derive.num.docs.per.chunk";
+    public static final String TARGET_MAX_CHUNK_SIZE_BYTES = "target.max.chunk.size.bytes";
+    public static final String TARGET_DOCS_PER_CHUNK = "target.docs.per.chunk";
 
     public static String getStarTreePrefix(int index) {
       return STAR_TREE_PREFIX + index;
