@@ -31,6 +31,8 @@ import javax.annotation.concurrent.ThreadSafe;
 
 @ThreadSafe
 public class PinotByteBuffer extends PinotDataBuffer {
+  public static final PinotDataBuffer EMPTY = PinotByteBuffer.wrap(ByteBuffer.allocate(0));
+
   private final ByteBuffer _buffer;
   private final boolean _flushable;
 
@@ -60,6 +62,14 @@ public class PinotByteBuffer extends PinotDataBuffer {
     }
   }
 
+  public static PinotByteBuffer wrap(byte[] buffer) {
+    return new PinotByteBuffer(ByteBuffer.wrap(buffer), false, false);
+  }
+
+  public static PinotByteBuffer wrap(ByteBuffer buffer) {
+    return new PinotByteBuffer(buffer, false, false);
+  }
+
   private PinotByteBuffer(ByteBuffer buffer, boolean closeable, boolean flushable) {
     super(closeable);
     _buffer = buffer;
@@ -237,7 +247,7 @@ public class PinotByteBuffer extends PinotDataBuffer {
   }
 
   @Override
-  public void copyTo(long offset, PinotDataBuffer buffer, long destOffset, long size) {
+  public void copyTo(long offset, DataBuffer buffer, long destOffset, long size) {
     assert offset <= Integer.MAX_VALUE;
     assert size <= Integer.MAX_VALUE;
     int start = (int) offset;
@@ -282,7 +292,7 @@ public class PinotByteBuffer extends PinotDataBuffer {
       ByteBuffer duplicate = _buffer.duplicate();
       int start = (int) offset;
       int end = start + (int) size;
-      ((Buffer) duplicate).position(start).limit(end);
+      duplicate.position(start).limit(end);
       channel.read(duplicate, srcOffset);
     }
   }
