@@ -84,6 +84,8 @@ public class HelixInstanceDataManager implements InstanceDataManager {
   private static final Logger LOGGER = LoggerFactory.getLogger(HelixInstanceDataManager.class);
 
   private final ConcurrentHashMap<String, TableDataManager> _tableDataManagerMap = new ConcurrentHashMap<>();
+  // TODO: Consider making segment locks per table instead of per instance
+  private final SegmentLocks _segmentLocks = new SegmentLocks();
 
   private HelixInstanceDataManagerConfig _instanceDataManagerConfig;
   private String _instanceId;
@@ -114,8 +116,8 @@ public class HelixInstanceDataManager implements InstanceDataManager {
     _instanceDataManagerConfig = new HelixInstanceDataManagerConfig(config);
     LOGGER.info("HelixInstanceDataManagerConfig: {}", _instanceDataManagerConfig);
     _instanceId = _instanceDataManagerConfig.getInstanceId();
-    _tableDataManagerProvider = new TableDataManagerProvider(_instanceDataManagerConfig);
     _helixManager = helixManager;
+    _tableDataManagerProvider = new TableDataManagerProvider(_instanceDataManagerConfig, helixManager, _segmentLocks);
     _segmentUploader = new PinotFSSegmentUploader(_instanceDataManagerConfig.getSegmentStoreUri(),
         ServerSegmentCompletionProtocolHandler.getSegmentUploadRequestTimeoutMs(), serverMetrics);
 
@@ -152,9 +154,9 @@ public class HelixInstanceDataManager implements InstanceDataManager {
 
     // Initialize the error cache
     _errorCache = CacheBuilder.newBuilder().maximumSize(_instanceDataManagerConfig.getErrorCacheSize())
-        .build(new CacheLoader<Pair<String, String>, SegmentErrorInfo>() {
+        .build(new CacheLoader<>() {
           @Override
-          public SegmentErrorInfo load(Pair<String, String> tableNameWithTypeSegmentNamePair) {
+          public SegmentErrorInfo load(Pair<String, String> tableSegmentPair) {
             // This cache is populated only via the put api.
             return null;
           }
@@ -225,34 +227,6 @@ public class HelixInstanceDataManager implements InstanceDataManager {
     LOGGER.info("Helix instance data manager shut down");
   }
 
-  @Override
-  public void addRealtimeSegment(String realtimeTableName, String segmentName)
-      throws Exception {
-    LOGGER.info("Adding segment: {} to table: {}", segmentName, realtimeTableName);
-    TableConfig tableConfig = ZKMetadataProvider.getTableConfig(_propertyStore, realtimeTableName);
-    Preconditions.checkState(tableConfig != null, "Failed to find table config for table: %s", realtimeTableName);
-    Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, tableConfig);
-    Preconditions.checkState(schema != null, "Failed to find schema for table: %s", realtimeTableName);
-    SegmentZKMetadata zkMetadata =
-        ZKMetadataProvider.getSegmentZKMetadata(_propertyStore, realtimeTableName, segmentName);
-    Preconditions.checkState(zkMetadata != null, "Failed to find ZK metadata for segment: %s, table: %s", segmentName,
-        realtimeTableName);
-    _tableDataManagerMap.computeIfAbsent(realtimeTableName, k -> createTableDataManager(tableConfig))
-        .addSegment(segmentName, new IndexLoadingConfig(_instanceDataManagerConfig, tableConfig, schema), zkMetadata);
-    LOGGER.info("Added segment: {} to table: {}", segmentName, realtimeTableName);
-  }
-
-  private TableDataManager createTableDataManager(TableConfig tableConfig) {
-    String tableNameWithType = tableConfig.getTableName();
-    LOGGER.info("Creating table data manager for table: {}", tableNameWithType);
-    TableDataManager tableDataManager =
-        _tableDataManagerProvider.getTableDataManager(tableConfig, _helixManager, _segmentPreloadExecutor, _errorCache,
-            _isServerReadyToServeQueries);
-    tableDataManager.start();
-    LOGGER.info("Created table data manager for table: {}", tableNameWithType);
-    return tableDataManager;
-  }
-
   @Override
   public void deleteTable(String tableNameWithType)
       throws Exception {
@@ -296,14 +270,52 @@ public class HelixInstanceDataManager implements InstanceDataManager {
   }
 
   @Override
-  public void offloadSegment(String tableNameWithType, String segmentName) {
-    LOGGER.info("Removing segment: {} from table: {}", segmentName, tableNameWithType);
+  public void addOnlineSegment(String tableNameWithType, String segmentName)
+      throws Exception {
+    _tableDataManagerMap.computeIfAbsent(tableNameWithType, this::createTableDataManager).addOnlineSegment(segmentName);
+  }
+
+  @Override
+  public void addConsumingSegment(String realtimeTableName, String segmentName)
+      throws Exception {
+    _tableDataManagerMap.computeIfAbsent(realtimeTableName, this::createTableDataManager)
+        .addConsumingSegment(segmentName);
+  }
+
+  private TableDataManager createTableDataManager(String tableNameWithType) {
+    LOGGER.info("Creating table data manager for table: {}", tableNameWithType);
+    TableConfig tableConfig = ZKMetadataProvider.getTableConfig(_propertyStore, tableNameWithType);
+    Preconditions.checkState(tableConfig != null, "Failed to find table config for table: %s", tableNameWithType);
+    TableDataManager tableDataManager =
+        _tableDataManagerProvider.getTableDataManager(tableConfig, _segmentPreloadExecutor, _errorCache,
+            _isServerReadyToServeQueries);
+    tableDataManager.start();
+    LOGGER.info("Created table data manager for table: {}", tableNameWithType);
+    return tableDataManager;
+  }
+
+  @Override
+  public void replaceSegment(String tableNameWithType, String segmentName)
+      throws Exception {
+    LOGGER.info("Replacing segment: {} in table: {}", segmentName, tableNameWithType);
     TableDataManager tableDataManager = _tableDataManagerMap.get(tableNameWithType);
     if (tableDataManager != null) {
-      tableDataManager.removeSegment(segmentName);
-      LOGGER.info("Removed segment: {} from table: {}", segmentName, tableNameWithType);
+      tableDataManager.replaceSegment(segmentName);
     } else {
-      LOGGER.warn("Failed to find data manager for table: {}, skipping removing segment: {}", tableNameWithType,
+      LOGGER.warn("Failed to find data manager for table: {}, skipping replacing segment: {}", tableNameWithType,
+          segmentName);
+    }
+  }
+
+  @Override
+  public void offloadSegment(String tableNameWithType, String segmentName)
+      throws Exception {
+    LOGGER.info("Offloading segment: {} from table: {}", segmentName, tableNameWithType);
+    TableDataManager tableDataManager = _tableDataManagerMap.get(tableNameWithType);
+    if (tableDataManager != null) {
+      tableDataManager.offloadSegment(segmentName);
+    } else {
+      LOGGER.warn("Failed to find data manager for table: {}, skipping offloading segment: {}", tableNameWithType,
           segmentName);
     }
   }
@@ -311,19 +323,29 @@ public class HelixInstanceDataManager implements InstanceDataManager {
   @Override
   public void deleteSegment(String tableNameWithType, String segmentName)
       throws Exception {
-    // This method might modify the file on disk. Use segment lock to prevent race condition
-    Lock segmentLock = SegmentLocks.getSegmentLock(tableNameWithType, segmentName);
+    LOGGER.info("Deleting segment: {} from table: {}", segmentName, tableNameWithType);
+    // Segment deletion is handled at instance level because table data manager might not exist. Acquire the lock here.
+    Lock segmentLock = _segmentLocks.getLock(tableNameWithType, segmentName);
+    segmentLock.lock();
     try {
-      segmentLock.lock();
-
+      // Check if the segment is still loaded, if so, offload it first.
+      // This might happen when the server disconnected from ZK and reconnected, and the segment is still loaded.
+      // TODO: Consider using table data manager to delete the segment. This will allow the table data manager to clean
+      //       up the segment data on all tiers. Note that table data manager might have not been created, and table
+      //       config might have been deleted at this point.
+      TableDataManager tableDataManager = _tableDataManagerMap.get(tableNameWithType);
+      if (tableDataManager != null && tableDataManager.hasSegment(segmentName)) {
+        LOGGER.warn("Segment: {} from table: {} is still loaded, offloading it first", segmentName, tableNameWithType);
+        tableDataManager.offloadSegment(segmentName);
+      }
       // Clean up the segment data on default tier unconditionally.
       File segmentDir = getSegmentDataDirectory(tableNameWithType, segmentName);
       if (segmentDir.exists()) {
         FileUtils.deleteQuietly(segmentDir);
         LOGGER.info("Deleted segment directory {} on default tier", segmentDir);
       }
-      // We might clean up further more with the specific segment loader. But note that tableDataManager object or
-      // even the TableConfig might not be present any more at this point.
+      // We might clean up further more with the specific segment loader. But note that table data manager might have
+      // not been created, and table config might have been deleted at this point.
       SegmentDirectoryLoader segmentLoader = SegmentDirectoryLoaderRegistry.getSegmentDirectoryLoader(
           _instanceDataManagerConfig.getSegmentDirectoryLoader());
       if (segmentLoader != null) {
@@ -333,11 +355,13 @@ public class HelixInstanceDataManager implements InstanceDataManager {
             .setTableDataDir(_instanceDataManagerConfig.getInstanceDataDir() + "/" + tableNameWithType).build();
         segmentLoader.delete(ctx);
       }
+      LOGGER.info("Deleted segment: {} from table: {}", segmentName, tableNameWithType);
     } finally {
       segmentLock.unlock();
     }
   }
 
+  // TODO: Move reload handling logic to table data manager
   @Override
   public void reloadSegment(String tableNameWithType, String segmentName, boolean forceDownload)
       throws Exception {
@@ -358,6 +382,7 @@ public class HelixInstanceDataManager implements InstanceDataManager {
     LOGGER.info("Reloaded single segment: {} in table: {}", segmentName, tableNameWithType);
   }
 
+  // TODO: Move reload handling logic to table data manager
   @Override
   public void reloadAllSegments(String tableNameWithType, boolean forceDownload,
       SegmentRefreshSemaphore segmentRefreshSemaphore)
@@ -367,6 +392,7 @@ public class HelixInstanceDataManager implements InstanceDataManager {
     reloadSegmentsWithMetadata(tableNameWithType, segmentsMetadata, forceDownload, segmentRefreshSemaphore);
   }
 
+  // TODO: Move reload handling logic to table data manager
   @Override
   public void reloadSegments(String tableNameWithType, List<String> segmentNames, boolean forceDownload,
       SegmentRefreshSemaphore segmentRefreshSemaphore)
@@ -470,56 +496,13 @@ public class HelixInstanceDataManager implements InstanceDataManager {
       }
     }
 
+    IndexLoadingConfig indexLoadingConfig = new IndexLoadingConfig(_instanceDataManagerConfig, tableConfig, schema);
+    indexLoadingConfig.setErrorOnColumnBuildFailure(true);
     SegmentZKMetadata zkMetadata =
         ZKMetadataProvider.getSegmentZKMetadata(_propertyStore, tableNameWithType, segmentName);
-    Preconditions.checkNotNull(zkMetadata);
-
-    // This method might modify the file on disk. Use segment lock to prevent race condition
-    Lock segmentLock = SegmentLocks.getSegmentLock(tableNameWithType, segmentName);
-    try {
-      segmentLock.lock();
-
-      // Reloads an existing segment, and the local segment metadata is existing as asserted above.
-      IndexLoadingConfig indexLoadingConfig = new IndexLoadingConfig(_instanceDataManagerConfig, tableConfig, schema);
-      indexLoadingConfig.setErrorOnColumnBuildFailure(true);
-      tableDataManager.reloadSegment(segmentName, indexLoadingConfig, zkMetadata, segmentMetadata, schema,
-          forceDownload);
-      LOGGER.info("Reloaded segment: {} of table: {}", segmentName, tableNameWithType);
-    } finally {
-      segmentLock.unlock();
-    }
-  }
-
-  @Override
-  public void addOrReplaceSegment(String tableNameWithType, String segmentName)
-      throws Exception {
-    LOGGER.info("Adding or replacing segment: {} for table: {}", segmentName, tableNameWithType);
-
-    // Get updated table config, schema and segment metadata from Zookeeper.
-    TableConfig tableConfig = ZKMetadataProvider.getTableConfig(_propertyStore, tableNameWithType);
-    Preconditions.checkState(tableConfig != null, "Failed to find table config for table: %s", tableNameWithType);
-    Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, tableConfig);
-    SegmentZKMetadata zkMetadata =
-        ZKMetadataProvider.getSegmentZKMetadata(_propertyStore, tableNameWithType, segmentName);
-    Preconditions.checkState(zkMetadata != null, "Failed to find ZK metadata for segment: %s, table: %s", segmentName,
+    Preconditions.checkState(zkMetadata != null, "Failed to find ZK metadata for segment: %s of table: %s", segmentName,
         tableNameWithType);
-
-    // This method might modify the file on disk. Use segment lock to prevent race condition
-    Lock segmentLock = SegmentLocks.getSegmentLock(tableNameWithType, segmentName);
-    try {
-      segmentLock.lock();
-
-      // But if table mgr is not created or the segment is not loaded yet, the localMetadata
-      // is set to null. Then, addOrReplaceSegment method will load the segment accordingly.
-      SegmentMetadata localMetadata = getSegmentMetadata(tableNameWithType, segmentName);
-
-      _tableDataManagerMap.computeIfAbsent(tableNameWithType, k -> createTableDataManager(tableConfig))
-          .addOrReplaceSegment(segmentName, new IndexLoadingConfig(_instanceDataManagerConfig, tableConfig, schema),
-              zkMetadata, localMetadata);
-      LOGGER.info("Added or replaced segment: {} of table: {}", segmentName, tableNameWithType);
-    } finally {
-      segmentLock.unlock();
-    }
+    tableDataManager.reloadSegment(segmentName, indexLoadingConfig, zkMetadata, segmentMetadata, schema, forceDownload);
   }
 
   @Override
