@@ -319,7 +319,7 @@ public class TablesResourceTest extends BaseResourceTest {
     Assert.assertEquals(validDocIdMetadata.get("totalDocs").asInt(), 100000);
     Assert.assertEquals(validDocIdMetadata.get("totalValidDocs").asInt(), 8);
     Assert.assertEquals(validDocIdMetadata.get("totalInvalidDocs").asInt(), 99992);
-    Assert.assertEquals(validDocIdMetadata.get("segmentCrc").asText(), "1265679343");
+    Assert.assertEquals(validDocIdMetadata.get("segmentCrc").asText(), "1894900283");
     Assert.assertEquals(validDocIdMetadata.get("validDocIdsType").asText(), "SNAPSHOT");
   }
 
@@ -345,7 +345,7 @@ public class TablesResourceTest extends BaseResourceTest {
     Assert.assertEquals(validDocIdsMetadata.get("totalDocs").asInt(), 100000);
     Assert.assertEquals(validDocIdsMetadata.get("totalValidDocs").asInt(), 8);
     Assert.assertEquals(validDocIdsMetadata.get("totalInvalidDocs").asInt(), 99992);
-    Assert.assertEquals(validDocIdsMetadata.get("segmentCrc").asText(), "1265679343");
+    Assert.assertEquals(validDocIdsMetadata.get("segmentCrc").asText(), "1894900283");
     Assert.assertEquals(validDocIdsMetadata.get("validDocIdsType").asText(), "SNAPSHOT");
   }
 
@@ -459,11 +459,12 @@ public class TablesResourceTest extends BaseResourceTest {
             V1Constants.VALID_DOC_IDS_SNAPSHOT_FILE_NAME);
     FileUtils.writeByteArrayToFile(validDocIdsSnapshotFile,
         RoaringBitmapUtils.serialize(validDocIdsSnapshot.getMutableRoaringBitmap()));
+    String expectedSegmentCrc = "1894900283";
 
     // Check no type (default should be validDocIdsSnapshot)
     ValidDocIdsBitmapResponse response = _webTarget.path(snapshotPath).request().get(ValidDocIdsBitmapResponse.class);
     Assert.assertNotNull(response);
-    Assert.assertEquals(response.getSegmentCrc(), "1265679343");
+    Assert.assertEquals(response.getSegmentCrc(), expectedSegmentCrc);
     Assert.assertEquals(response.getSegmentName(), segment.getSegmentName());
     byte[] validDocIdsSnapshotBitmap = response.getBitmap();
     Assert.assertNotNull(validDocIdsSnapshotBitmap);
@@ -475,7 +476,7 @@ public class TablesResourceTest extends BaseResourceTest {
         _webTarget.path(snapshotPath).queryParam("validDocIdsType", ValidDocIdsType.SNAPSHOT.toString()).request()
             .get(ValidDocIdsBitmapResponse.class);
     Assert.assertNotNull(response);
-    Assert.assertEquals(response.getSegmentCrc(), "1265679343");
+    Assert.assertEquals(response.getSegmentCrc(), expectedSegmentCrc);
     Assert.assertEquals(response.getSegmentName(), segment.getSegmentName());
     validDocIdsSnapshotBitmap = response.getBitmap();
     Assert.assertNotNull(validDocIdsSnapshotBitmap);
@@ -487,7 +488,7 @@ public class TablesResourceTest extends BaseResourceTest {
         _webTarget.path(snapshotPath).queryParam("validDocIdsType", ValidDocIdsType.IN_MEMORY.toString()).request()
             .get(ValidDocIdsBitmapResponse.class);
     Assert.assertNotNull(response);
-    Assert.assertEquals(response.getSegmentCrc(), "1265679343");
+    Assert.assertEquals(response.getSegmentCrc(), expectedSegmentCrc);
     Assert.assertEquals(response.getSegmentName(), segment.getSegmentName());
     validDocIdsSnapshotBitmap = response.getBitmap();
     Assert.assertNotNull(validDocIdsSnapshotBitmap);
@@ -499,7 +500,7 @@ public class TablesResourceTest extends BaseResourceTest {
         _webTarget.path(snapshotPath).queryParam("validDocIdsType", ValidDocIdsType.IN_MEMORY_WITH_DELETE.toString())
             .request().get(ValidDocIdsBitmapResponse.class);
     Assert.assertNotNull(response);
-    Assert.assertEquals(response.getSegmentCrc(), "1265679343");
+    Assert.assertEquals(response.getSegmentCrc(), expectedSegmentCrc);
     Assert.assertEquals(response.getSegmentName(), segment.getSegmentName());
     validDocIdsSnapshotBitmap = response.getBitmap();
     Assert.assertNotNull(validDocIdsSnapshotBitmap);
