@@ -39,6 +39,10 @@ import javax.annotation.Nullable;
  *                 the excluded paths will also be excluded, e.g. "$.a.b.c" will be excluded when "$.a.b" is configured
  *                 to be excluded.
  * - excludeFields: Exclude the given fields, e.g. "b", "c", even if it is under the included paths.
+ * - maxValueLength: Exclude field values which are longer than this length. A value of "0" disables this filter.
+ *                   Excluded values will be replaced with JsonUtils.SKIPPED_VALUE_REPLACEMENT.
+ * - skipInvalidJson: If the raw data is not a valid json string, then replace with {"":SKIPPED_VALUE_REPLACEMENT}
+ *                    and continue indexing on following Json records.
  */
 public class JsonIndexConfig extends IndexConfig {
   public static final JsonIndexConfig DISABLED = new JsonIndexConfig(true);
@@ -49,6 +53,8 @@ public class JsonIndexConfig extends IndexConfig {
   private Set<String> _includePaths;
   private Set<String> _excludePaths;
   private Set<String> _excludeFields;
+  private int _maxValueLength = 0;
+  private boolean _skipInvalidJson = false;
 
   public JsonIndexConfig() {
     super(false);
@@ -64,7 +70,9 @@ public class JsonIndexConfig extends IndexConfig {
       @JsonProperty("disableCrossArrayUnnest") boolean disableCrossArrayUnnest,
       @JsonProperty("includePaths") @Nullable Set<String> includePaths,
       @JsonProperty("excludePaths") @Nullable Set<String> excludePaths,
-      @JsonProperty("excludeFields") @Nullable Set<String> excludeFields) {
+      @JsonProperty("excludeFields") @Nullable Set<String> excludeFields,
+      @JsonProperty("maxValueLength") int maxValueLength,
+      @JsonProperty("skipInvalidJson") boolean skipInvalidJson) {
     super(disabled);
     _maxLevels = maxLevels;
     _excludeArray = excludeArray;
@@ -72,6 +80,8 @@ public class JsonIndexConfig extends IndexConfig {
     _includePaths = includePaths;
     _excludePaths = excludePaths;
     _excludeFields = excludeFields;
+    _maxValueLength = maxValueLength;
+    _skipInvalidJson = skipInvalidJson;
   }
 
   public int getMaxLevels() {
@@ -130,6 +140,22 @@ public class JsonIndexConfig extends IndexConfig {
     _excludeFields = excludeFields;
   }
 
+  public int getMaxValueLength() {
+    return _maxValueLength;
+  }
+
+  public void setMaxValueLength(int maxValueLength) {
+    _maxValueLength = maxValueLength;
+  }
+
+  public boolean getSkipInvalidJson() {
+    return _skipInvalidJson;
+  }
+
+  public void setSkipInvalidJson(boolean skipInvalidJson) {
+    _skipInvalidJson = skipInvalidJson;
+  }
+
   @Override
   public boolean equals(Object o) {
     if (this == o) {
@@ -145,12 +171,13 @@ public class JsonIndexConfig extends IndexConfig {
     return _maxLevels == config._maxLevels && _excludeArray == config._excludeArray
         && _disableCrossArrayUnnest == config._disableCrossArrayUnnest && Objects.equals(_includePaths,
         config._includePaths) && Objects.equals(_excludePaths, config._excludePaths) && Objects.equals(_excludeFields,
-        config._excludeFields);
+        config._excludeFields) && _maxValueLength == config._maxValueLength
+        && _skipInvalidJson == config._skipInvalidJson;
   }
 
   @Override
   public int hashCode() {
     return Objects.hash(super.hashCode(), _maxLevels, _excludeArray, _disableCrossArrayUnnest, _includePaths,
-        _excludePaths, _excludeFields);
+        _excludePaths, _excludeFields, _maxValueLength, _skipInvalidJson);
   }
 }
