@@ -65,7 +65,7 @@ public class TableConfig extends BaseJsonConfig {
   /* MANDATORY FIELDS */
 
   @JsonPropertyDescription("The name for the table (with type suffix), e.g. \"myTable_OFFLINE\" (mandatory)")
-  private final String _tableName;
+  private String _tableName;
 
   @JsonPropertyDescription(value = "The type of the table (OFFLINE|REALTIME) (mandatory)")
   private final TableType _tableType;
@@ -118,7 +118,7 @@ public class TableConfig extends BaseJsonConfig {
   public TableConfig(@JsonProperty(value = TABLE_NAME_KEY, required = true) String tableName,
       @JsonProperty(value = TABLE_TYPE_KEY, required = true) String tableType,
       @JsonProperty(value = VALIDATION_CONFIG_KEY, required = true)
-          SegmentsValidationAndRetentionConfig validationConfig,
+      SegmentsValidationAndRetentionConfig validationConfig,
       @JsonProperty(value = TENANT_CONFIG_KEY, required = true) TenantConfig tenantConfig,
       @JsonProperty(value = INDEXING_CONFIG_KEY, required = true) IndexingConfig indexingConfig,
       @JsonProperty(value = CUSTOM_CONFIG_KEY, required = true) TableCustomConfig customConfig,
@@ -127,7 +127,7 @@ public class TableConfig extends BaseJsonConfig {
       @JsonProperty(ROUTING_CONFIG_KEY) @Nullable RoutingConfig routingConfig,
       @JsonProperty(QUERY_CONFIG_KEY) @Nullable QueryConfig queryConfig,
       @JsonProperty(INSTANCE_ASSIGNMENT_CONFIG_MAP_KEY) @Nullable
-          Map<String, InstanceAssignmentConfig> instanceAssignmentConfigMap,
+      Map<String, InstanceAssignmentConfig> instanceAssignmentConfigMap,
       @JsonProperty(FIELD_CONFIG_LIST_KEY) @Nullable List<FieldConfig> fieldConfigList,
       @JsonProperty(UPSERT_CONFIG_KEY) @Nullable UpsertConfig upsertConfig,
       @JsonProperty(DEDUP_CONFIG_KEY) @Nullable DedupConfig dedupConfig,
@@ -137,9 +137,9 @@ public class TableConfig extends BaseJsonConfig {
       @JsonProperty(IS_DIM_TABLE_KEY) boolean dimTable,
       @JsonProperty(TUNER_CONFIG_LIST_KEY) @Nullable List<TunerConfig> tunerConfigList,
       @JsonProperty(INSTANCE_PARTITIONS_MAP_CONFIG_KEY) @Nullable
-          Map<InstancePartitionsType, String> instancePartitionsMap,
+      Map<InstancePartitionsType, String> instancePartitionsMap,
       @JsonProperty(SEGMENT_ASSIGNMENT_CONFIG_MAP_KEY) @Nullable
-          Map<String, SegmentAssignmentConfig> segmentAssignmentConfigMap) {
+      Map<String, SegmentAssignmentConfig> segmentAssignmentConfigMap) {
     Preconditions.checkArgument(tableName != null, "'tableName' must be configured");
     Preconditions.checkArgument(!tableName.contains(TABLE_NAME_FORBIDDEN_SUBSTRING),
         "'tableName' cannot contain double underscore ('__')");
@@ -202,6 +202,10 @@ public class TableConfig extends BaseJsonConfig {
     return _tableName;
   }
 
+  public void setTableName(String tableNameWithType) {
+    _tableName = tableNameWithType;
+  }
+
   @JsonProperty(TABLE_TYPE_KEY)
   public TableType getTableType() {
     return _tableType;
@@ -294,8 +298,7 @@ public class TableConfig extends BaseJsonConfig {
     return _instanceAssignmentConfigMap;
   }
 
-  public void setInstanceAssignmentConfigMap(
-      Map<String, InstanceAssignmentConfig> instanceAssignmentConfigMap) {
+  public void setInstanceAssignmentConfigMap(Map<String, InstanceAssignmentConfig> instanceAssignmentConfigMap) {
     _instanceAssignmentConfigMap = instanceAssignmentConfigMap;
   }
 
@@ -386,6 +389,11 @@ public class TableConfig extends BaseJsonConfig {
     return _upsertConfig == null ? null : _upsertConfig.getComparisonColumns();
   }
 
+  @JsonIgnore
+  public double getUpsertMetadataTTL() {
+    return _upsertConfig == null ? 0 : _upsertConfig.getMetadataTTL();
+  }
+
   @JsonIgnore
   @Nullable
   public String getUpsertDeleteRecordColumn() {
