@@ -29,7 +29,7 @@ import org.apache.pinot.spi.utils.ByteArray;
 /**
  * The primary key of a record. Note that the value used in the primary key must be single-value.
  */
-public class PrimaryKey {
+public final class PrimaryKey {
   private final Object[] _values;
 
   public PrimaryKey(Object[] values) {
