@@ -213,9 +213,11 @@ public class PinotConfiguration {
       PropertiesConfiguration propertiesConfiguration;
       if (configPath.startsWith("classpath:")) {
         propertiesConfiguration = CommonsConfigurationUtils.fromInputStream(
-            PinotConfiguration.class.getResourceAsStream(configPath.substring("classpath:".length())), true, true);
+            PinotConfiguration.class.getResourceAsStream(configPath.substring("classpath:".length())), true,
+            PropertyIOFactoryKind.ConfigFileIOFactory);
       } else {
-        propertiesConfiguration = CommonsConfigurationUtils.fromPath(configPath, true, true);
+        propertiesConfiguration = CommonsConfigurationUtils.fromPath(configPath, true,
+            PropertyIOFactoryKind.ConfigFileIOFactory);
       }
       return propertiesConfiguration;
     } catch (ConfigurationException e) {
@@ -231,7 +233,15 @@ public class PinotConfiguration {
 
   private static Map<String, String> relaxEnvironmentVariables(Map<String, String> environmentVariables) {
     return environmentVariables.entrySet().stream()
-        .collect(Collectors.toMap(PinotConfiguration::relaxEnvVarName, Entry::getValue));
+        // Sort by key to ensure choosing a consistent env var if there are collisions
+        .sorted(Entry.comparingByKey())
+        .collect(Collectors.toMap(
+            PinotConfiguration::relaxEnvVarName,
+            Entry::getValue,
+            // Nothing prevents users from setting env variables like foo_bar and foo.bar
+            // This should not prevent Pinot from starting.
+            (existingValue, newValue) -> existingValue
+        ));
   }
 
   private static String relaxEnvVarName(Entry<String, String> envVarEntry) {
