@@ -38,6 +38,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.OutputStream;
 import java.nio.charset.StandardCharsets;
 import java.util.AbstractMap;
 import java.util.ArrayList;
@@ -239,6 +240,11 @@ public class JsonUtils {
     return DEFAULT_WRITER.writeValueAsString(object);
   }
 
+  public static void objectToOutputStream(Object object, OutputStream outputStream)
+      throws IOException {
+    DEFAULT_WRITER.writeValue(outputStream, object);
+  }
+
   public static String objectToPrettyString(Object object)
       throws JsonProcessingException {
     return DEFAULT_PRETTY_WRITER.writeValueAsString(object);
@@ -378,6 +384,10 @@ public class JsonUtils {
       return Collections.emptyList();
     }
 
+    if (node.isMissingNode()) {
+      return Collections.emptyList();
+    }
+
     // Value
     if (node.isValueNode()) {
       String valueAsText = node.asText();
